
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : mupad.scm
;; DESCRIPTION : Initialize mupad plugin
;; COPYRIGHT   : (C) 1999  Joris van der Hoeven
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Help menus
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(menu-bind mupad-help-menu
  ("Tutorial" (system "tm_mupad_help tutorium &"))
  ("Quick reference" (system "tm_mupad_help quickref &"))
  ("Standard library" (system "tm_mupad_help stdlib &"))
  ("What's new" (system "tm_mupad_help changes &"))
  ("SciLab interface" (system "tm_mupad_help scilab &"))
  ---
  ("Solving equations" (system "tm_mupad_help solvelib &"))
  ("Numerical calculations" (system "tm_mupad_help numeric &"))
  ("Plots" (system "tm_mupad_help plot &"))
  ---
  (-> "Algebra" (link mupad-help-algebra))
  (-> "Polynomials" (link mupad-help-polynomials))
  (-> "Calculus" (link mupad-help-calculus))
  (-> "Programming" (link mupad-help-programming))
  (-> "Domains" (link mupad-help-domains))
  (-> "Input-output" (link mupad-help-io)))

(menu-bind mupad-help-algebra
  ("Linear algebra" (system "tm_mupad_help linalg &"))
  ("Linear optimization" (system "tm_mupad_help linopt &"))
  ("Number theory" (system "tm_mupad_help numlib &"))
  ("Combinatorics" (system "tm_mupad_help combinat &"))
  ("Graph theory" (system "tm_mupad_help Network &"))
  ("Statistics" (system "tm_mupad_help stats &"))
  ("Miscellania" (system "tm_mupad_help misc &")))

(menu-bind mupad-help-polynomials
  ("Polynomials" (system "tm_mupad_help polylib &"))
  ("Orthogonal polynomials" (system "tm_mupad_help orthpoly &"))
  ("Groebner bases" (system "tm_mupad_help groebner &")))

(menu-bind mupad-help-calculus
  ("Integration" (system "tm_mupad_help intlib &"))
  ("Educational tools" (system "tm_mupad_help student &"))
  ("Integral transforms" (system "tm_mupad_help transform &"))
  ("Differential equations" (system "tm_mupad_help detools &")))

(menu-bind mupad-help-programming
  ("Data types" (system "tm_mupad_help datatypes &"))
  ("Strings" (system "tm_mupad_help stringlib &"))
  ("Lists" (system "tm_mupad_help listlib &"))
  ("Programmer's toolbox" (system "tm_mupad_help prog &"))
  ("Abstract data types" (system "tm_mupad_help adt &"))
  ("Functional programming" (system "tm_mupad_help fp &"))
  ("Type checking" (system "tm_mupad_help Type &"))
  ("Properties" (system "tm_mupad_help property &"))
  ("Pattern matching" (system "tm_mupad_help matchlib &"))
  ("User preferences" (system "tm_mupad_help Pref &"))
  ("Dynamic modules" (system "tm_mupad_help module &")))

(menu-bind mupad-help-domains
  ("Axioms, categories and domains" (system "tm_mupad_help domainref &"))
  ("Domains" (system "tm_mupad_help Dom &"))
  ("Categories" (system "tm_mupad_help Cat &"))
  ("Axioms" (system "tm_mupad_help Ax &")))

(menu-bind mupad-help-io
  ("Formatted output" (system "tm_mupad_help output &"))
  ("Importing data" (system "tm_mupad_help import &"))
  ("Generating input for other programs"
   (system "tm_mupad_help generate &")))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Specific conversion routines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (mupad-input-var-row r)
  (if (not (null? r))
      (begin
	(display ", ")
	(plugin-input (car r))
	(mupad-input-var-row (cdr r)))))

(define (mupad-input-row r)
  (display "[")
  (plugin-input (car r))
  (mupad-input-var-row (cdr r))
  (display "]"))

(define (mupad-input-var-rows t)
  (if (not (null? t))
      (begin
	(display ", ")
	(mupad-input-row (car t))
	(mupad-input-var-rows (cdr t)))))

(define (mupad-input-rows t)
  (display "matrix([")
  (mupad-input-row (car t))
  (mupad-input-var-rows (cdr t))
  (display "])"))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Initialization
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(plugin-configure mupad
  (:launch "tm_mupad")
  (:format "verbatim" "generic")
  (:session "Mupad"))

(plugin-input-converters mupad
  (rows mupad-input-rows))

(menu-extend texmacs-session-help-icons
  (if (in-mupad?)
      |
      (=> (balloon (icon "tm_help.xpm") "MuPAD documentation")
	  (link mupad-help-menu))))
