
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : def-data.scm
;; DESCRIPTION : Macros for defining TeXmacs data
;; COPYRIGHT   : (C) 1999  Joris van der Hoeven
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition of groups (should use logical programming soon)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define group-table (make-ahash-table))

(define (drd-group-decls name l)
  (define (insert member)
    (ahash-set! group-table (list name member) #t))
  (for-each insert l))

(define-macro (drd-group name . l)
  `(begin
    (drd-group-decls ',name ,(list 'quasiquote l))))

(define-macro (drd-in? x name)
  `(ahash-ref group-table (list ',name ,x)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition of tables (should use logical programming soon)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define drd-table-table (make-ahash-table))

(define (drd-table-decls name l)
  (define (insert binding)
    (let ((key (car binding))
	  (im (cadr binding)))
      (if (and (list? key) (not (null? key)) (== (car key) :or))
	  (if (not (null? (cdr key)))
	      (begin
		(insert (list (cadr key) im))
		(insert (list (cons :or (cddr key)) im))))
	  (ahash-set! drd-table-table (list name key) im))))
  (for-each insert l))

(define-macro (drd-table name . l)
  `(begin
     (drd-table-decls ',name ,(list 'quasiquote l))))

(define-macro (drd-ref name key)
  `(ahash-ref drd-table-table (list ',name ,key)))

(define-macro (drd-dispatch name . args)
  (if (= (length args) 1)
      `(eval (list (drd-ref ,name (car ,(car args))) ,(car args)))
      `(eval (list (drd-ref ,name ,(car args)) ,@(cdr args)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition of tables (will this be useful one day?)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (define-table-decls h l)
  (define (insert binding)
    (ahash-set! h (car binding) (cadr binding)))
  (for-each insert l))

(define-macro (define-table name . l)
  `(begin
     (define ,name (make-ahash-table))
     (define-table-decls ,name ,(list 'quasiquote l))))
