
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : def-kbd.scm
;; DESCRIPTION : Definition of keyboard shortcuts/wildcards
;; COPYRIGHT   : (C) 1999  Joris van der Hoeven
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition of keyboard wildcards
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (kbd-wildcards-sub l post)
  (if (not (null? l))
      (let* ((w (car l))
	     (key (car w))
	     (im (cadr w))
	     (left (if (>= (length w) 3) (caddr w) #f))
	     (right (if (>= (length w) 4) (cadddr w) #t)))
	(insert-kbd-wildcard key im post left right)
	(kbd-wildcards-sub (cdr l) post))))

(define (kbd-wildcards-body l)
  (cond ((null? l) (noop))
	((== (car l) 'pre) (kbd-wildcards-sub (cdr l) #f))
	((== (car l) 'post) (kbd-wildcards-sub (cdr l) #t))
	(else (kbd-wildcards-sub l #t))))

(define-macro (kbd-wildcards . l)
  `(kbd-wildcards-body ,(list 'quasiquote l)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definition of keyboard shortcuts
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (kbd-map-sub l)
  (if (and (>= (length l) 2) (list? (cadr l)) (string? (caadr l)))
      (system-set-keymap (car l) (cdr l))
      (system-set-keymap (eval (car l)) (eval (cadr l)))))

(define (kbd-map-body l)
  (cond ((null? l) (noop))
	((symbol? (car l)) (kbd-map-sub (cons (list (car l)) (cdr l))))
	((and (list? (car l)) (not (null? (car l))) (== (caar l) :or))
	 (kbd-map-sub (cons (cdar l) (cdr l))))
	(else (kbd-map-sub (cons '(always?) l)))))

(define-macro (kbd-map . l)
  `(kbd-map-body ,(list 'quasiquote l)))

(define (kbd-remove-body l)
  (if (and (>= (length l) 2) (string? (cadr l)))
      (system-remove-keymap (car l) (cdr l))
      (system-remove-keymap (eval (car l)) (eval (cadr l)))))

(define-macro (kbd-remove . l)
  `(kbd-remove-body ,(list 'quasiquote l)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Deprecated: for compatability with older versions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define-macro (set-pre-wildcards . l) (kbd-wildcards-sub l #f))
(define-macro (set-post-wildcards . l) (kbd-wildcards-sub l #t))
(define-macro (set-keymap . l) (kbd-map-sub l))
(define-macro (remove-keymap . l) (kbd-remove-body l))
