
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : ahash-table.scm
;; DESCRIPTION : adaptive hash tables
;; COPYRIGHT   : (C) 2003  Joris van der Hoeven
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Adaptive hash tables
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (make-ahash-table)
  (cons (make-hash-table 1) 0))

(define (ahash-ref h key)
  (hash-ref (car h) key))

(define (ahash-get-handle h key)
  (hash-get-handle (car h) key))

(define (ahash-size h)
  (cdr h))

(define (ahash-slots! h new-size)
  (let ((new-h (make-hash-table new-size)))
    (hash-fold (lambda (key value dummy) (hash-set! new-h key value))
	       #f (car h))
    (set-car! h new-h)))

(define (ahash-set! h key value)
  (if (hash-get-handle (car h) key)
      (hash-set! (car h) key value)
      (begin
	(if (>= (cdr h) (vector-length (car h)))
	    (ahash-slots! h (+ (* 2 (vector-length (car h))) 1)))
	(set-cdr! h (+ (cdr h) 1))
	(hash-set! (car h) key value))))

(define (ahash-remove! h key)
  (let ((removed (hash-remove! (car h) key)))
    (if removed
	(begin
	  (set-cdr! h (- (cdr h) 1))
	  (if (< (+ (* 4 (cdr h)) 1) (vector-length (car h)))
	      (ahash-slots! h (quotient (vector-length (car h)) 2)))))
    removed))

(define (ahash-fold h)
  (hash-fold (car h)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Simple definition of hash tables
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (define-table-decls h l)
  (define (insert binding)
    (ahash-set! h (car binding) (cdr binding)))
  (for-each insert l))

(define-macro (define-table name . l)
  `(begin
     (define ,name (make-ahash-table))
     (define-table-decls ,name ,(list 'quasiquote l))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Protected objects
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;(define object-serial 0)
;(define object-table (make-ahash-table))
;
;(define (object-create obj)
;  (display* "Protect " obj "\n")
;  (set! object-serial (+ object-serial 1))
;  (ahash-set! object-table object-serial obj)
;  (display* object-table "\n")
;  object-serial)
;
;(define (object-lookup handle)
;  (ahash-ref object-table handle))
;
;(define (object-destroy handle)
;  (display* "Unprotect " (object-lookup handle) "\n")
;  (ahash-remove! object-table handle)
;  (display* object-table "\n"))

;(define object-stack '(()))
;
;(define (object-create obj)
;  (display* "Protect " obj "\n")
;  (set! object-stack (cons (list obj) object-stack))
;  (display* "  -> " object-stack "\n")
;  object-stack)
;
;(define (object-lookup handle)
;  (caar handle))
;
;(define (object-shrink handle)
;  (if (and (pair? handle) (null? (car handle)))
;      (object-shrink (cdr handle))
;      handle))
;
;(define (object-destroy handle)
;  (display* "Unprotect " (object-lookup handle) "\n")
;  (set-car! handle '())
;  (set-cdr! handle (object-shrink (cdr handle)))
;  (display* "  -> " object-stack "\n"))
