
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : init-texmacs.scm
;; DESCRIPTION : This is the standard TeXmacs initialization file
;; COPYRIGHT   : (C) 1999  Joris van der Hoeven, David Allouche
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(debug-set! stack 1000000)

;;; make eval from guile>=1.6.0 backwards compatible
(catch 'wrong-number-of-args
       (lambda () (eval 1))
       (lambda arg
	 (let ((default-eval eval))
	   (set! eval (lambda (form . env)
			(cond ((null? form) (list))
			      ((null? env) (primitive-eval form))
			      (else (default-eval form (car env)))))))))

;(display "Booting TeXmacs extensions to scheme\n")
(exec-file "$TEXMACS_PATH/progs/extensions/boot" "abbrevs.scm")
(exec-file "$TEXMACS_PATH/progs/extensions/boot" "debug.scm")
(exec-file "$TEXMACS_PATH/progs/extensions/boot" "srfi.scm")
(exec-file "$TEXMACS_PATH/progs/extensions/boot" "ahash-table.scm")
(exec-file "$TEXMACS_PATH/progs/extensions/regexp" "match.scm")
(exec-file "$TEXMACS_PATH/progs/extensions/drd" "bind.scm")
(exec-file "$TEXMACS_PATH/progs/extensions/drd" "unify.scm")
(exec-file "$TEXMACS_PATH/progs/extensions/drd" "rules.scm")
(exec-file "$TEXMACS_PATH/progs/extensions/drd" "query.scm")
(exec-file "$TEXMACS_PATH/progs/extensions/drd" "def-data.scm")
(exec-file "$TEXMACS_PATH/progs/extensions/texmacs" "def-funcs.scm")
(exec-file "$TEXMACS_PATH/progs/extensions/library" "base.scm")
(exec-file "$TEXMACS_PATH/progs/extensions/library" "list.scm")
(exec-file "$TEXMACS_PATH/progs/extensions/gui" "menu-define.scm")
(exec-file "$TEXMACS_PATH/progs/extensions/gui" "menu-widget.scm")
(exec-file "$TEXMACS_PATH/progs/extensions/gui" "kbd-define.scm")

;(display "Booting TeXmacs primitives\n")
(exec-file "$TEXMACS_PATH/progs/texmacs/base" "general.scm")
(exec-file "$TEXMACS_PATH/progs/texmacs/edit" "hybrid.scm")
(exec-file "$TEXMACS_PATH/progs/texmacs/edit" "modify.scm")
(exec-file "$TEXMACS_PATH/progs/texmacs/edit" "table.scm")
(exec-file "$TEXMACS_PATH/progs/texmacs/edit" "style.scm")
(exec-file "$TEXMACS_PATH/progs/texmacs/edit" "fold.scm")
(exec-file "$TEXMACS_PATH/progs/texmacs/plugin" "plugin.scm")
(exec-file "$TEXMACS_PATH/progs/texmacs/plugin" "convert.scm")
(exec-file "$TEXMACS_PATH/progs/texmacs/base" "checkmarks.scm")
(exec-file "$TEXMACS_PATH/progs/texmacs/base" "preferences.scm")
(pre-initialize-preferences)

;(display "Booting menus\n")
(exec-file "$TEXMACS_PATH/progs/menu" "basic-menus.scm")
(exec-file "$TEXMACS_PATH/progs/menu" "insert-menu.scm")
(exec-file "$TEXMACS_PATH/progs/menu" "text-menu.scm")
(exec-file "$TEXMACS_PATH/progs/menu" "mathematics-menu.scm")
(exec-file "$TEXMACS_PATH/progs/menu" "table-menu.scm")
(exec-file "$TEXMACS_PATH/progs/menu" "format-menu.scm")
(exec-file "$TEXMACS_PATH/progs/menu" "document-menu.scm")
(exec-file "$TEXMACS_PATH/progs/menu" "main-menu.scm")
(exec-file "$TEXMACS_PATH/progs/menu" "icon-menu.scm")

;(display "Booting keyboard\n")
(exec-file "$TEXMACS_PATH/progs/keyboard" "wildcards.scm")
(exec-file "$TEXMACS_PATH/progs/keyboard" "latex.scm")
(exec-file "$TEXMACS_PATH/progs/keyboard" "circulate.scm")
(exec-file "$TEXMACS_PATH/progs/keyboard" "kbd-general.scm")
(exec-file "$TEXMACS_PATH/progs/keyboard" "kbd-preamble.scm")
(exec-file "$TEXMACS_PATH/progs/keyboard" "kbd-text.scm")
(exec-file "$TEXMACS_PATH/progs/keyboard" "kbd-math.scm")
(exec-file "$TEXMACS_PATH/progs/keyboard" "kbd-table.scm")

;(display "Booting fonts\n")
(if (use-ec-fonts?)
    (exec-file "$TEXMACS_PATH/progs/fonts" "ec-fonts.scm")
    (exec-file "$TEXMACS_PATH/progs/fonts" "cm-fonts.scm"))
(exec-file "$TEXMACS_PATH/progs/fonts" "adobe-fonts.scm")
(exec-file "$TEXMACS_PATH/progs/fonts" "x-fonts.scm")
(exec-file "$TEXMACS_PATH/progs/fonts" "math-fonts.scm")
(exec-file "$TEXMACS_PATH/progs/fonts" "foreign-fonts.scm")
(exec-file "$TEXMACS_PATH/progs/fonts" "misc-fonts.scm")

;(display "Booting converters\n")
(exec-file "$TEXMACS_PATH/progs/convert/tools" "tmpre.scm")
(exec-file "$TEXMACS_PATH/progs/convert/tools" "tmtable.scm")
(exec-file "$TEXMACS_PATH/progs/convert/tools" "tmlength.scm")
(exec-file "$TEXMACS_PATH/progs/convert/tmtm" "tmtm-eqns.scm")
(exec-file "$TEXMACS_PATH/progs/convert/tmtm" "tmtm-brackets.scm")
(exec-file "$TEXMACS_PATH/progs/convert/tmtex" "tmtex.scm")
(exec-file "$TEXMACS_PATH/progs/convert/tmtex" "tmtex-preamble.scm")
(exec-file "$TEXMACS_PATH/progs/convert/tmtex" "texout.scm")
(exec-file "$TEXMACS_PATH/progs/convert/tmhtml" "tmhtml.scm")
(exec-file "$TEXMACS_PATH/progs/convert/tmhtml" "htmlout.scm")
(exec-file "$TEXMACS_PATH/progs/convert/tools" "output.scm")
(exec-file "$TEXMACS_PATH/progs/convert/doc" "tmdoc.scm")

;(display "Booting miscellaneous stuff\n")
(connection-declare "ispell" "ispell -a")
(connection-format "ispell" "verbatim" "ispell")
(set-printer-paper-type (get-default-paper-size))

;(display "Initializing user preferences\n")
(initialize-preferences)
(menu-extend page-setup-menu ,@(compute-preferences-menu page-setup-tree))
(menu-extend preferences-menu ,@(compute-preferences-menu preferences-tree))
;(display "Initialization done\n")
