
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : basic-menus.scm
;; DESCRIPTION : basic menus
;; COPYRIGHT   : (C) 1999  Joris van der Hoeven
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The File menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(menu-bind print-menu
  ("Preview with ghostview" (preview-with-ghostview))
  ---
  ("Print all" (print))
  ("Print page selection" ...
   (interactive '("First page:" "Last page:") 'print-pages))
  ("Print all to file" ...
   (choose-file "Print all to file" "postscript" 'print-to-file))
  ("Print page selection to file" ...
   (interactive '("First page:" "Last page:")
		'choose-file-and-print-page-selection)))

(menu-bind file-menu
  ("New" (new-buffer))
  ("Load" ... (choose-file "Load file" "" 'load-buffer))
  ("Save" (save-buffer))
  ("Save as" ...
   (choose-file "Save TeXmacs file" "TeXmacs" 'save-buffer))
  ("Revert" ... (revert-buffer))
  ---
  (-> "Page setup" (link page-setup-menu))
  (-> "Print" (link print-menu))
  (-> "Import"
      ("Verbatim" ...
       (choose-file "Load verbatim file" "" 'load-verbatim-buffer))
      ("LaTeX" ... (choose-file "Load LaTeX file" "latex"
				'load-latex-buffer))
      ("HTML" ... (choose-file "Load html file" "html" 'load-html-buffer))
      ("Scheme" ...
       (choose-file "Load scheme file" "scheme" 'load-scheme-buffer)))
  (-> "Export"
      ("Verbatim" ...
       (choose-file "Save verbatim file" "" 'save-verbatim-buffer))
      ("LaTeX" ...
       (choose-file "Save LaTeX file" "latex" 'save-latex-buffer))
      ("HTML" ...
       (choose-file "Save HTML file" "html" 'save-html-buffer))
      ("Scheme" ...
       (choose-file "Save scheme file" "scheme" 'save-scheme-buffer))
      ("PostScript" ...
       (choose-file "Save postscript file" "postscript" 'print-to-file)))
  ---
  ("Close buffer" (safely-kill-buffer))
  ("Close TeXmacs" (safely-quit-TeXmacs)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The Go menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(menu-bind bookmarks-menu)

(menu-bind go-menu
  (link buffer-menu)
  (if (not (null? (menu-get 'bookmarks-menu)))
      ---
      (link bookmarks-menu)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The Edit menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(menu-bind edit-menu
  ("Undo" (undo))
  ("Redo" (redo))
  ---
  (when (selection-active-any?)
	("Copy" (copy "primary"))
	("Cut" (cut "primary")))
  ("Paste" (paste "primary"))
  ("Clear" (selection-clear "primary"))
  ---
  ("Search" ... (search-start #t))
  ("Replace" ...
   (interactive '("Replace:" "Replace by:") 'replace-start-forward))
  ("Spell" ... (spell-start))
  ---
  (when (selection-active-any?)
	(-> "Copy to"
	    ("TeXmacs" (copy-export "TeXmacs" "primary"))
	    ("LaTeX" (copy-export "latex" "primary"))
	    ("HTML" (copy-export "html" "primary"))
	    ("Scheme" (copy-export "scheme" "primary"))
	    ("Verbatim" (copy-export "verbatim" "primary"))
	    ---
	    ("Primary" (copy "primary"))
	    ("Secondary" (copy "secondary"))
	    ("Ternary" (copy "ternary"))
	    ---
	    ("Other" ... (interactive '("Copy to:") 'copy)))
	(-> "Cut to"
	    ("TeXmacs" (cut-export "TeXmacs" "primary"))
	    ("LaTeX" (cut-export "latex" "primary"))
	    ("HTML" (cut-export "html" "primary"))
	    ("Scheme" (cut-export "scheme" "primary"))
	    ("Verbatim" (cut-export "verbatim" "primary"))
	    ---
	    ("Primary" (cut "primary"))
	    ("Secondary" (cut "secondary"))
	    ("Ternary" (cut "ternary"))
	    ---
	    ("Other" ... (interactive '("Cut to:") 'cut))))
  (-> "Paste from"
      ("TeXmacs" (paste-import "TeXmacs" "primary"))
      ("LaTeX" (paste-import "latex" "primary"))
      ("HTML" (paste-import "html" "primary"))
      ("Scheme" (paste-import "scheme" "primary"))
      ("Verbatim" (paste-import "verbatim" "primary"))
      ---
      ("Primary" (paste "primary"))
      ("Secondary" (paste "secondary"))
      ("Ternary" (paste "ternary"))
      ---
      ("Other" ... (interactive '("Paste from:") 'paste)))
  ---
  (-> "Preferences"
      (link preferences-menu)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The Project menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(menu-bind project-menu
  (link project-buffer-menu))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The View menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(menu-bind view-menu
  ("Open new window" (open-window))
  ("Load in new window" ... (choose-file "Load file" "" 'load-in-new-window))
  ("Clone window" (clone-window))
  ("Close window" (safely-kill-window))
  ---
  ("Header" (toggle-visible-header))
  (when (visible-header?)
	("Main icon bar" (toggle-visible-icon-bar 0))
	("Context dependent icons" (toggle-visible-icon-bar 1))
	("User provided icons" (toggle-visible-icon-bar 2)))
  ("Status bar" (toggle-visible-footer))
  ---
  ("Presentation mode" (toggle-full-screen-mode))
  (-> "Shrinking factor"
      ("1" (set-shrinking-factor 1))
      ("2" (set-shrinking-factor 2))
      ("3" (set-shrinking-factor 3))
      ("4" (set-shrinking-factor 4))
      ("5" (set-shrinking-factor 5))
      ("7" (set-shrinking-factor 7))
      ("10" (set-shrinking-factor 10))
      ---
      ("Other" ...
       (interactive '("Shrinking factor:") 'other-shrinking-factor))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The Preferences menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define page-setup-tree
  '((string ("Printing command" "printing command"))
    (enum ("Paper type" "paper type")
	  "A3" "A4" "A5" "B4" "B5" "B6"
	  "Letter" "Legal" "Executive" "Lecture note"
	  *)
    (enum ("Printer dpi" "printer dpi")
	  "150" "200" "300" "400" "600" "800" "1200"
	  *)))

(define preferences-tree
  `((enum ("Look and feel" "look and feel")
	  ("Emacs" "emacs")
;	  ("Windows" "windows")
	  ("Old style" "old style"))
;   (enum ("Profile" "profile")
;	  ("Beginner" "beginner")
;	  ("Normal" "normal")
;	  ("Advanced" "advanced"))
    (enum ("Language" "language")
	  ("British" "british")
	  ("Czech" "czech")
	  ("Dutch" "dutch")
	  ("English" "english")
	  ("Finnish" "finnish")
	  ("French" "french")
	  ("German" "german")
	  ("Hungarian" "hungarian")
	  ("Italian" "italian")
	  ("Polish" "polish")
	  ("Portuguese" "portuguese")
	  ("Romanian" "romanian")
	  ("Russian" "russian")
	  ("Spanish" "spanish")
	  ("Swedish" "swedish")
	  ("Ukrainian" "ukrainian"))
    (-> "Keyboard"
	(enum ("A modifier" "A")
	      ("Default" "default")
	      ("Equivalent for#Mod1" "Mod1")
	      ("Equivalent for#Mod2" "Mod2")
	      ("Equivalent for#Mod3" "Mod3")
	      ("Equivalent for#Mod4" "Mod4")
	      ("Equivalent for#Mod5" "Mod5"))
	(enum ("M modifier" "M")
	      ("Default" "default")
	      ("Equivalent for#Mod1" "Mod1")
	      ("Equivalent for#Mod2" "Mod2")
	      ("Equivalent for#Mod3" "Mod3")
	      ("Equivalent for#Mod4" "Mod4")
	      ("Equivalent for#Mod5" "Mod5"))
	(enum ("H modifier" "H")
	      ("Default" "default")
	      ("Equivalent for#Mod1" "Mod1")
	      ("Equivalent for#Mod2" "Mod2")
	      ("Equivalent for#Mod3" "Mod3")
	      ("Equivalent for#Mod4" "Mod4")
	      ("Equivalent for#Mod5" "Mod5"))
	---
	(enum ("Alt key" "alt")
	      ("Don't remap" "default")
	      ("Map to#A modifier" "A")
	      ("Map to#M modifier" "M")
	      ("Map to#H modifier" "H"))
	(enum ("Meta key" "meta")
	      ("Don't remap" "default")
	      ("Map to#A modifier" "A")
	      ("Map to#M modifier" "M")
	      ("Map to#H modifier" "H"))
	(enum ("Windows key" "windows")
	      ("Don't remap" "default")
	      ("Map to#A modifier" "A")
	      ("Map to#M modifier" "M")
	      ("Map to#H modifier" "H"))
	(enum ("Caps-lock key" "caps-lock")
	      ("Don't remap" "default")
	      ("Map to#A modifier" "A")
	      ("Map to#M modifier" "M")
	      ("Map to#H modifier" "H"))
	---
	(enum ("Cyrillic input method" "cyrillic input method")
	      ("Translit" "translit")
	      ("Jcuken" "jcuken")
	      ("Yawerty" "yawerty")
	      ("Koi8-r" "koi8-r")
	      ("Cp1251" "cp1251")))
    (-> "Printer" . ,page-setup-tree)
    (-> "View"
	(toggle ("Header" "header"))
	(toggle ("Main icon bar" "main icon bar"))
	(toggle ("Context dependent icons" "context dependent icons"))
	(toggle ("User provided icons" "user provided icons"))
	(toggle ("Status bar" "status bar"))
	(enum ("Shrinking factor" "shrinking factor")
	      "1" "2" "3" "4" "5" "7" "10" *))
    (enum ("Security" "security")
	  ("Accept no scripts" "accept no scripts")
	  ("Prompt on scripts" "prompt on scripts")
	  ("Accept all scripts" "accept all scripts"))))

(menu-bind page-setup-menu)
(menu-bind preferences-menu)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The Tools menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(menu-bind test-menu)

(menu-bind tools-menu
  (-> "Execute"
      ("Execute system command" ... (interactive '("Command:") 'system))
      ("Evaluate scheme expression" ...
       (interactive '("Command:") 'TeXmacs-eval)))
  (-> "Selections"
      (-> "Import"
	  ("TeXmacs" (selection-set-import "TeXmacs"))
	  ("LaTeX" (selection-set-import "latex"))
	  ("HTML" (selection-set-import "html"))
	  ("Scheme" (selection-set-import "scheme"))
	  ("Verbatim" (selection-set-import "verbatim")))
      (-> "Export"
	  ("TeXmacs" (selection-set-export "TeXmacs"))
	  ("LaTeX" (selection-set-export "latex"))
	  ("HTML" (selection-set-export "html"))
	  ("Scheme" (selection-set-export "scheme"))
	  ("Verbatim" (selection-set-export "verbatim"))))
  (-> "Update"
      ("Image links" (postscript-gc))
      ("Inclusions" (inclusions-gc))
      ("Styles" (style-clear-cache)))
  (if (not (null? (menu-get 'test-menu)))
      ---
      (link test-menu)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The Help menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(menu-bind help-menu
  ("Welcome" (load-help-buffer "about/welcome/welcome"))
  ---
  (-> "Configuration"
      ("Browse" (load-help-buffer "main/config/man-configuration"))
      ---
      ("Introduction"
       (load-help-article "main/config/man-intro"))
      ("Keyboard modifiers"
       (load-help-article "main/config/man-config-kbd-modkeys"))
      ("Users of Cyrillic languages"
       (load-help-article "main/config/man-russian")))
  (-> "Manual"
      ("Browse" (load-help-buffer "main/man-manual"))
      ---
      ("Getting started"
       (load-help-article "main/start/man-getting-started"))
      ("Typing simple texts"
       (load-help-article "main/text/man-text"))
      ("Mathematical formulas"
       (load-help-article "main/math/man-math"))
      ("Tabular material"
       (load-help-article "main/table/man-table"))
      ("Automatically generated content"
       (load-help-article "main/links/man-links"))
      ("Editing tools"
       (load-help-article "main/editing/man-editing-tools"))
      ("Advanced layout features"
       (load-help-article "main/layout/man-layout"))
      ("TeXmacs as an interface"
       (load-help-article "main/interface/man-itf"))
      ("TeXmacs style files"
       (load-help-article "devel/style/style"))
      ("Summary of the principal TeXmacs tags"
       (load-help-article "main/styles/std-dtds/dtds"))
      ("Customizing TeXmacs"
       (load-help-article "main/scheme/man-scheme"))
      ("Compatibility with other formats"
       (load-help-article "main/convert/man-convert")))
  (when (help-file-exists? "tutorial/tut-tutorial.en.tm")
	(-> "Tutorial"
	    ("Browse" (load-help-buffer "tutorial/tut-tutorial"))
	    ---
	    ("First contact"
	     (load-help-article "tutorial/start/tut-start"))
	    ("Typing a simple text"
	     (load-help-article "tutorial/start/tut-simple-text"))))
  (-> "Styles"
      ("Browse" (load-help-buffer "devel/style/style"))
      ---
      ("Introduction"
       (load-help-article "devel/style/style-intro"))
      ("The standard TeXmacs styles"
       (load-help-article "devel/style/standard/standard"))
      ("Writing your own style files"
       (load-help-article "devel/style/design/design"))
      ("Customizing the TeXmacs styles"
       (load-help-article "devel/style/customize/customize")))
  (-> "About"
      ("Browse" (load-help-buffer "about/about"))
      ---
      ("Summary"
       (load-help-article "about/about-summary"))
      ("License"
       (load-help-buffer "$TEXMACS_PATH/LICENSE"))
      ("Philosophy"
       (load-help-article "about/philosophy/philosophy"))
      ("The TeXmacs authors"
       (load-help-article "about/authors/authors"))
      ---
      (when (help-file-exists? "about/changes/changes-recent")
	    ("What is new"
	     (load-help-article "about/changes/changes-recent")))
      ("Major changes"
       (load-help-article "about/changes/changes-main"))
      ("Change log"
       (load-help-article "about/changes/change-log"))
      ---
      ("Original welcome message"
       (load-help-article "about/welcome/first")))
  ---
  (-> "Help us"
      ("Browse" (load-help-buffer "about/contribute/contribute"))
      ---
      ("Use TeXmacs"
       (load-help-article "about/contribute/using/using"))
      ("Making donations"
       (load-help-article "about/contribute/material/donations"))
      ("Documentation"
       (load-help-article "about/contribute/documentation/documentation"))
      ("Internationalization"
       (load-help-article "about/contribute/translate/translate"))
      ("Writing data converters"
       (load-help-article "about/contribute/converters/converters"))
      ("Porting TeXmacs to other platforms"
       (load-help-article "about/contribute/porting/porting"))
      ("Interfacing TeXmacs with other systems"
       (load-help-article "about/contribute/interfaces/interfaces"))
      ("Become a TeXmacs developer"
       (load-help-article "about/contribute/develop/develop")))
  (-> "Projects"
      ("Browse" (load-help-buffer "about/projects/projects"))
      ---
      ("Improving the current implementation"
       (load-help-buffer "about/projects/improvements"))
      ("Plans for the future"
       (load-help-buffer "about/projects/future")))
  (-> "Document format"
      ("Browse" (load-help-buffer "devel/format/format"))
      ---
      ("Documents are trees"
       (load-help-article "devel/format/trees"))
      ("The leaves of TeXmacs trees"
       (load-help-article "devel/format/leaves"))
      ("The primitive TeXmacs constructs"
       (load-help-article "devel/format/primitives"))
      ("System environment variables"
       (load-help-article "devel/format/env-vars"))
      ("Planned changes"
       (load-help-article "devel/format/planned-changes")))
  (-> "Interfacing"
      ("Browse" (load-help-buffer "devel/plugin/plugin"))
      ---
      ("The mycas example"
       (load-help-article "devel/plugin/mycas"))
      ("Your first interface"
       (load-help-article "devel/plugin/first"))
      ("Inclusion into TeXmacs"
       (load-help-article "devel/plugin/include"))
      ("Further customization"
       (load-help-article "devel/plugin/customization"))
      ("Dynamic libraries"
       (load-help-article "devel/plugin/dynlibs"))
      ("Plans for the future"
       (load-help-article "devel/plugin/plans")))
  (-> "Source code"
      ("Browse" (load-help-buffer "devel/source/source"))
      ---
      ("General architecture of TeXmacs"
       (load-help-article "devel/source/architecture"))
      ("Basic data types"
       (load-help-article "devel/source/types"))
      ("Converters to other data formats"
       (load-help-article "devel/source/conversions"))
      ("The graphical user interface"
       (load-help-article "devel/source/gui"))
      ("TeXmacs fonts"
       (load-help-article "devel/source/fonts"))
      ("Mathematical typesetting"
       (load-help-article "devel/source/maths"))
      ("The boxes produced by the typesetter"
       (load-help-article "devel/source/boxes")))
  ---
  (-> "Full manuals"
      ("User manual" (load-help-book "main/man-user-manual"))
      (when (help-file-exists? "tutorial/tut-tutorial.en.tm")
	    ("Tutorial" (load-help-book "tutorial/tut-tutorial")))
      ("Developers guide" (load-help-book "devel/source/source"))
      ---
      (when (style-has? "tmdoc-style")
	    ("Compile article" (tmdoc-expand-this 'tmdoc-title))
	    ("Compile book" (tmdoc-expand-this 'title))))
  (when (file-exists? "$PATH" "wget")
	(-> "Online help"
	    ("Browse web" (load-help-online "index.en.tm"))
	    ("Update from web" (update-help-online)))))
