
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : icon-menu.scm
;; DESCRIPTION : the default icon menus of TeXmacs
;; COPYRIGHT   : (C) 1999  Joris van der Hoeven
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Main icons
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(menu-bind texmacs-main-icons
  (=> (balloon (icon "tm_new.xpm") "Open a new buffer")
      ("New buffer" (new-buffer))
      ("Open new window" (open-window))
      ("Clone window" (clone-window)))
  (=> (balloon (icon "tm_open.xpm") "Load a file")
      ("Load buffer" ... (choose-file "Load file" "" 'load-buffer))
      ("Revert buffer" ... (revert-buffer))
      ("Load in new window" ...
       (choose-file "Load file" "" 'load-in-new-window))
      ---
      ("Import#Verbatim" ...
       (choose-file "Load verbatim file" "" 'load-verbatim-buffer))
      ("Import#LaTeX" ...
       (choose-file "Load LaTeX file" "latex" 'load-latex-buffer))
      ("Import#HTML" ...
       (choose-file "Load html file" "html" 'load-html-buffer))
      ("Import#Scheme" ...
       (choose-file "Load scheme file" "scheme" 'load-scheme-buffer)))
  (=> (balloon (icon "tm_save.xpm") "Save this buffer")
      ("Save buffer" (save-buffer))
      ("Save buffer as" ...
       (choose-file "Save TeXmacs file" "TeXmacs" 'save-buffer))
      ---
      ("Export as#Verbatim" ...
       (choose-file "Save verbatim file" "" 'save-verbatim-buffer))
      ("Export as#LaTeX" ...
       (choose-file "Save LaTeX file" "latex" 'save-latex-buffer))
      ("Export as#HTML" ...
       (choose-file "Save html file" "html" 'save-html-buffer))
      ("Export as#Scheme" ...
       (choose-file "Save scheme file" "scheme" 'save-scheme-buffer))
      ("Export as#PostScript" ...
       (choose-file "Save postscript file" "postscript" 'print-to-file)))
  (=> (balloon (icon "tm_print.xpm") "Print") (link print-menu))
  (=> (balloon (icon "tm_style.xpm") "Select a document style")
      ("No style" (init-style "none"))
      ---
      (link style-menu)
      ---
      ("Other style" ... (interactive '("Document style:") 'init-style)))
  (=> (balloon (icon "tm_language.xpm") "Select a language")
      ("British"
       (begin
	 (init-language "british")
	 (set-output-language "british")))
      ("Czech"
       (begin
	 (init-language "czech")
	 (set-output-language "czech")))
      ("Dutch"
       (begin
	 (init-language "dutch")
	 (set-output-language "dutch")))
      ("English"
       (begin
	 (init-language "english")
	 (set-output-language "english")))
      ("Finnish"
       (begin
	 (init-language "finnish")
	 (set-output-language "finnish")))
      ("French"
       (begin
	 (init-language "french")
	 (set-output-language "french")))
      ("German"
       (begin
	 (init-language "german")
	 (set-output-language "german")))
      ("Hungarian"
       (begin
	 (init-language "hungarian")
	 (set-output-language "hungarian")))
      ("Italian"
       (begin
	 (init-language "italian")
	 (set-output-language "italian")))
      ("Polish"
       (begin
	 (init-language "polish")
	 (set-output-language "polish")))
      ("Portuguese"
       (begin
	 (init-language "portuguese")
	 (set-output-language "portuguese")))
      ("Romanian"
       (begin
	 (init-language "romanian")
	 (set-output-language "romanian")))
      ("Russian"
       (begin
	 (init-language "russian")
	 (set-output-language "russian")))
      ("Spanish"
       (begin
	 (init-language "spanish")
	 (set-output-language "spanish")))
      ("Swedish"
       (begin
	 (init-language "swedish")
	 (set-output-language "swedish")))
      ("Ukrainian"
       (begin
	 (init-language "ukrainian")
	 (set-output-language "ukrainian"))))
  (=> (balloon (icon "tm_cancel.xpm") "Close")
      ("Close buffer" (safely-kill-buffer))
      ("Close window" (safely-kill-window))
      ("Close TeXmacs" (safely-quit-TeXmacs)))
  |
  ((balloon (icon "tm_cut.xpm") "Cut text#(C-w)") (cut "primary"))
  ((balloon (icon "tm_copy.xpm") "Copy text#(M-w)") (copy "primary"))
  ((balloon (icon "tm_paste.xpm") "Paste text#(C-y)") (paste "primary"))
  (if (not (in-search-mode?))
      ((balloon (icon "tm_find.xpm") "Find text#(C-s)") (search-start #t)))
  (if (in-search-mode?)
      ((balloon (icon "tm_find_next.xpm") "Find next match#(C-s)")
       (search-button-next)))
  ((balloon (icon "tm_replace.xpm") "Query replace#(C-=)")
   (interactive '("Replace:" "Replace by:") 'replace-start-forward))
  ((balloon (icon "tm_spell.xpm") "Check text for spelling errors#(M-$)")
   (spell-start))
  (if (not (help-buffer?))
      ((balloon (icon "tm_undo.xpm") "Undo last changes#(M-[)") (undo))
      ((balloon (icon "tm_redo.xpm") "Redo undone changes#(M-])") (redo)))
  (if (help-buffer?)
      ((balloon (icon "tm_back.xpm") "Browse back") (browse-help -1))
      ((balloon (icon "tm_forward.xpm") "Browse forward") (browse-help 1)))
  |
  (=> (balloon (icon "tm_table.xpm") "Insert a table")
      (link insert-table-menu))
  (=> (balloon (icon "tm_image.xpm") "Insert a picture")
      (link insert-image-menu))
  (=> (balloon (icon "tm_link.xpm") "Insert a link")
      (link insert-link-menu))
  (=> (balloon (icon "tm_switch.xpm") "Switching and folding")
      (link insert-switch-menu)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Icons for text mode
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(menu-bind texmacs-text-icons
  (if (and (style-has? "header-title-dtd")
	   (not (style-has? "header-letter-dtd"))
	   (not (style-has? "header-exam-dtd")))
      (=> (balloon (icon "tm_title.xpm") "Enter a title")
	  (link title-menu)))
  (if (style-has? "header-letter-dtd")
      (=> (balloon (icon "tm_title.xpm") "Make a letter environment")
	  (link letter-header-menu)))
  (if (style-has? "header-exam-dtd")
      (=> (balloon (icon "tm_title.xpm") "Enter title information")
	  (link exam-header-menu)))
  (if (style-has? "book-style")
      (=> (balloon (icon "tm_chapter.xpm") "Start a new chapter")
	  (link chapter-menu)))
  (if (and (style-has? "section-latex-dtd")
	   (not (style-has? "header-exam-dtd")))
      (=> (balloon (icon "tm_section.xpm") "Start a new section")
	  (link section-menu)))
  (if (style-has? "std-markup-dtd")
      (=> (balloon (icon "tm_theorem.xpm") "Insert an environment")
	  (link environment-menu)))
  (=> (balloon (icon "tm_parstyle.xpm") "Set paragraph mode")
      ((balloon (icon "tm_left.xpm") "Align text to the left")
       (make-line-with "paragraph mode" "left"))
      ((balloon (icon "tm_center.xpm") "Center text")
       (make-line-with "paragraph mode" "center"))
      ((balloon (icon "tm_right.xpm") "Align text to the right")
       (make-line-with "paragraph mode" "right"))
      ((balloon (icon "tm_justify.xpm") "Justify text")
       (make-line-with "paragraph mode" "justify")))
  (=> (balloon (icon "tm_parindent.xpm") "Set paragraph margins")
      ("Left margin" (interactive '("Left margin:") 'set-left-margin))
      ("Right margin" (interactive '("Right margin:") 'set-right-margin))
      ("First indentation"
       (interactive '("First indentation:") 'set-first-indent)))
  (if (style-has? "env-float-dtd")
      (if (not (inside? "float"))
	  (=> (balloon (icon "tm_pageins.xpm") "Make a page insertion")
	      (link insert-page-insertion-menu)))
      (if (inside? "float")
	  (=> (balloon (icon "tm_floatpos.xpm") "Position floating object")
	      (link position-float-menu))))
;;((balloon (icon "tm_footnote.xpm") "Insert a footnote") ())
;;((balloon (icon "tm_margin.xpm") "Insert a marginal note") ())
;;((balloon (icon "tm_floating.xpm") "Insert a floating object") ())
;;((balloon (icon "tm_multicol.xpm") "Start multicolumn context") ())
  (if (style-has? "section-automatic-dtd")
      (=> (balloon (icon "tm_index.xpm")
		   "Insert automatically generated content")
	  (link automatic-menu)))
  (if (style-has? "std-list-dtd")
      |
      (=> (balloon (icon "tm_itemize.xpm") "Itemize")
	  (link itemize-menu))
      (=> (balloon (icon "tm_enumerate.xpm") "Enumerate")
	  (link enumerate-menu))
      (=> (balloon (icon "tm_description.xpm") "Description")
	  (link description-menu))
      (if (or (inside-list?) (inside-description?))
	  ((balloon (icon "tm_item.xpm") "Insert a new item#(A-;)")
	   (make-item))))
  |
  (=> (balloon (icon "tm_math.xpm") "Insert mathematics")
      (link insert-mathematics-menu))
  (if (style-has? "program-dtd")
      (=> (balloon (icon "tm_shell.xpm")
		   "Start an interactive session")
	  (link session-menu)
	  ---
	  ("Other" ...
	   (interactive
	    '("Session type:" "Session name:") 'make-session)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Icons for math mode
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(menu-bind texmacs-math-icons
  ((balloon (icon "tm_fraction.xpm") "Insert a fraction#(A-f)")
   (make-fraction))
  ((balloon (icon "tm_sqrt.xpm") "Insert a square root#(A-s)")
   (make-sqrt))
  ((balloon (icon "tm_root.xpm") "Insert an n-th root#(A-S)")
   (make-var-sqrt))
  ((balloon (icon "tm_sub.xpm") "Make a subscript#(_)")
   (make-script #f #t))
  ((balloon (icon "tm_sup.xpm") "Make a superscript#(^)")
   (make-script #t #t))
  (=> (balloon (icon "tm_hat.xpm") "Insert an accent")
      ((icon "tm_hat.xpm") "A-^" (make-wide "^"))
      ((icon "tm_tilda.xpm") "A-~" (make-wide "~"))
      ((icon "tm_bar.xpm") "A-B" (make-wide "<bar>"))
      ((icon "tm_vect.xpm") "A-V" (make-wide "<vect>"))
      ((icon "tm_check.xpm") "A-C" (make-wide "<check>"))
      ((icon "tm_breve.xpm") "A-U" (make-wide "<breve>"))
      ((icon "tm_dot.xpm") "A-." (make-wide "<dot>"))
      ((icon "tm_ddot.xpm") "A-\"" (make-wide "<ddot>"))
      ((icon "tm_acute.xpm") "A-'" (make-wide "<acute>"))
      ((icon "tm_grave.xpm") "A-`" (make-wide "<grave>")))
  |
  (=> (balloon (icon "tm_bigop.xpm") "Insert a big operator")
      (tile 8 (link big-operator-menu)))
  (=> (balloon (icon "tm_bigleft.xpm") "Insert a large left delimiter")
      (tile 8 (link left-delimiter-menu)))
  (=> (balloon (icon "tm_bigsep.xpm") "Insert a large separator")
      (tile 8 (link middle-delimiter-menu)))
  (=> (balloon (icon "tm_bigright.xpm") "Insert a large right delimiter")
      (tile 8 (link right-delimiter-menu)))
  (=> (balloon (icon "tm_binop.xpm") "Insert a binary operation")
      (tile 8 (link binary-operation-menu)))
  (=> (balloon (icon "tm_binrel.xpm") "Insert a binary relation")
      (tile 8 (link binary-relation-menu-1))
      ---
      (tile 8 (link binary-relation-menu-2)))
  (=> (balloon (icon "tm_arrow.xpm") "Insert an arrow")
      (tile 9 (link horizontal-arrow-menu))
      ---
      (tile 8 (link vertical-arrow-menu))
      ---
      (tile 6 (link long-arrow-menu)))
  (=> (balloon (icon "tm_miscsymb.xpm") "Insert a miscellaneous symbol")
      (tile 8 (link miscellaneous-symbol-menu))
      ---
      (tile 6 (link dots-menu))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Icons for computer algebra sessions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(menu-bind texmacs-session-icons
  (if (in-plugin-with-converters?)
      (=> (balloon (icon "tm_plugin_input.xpm") "Input mode")
	  ("Textual input" (session-use-math-input #f))
	  ("Mathematical input" (session-use-math-input #t))))
  ((balloon (icon "tm_stop.xpm") "Interrupt execution")
   (connection-interrupt))
  ((balloon (icon "tm_clsession.xpm") "Close session")
   (connection-stop))
  (if (in-math?)
      |))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Icons for modifying text properties
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(menu-bind texmacs-modifier-icons
  |
  (if (in-prog?)
      ((balloon (icon "tm_italic.xpm") "Write italic text#(A-C-i)")
       (make-with "prog font shape" "italic"))
      ((balloon (icon "tm_bold.xpm") "Write bold text#(A-C-b)")
       (make-with "prog font series" "bold"))
      ((balloon (icon "tm_sansserif.xpm") "Use a sans serif font#(A-C-s)")
       (make-with "prog font family" "ss")))
  (if (and (in-text?) (style-has? "std-markup-dtd"))
      ;;((balloon
      ;;(text (roman rm bold right 12 600) "S")
      ;;"Write bold text#(A-C-b)")
      ;;(make-with "font series" "bold"))
      ((balloon (icon "tm_emphasize.xpm") "Emphasize text#(F5)")
       (make-expand-arg "em"))
      ((balloon (icon "tm_strong.xpm") "Write strong text#(F6)")
       (make-expand-arg "strong"))
      ((balloon (icon "tm_verbatim.xpm") "Write verbatim text#(F7)")
       (make-expand-arg "verbatim"))
      ((balloon (icon "tm_sansserif.xpm") "Write sample text#(F8)")
       (make-expand-arg "samp"))
      ((balloon (icon "tm_name.xpm") "Write a name#(S-F6)")
       (make-expand-arg "name")))
  (if (and (in-text?) (not (style-has? "std-markup-dtd")))
      ((balloon (icon "tm_italic.xpm") "Write italic text#(A-C-i)")
       (make-with "font shape" "italic"))
      ((balloon (icon "tm_bold.xpm") "Write bold text#(A-C-b)")
       (make-with "font series" "bold"))
      ((balloon (icon "tm_typewriter.xpm") "Use a typewriter font#(A-C-t)")
       (make-with "font family" "tt"))
      ((balloon (icon "tm_sansserif.xpm") "Use a sans serif font#(A-C-s)")
       (make-with "font family" "ss"))
      ((balloon (icon "tm_smallcaps.xpm") "Use small capitals#(A-C-p)")
       (make-with "font shape" "small-caps")))
  (if (in-math?)
      (=> (balloon (icon "tm_greek.xpm") "Insert a greek character")
	  (tile 8 (link lower-greek-menu))
	  ---
	  (tile 8 (link upper-greek-menu)))
      (=> (balloon (icon "tm_mathbold.xpm")
		   "Insert a bold character")
	  (tile 15 (link bold-num-menu))
	  ---
	  (tile 13 (link bold-alpha-menu))
	  ---
	  (tile 15 (link bold-greek-menu))
	  ---
	  ("use a bold font" (make-with "math font series" "bold")))
      (=> (balloon (icon "tm_cal.xpm")
		   "Insert a calligraphic character")
	  (tile 13 (link cal-menu))
	  ---
	  ("use a calligraphic font" (make-with "math font" "cal")))
      (=> (balloon (icon "tm_frak.xpm")
		   "Insert a fraktur character")
	  (tile 13 (link frak-menu))
	  ---
	  ("use the fraktur font" (make-with "math font" "Euler")))
      (=> (balloon (icon "tm_bbb.xpm")
		   "Insert a blackboard bold character")
	  (tile 13 (link bbb-menu))
	  ---
	  ("use the blackboard bold font" (make-with "math font" "Bbb*"))))
  (=> (balloon (icon "tm_color.xpm") "Select a foreground color")
      (link color-menu)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Icons for manipulation of tables
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(menu-bind texmacs-table-insert-icons
  ((balloon (icon "tm_table_arow.xpm")
	    "Insert row above (A-up)")
   (table-insert-row #f))
  ((balloon (icon "tm_table_brow.xpm")
	    "Insert row below (A-down)")
   (table-insert-row #t))
  ((balloon (icon "tm_table_lcol.xpm")
	    "Insert column to the left (A-left)")
   (table-insert-column #f))
  ((balloon (icon "tm_table_rcol.xpm")
	    "Insert column to the right (A-right)")
   (table-insert-column #t))
  ---
  ((balloon (icon "tm_table_drow.xpm")
	    "Delete row (M-t h delete)")
   (table-delete-row #f))
  ((balloon (icon "tm_table_dcol.xpm")
	    "Delete column (M-t v delete)")
   (table-delete-column #f)))

(menu-bind texmacs-table-hpos-icons
  ((balloon (icon "tm_table_left.xpm")
	    "Align table at the left (M-t H l)")
   (table-set-halign "l"))
  ((balloon (icon "tm_table_hcenter.xpm")
	    "Align table at the horizontal center (M-t H c)")
   (table-set-halign "c"))
  ((balloon (icon "tm_table_right.xpm")
	    "Align table to the right (M-t H r)")
   (table-set-halign "r"))
  ((icon "tm_empty.xpm") (noop))
  ((balloon (icon "tm_table_bleft.xpm")
	    "Align table at base of left column (M-t H L)")
   (table-set-halign "L"))
  ((balloon (icon "tm_table_bhcenter.xpm")
	    "Align table at base of middle column (M-t H C)")
   (table-set-halign "C"))
  ((balloon (icon "tm_table_bright.xpm")
	    "Align table at base of right column (M-t H R)")
   (table-set-halign "R"))
  ((balloon (icon "tm_table_hbase.xpm")
	    "Align table at base of origin column (M-t H O)")
   (table-set-halign "O")))

(menu-bind texmacs-table-vpos-icons
  ((balloon (icon "tm_table_bottom.xpm")
	    "Align table at the bottom (M-t V b)")
   (table-set-valign "b"))
  ((balloon (icon "tm_table_vcenter.xpm")
	    "Align table at the center (M-t V c)")
   (table-set-valign "c"))
  ((balloon (icon "tm_table_top.xpm")
	    "Align table at the top (M-t V t)")
   (table-set-valign "t"))
  ((balloon (icon "tm_table_vfrac.xpm")
	    "Align table at fraction bar height (M-t V f)")
   (table-set-valign "f"))
  ((balloon (icon "tm_table_bbottom.xpm")
	    "Align table at the base of bottom row (M-t V B)")
   (table-set-valign "B"))
  ((balloon (icon "tm_table_bvcenter.xpm")
	    "Align table at the base of middle row (M-t V C)")
   (table-set-valign "C"))
  ((balloon (icon "tm_table_btop.xpm")
	    "Align table at the base of top row (M-t V T)")
   (table-set-valign "T"))
  ((balloon (icon "tm_table_vbase.xpm")
	    "Align table at the base of origin row (M-t V O)")
   (table-set-valign "O")))

(menu-bind texmacs-cell-mode-icons
  ((balloon (icon "tm_cell_by_cell.xpm")
	    "Perform operations on cells (M-t m c)")
   (set-cell-mode "cell"))
  ((balloon (icon "tm_cell_by_row.xpm")
	    "Perform operations on rows (M-t m h)")
   (set-cell-mode "row"))
  ((balloon (icon "tm_cell_by_column.xpm")
	    "Perform operations on columns (M-t m v)")
   (set-cell-mode "column"))
  ((balloon (icon "tm_cell_by_table.xpm")
	    "Perform operations on entire table (M-t m t)")
   (set-cell-mode "table")))

(menu-bind texmacs-cell-pos-icons
  ((balloon (icon "tm_cell_left.xpm")
	    "Align cell to the left (M-t h l)")
   (cell-set-halign "l"))
  ((balloon (icon "tm_cell_hcenter.xpm")
	    "Horizontally center the cell (M-t h c)")
   (cell-set-halign "c"))
  ((balloon (icon "tm_cell_hbase.xpm")
	    "Horizontally align cell to the base (M-t h R)")
   (cell-set-halign "R"))
  ((balloon (icon "tm_cell_right.xpm")
	    "Align cell to the right (M-t h r)")
   (cell-set-halign "r"))
  ((balloon (icon "tm_cell_bottom.xpm")
	    "Align cell to the bottom (M-t v b)")
   (cell-set-valign "b"))
  ((balloon (icon "tm_cell_vbase.xpm")
	    "Vertically align cell to the base (M-t v B)")
   (cell-set-valign "B"))
  ((balloon (icon "tm_cell_vcenter.xpm")
	    "Vertically center the cell (M-t v c)")
   (cell-set-valign "c"))
  ((balloon (icon "tm_cell_top.xpm")
	    "Align cell to the top (M-t v t)")
   (cell-set-valign "t")))

(menu-bind texmacs-cell-size-icons
  ((balloon (icon "tm_cell_width.xpm")
	    "Set width of cell (M-t h s)")
   (cell-set-width-ia))
  ((balloon (icon "tm_cell_hmin.xpm")
	    "Width is minimum of specified width and box width (M-t h m)")
   (cell-set-hmode "min"))
  ((balloon (icon "tm_cell_hexact.xpm")
	    "Width is exactly the specified width (M-t h e)")
   (cell-set-hmode "exact"))
  ((balloon (icon "tm_cell_hmax.xpm")
	    "Width is maximum of specified width and box width (M-t h M)")
   (cell-set-hmode "max"))
  ((balloon (icon "tm_cell_height.xpm")
	    "Set height of cell (M-t v s)")
   (cell-set-height-ia))
  ((balloon (icon "tm_cell_vmin.xpm")
	    "Height is minimum of specified height and box height (M-t v m)")
   (cell-set-vmode "min"))
  ((balloon (icon "tm_cell_vexact.xpm")
	    "Height is exactly the specified height (M-t v e)")
   (cell-set-vmode "exact"))
  ((balloon (icon "tm_cell_vmax.xpm")
	    "Height is maximum of specified height and box height (M-t v M)")
   (cell-set-vmode "max")))

(menu-bind texmacs-cell-decoration-icons
  ((balloon (icon "tm_decorate_left.xpm")
	    "Use left hand column as border (M-t b left)")
   (table-column-decoration #f))
  ((balloon (icon "tm_decorate_right.xpm")
	    "Use right hand column as border (M-t b right)")
   (table-column-decoration #t))
  ((balloon (icon "tm_decorate_up.xpm")
	    "Use row above as border (M-t b up)")
   (table-row-decoration #f))
  ((balloon (icon "tm_decorate_down.xpm")
	    "Use row below as border (M-t b down)")
   (table-row-decoration #t)))

(menu-bind texmacs-table-icons
  |
  (=> (balloon (icon "tm_table_insert.xpm")
	       "Insert or delete rows or columns")
      (link texmacs-table-insert-icons))
  (=> (balloon (icon "tm_table_pos.xpm") "Position the table")
      (tile 4 (link texmacs-table-hpos-icons))
      ---
      (tile 4 (link texmacs-table-vpos-icons)))
  (=> (balloon (icon "tm_table_special.xpm") "Set special table properties")
      (link table-special-menu))
  |
  (if (== (get-cell-mode) "cell")
      (=> (balloon (icon "tm_cell_cell.xpm")
		   "Change cell operation mode")
	  (link texmacs-cell-mode-icons)))
  (if (== (get-cell-mode) "row")
      (=> (balloon (icon "tm_cell_row.xpm")
		   "Change cell operation mode")
	  (link texmacs-cell-mode-icons)))
  (if (== (get-cell-mode) "column")
      (=> (balloon (icon "tm_cell_column.xpm")
		   "Change cell operation mode")
	  (link texmacs-cell-mode-icons)))
  (if (== (get-cell-mode) "table")
      (=> (balloon (icon "tm_cell_table.xpm")
		   "Change cell operation mode")
	  (link texmacs-cell-mode-icons)))
  (=> (balloon (icon "tm_cell_pos.xpm") "Modify cell alignment")
      (tile 4 (link texmacs-cell-pos-icons)))
  (=> (balloon (icon "tm_cell_size.xpm") "Modify cell size")
      (tile 4 (link texmacs-cell-size-icons)))
  (=> (balloon (icon "tm_cell_border.xpm") "Change border of cell")
      (link cell-border-menu))
  (=> (balloon (icon "tm_cell_background.xpm")
	       "Set background color of cell")
      (link cell-color-menu))
  (=> (balloon (icon "tm_cell_special.xpm") "Set special cell properties")
      (link  cell-special-menu)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Help icon
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(menu-bind texmacs-session-help-icons)

(menu-bind texmacs-help-icons
  (link texmacs-session-help-icons))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Interface
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(menu-bind texmacs-context-icons
  (if (in-text?) (link texmacs-text-icons))
  (if (in-session?) (link texmacs-session-icons))
  (if (in-math?) (link texmacs-math-icons))
  (link texmacs-modifier-icons)
  (if (inside? "table") (link texmacs-table-icons))
  (link texmacs-help-icons))
