
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : table-menu.scm
;; DESCRIPTION : the Table menu
;; COPYRIGHT   : (C) 1999  Joris van der Hoeven
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The Table menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(menu-bind table-insert-row-menu
  ("Above" (table-insert-row #f))
  ("Below" (table-insert-row #t)))

(menu-bind table-insert-column-menu
  ("To the left" (table-insert-column #f))
  ("To the right" (table-insert-column #t)))

(menu-bind table-halign-menu
  ("Left" (table-set-halign "l"))
  ("Middle" (table-set-halign "c"))
  ("Right" (table-set-halign "r")))

(menu-bind table-valign-menu
  ("Bottom" (table-set-valign "b"))
  ("Middle" (table-set-valign "c"))
  ("Fraction bar height" (table-set-valign "f"))
  ("Top" (table-set-valign "t")))

(menu-bind table-border-menu
  ("Border" ... (table-set-border-ia))
  ("Left border" ... (table-set-lborder-ia))
  ("Right border" ... (table-set-rborder-ia))
  ("Bottom border" ... (table-set-bborder-ia))
  ("Top border" ... (table-set-tborder-ia))
  ---
  ("Padding" ... (table-set-padding-ia))
  ("Left padding" ... (table-set-lpadding-ia))
  ("Right padding" ... (table-set-rpadding-ia))
  ("Bottom padding" ... (table-set-bpadding-ia))
  ("Top padding" ... (table-set-tpadding-ia)))

(menu-bind table-limits-menu
  ("Minimal number of rows" ... (table-set-min-rows-ia))
  ("Minimal number of columns" ... (table-set-min-columns-ia))
  ("Maximal number of rows" ... (table-set-max-rows-ia))
  ("Maximal number of columns" ... (table-set-max-columns-ia)))

(menu-bind table-special-menu
  ("Deactivate" (table-disactivate))
  ("Set extension center" (table-format-center))
  ("Extract format" (table-extract-format))
  ---
  ("Use paragraph width" (table-use-paragraph-width))
  ("Width" ... (table-set-width-ia))
  ("Height" ... (table-set-height-ia))
  ---
  ("Hyphenation" (toggle-table-hyphen))
  (-> "Border" (link table-border-menu))
;;(-> "Origin"
;;    ("Row" (table-set-row-origin-ia))
;;    ("Column" (table-set-column-origin-ia)))
  (-> "Size limits" (link table-limits-menu)))

(menu-bind cell-mode-menu
  ("Cells" (set-cell-mode "cell"))
  ("Rows" (set-cell-mode "row"))
  ("Columns" (set-cell-mode "column"))
  ("Entire table" (set-cell-mode "table")))

(menu-bind cell-halign-menu
  ("Left" (cell-set-halign "l"))
  ("Center" (cell-set-halign "c"))
  ("Baseline" (cell-set-halign "R"))
  ("Right" (cell-set-halign "r")))

(menu-bind cell-valign-menu
  ("Bottom" (cell-set-valign "b"))
  ("Baseline" (cell-set-valign "B"))
  ("Center" (cell-set-valign "c"))
  ("Top" (cell-set-valign "t")))

(menu-bind cell-width-menu
  ("Set width" ... (cell-set-width-ia))
  ---
  ("Minimum mode" (cell-set-hmode "min"))
  ("Exact mode" (cell-set-hmode "exact"))
  ("Maximum mode" (cell-set-hmode "max")))

(menu-bind cell-height-menu
  ("Set height" ... (cell-set-height-ia))
  ---
  ("Minimum mode" (cell-set-vmode "min"))
  ("Exact mode" (cell-set-vmode "exact"))
  ("Maximum mode" (cell-set-vmode "max")))

(menu-bind cell-border-menu
  ("Border" ... (cell-set-border-ia))
  ("Left border" ... (cell-set-lborder-ia))
  ("Right border" ... (cell-set-rborder-ia))
  ("Bottom border" ... (cell-set-bborder-ia))
  ("Top border" ... (cell-set-tborder-ia))
  ---
  ("Padding" ... (cell-set-padding-ia))
  ("Left padding" ... (cell-set-lpadding-ia))
  ("Right padding" ... (cell-set-rpadding-ia))
  ("Bottom padding" ... (cell-set-bpadding-ia))
  ("Top padding" ... (cell-set-tpadding-ia)))

(menu-bind cell-color-menu
  ("None" (cell-set-background ""))
  ---
  ("Black" (cell-set-background "black"))
  ("White" (cell-set-background "white"))
  ("Grey" (cell-set-background "grey"))
  ("Red" (cell-set-background "red"))
  ("Blue" (cell-set-background "blue"))
  ("Yellow" (cell-set-background "yellow"))
  ("Green" (cell-set-background "green"))
  ("Orange" (cell-set-background "orange"))
  ("Magenta" (cell-set-background "magenta"))
  ("Brown" (cell-set-background "brown"))
  ("Pink" (cell-set-background "pink"))
  ---
  ("Foreground" (cell-set-background "foreground")))

(menu-bind cell-special-menu
  ("Set span" ... (cell-set-span-ia))
  ("Subtable" (make-sub-table))
  ---
  (-> "Text height correction"
      ("Off" (cell-set-vcorrect "n"))
      ---
      ("Bottom" (cell-set-vcorrect "b"))
      ("Top" (cell-set-vcorrect "t"))
      ("Both" (cell-set-vcorrect "a")))
  (-> "Hyphenation"
      ("Off" (cell-set-hyphen "n"))
      ---
      ("Top" (cell-set-hyphen "t"))
      ("Center" (cell-set-hyphen "c"))
      ("Bottom" (cell-set-hyphen "b"))
      ---
      ("Multi-paragraph" (cell-toggle-multi-paragraph)))
  (-> "Distribute unused space"
      ("Horizontal part" ... (cell-set-hpart-ia))
      ("Vertical part" ... (cell-set-vpart-ia)))
  (-> "Glue decorations" (tile 2 (link texmacs-cell-decoration-icons))))

(menu-bind table-menu
  (-> "Insert row" (link table-insert-row-menu))
  (-> "Insert column" (link table-insert-column-menu))
  (-> "Horizontal table alignment" (link table-halign-menu))
  (-> "Vertical table alignment" (link table-valign-menu))
  (-> "Special table properties" (link table-special-menu))
  ---
  (-> "Cell operation mode" (link cell-mode-menu))
  (-> "Horizontal cell alignment" (link cell-halign-menu))
  (-> "Vertical cell alignment" (link cell-valign-menu))
  (-> "Cell width" (link cell-width-menu))
  (-> "Cell height" (link cell-height-menu))
  (-> "Cell border" (link cell-border-menu))
  (-> "Cell background color" (link cell-color-menu))
  (-> "Special cell properties" (link cell-special-menu)))
