
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : modify.scm
;; DESCRIPTION : routines for modifying a document
;; COPYRIGHT   : (C) 2001  Joris van der Hoeven
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; General subroutines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (insert-object obj)
  (insert-tree (object->tree obj)))

(define (insert-object-go-to obj p)
  (insert-tree-go-to (object->tree obj) p))

(define (insert-tree-at t p)
  (let* ((pos (tm-position-new))
	 (old (tm-position-get pos)))
    (tm-go-to p)
    (insert-tree t)
    (tm-go-to old)
    (tm-position-delete pos)))

(define (insert-object-at obj p)
  (insert-tree-at (object->tree obj) p))

(define (make-expand-arg s)
  (make-expand-arity s 1))

(define (make-inactive name nrargs)
  (make-deactivated name nrargs name))

(define (make-inactive-arg name arg nrargs)
  (make-deactivated-arg name nrargs name arg))

(define (make-inactive-message name nrargs message)
  (make-deactivated name nrargs message))

(define (make-inactive-apply-arg s)
  (make-inactive-apply)
  (insert-string s)
  (insert-argument)
  (set-message "Press <Return> to activate" s))

(define (make-inactive-apply-args s n)
  (make-inactive "apply" n)
  (insert-string s)
  (insert-argument)
  (set-message "Press <Return> to activate" s))

(tm-define (make-line-with var val)
  (:synopsis "Make 'with' with one or more paragraphs as its scope")
  (:check-mark "o" test-env?)
  (if (not (selection-active-normal?))
      (select-line))
  (make-with var val)
  (insert-return)
  (remove-backwards))

(define (make-inactive-assign-arg s)
  (insert-object-go-to `(inactive (assign ,s "")) '(0 1 0))
  (set-message "return: activate" "assign"))

(define (make-inactive-assign-function s)
  (insert-object-go-to
   `(inactive (assign ,s (inactive (func ""))))
   '(0 1 0 0 0))
  (set-message "return (2x): activate" "assign#function"))

(define (make-inactive-assign-function-arg s)
  (insert-object-go-to
   `(inactive (assign ,s (inactive (func "s" ""))))
   '(0 1 0 1 0))
  (set-message "return (2x): activate" "assign#function"))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Routines for formatting markup
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (make-inactive-surround) (make-inactive "surround" 3))
(define (make-inactive-group) (make-inactive "group" 1))
(define (make-inactive-float) (make-inactive "float" 3))
(define (make-inactive-repeat) (make-inactive "repeat" 2))
(define (make-inactive-decorate-atoms) (make-inactive "datoms" 2))
(define (make-inactive-decorate-lines) (make-inactive "dlines" 2))
(define (make-inactive-decorate-pages) (make-inactive "dpages" 2))

(define (make-page-break-before)
  (make-format "page break before"))

(define (make-page-break)
  (make-format "page break")
  (insert-return))

(define (make-new-page-before)
  (make-format "new page before"))

(define (make-new-page)
  (make-format "new page")
  (insert-return))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Routines for macro handling and main executable markup
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (make-inactive-assign) (make-inactive "assign" 2))
(define (make-inactive-with) (make-inactive "with" 3))
(define (make-inactive-expand) (make-inactive "expand" 1))
(define (make-inactive-var-expand) (make-inactive "var_expand" 1))
(define (make-inactive-hide-expand) (make-inactive "hide_expand" 1))
(define (make-inactive-apply) (make-inactive "apply" 1))
(define (make-inactive-include) (make-inactive "include" 1))
(define (make-inactive-macro) (make-inactive "macro" 1))
(define (make-inactive-function) (make-inactive "func" 1))
(define (make-inactive-eval) (make-inactive-message "eval" 1 "evaluate"))
(define (make-inactive-value) (make-inactive "value" 1))
(define (make-inactive-argument) (make-inactive-message "arg" 1 "argument"))
(define (make-inactive-quote) (make-inactive "quote" 1))
(define (make-inactive-delay) (make-inactive "delay" 1))
(define (make-inactive-hold) (make-inactive "hold" 1))
(define (make-inactive-release) (make-inactive "release" 1))

(define (make-include s) (insert-object `(include ,s)))
(define (make-tuple) (make-active "tuple" 1))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Routines for inserting computational markup
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (make-inactive-or) (make-inactive "or" 2))
(define (make-inactive-xor) (make-inactive "xor" 2))
(define (make-inactive-and) (make-inactive "and" 2))
(define (make-inactive-not) (make-inactive "not" 1))
(define (make-inactive-plus) (make-inactive "plus" 2))
(define (make-inactive-minus) (make-inactive "minus" 2))
(define (make-inactive-times) (make-inactive "times" 2))
(define (make-inactive-over) (make-inactive "over" 2))
(define (make-inactive-div) (make-inactive "div" 2))
(define (make-inactive-mod) (make-inactive "mod" 2))
(define (make-inactive-merge) (make-inactive "merge" 2))
(define (make-inactive-length) (make-inactive "length" 1))
(define (make-inactive-range) (make-inactive "range" 3))
(define (make-inactive-number) (make-inactive "number" 2))
(define (make-inactive-date) (make-inactive "date" 1))
(define (make-inactive-translate) (make-inactive "translate" 3))
(define (make-inactive-find-file) (make-inactive "find_file" 2))
(define (make-inactive-is-tuple) (make-inactive "is_tuple" 1))
(define (make-inactive-look-up) (make-inactive "look_up" 2))
(define (make-inactive-equal) (make-inactive "equal" 2))
(define (make-inactive-unequal)
  (make-inactive-message "unequal" 2 "not equal"))
(define (make-inactive-less) (make-inactive "less" 2))
(define (make-inactive-lesseq)
  (make-inactive-message "lesseq" 2 "less or equal"))
(define (make-inactive-greater) (make-inactive "greater" 2))
(define (make-inactive-greatereq)
  (make-inactive-message "greatereq" 2 "greater or equal"))
(define (make-inactive-if) (make-inactive "if" 2))
(define (make-inactive-var-if) (make-inactive "var_if" 2))
(define (make-inactive-case) (make-inactive "case" 2))
(define (make-inactive-for) (make-inactive "for" 4))
(define (make-inactive-while) (make-inactive "while" 2))
(define (make-inactive-extern) (make-inactive "extern" 1))
(define (make-inactive-authorize) (make-inactive "authorize" 2))

(define (make-date)
  (if (== (get-env "language") "english")
      (insert-tree (object->tree '(date "%B %d, %Y")))
      (insert-tree (object->tree '(date "%d %B %Y")))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Routines for inserting miscellaneous content
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (make-inactive-symbol) (make-inactive "symbol" 1))
(define (make-inactive-latex)
  (make-inactive "latex" 1)
  (set-message "Type a latex command followed by return" "latex"))
(define (make-inactive-hybrid)
  (make-inactive "hybrid" 1)
  (set-message
   "tab: insert argument, return: activate or execute latex command"
   "hybrid"))
(define (make-inactive-label) (make-inactive "label" 1))
(define (make-inactive-reference) (make-inactive "reference" 1))
(define (make-inactive-pageref) (make-inactive "pageref" 1))
(define (make-inactive-write) (make-inactive "write" 1))
(define (make-inactive-action) (make-inactive "action" 2))
(define (make-inactive-specific s) (make-inactive-arg "specific" s 2))
(define (make-inactive-hyperlink) 
  (make-inactive-message "hlink" 2 "hyperlink"))
(define (make-inactive-tag) (make-inactive "tag" 2))
(define (make-inactive-meaning) (make-inactive "meaning" 2))
(define (make-inactive-superpose) (make-inactive "superpose" 1))
(define (make-inline-image l)
  (eval (cons 'make-postscript (cons (car l) (cons #f (cdr l))))))
(define (make-link-image l)
  (eval (cons 'make-postscript (cons (car l) (cons #t (cdr l))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Making titles
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (go-end-of-header-element)
  (if (inside? "address") (go-end-of "address"))
  (if (inside? "destination") (go-end-of "destination"))
  (if (inside? "cc") (go-end-of "cc"))
  (if (inside? "encl") (go-end-of "encl"))
  (go-end-line))

(define (make-header-expand s)
  (go-end-of-header-element)
  (if (not (== (tree->object (the-line)) "")) (insert-return))
  (make-expand-arg s))

(define (make-header-apply s)
  (go-end-of-header-element)
  (make-inactive-apply-arg s))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Routines for floats
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;(define (make-insertion s)
;  (insert-object-go-to
;   (list 'float s
;	 (if (string=? s "float") "tbh" "")
;	 (list 'document ""))
;   (list 2 0 0)))

;(define (position-insertion what flag)
;"Allow/disallow a position for the inner float the caret in is.
;what <char>   : position to allow/disallow
;flag <boolean>: allow if true, disallow is false."
;
;  (let ((p (search-upwards "float")))
;    (if (not (null? p))
;	(tm-assign
;	 (rcons p 1)
;	 (string->tree ((if flag
;			    string-include
;			    string-exclude) (tree->string
;					     (subtree (the-buffer)
;						      (rcons p 1)))
;			    what))))))

(define (test-insertion-position? what)
  (let ((p (search-upwards "float"))
	(c (string-ref what 0)))
    (if (not (null? p))
	(char-in-string? c (tree->string
			    (subtree (the-buffer) (rcons p 1)))))))
