
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : plugin.scm
;; DESCRIPTION : General tools for links with plugins
;; COPYRIGHT   : (C) 1999  Joris van der Hoeven
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; tab completion
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define plugin-supports-completions (make-ahash-table))

(define (plugin-supports-completions-set! key)
  (ahash-set! plugin-supports-completions key #t))

(define (plugin-supports-completions? key)
  (ahash-ref plugin-supports-completions key))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; operations on selections (not finished)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (tree->plugin lan t)
  (plugin-math-input (list 'tuple lan (tree->object t))))

(define (input->plugin lan t)
  (if (and ((== (tree-get-label t) 'expand)
	    (= (tree-arity t) 2)
	    (== (tree-ref t 0) (tree "math"))))
      (tree->plugin lan t)
      (tree->verbatim t)))

(define (maxima-expand)
  (if (selection-active-any?)
      (let* ((t (the-selection))
	     (s (tree->plugin "maxima" t))
	     (r (string-append s ";"))
	     (u (connection-eval "maxima" "default" (string->tree r))))
	(cut "primary")
	(insert-tree u))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; configuration of plugins
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (plugin-configure-cmd name cmd)
  (cond ((func? cmd :launch 1)
	 (connection-declare name (second cmd)))
	((func? cmd :format 2)
	 (connection-format name (second cmd) (third cmd)))
	((func? cmd :session 1)
	 (menu-extend session-menu
	   ,(list (second cmd) (lambda () (make-session name "default")))))
	((func? cmd :filter-in 1) ; deprecated
	 (connection-filter-in name (second cmd)))
	((func? cmd :tab-completion 1)
	 (if (second cmd) (plugin-supports-completions! name)))))

(define-macro (plugin-configure name2 . options)
  (let* ((name (if (string? name2) name2 (symbol->string name2)))
	 (in-name (string->symbol (string-append "in-" name "?"))))
    `(begin
       (define (,in-name) (== (get-env "prog language") ,name))
       (for-each (lambda (x) (plugin-configure-cmd ,name x)) ',options))))
