#define _GNU_SOURCE

#include <sys/syscall.h>
#include <signal.h>
#include <grp.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "common.h"
#include "main.h"
#include "ipqapi.h"
#include "client.h"
#include "icmp.h"
#include "packet.h"

volatile int signum = 0;

fd_set listen_wfds, listen_rfds, select_wfds, select_rfds;

gboolean fail = FALSE;

guint32 default_verdict = 15; // ICMP packet filtered
//guint32 default_verdict = 1; // Accept

static void sigfunc(int s) {
    signum = s;
}

static gboolean _verdict_func(ipq_packet_msg_t*m) {
    g_message("Packet %lu still unanswered, verdicting.", m->packet_id);
    ipqapi_verdict(m, default_verdict);
    return FALSE;
}

int main(int argc, char *argv[]) {
    int r = 1;
    struct group *gr;
    
    setuid(geteuid());
    
    signal(SIGINT, sigfunc);
    siginterrupt(SIGINT, TRUE);
    signal(SIGQUIT, sigfunc);
    siginterrupt(SIGQUIT, TRUE);
    signal(SIGPIPE, SIG_IGN);
    siginterrupt(SIGPIPE, FALSE);

    if (!(gr = getgrnam("fieryfilter"))) {
        g_message("Could not find group fieryfilter");
        goto finish;
    }
    
    if (setgid(gr->gr_gid) != 0) {
        g_message("setgid(): %s", strerror(errno)); 
        goto finish; 
    }

    FD_ZERO(&listen_wfds);
    FD_ZERO(&listen_rfds);
    
    if (ipqapi_init() < 0)
        goto finish;

    if (client_init() < 0 )
        goto finish;
    
    if (icmp_init() < 0)
        goto finish;
    

    g_message("fieryfilterd started successfully.");
    
    while (!fail) {
        select_rfds = listen_rfds;
        select_wfds = listen_wfds;
        
        if (select(FD_SETSIZE, &select_rfds, &select_wfds, NULL, NULL) < 0) {
            if (errno == EINTR) {
                switch (signum) {
                    case SIGINT:
                    case SIGQUIT:
                        g_message("Recieved signal <%s>, exiting cleanly.", strsignal(signum));
                        r = 0;
                        goto finish;
                }
            } else {
                g_message("select: %s", strerror(errno)); 
                goto finish;
            }
        } else {
            if (client_work() < 0)
                goto finish;

            if (ipqapi_work() < 0)
                goto finish;
        }

        packet_foreach(client_is_connected(), _verdict_func);
    }

    r = 0;
    
finish:

    g_message("fieryfilterd exiting.");

    ipqapi_done();
    client_done();
    icmp_done();
    packets_free();
    

    syscall(SYS_exit, r);
    
    return r;
}
