/*
    kopeteevent.cpp - Kopete Event

    Copyright (c) 2002      by Duncan Mac-Vicar Prett <duncan@kde.org>
    Copyright (c) 2002      by Hendrik vom Lehn <hvl@linux-4-ever.de>
    Copyright (c) 2002-2003 by Martijn Klingens       <klingens@kde.org>

    Kopete    (c) 2002-2003 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include "kopeteevent.h"

#include <qwidget.h>
#include <qpoint.h>

#include <kdebug.h>
#include <knotifyclient.h>

#include "kopeteprefs.h"
#include "kopeteaway.h"
#include "systemtray.h"

KopeteEvent::KopeteEvent( QString text, QString picture, QObject *object, const char *slot )
: mBalloon( text, picture )
{
	mPlaying = false;
	mUsedBalloon = false;
	mUsedFlash = false;

	mText = text;
	mPicture = picture;
	mObject = object;
	mSlot = slot;

	if (mObject && mSlot)
	{
		connect( &mBalloon, SIGNAL(signalButtonClicked()), mObject, mSlot );
		connect( &mBalloon, SIGNAL(signalButtonClicked()), this, SLOT(deleteLater()) );
	}

	connect( &mBalloon, SIGNAL(signalBalloonClicked()), this, SLOT(deleteLater()) );
}
KopeteEvent::~KopeteEvent()
{
//	kdDebug(14010) << "KopeteEvent: done() I'm dying! :-)" << endl;
	/* Balloon Style */
	mBalloon.hide();

	emit done(this);
}

void KopeteEvent::start()
{
//	kdDebug(14010) << "KopeteEvent: start(), showing Balloon at (" << ( KopeteSystemTray::systemTray()->mapToGlobal(KopeteSystemTray::systemTray()->pos())).x() << " , " << (KopeteSystemTray::systemTray()->mapToGlobal(KopeteSystemTray::systemTray()->pos())).y() << ")" << endl;
	mPlaying = true;
	mBalloon.setAnchor( KopeteSystemTray::systemTray()->mapToGlobal(
		KopeteSystemTray::systemTray()->pos() ) );
	/* Balloon Style */
	if ( KopetePrefs::prefs()->balloonNotify() )
	{
		mBalloon.show();
		mUsedBalloon = true;
	}
	/* Tray Blink Style */
	if ( KopetePrefs::prefs()->trayflashNotify() )
	{
		connect( KopeteSystemTray::systemTray(),
			SIGNAL( doubleClicked( const QPoint ) ), object(), slot() );
		connect( KopeteSystemTray::systemTray(),
			SIGNAL( doubleClicked( const QPoint ) ),
			this, SLOT( deleteLater() ) );
		KopeteSystemTray::systemTray()->startBlink();
		mUsedFlash = true;
	}
}

void KopeteEvent::startNow()
{
	/* Beep */
	if ( KopetePrefs::prefs()->beepNotify() )
	{
		KNotifyClient::beep();
	}
	/* Sound */
	if ( KopetePrefs::prefs()->soundNotify() )
	{
		if ( !( (KopetePrefs::prefs()->soundIfAway() == false) && (KopeteAway::globalAway()) ) )
		{
			KNotifyClient::event( QString::fromLatin1( "kopete_event" ) );
		}
	}
}

void KopeteEvent::stop()
{
//	kdDebug(14010) << "KopeteEvent: stop() Hiding Balloon" << endl;
	mPlaying = false;

	/* We dont have to stop each style reading prefs
		because they can chang in real time while queued event */

	/* Balloon Style */
	if ( mUsedBalloon )
	{
		mBalloon.hide();
		mUsedBalloon = false;
	}
	/* Tray Blink Style */
	if ( mUsedFlash )
	{
		disconnect( KopeteSystemTray::systemTray(), SIGNAL(doubleClicked(const QPoint)), object(), slot() );
		disconnect( KopeteSystemTray::systemTray(), SIGNAL(doubleClicked(const QPoint)), this, SLOT(deleteLater()) );
		KopeteSystemTray::systemTray()->stopBlink();
		mUsedFlash = false;
	}
}

#include "kopeteevent.moc"

// vim: set noet ts=4 sts=4 sw=4:

