/*
    kopeteevent.h - Kopete Event

    Copyright (c) 2002 by Duncan Mac-Vicar Prett <duncan@kde.org>
    Copyright (c) 2002 by Hendrik vom Lehn <hvl@linux-4-ever.de>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#ifndef KOPETEEVENT_H
#define KOPETEEVENT_H

#include <qobject.h>
#include <qcstring.h>
#include <qstring.h>

#include "kopeteballoon.h"
#include "systemtray.h"

/**
 * @author Duncan Mac-Vicar P. <duncan@kde.org>
 */
class KopeteEvent : public QObject
{
	Q_OBJECT

public:
	KopeteEvent( QString, QString, QObject *object = 0L, const char *slot = 0L);
	~KopeteEvent();

	/**
	* KopeteNotifier will start() and event after queuing it
	* So all effects like visual messages (you cannot show more
	* than one balloon in the screen, and that need a stop()
	* part too (ie. removing the balloon from screen), should be here.
	**/
	void start();

	/**
	* All effects that should be played inmediatly
	* when an event arrive to the notifier, should
	* be here, ie: sound effects.
	**/
	void startNow();

	void stop();
	QCString slot() const { return mSlot; }
	QString text() const { return mText; }
	QString picture() const { return mPicture; }
	QObject * object() const { return mObject; }
	bool isPlaying() const { return mPlaying; }

signals:
	void done(KopeteEvent *);

private:
	QString mText;
	QString mPicture;
	QCString mSlot;
	QObject *mObject;
	bool mPlaying;
	KopeteBalloon mBalloon;
	bool mUsedBalloon;
	bool mUsedFlash;
};

#endif

// vim: set noet ts=4 sts=4 sw=4:

