/*
	kopeteidentitymanager.cpp - Kopete Identity Manager

	Copyright (c) 2002 by Martijn Klingens       <klingens@kde.org>

	Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

	*************************************************************************
	*                                                                       *
	* This program is free software; you can redistribute it and/or modify  *
	* it under the terms of the GNU General Public License as published by  *
	* the Free Software Foundation; either version 2 of the License, or     *
	* (at your option) any later version.                                   *
	*                                                                       *
	*************************************************************************
	*/

#include "kopeteidentitymanager.h"

#include <qapplication.h>

#include <kdebug.h>

#include "kopeteaway.h"
#include "kopeteprotocol.h"
#include "pluginloader.h"

KopeteIdentityManager* KopeteIdentityManager::s_manager = 0L;

KopeteIdentityManager* KopeteIdentityManager::manager()
{
		if( !s_manager )
				s_manager = new KopeteIdentityManager;

		return s_manager;
}

KopeteIdentityManager::KopeteIdentityManager()
				: QObject( qApp, "KopeteIdentityManager" )
{
}

KopeteIdentityManager::~KopeteIdentityManager()
{
		s_manager = 0L;
}

void KopeteIdentityManager::connectAll()
{
		QPtrList<KopetePlugin> plugins = LibraryLoader::pluginLoader()->plugins();
		for( KopetePlugin *p = plugins.first() ; p ; p = plugins.next() )
		{
				KopeteProtocol *proto = dynamic_cast<KopeteProtocol*>( p );
				if( !proto )
						continue;

				if( !proto->isConnected() )
				{
						kdDebug(14010) << "KopeteIdentityManager::connectAll: "
													 << "Connecting plugin: " << proto->pluginId() << endl;

						proto->connect();
				}
		}
}

void KopeteIdentityManager::disconnectAll()
{
		QPtrList<KopetePlugin> plugins = LibraryLoader::pluginLoader()->plugins();
		for( KopetePlugin *p = plugins.first() ; p ; p = plugins.next() )
		{
				KopeteProtocol *proto = dynamic_cast<KopeteProtocol*>( p );
				if( !proto )
						continue;

				if( proto->isConnected() )
				{
						kdDebug(14010) << "KopeteIdentityManager::disconnectAll: "
													 << "Disonnecting plugin: " << proto->pluginId() << endl;

						proto->disconnect();
				}
		}
}

void KopeteIdentityManager::setAwayAll()
{
		KopeteAway::setGlobalAway( true );
		QPtrList<KopetePlugin> plugins = LibraryLoader::pluginLoader()->plugins();
		for( KopetePlugin *p = plugins.first() ; p ; p = plugins.next() )
		{
				KopeteProtocol *proto = dynamic_cast<KopeteProtocol*>( p );
				if( !proto )
						continue;

				if( proto->isConnected() && !proto->isAway() )
				{
						kdDebug(14010) << "KopeteIdentityManager::setAwayAll: "
													 << "Setting plugin to away: " << proto->pluginId() << endl;

						proto->setAway();
				}
		}
}

void KopeteIdentityManager::setAvailableAll()
{
		KopeteAway::setGlobalAway( false );
		QPtrList<KopetePlugin> plugins = LibraryLoader::pluginLoader()->plugins();
		for( KopetePlugin *p = plugins.first() ; p ; p = plugins.next() )
		{
				KopeteProtocol *proto = dynamic_cast<KopeteProtocol*>( p );
				if( !proto )
						continue;

				if( proto->isConnected() && proto->isAway() )
				{
						kdDebug(14010) << "KopeteIdentityManager::setAvailableAll: "
													 << "Setting plugin to available: " << proto->pluginId() << endl;

						proto->setAvailable();
				}
		}
}

#include "kopeteidentitymanager.moc"
/*
 * Local variables:
 * mode: c++
 * c-indentation-style: k&r
 * c-basic-offset: 4
 * indent-tabs-mode: t
 * End:
 */
// vim: set noet ts=4 sts=4 sw=4:

