/*
	kopeteidentitymanager.h - Kopete Identity Manager

	Copyright (c) 2002 by Martijn Klingens       <klingens@kde.org>

	Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

	*************************************************************************
	*                                                                       *
	* This program is free software; you can redistribute it and/or modify  *
	* it under the terms of the GNU General Public License as published by  *
	* the Free Software Foundation; either version 2 of the License, or     *
	* (at your option) any later version.                                   *
	*                                                                       *
	*************************************************************************
	*/

#ifndef __kopeteidentitymanager_h__
#define __kopeteidentitymanager_h__

#include <qobject.h>

/**
 * @author Martijn Klingens <klingens@kde.org>
 *
 * KopeteIdentityManager manages all defined identities in Kopete. You can
 * query them, and globally set them all online or offline from here.
 *
 * NOTE: At this moment (december 2002) there is no class KopeteIdentity
 *       yet. Instead there is a single identity built into KopeteProtocol.
 *       Because this will need to change later I named the class after its
 *       future additional use already.
 */
class KopeteIdentityManager : public QObject
{
		Q_OBJECT

public:
		/**
		 * Retrieve the instance of KopeteIdentityManager.
		 * The identity manager is a singleton class of which only a single
		 * instance will exist. If no manager exists yet this function will
		 * create one for you.
		 */
		static KopeteIdentityManager* manager();
		
		~KopeteIdentityManager();
		
public slots:
		/**
		 * Connect all identities at once.
		 * This is a slot, so you can connect directly to it from e.g. a KAction.
		 */
		void connectAll();
		
		/**
		 * Disconnect all identities at once.
		 * This is a slot, so you can connect directly to it from e.g. a KAction.
		 */
		void disconnectAll();

		/**
		 * Set all identities to away at once.
		 * This is a slot, so you can connect directly to it from e.g. a KAction.
		 */
		void setAwayAll();

		/**
		 * Remove the away status from all identities at once.
		 * This is a slot, so you can connect directly to it from e.g. a KAction.
		 */
		void setAvailableAll();

private:
		/**
		 * Private constructor, because we're a singleton
		 */
		KopeteIdentityManager();

		static KopeteIdentityManager *s_manager;
};

#endif
/*
 * Local variables:
 * mode: c++
 * c-indentation-style: k&r
 * c-basic-offset: 4
 * indent-tabs-mode: t
 * End:
 */
// vim: set noet ts=4 sts=4 sw=4:

