/*
    messagelog.cpp  -  Kopete Logger

    Copyright (c) 2002      by Ryan Cumming          <ryan@kde.org>

    Kopete    (c) 2002-2003 by the Kopete developers <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include "kopetemessagelog.h"

#include <qdatetime.h>
#include <qdir.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qstring.h>
#include <qstylesheet.h>
#include <qtextstream.h>

#include <kdebug.h>
#include <kstandarddirs.h>

KopeteMessageLog::KopeteMessageLog(QString filename, QObject *parent, const char *name) : QObject(parent, name)
{
	QFileInfo fileInfo(filename);

	QString realFileName = locateLocal( "data", fileInfo.dirPath() + QString::fromLatin1( "/" ) );
	QDir dir;

	if (dir.exists(realFileName) == false)
		dir.mkdir(realFileName);

	realFileName.append(fileInfo.fileName());

	kdDebug(14010) << "Opening log file " << realFileName << endl;

	mLogFile.setName(realFileName);
	mLogFile.open(IO_ReadWrite | IO_Append);

	if (!ok())
		kdWarning() << "Unable to open message log file" << endl;
}

bool KopeteMessageLog::ok()
{
	return (mLogFile.status() == IO_Ok);
}

bool KopeteMessageLog::append(const KopeteMessage &m)
{
	if (!ok())
		return false;

	QTextStream stream(&mLogFile);
	stream.setEncoding(QTextStream::UnicodeUTF8);

	// This is a new message
	if (m.direction() == KopeteMessage::Inbound)
	{
		// Incoming message
		stream << "<message direction=\"inbound\">" << endl;
	}
	else if (m.direction() == KopeteMessage::Outbound)
	{
		// Outgoing message
		stream << "<message direction=\"outbound\">" << endl;
	}
	else {
		// Unknown direction (probably shouldn't happen)
		stream << "<message>" << endl;
	}

	if (!m.from()) return false;
	if (!m.from()->displayName().isNull())
	{
		// Log the source user
		stream << "\t<srcnick>" << QStyleSheet::escape(m.from()->displayName()) << "</srcnick>" << endl;
	}

	const KopeteContact *to = m.to().first();
	if( !to ) return false;

	if (!to->displayName().isNull())
	{
		// Log the destination user
		stream << "\t<destnick>" << QStyleSheet::escape(to->displayName()) << "</destnick>" << endl;
	}

	if (m.timestamp().isValid())
	{
		// Log the time
		stream << "\t<date>" << QStyleSheet::escape(m.timestamp().toString()) << "</date>" << endl;
	}

	// Time for the actual message
	stream << "\t<body>" << endl;
	stream << "\t\t" << QStyleSheet::escape(m.body()) << endl;
	stream << "\t</body>" << endl;

	stream << "</message>" << endl;
	stream << endl;

	/* Sync to disk */
	stream.device()->flush();

	return true;
}

#include "kopetemessagelog.moc"

// vim: set noet ts=4 sts=4 sw=4:

