/*
    kopetenotifier.cpp  -  Event Notifier

    Copyright (c) 2002 by Duncan Mac-Vicar Prett   <duncan@kde.org>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include "kopetenotifier.h"

#include <qapplication.h>

#include <kdebug.h>

#include "systemtray.h"

KopeteNotifier* KopeteNotifier::s_notifier = 0L;

KopeteNotifier* KopeteNotifier::notifier()
{
	if( !s_notifier )
		s_notifier = new KopeteNotifier( qApp, "KopeteNotifier" );

	return s_notifier;
}

KopeteNotifier::KopeteNotifier( QObject *parent, const char *name )
: QObject( parent, name )
{
	mBusy = false;
}

KopeteNotifier::~KopeteNotifier()
{
	s_notifier = 0L;
}

void KopeteNotifier::slotEventDone(KopeteEvent *event)
{
	removeEvent( event);
}

void KopeteNotifier::notifyEvent(KopeteEvent *event)
{
	if ( !event)
	{
		kdDebug(14010) << "Kopete Notifier: I cant process a null event" << endl;
		return;
	}

	// This process the notificaztions that must be played at arrive time
	// for example sounds.

	event->startNow();

	if ( mBusy && ( mCurrentEvent ) )
	{
		kdDebug(14010) << "Kopete Notifier: New event! , Notifier busy .Adding to queue" << endl;
		mEventList.append( event );
	}
	else
	{
		kdDebug(14010) << "Kopete Notifier: New event! processing immediately" << endl;
		mCurrentEvent = event;
		connect( mCurrentEvent , SIGNAL(done(KopeteEvent *)), this, SLOT(slotEventDone( KopeteEvent *)));
		mCurrentEvent->start();
		kdDebug(14010) << "Kopete Notifier: event started." << endl;
		kdDebug(14010) << "Kopete Notifier: now busy" << endl;
		mBusy = true;
	}
}

void KopeteNotifier::removeEvent(KopeteEvent *event)
{
	if ( !event)
	{
		kdDebug(14010) << "Kopete Notifier: I cant cancel a null event" << endl;
		return;
	}

	int index = mEventList.find(event);
	if ( index != -1 )
	{
		kdDebug(14010) << "Kopete Notifier: Removing a queued event." << endl;
		if (event != mCurrentEvent)
		{
			mEventList.remove( index );
			mBusy = false;
		}
		else
		{
			kdDebug(14010) << "Kopete Notifier: Weird! Event was both in queue and playing....CLEANING UP!" << endl;

			mCurrentEvent = NULL;
			mEventList.remove( index );
			mBusy = false;
		}
	}
	else if( index == -1 )
	{
		if ( event == mCurrentEvent )
		{
			kdDebug(14010) << "Kopete Notifier: Stopping current event (non-queued event.) All OK" << endl;
			/* The event was not queued */
			if ( event->isPlaying() )
			{
				event->stop();
			}
			kdDebug(14010) << "Kopete Notifier: Notifier free :-)" << endl;
			mCurrentEvent = NULL;
			mBusy = false;

			/* We take the next non-null one */
			kdDebug(14010) << "Kopete Notifier: Trying to get next event" << endl;
			while ( !(mCurrentEvent = mEventList.take(0))  )
			{
				kdDebug(14010) << "Kopete Notifier: Null event in queue" << endl;
				if ( mEventList.isEmpty() )
				{
					kdDebug(14010) << "Kopete Notifier: No more events" << endl;
					break;
				}
			}
			if (mCurrentEvent)
			{
				kdDebug(14010) << "KopeteNotifier: Got new queued event..." << endl;
				kdDebug(14010) << "Kopete Notifier: Now busy" << endl;
				mBusy = true;
				connect( mCurrentEvent , SIGNAL(done(KopeteEvent *)), this, SLOT(slotEventDone( KopeteEvent *)));
				mCurrentEvent->start();
			}
		}
		else
		{
			kdDebug(14010) << "Kopete Notifier: Event was not found! HEEELP!!" << endl;
		}
	}
}

#include "kopetenotifier.moc"

// vim: set noet ts=4 sts=4 sw=4:

