/*
    kopetenotifier.h  -  Event Notifier

    Copyright (c) 2002 by Duncan Mac-Vicar Prett   <duncan@kde.org>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#ifndef KOPETENOTIFIER_H
#define KOPETENOTIFIER_H

#include <qobject.h>
#include <qstring.h>
#include <qpixmap.h>
#include <qptrlist.h>


#include "kopeteevent.h"

class KopeteEvent;

/**
 * @author Duncan Mac-Vicar P. <duncan@kde.org>
 *
 * This is the Kopete Notifier System.
 * Plugins can use it to notify events to the user.
 * Dont use it directly.
 * See KopeteEvent. To remove events, just delete them.
 */
class KopeteNotifier : public QObject
{
	Q_OBJECT

public:
	/**
	 * get the KopeteNotifier instance
	 */
	static KopeteNotifier* notifier();

	~KopeteNotifier();

signals:
	void signalShowEvent();

public slots:
	/**
	 * Only use notify event for system-wide messages
	 * and things like online notification
	 * Messages from specific contacts should use KopeteContact's
	 * incomingEvent signal
	 */
	void notifyEvent( KopeteEvent * );

	void slotEventDone( KopeteEvent *event );

private:
	/**
	 * Private constructor, this is a singleton
	 */
	KopeteNotifier( QObject *parent, const char *name );

	void removeEvent( KopeteEvent * );
	QPtrList<KopeteEvent> mEventList;
	KopeteEvent *mCurrentEvent;
	bool mBusy;

	static KopeteNotifier *s_notifier;
};

#endif

// vim: set noet ts=4 sts=4 sw=4:

