/*
    kopeteprefs.cpp - Kopete Preferences Container-Class

    Copyright (c) 2002      by Stefan Gehn            <sgehn@gmx.net>
    Kopete    (c) 2002-2003 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#ifndef __KOPETEPREFS_H__
#define __KOPETEPREFS_H__

#include <qobject.h>
#include <kdeversion.h>
#include <qcolor.h>

class KConfig;

class KopetePrefs : public QObject
{
	Q_OBJECT

public:
	/**
	 * The prefs container-class is a singleton object. Use this method to retrieve
	 * the instance.
	 */
	static KopetePrefs *prefs();

	~KopetePrefs();

	/**
	 * Reads all pref-variables from KConfig
	 * usually you don't need this as KopePrefs loads settings
	 * when an instance is created
	 */
	void load();
	
	/**
	 * Stores all pref-variables into KConfig
	 */
	void save();

	QString iconTheme() const { return mIconTheme; }
	bool useEmoticons() const { return mUseEmoticons; }
	bool showOffline() const { return mShowOffline; }
	#if KDE_VERSION >= 306
	bool notifyOnline() const { return mNotifyOnline; }
	#endif
	bool treeView() const { return mTreeView; }
	bool sortByGroup() const { return mSortByGroup; }
	bool greyIdleMetaContacts() const { return mGreyIdle; }
	bool startDocked() const { return mStartDocked; }
	bool useQueue() const { return mUseQueue; }
	bool raiseMsgWindow() const{ return mRaiseMsgWindow; }
	bool showEvents() const{ return mShowEvents; }
	bool trayflashNotify() const { return mTrayflashNotify; }
	bool balloonNotify() const { return mBalloonNotify; }
	bool soundNotify() const { return mSoundNotify; }
	bool soundIfAway() const { return mSoundIfAway; }
	bool beepNotify() const { return mBeepNotify; }
	bool transparencyEnabled() const { return mTransparencyEnabled; }
	int transparencyValue() const { return mTransparencyValue; }
	QColor transparencyColor() const { return mTransparencyColor; }
	bool bgOverride() const { return mBgOverride; }
/*
	bool ctransparencyEnabled() const { return mCTransparencyEnabled; }
	int ctransparencyValue() const { return mCTransparencyValue; }
	QColor ctransparencyColor() const { return mCTransparencyColor; }
*/

	int chatWindowPolicy() const { return mChatWindowPolicy; }
	QString kindMessagesHtml() const { return mKindMessagesHtml; }

	QString defaultTheme() const { return QString::fromLatin1( "Default" ); }

	void setIconTheme( const QString &value );
	void setUseEmoticons( bool value );
	void setShowOffline( bool value );
	#if KDE_VERSION >= 306
	void setNotifyOnline( bool value );
	#endif
	void setTreeView( bool value );
	void setSortByGroup( bool value );
	void setGreyIdleMetaContacts( bool value );
	void setStartDocked( bool value );
	void setUseQueue( bool value );
	void setRaiseMsgWindow( bool value );
	void setShowEvents( bool value );
	void setTrayflashNotify( bool value );
	void setBalloonNotify( bool value );
	void setSoundNotify( bool value );
	void setSoundIfAway( bool value );
	void setBeepNotify( bool value );
	void setChatWindowPolicy( int value );
	void setKindMessagesHtml ( const QString &value );
	void setTransparencyEnabled( bool value );
	void setTransparencyColor( const QColor &value );
	void setTransparencyValue( int value );
/*	void setCTransparencyEnabled( bool value );
	void setCTransparencyColor( const QColor &value );
	void setCTransparencyValue( int value );*/
	void setBgOverride( bool value );
	
	signals:
	/**
	 * Emitted when config gets saved by save()
	 */
	void saved();
	/**
	 * Emitted when the queue-preference changed
	 */
	void queueChanged();

private:
	/**
	 * Private constructor: we are a singleton
	 */
	KopetePrefs();

	/**
	 * Our instance
	 */
	static KopetePrefs *s_prefs;

	KConfig *config;

	QString mIconTheme;
	bool mUseEmoticons;
	bool mShowOffline;
	bool mGreyIdle;
	#if KDE_VERSION >= 306
	bool mNotifyOnline;
	#endif
	bool mTreeView;
	bool mSortByGroup;
	bool mStartDocked;
	bool mUseQueue;
	bool mRaiseMsgWindow;
	bool mShowEvents;
	bool mTrayflashNotify;
	bool mBalloonNotify;
	bool mSoundNotify;
	bool mSoundIfAway;
	bool mBeepNotify;
	bool mTransparencyEnabled;
	int mTransparencyValue;
	QColor mTransparencyColor;
	bool mBgOverride;
/*	bool mCTransparencyEnabled;
	int mCTransparencyValue;
	QColor mCTransparencyColor;*/

	int mChatWindowPolicy;
	QString mKindMessagesHtml;
};

#endif

// vim: set noet ts=4 sts=4 sw=4:

