/*
    kopetechatwindow.h - Chat Window

    Copyright (c) 2002 by Olivier Goffart <ogoffart@tiscalinet.be>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#ifndef KOPETECHATWINDOW_H
#define KOPETECHATWINDOW_H

#include <qstring.h>
#include <kmainwindow.h>
#include "kopetecontact.h"

namespace Kopete {
	class ChatView;
}

using Kopete::ChatView;

class KopeteMessage;
class KopeteMessageManager;
class KAction;
class KToggleAction;
class KActionMenu;
class KopeteContact;
class KTempFile;
class QPixmap;
class QTabWidget;

typedef  QPtrList<KopeteContact>  KopeteContactPtrList;

/**
 * @author Olivier Goffart
 */
class KopeteChatWindow : public KMainWindow
{
	Q_OBJECT

public:
	KopeteChatWindow(QWidget *parent = 0, const char* name = "KopeteChatWindow" );
	~KopeteChatWindow();

	KopeteMessage currentMessage();
	ChatView *addChatView( KopeteMessageManager *manager );
	void addChatView( ChatView* );
	const int chatViewCount() { return chatViewList.count(); }
	ChatView *activeView();
	KTempFile *backgroundFile;
	QPtrList<ChatView> chatViewList;
	void updateMembersActions();

	KAction *chatSend;
	KAction *historyUp;
	KAction *historyDown;

	/**
	 *
	 * Code:
	 * %M : insert the Message
	 * %T : insert Timestamp
	 * %F : insert Fonts (the first open and the second close)
	 * %b : background color ( bgcolor="XXXX" )
	 * %f : insert the displayName of the sender
	 * %t : insert the displayName of the reciever
	 *
	 * %i ... %i : code between is parsed only if inbound
	 * %o ... %o : code between is parsed only if outbound
	 * %s ... %s : code between is parsed only if internal
	 * %e ... %e : code between is parsed only if not internal (external)
	 *
	 * %% : insert a '%'
	 */
	static QString KindMessagesHTML(unsigned int k);

private:
	void initActions(void);
	void saveOptions(void);
	void readOptions(void);
	void checkDetachEnable();

	ChatView *m_activeView;
	bool widgetSet;
	bool tabsEnabled;
	const KopeteContact	*m_us;
	bool updateBg;
	QTabWidget *m_tabBar;

	KAction* editCut;
	KAction* editCopy;
	KAction* editPaste;
	KAction* editFont;
	KAction* editColor;
	KAction* editBgColor;

	KAction* chatSave;
	KAction* chatPrint;
	KAction* chatCloseAll;

	KAction *tabLeft;
	KAction *tabRight;
	KAction *tabDetach;
	KAction* tabClose;

	KToggleAction* viewToolBar;
	KToggleAction* viewMenuBar;
	KToggleAction* viewTabToolBar;
	
	KToggleAction* membersLeft;
	KToggleAction* membersRight;
	KToggleAction* membersNone;

	KActionMenu *actionSmileyMenu;
	KActionMenu *actionActionMenu;
	KActionMenu *actionContactMenu;
	KActionMenu *actionDetachMenu;

	QString statusMsg;
	QPixmap protocolIcon;

signals:
	void closing(KopeteChatWindow*);

public slots:
	void slotSmileyActivated(int sm);
	void setActiveView( QWidget *active );
	void updateBackground( const QPixmap &pm );

private slots:

	void slotPrepareSmileyMenu();
	void slotPrepareContactMenu();
	void slotPrepareActionMenu();
	void slotPrepareDetachMenu();
	
	void slotCut();
	void slotCopy();
	void slotPaste();
	
	void slotSetBgColor();
	void slotSetFgColor();
	void slotSetFont();

	void slotHistoryUp();
	void slotHistoryDown();

	void slotSendMessage();
	void slotChatSave();
	void slotChatPrint();
	void slotChatClose( QWidget* closedView = 0L );

	void slotPreviousTab();
	void slotNextTab();
	void slotDetachChat( int newWindowIndex = -1 );

	void slotViewMenuBar();
	void slotViewToolBar();
	void slotViewTabToolBar();
	
	void slotConfKeys();
	void slotConfToolbar();

	void slotViewMembersLeft();
	void slotViewMembersRight();
	void slotViewMembersNone();
	const void slotEnableUpdateBg() { updateBg = true; }

	void slotSetCaption( bool );

protected:
	virtual void closeEvent( QCloseEvent *e );
	virtual bool queryExit( );

};

#endif
