/*
    kopeteemailwindow.cpp - Generic Email-like window for most of the Kopete
                            plugins

    Copyright (c) 2002      by Daniel Stone          <dstone@kde.org>

    Kopete    (c) 2002-2003 by the Kopete developers <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include <kcolorbutton.h>
#include <kdebug.h>
#include <klineedit.h>
#include <klocale.h>
#include <kglobal.h>
#include <kconfig.h>
#include <ktoolbar.h>

#include "emailwindow.h"
#include "kopeteemailwindow.h"
#include "kopetecontact.h"
#include "kopetemessage.h"
	
KopeteEmailWindow::KopeteEmailWindow (const KopeteContact *us, KopeteContactPtrList them,
		QWidget *parent, const char *name) : dlgSendEmail (parent, name) {
	kdDebug(14010) << "[KopeteEmailWindow]: I just hit my head on a tree branch. It was solid. My head hurts." << endl;
	mUs = us;
	mThem = them;
	mMsgsWaiting = 0;
	leFrom->setText(mUs->displayName());
	leTo->setText((mThem.first())->displayName());
	mInMessage = false;
	connect(btnReadNext, SIGNAL(clicked()), this, SLOT(slotReadNext()));

	slotContactChanged();
}

KopeteEmailWindow::~KopeteEmailWindow() {
	emit deleting();
}

void KopeteEmailWindow::messageReceived(const KopeteMessage &message) {
	
	kdDebug(14010) << "KopeteEmailWindow: Message received from " << message.from()->displayName() << "\n";

	if (mInMessage == false) {
		leSubject->setText(message.subject());
		leFrom->setText((message.from())->displayName());
		leTo->setText((message.to().first())->displayName());
		teMessage->setText(message.plainBody());
		mInMessage = true;
	}
	else {
		messageQueue.append(message);
		mMsgsWaiting++;
		btnReadNext->setEnabled(true);
		btnReadNext->setCaption(i18n("Read Next (%1)").arg(mMsgsWaiting, 1));
	}
}

void KopeteEmailWindow::slotReadNext() {
	QValueList<KopeteMessage>::Iterator tmpMessage = messageQueue.begin();
	KopeteMessage message = (*tmpMessage);
	leSubject->setText(message.subject());
	leFrom->setText((message.from())->displayName());
	leTo->setText((message.to().first())->displayName());
	teMessage->setText(message.plainBody());
	messageQueue.remove(tmpMessage);
	mMsgsWaiting--;
	if (mMsgsWaiting == 0) {
		btnReadNext->setEnabled(false);
		btnReadNext->setCaption( QString::fromLatin1( "Read Next" ) );
	}
}

void KopeteEmailWindow::slotCloseClicked() {
	emit closeClicked();
}

void KopeteEmailWindow::slotSendClicked() {
	teMessage->setFocus();
	if (teMessage->text().isEmpty())
		return;
	/* o/~ free-free-free, fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr
	 * fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr
	 * fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr
	 * fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr-fr
	 * free-free-freestyyyler ~/o */
	KopeteMessage tmpMessage = KopeteMessage(mUs, mThem, teMessage->text(), leSubject->text(), KopeteMessage::Outbound);
	emit(sendMessage(tmpMessage));
	teMessage->clear();
	emit closeClicked(); /* Ugh. */
}

void KopeteEmailWindow::slotReplyClicked() {
	emit(replyClicked());
}

void KopeteEmailWindow::slotContactChanged() {
	KopeteContact *other = mThem.first();
	setCaption( i18n( "%1 (%2)" ).arg( other->displayName() ).arg( other->statusText() ));
	setIconText( caption() );
	setIcon( other->statusIcon() );
}

void KopeteEmailWindow::setSendEnabled(bool b) {
	bool c;
	mSendEnabled = b;
	kdDebug(14010) << "[KopeteEmailWindow] About to enable a lot of stuff. Whooha! (" << b << ")" << endl;
	if (b == false) /* Gar, silly inversion. */
		c = true;
	else
		c = false;
	if (b == false) {
		btnSend->hide();
		btnReadNext->show();
		btnReply->show();
	}
	else {
		btnSend->show();
		btnReadNext->hide();
		btnReply->hide();
	}
	leFrom->setEnabled(b);
	leTo->setEnabled(b);
	leSubject->setEnabled(b);
	leSubject->setReadOnly(c);
	teMessage->setEnabled(b);
	teMessage->setReadOnly(c);
	btnSend->setEnabled(b);
}

void KopeteEmailWindow::setReplyMode(bool b) {
	if (b == true) {
		btnReply->hide();
	}
	else {
		btnReply->show();
	}
}

void KopeteEmailWindow::showEvent(QShowEvent * /* e */) {
/*        kdDebug(14010) << "[KopeteEmailWindow] showEvent( e )" << endl;
		KDialogBase::showEvent ( e );
		emit shown(); */
}

#include "kopeteemailwindow.moc"


/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 8
 * indent-tabs-mode: t
 * End:
 */
// vim: set noet ts=4 sts=4 sw=4:

