/*
	kopeteemailwindow.cpp - Generic Email-like Window for most of the Kopete
	                        plugins

	Copyright   : (c) 2002 by Daniel Stone <dstone@kde.org>

	Based on Martijn's KopeteChatWindow stuff.

	*************************************************************************
	*                                                                       *
	* This program is free software; you can redistribute it and/or modify  *
	* it under the terms of the GNU General Public License as published by  *
	* the Free Software Foundation; either version 2 of the License, or     *
	* (at your option) any later version.                                   *
	*                                                                       *
	*************************************************************************
*/

#ifndef __KOPETEEMAILWINDOW_H__
#define __KOPETEEMAILWINDOW_H__

#include <emailwindow.h>
#include <qlineedit.h>
#include <qtextedit.h>

class KopeteContact;
class KopeteMessageLog;
class KopeteMessage;

typedef QPtrList<KopeteContact>        KopeteContactPtrList;
typedef QValueList<KopeteMessage>        KopeteMessageList;

/**
 * @author Daniel Stone <dstone@kde.org>
 *
 */

class KopeteEmailWindow : public dlgSendEmail
{
	Q_OBJECT

public:
    KopeteEmailWindow(const KopeteContact *us, KopeteContactPtrList them, QWidget
			          *parent = 0, const char* name = 0);
	~KopeteEmailWindow();

	void setSendEnabled(bool sendEnabled);
	bool sendEnabled() { return mSendEnabled; }
	void setReplyMode(bool replyMode);

public slots:
	void messageReceived(const KopeteMessage &message);

signals:
	void sendMessage(const KopeteMessage &message);
	void shown();
	void deleting();
	void closeClicked();
	void replyClicked();

protected:
	virtual void showEvent (QShowEvent *);

private slots:
	void slotContactChanged();
	void slotCloseClicked();
	void slotSendClicked();
	void slotReplyClicked();
	void slotReadNext();

private:
	bool mSendEnabled, mInMessage;
	const KopeteContact *mUs;
	KopeteContactPtrList mThem;
	QValueList<KopeteMessage> messageQueue;
	int mMsgsWaiting;
};

#endif



/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 8
 * indent-tabs-mode: t
 * End:
 */
// vim: set noet ts=4 sts=4 sw=4:

