/*
    kopetehistorywidget.cpp - Kopete History Widget

    Copyright (c) 2002      by Richard Stellingwerff <remenic@linuxfromscratch.org>
    Copyright (c) 2002-2003 by Martijn Klingens      <klingens@kde.org>

    Kopete    (c) 2002-2003 by the Kopete developers <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include "kopetehistorywidget.h"
#include "kopetemessage.h"
#include "kopeteprotocol.h"

#include <sys/time.h>

#include <qapplication.h>
#include <qtextstream.h>

#include <kdebug.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kstandarddirs.h>
#include <qdialog.h>
#include <qdom.h>
#include <qpushbutton.h>
#include <qprogressbar.h>
#include <qregexp.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qgroupbox.h>
#include <qlayout.h>
#include <qfile.h>
#include <qptrlist.h>

#include <ktextbrowser.h>

#define CBUFLENGTH 512 // buffer length for fgets()

KopeteHistoryWidget::KopeteHistoryWidget( KopeteContact *mContact, int count, QWidget* parent, const char* name )
: QWidget( parent, name )
{
	//Create the log file name for this contact
	QString logFileName = QString::fromLatin1( mContact->protocol()->pluginId() ) + QString::fromLatin1( "/" ) +
		mContact->contactId().replace( QRegExp( QString::fromLatin1( "[./~]" ) ), QString::fromLatin1( "-" ) ) +
		QString::fromLatin1( ".log" );

	//Add it to the QStringList
	logFileNames.append( locateLocal( "data", QString::fromLatin1( "kopete/" ) + logFileName ) );

	buildWidget(count);
}

KopeteHistoryWidget::KopeteHistoryWidget( KopeteMetaContact *mContact, int count, QWidget* parent, const char* name )
: QWidget( parent, name )
{
	//Lets iterate through the contacts of this guy
	QPtrList<KopeteContact> m_contacts = mContact->contacts();
	QPtrListIterator<KopeteContact> it( m_contacts );
	for( ; it.current(); ++it )
	{
		//Create the log file name for this contact
		QString logFileName = QString::fromLatin1( it.current()->protocol()->pluginId() ) + QString::fromLatin1( "/" ) +
			it.current()->contactId().replace( QRegExp( QString::fromLatin1( "[./~]" ) ), QString::fromLatin1( "-" ) ) +
			QString::fromLatin1( ".log" );

		//Add it to the QStringList
		logFileNames.append( locateLocal( "data", QString::fromLatin1( "kopete/" ) + logFileName ) );
	}

	buildWidget(count);
}

void KopeteHistoryWidget::buildWidget(int count)
{
	msgStart = 0; // always display newest message first
	msgCount = count; // 50 by default
	mUser = "";
	mSuperBuffer = "";

	layout = new QGridLayout( this, 1, 1, 6, 6 );
	mHistoryView = new KTextBrowser(this, "mHistoryView");

	layout->addMultiCellWidget( mHistoryView, 0, 0, 0, 5);

	optionsBox = new QGroupBox(this, "optionsBox");
	optionsBox->setTitle( i18n("Options") );
	optionsBox->setColumnLayout(0, Qt::Vertical);
	optionsBox->layout()->setSpacing( 6 );
	optionsBox->layout()->setMargin( 11 );

	optionsLayout = new QGridLayout( optionsBox->layout() );
	optionsLayout->setAlignment( Qt::AlignTop );

	optionsCBLayout = new QHBoxLayout(0, 0, 6, "optionsCBLayout");

	mSearchLabel = new QLabel(optionsBox, "mSearchLabel");
	mSearchLabel->setText( i18n("Search:") );
	optionsCBLayout->addWidget(mSearchLabel);

	mSearchInput = new QLineEdit(optionsBox, "mSearchInput");
	optionsCBLayout->addWidget(mSearchInput);
	mSearchInput->setFocus();

	mSearchButton = new QPushButton( optionsBox, "mSearchButton" );
	mSearchButton->setText( i18n("&Search") );
	mSearchButton->setDefault( true );

	optionsCBLayout->addWidget( mSearchButton );

	optionsLayout->addMultiCellLayout( optionsCBLayout, 0, 0, 0, 1 );

	mReverse = new QCheckBox( optionsBox, "mReverse" );
	mReverse->setText( i18n("Show &oldest message first") );

	optionsLayout->addWidget( mReverse, 1, 0 );

	mIncoming = new QCheckBox( optionsBox, "mIncoming" );
	mIncoming->setText( i18n("Only show &incoming messages") );

	optionsLayout->addWidget( mIncoming, 1, 1 );

	layout->addMultiCellWidget( optionsBox, 1, 1, 0, 5 );

	mBack = new QPushButton( this, "mBack" );
	mBack->setPixmap( SmallIcon( QString::fromLatin1( "2leftarrow" ) ) );

	layout->addWidget( mBack, 2, 0 );

	mPrevious = new QPushButton(this, "mPrevious");
	mPrevious->setPixmap( SmallIcon( QString::fromLatin1( "1leftarrow" ) ) );

	layout->addWidget( mPrevious, 2, 1 );

	mNext = new QPushButton(this, "mNext");
	mNext->setPixmap( SmallIcon( QString::fromLatin1( "1rightarrow" ) ) );

	layout->addWidget( mNext, 2, 2 );

	mForward = new QPushButton(this, "mForward");
	mForward->setPixmap( SmallIcon( QString::fromLatin1( "2rightarrow" ) ) );

	layout->addWidget( mForward, 2, 3 );

	mProgress = new QProgressBar(50, this, "progress");
	mProgress->setCenterIndicator( true );

	layout->addMultiCellWidget(mProgress, 2, 2, 4, 5);

	// all buttons disabled by default
	mNext->setEnabled( false );
	mPrevious->setEnabled( false );
	mBack->setEnabled( false );
	mForward->setEnabled( false );
	optionsBox->setEnabled( false );

	connect( mNext, SIGNAL(clicked()), this, SLOT(slotNextClicked()));
	connect( mPrevious, SIGNAL(clicked()), this, SLOT(slotPrevClicked()));
	connect( mForward, SIGNAL(clicked()), this, SLOT(slotForwardClicked()));
	connect( mBack, SIGNAL(clicked()), this, SLOT(slotBackClicked()));

	connect( mReverse, SIGNAL(toggled(bool)), this, SLOT(slotReversedToggled(bool)));
	connect( mIncoming, SIGNAL(toggled(bool)), this, SLOT(slotIncomingToggled(bool)));
	connect( mSearchButton, SIGNAL(clicked()), this, SLOT(slotSearchClicked()));
}
	
void KopeteHistoryWidget::init()
{
	kdDebug(14010) << "[KopeteHistoryWidget] START init()" << endl;
	
	xmllist = new QDomDocument( QString::fromLatin1( "History" ) );

	messages = countMessages();
	
	if ( messages == 0 )
	{
		//There are no messages for this contact
		mHistoryView->setText(i18n("No history for user %1.").arg(mUser));
		mPrevious->setEnabled(false);
		mNext->setEnabled(false);
	}
	else
	{
		// start displaying the log
		readLog(msgStart);

		// enable the options GroupBox
		optionsBox->setEnabled( true );
	}

	kdDebug(14010) << "[KopeteHistoryWidget] END init()" << endl;
}

KopeteHistoryWidget::~KopeteHistoryWidget()
{
	kdDebug(14010) << "KopeteHistoryWidget::~KopeteHistoryWidget()" << endl;
}

// return wether or not it failed?
void KopeteHistoryWidget::readLog(int start)
{
	kdDebug(14010) << "[KopeteHistoryWidget] readLog()" << endl;

	QDomElement msgelement;
	QDomNode node;

	KopeteMessage::MessageDirection dir;

	QString body, date, nick;
	QString buffer, msgBlock;

	char cbuf[CBUFLENGTH]; // buffer for the log file

	int currentPos=0;
	int startPos;
	int endPos;

	msgStart = start;

	if ( ! mReverse->isOn() )
	{
		startPos = messages - (msgCount + msgStart);
		endPos = messages - msgStart;

		if (startPos < 0)
			startPos = 0;

		if (endPos > messages)
			endPos = messages;
	}
	else
	{
		startPos = msgStart;
		endPos = msgStart + msgCount;

		if (startPos < 0)
			startPos = 0;

		if (endPos > messages)
			endPos = messages;
	}

	int steps = msgCount / 10; // split the progressbar into 10 parts.
	int sPos = 0;

	if (steps < 0) steps = 1;

	if ( mReverse->isOn() )
	{
		if (startPos == 0)
		{
			mPrevious->setEnabled(false);
			mBack->setEnabled(false);
		}
		else
		{
			mPrevious->setEnabled(true);
			mBack->setEnabled(true);
		}

		if (endPos >= messages)
		{
			mNext->setEnabled( false );
			mForward->setEnabled( false );
		}
		else
		{
			mForward->setEnabled( true );
			mNext->setEnabled(true);
		}
	}
	else // normal display
	{
		if (startPos == 0)
		{
			mNext->setEnabled(false);
			mForward->setEnabled(false);
		}
		else
		{
			mNext->setEnabled(true);
			mForward->setEnabled(true);
		}

		if (endPos >= messages)
		{
			mPrevious->setEnabled(false);
			mBack->setEnabled(false);
		}
		else
		{
			mPrevious->setEnabled(true);
			mBack->setEnabled(true);
		}
	}


	mProgress->setTotalSteps(10);

	// show the user that it's being processed (i know, this method is ugly)
	mProgress->setProgress(1);
		
	//Loop through the logfiles in the list, process each one
	for ( QStringList::Iterator it = logFileNames.begin(); it != logFileNames.end(); ++it ) 
	{
		QString logFileName = *it;
		
		// open the file
		FILE *f = fopen(QFile::encodeName(logFileName), "r");

		// find the first message to display ( this needs to be faster! )
		while ( !feof( f ) && currentPos != startPos)
		{
			fgets(cbuf, CBUFLENGTH, f);

			buffer = QString::fromUtf8(cbuf);

			while ( strchr(cbuf, '\n') == NULL && !feof(f) ) // Keep reading till we got one line.
			{
				fgets( cbuf, CBUFLENGTH, f );
				buffer += QString::fromUtf8(cbuf);
			}

			if (mIncoming->isOn())
			{
				if( buffer.startsWith( QString::fromLatin1( "<message " ) ) )
				{
					if( buffer.contains( QString::fromLatin1( "direction=\"inbound\"" ) ) )
						currentPos++;
				}
			}
			else
			{
				if( buffer.startsWith( QString::fromLatin1( "<message " ) ) )
					currentPos++;
			}
		}

		currentPos++;

		// create a new <message> block
		while ( ! feof( f ) && currentPos <= endPos)
		{
			fgets(cbuf, CBUFLENGTH, f);
			buffer = QString::fromUtf8(cbuf);

			while ( strchr(cbuf, '\n') == NULL && !feof(f) )
			{
				fgets( cbuf, CBUFLENGTH, f );
				buffer += QString::fromUtf8(cbuf);
			}

			if( buffer.startsWith( QString::fromLatin1( "<message " ) ) )
			{
				msgBlock = buffer;

				// find the end of the message block
				while( !feof( f ) && buffer != QString::fromLatin1( "</message>\n" ) /*strcmp("</message>\n", cbuf )*/ )
				{
					fgets(cbuf, CBUFLENGTH, f);
					buffer = QString::fromUtf8(cbuf);

					while ( strchr(cbuf, '\n') == NULL && !feof(f) )
					{
						fgets( cbuf, 511, f );
						buffer += QString::fromUtf8(cbuf);
					}

					msgBlock.append(buffer);
				}

				// now let's work on this new block
				xmllist->setContent(msgBlock, false);
				msgelement = xmllist->documentElement();
				node = msgelement.firstChild();

				if( msgelement.attribute( QString::fromLatin1( "direction" ) ) == QString::fromLatin1( "inbound" ) )
					dir = KopeteMessage::Inbound;
				else
				{
					dir = KopeteMessage::Outbound;

					// skip outbound messages if only incoming messages want to be seen
					if (mIncoming->isOn())
						continue;
				}

				// Read all the elements.
				QString tagname;
				QDomElement element;

				while ( ! node.isNull() )
				{

					if ( node.isElement() )
					{
						element = node.toElement();
						tagname = element.tagName();

						if( dir == KopeteMessage::Inbound && tagname == QString::fromLatin1( "srcnick" ) )
							nick = element.text();
						else if( dir == KopeteMessage::Outbound && tagname == QString::fromLatin1( "destnick" ) )
							nick = element.text();
						else if( tagname == QString::fromLatin1( "date" ) )
							date = element.text();
						else if( tagname == QString::fromLatin1( "body" ) )
							body = element.text();
					}

					node = node.nextSibling();
				}

				// add the message to the history view
				addMessage(dir, nick, date, body);
				currentPos++;

				// update the progress bar
				sPos++;
				mProgress->setProgress( sPos / steps );
			}
		}

		fclose( f );
	}

	mProgress->reset();
	mHistoryView->setText(mSuperBuffer);

}

int KopeteHistoryWidget::countMessages()
{
	QString buffer;
	char cbuf[CBUFLENGTH];
	int n = 0;
			
	kdDebug(14010) << "[KopeteHistoryWidget] START countMessages" << endl;
	
	for ( QStringList::Iterator it = logFileNames.begin(); it != logFileNames.end(); ++it )
	{
		kdDebug(14010) << "Opening File: " << QFile::encodeName(*it) << endl;
		
		//Check if thie file exists
		FILE *f = fopen(QFile::encodeName(*it), "r");

		// FIXME: fopen instead of QFile ??????? - Martijn
		if (f == NULL)
		{
			// Oops, no file
		}
		else
		{
			if (mIncoming->isOn())
			{
				while ( !feof(f) )
				{
					fgets(cbuf, CBUFLENGTH, f);
					buffer = cbuf;

					while ( strchr(cbuf, '\n') == NULL  && !feof(f) )
					{
						fgets(cbuf, CBUFLENGTH, f);
						buffer += QString::fromUtf8( cbuf );
					}

					if ( !strncmp( "<message ", cbuf, 9 ) )
					{
						if( buffer.contains( QString::fromLatin1( "inbound" ) ) )
							n++;
						else
							continue;
					}
				}
			}
			else
			{
				kdDebug(14010) << "Loop file..." << endl;

				while ( !feof(f) )
				{
					fgets(cbuf, CBUFLENGTH, f);
					buffer = cbuf;

					while ( strchr(cbuf, '\n') == NULL && !feof( f ) )
					{
						fgets(cbuf, CBUFLENGTH, f);
						buffer += QString::fromUtf8( cbuf );
					}

					if ( /*buffer.startsWith("<message ")*/ !strncmp("<message ", cbuf, 9 ) )
						n++;
					else
						continue;
				}
			}

			// Close file only when it exists
			fclose(f);
		}
	}

	kdDebug(14010) << "[KopeteHistoryWidget] " << n << " Messages in log" << endl;

	return n;
}

void KopeteHistoryWidget::addMessage(KopeteMessage::MessageDirection dir, QString nick, QString date, QString body)
{
	QString message = QString::fromLatin1( "<table width=\"100%\" cellspacing=\"0\" cellpadding=\"0\">" );

	if( dir == KopeteMessage::Inbound )
	{
		message += QString::fromLatin1( "<tr><td><font color=\"#0360B1\"><b>" ) +
			i18n( "Message from %1 at %2:" ).arg( nick ).arg( date );
	}
	else
	{
		message += QString::fromLatin1( "<tr><td><font color=\"#E11919\"><b>" ) +
			i18n( "Message to %1 at %2:" ).arg( nick ).arg( date );
	}

	message += QString::fromLatin1(
		"</b></font></td></tr></table>\n"
		"<table width=\"100%\" cellspacing=\"0\" cellpadding=\"0\"><tr><td>" ) +
		body.stripWhiteSpace() + QString::fromLatin1( "</tr></td></table><br><br>" );

	if( mSuperBuffer.isEmpty() )
	{
		mSuperBuffer = message + QString::fromLatin1( "\n" );
	}
	else
	{
		if( mReverse->isOn() )
			mSuperBuffer += message + '\n';
		else
			mSuperBuffer.prepend( message + '\n' );
	}
}

void KopeteHistoryWidget::slotPrevClicked()
{
	msgStart -= 50;
	mSuperBuffer = "";
	readLog(msgStart);
}

void KopeteHistoryWidget::slotNextClicked()
{
	msgStart += 50;
	mSuperBuffer = "";
	readLog(msgStart);
}

void KopeteHistoryWidget::slotBackClicked()
{
	msgStart = 0;
	mSuperBuffer = "";
	readLog(msgStart);
}

void KopeteHistoryWidget::slotForwardClicked()
{
	msgStart = messages - msgCount;
	mSuperBuffer = "";
	readLog(msgStart);
}

void KopeteHistoryWidget::slotSearchClicked()
{
	kdDebug(14010) << "[KopeteHistoryWidget] slotSearchClicked()" << endl;

	if (mSearchInput->text().isEmpty())
		return;

	mPrevious->setEnabled( false );
	mNext->setEnabled( false );

	mSuperBuffer = "";

	QDomElement msgelement;
	QDomNode node;

	KopeteMessage::MessageDirection dir;

	int currentPos=0;
	int steps = messages / 20; // split the scrollbar into 20 parts.

	if ( steps == 0 )
		steps = 1; // avoid division by zero

	QString body, date, nick;
	QString buffer, msgBlock;

	char cbuf[CBUFLENGTH]; // buffer for the log file

	mProgress->setTotalSteps(20); // split the scrollbar into 20 parts.

	//Loop through the logfiles in the list, process each one
	for ( QStringList::Iterator it = logFileNames.begin(); it != logFileNames.end(); ++it ) 
	{
		QString logFileName = *it;
	
		// open the file
		FILE *f = fopen(QFile::encodeName(logFileName), "r");

		// create a new <message> block
		while ( ! feof( f ) )
		{
			fgets(cbuf, CBUFLENGTH, f);
			buffer = QString::fromUtf8(cbuf);

			while ( strchr(cbuf, '\n') == NULL && !feof(f) )
			{
				fgets( cbuf, CBUFLENGTH, f );
				buffer += QString::fromUtf8(cbuf);
			}

			if( buffer.startsWith( QString::fromLatin1( "<message " ) ) )
			{
				msgBlock = buffer;

				// find the end of the message block
				while ( ! feof( f ) && /*buffer != "</message>\n"*/ strcmp("</message>\n", cbuf ) )
				{
					fgets(cbuf, CBUFLENGTH, f);
					buffer = QString::fromUtf8(cbuf);

					while ( strchr(cbuf, '\n') == NULL && !feof(f) )
					{
						fgets( cbuf, CBUFLENGTH, f );
						buffer += QString::fromUtf8(cbuf);
					}

					msgBlock.append(buffer);
				}

				// now let's work on this new block
				xmllist->setContent(msgBlock, false);
				msgelement = xmllist->documentElement();
				node = msgelement.firstChild();

				if( msgelement.attribute( QString::fromLatin1( "direction" ) ) == QString::fromLatin1( "inbound" ) )
					dir = KopeteMessage::Inbound;
				else
				{
					dir = KopeteMessage::Outbound;

					// skip outbound messages if only incoming messages want to be seen
					if (mIncoming->isOn())
						continue;
				}

				nick = date = body = "";

				// Read all the elements.
				while ( ! node.isNull() )
				{
					QString tagname;
					QDomElement element;

					if ( node.isElement() )
					{
						element = node.toElement();
						tagname = element.tagName();

						if( dir == KopeteMessage::Inbound && tagname == QString::fromLatin1( "srcnick" ) )
							nick = element.text();
						else if( dir == KopeteMessage::Outbound && tagname == QString::fromLatin1( "destnick" ) )
							nick = element.text();
						else if( tagname == QString::fromLatin1( "date" ) )
							date = element.text();
						else if( tagname == QString::fromLatin1( "body" ) )
						{
							if (!element.text().contains(mSearchInput->text()))
								break;

							body = element.text();
						}
					}

					node = node.nextSibling();
				}

				// add the message to the history view

				if (!body.isEmpty())
					addMessage(dir, nick, date, body);

				currentPos++;

				mProgress->setProgress (currentPos / steps);
				qApp->processEvents();
			}
		}

		fclose( f );
	}

	mProgress->reset();

	if ( ! mSuperBuffer.isEmpty() )
		mHistoryView->setText(mSuperBuffer);
	else
		mHistoryView->setText(i18n("No matches found."));

}

void KopeteHistoryWidget::slotReversedToggled( bool /* b */ )
{
	// FIXME: Honour the bool!
	mSuperBuffer = "";
	msgStart = 0;
	readLog(msgStart);
}

void KopeteHistoryWidget::slotIncomingToggled( bool /* b */ )
{
	// FIXME: Honour the bool!
	// recount messages
	messages = countMessages();

	mSuperBuffer = "";
	msgStart = 0;
	readLog(msgStart);
}

#include "kopetehistorywidget.moc"

// vim: set noet ts=4 sts=4 sw=4:

