/*
    kopetehistorywidget.h

	Kopete History Widget

	Copyright (c) 2001-2002 by
	Richard Stellingwerff <remenic@linuxfromscratch.org>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#ifndef _HISTORYWIDGET_H
#define _HISTORYWIDGET_H

#include <qwidget.h>
#include "kopetecontact.h"
#include "kopetemetacontact.h"
#include "kopetemessage.h"

class QGridLayout;
class QPushButton;
class KTextBrowser;
class QDomDocument;
class QFile;
class QProgressBar;
class QLabel;
class QLineEdit;
class QGroupBox;
class QCheckBox;
class QBoxLayout;

/**
 * @author Richard Stellingwerff <remenic@linuxfromscratch.org>
 *
 */

class KopeteHistoryWidget : public QWidget
{
	Q_OBJECT
public:
	KopeteHistoryWidget( KopeteContact *mContact, int count = 50, QWidget* parent=0, const char* name= "KopeteHistoryWidget");
	KopeteHistoryWidget( KopeteMetaContact *mContact, int count = 50,  QWidget* parent = 0, const char* name="KopeteHistoryWidget" );
	~KopeteHistoryWidget();
	void init();

private:

	QDomDocument *xmllist;			// xml stuff

	KTextBrowser *mHistoryView;	// the actual log view

	int messages;			// total amount of messages in the log
	int msgStart;			// where to start reading
	int msgCount;			// amount of entries to read at once

	QGridLayout *layout;					// main layout

	// BASIC CONTROLS
	QPushButton *mPrevious;			// previous button "<"
	QPushButton *mNext;				// next button ">"
	QPushButton *mBack;				// all the way back "<<"
	QPushButton *mForward;			// all the way forward ">>"

	QPushButton *mSearchButton;	// search button
	QProgressBar *mProgress;		// progress bar

	QLabel *mSearchLabel;			// "Search: " label
	QLineEdit *mSearchInput;		// input field for search

	QGroupBox *optionsBox;			// the options groupbox
	QGridLayout *optionsLayout;		// the main layout for the options groupbox
	QBoxLayout *optionsCBLayout;	// check boxes layout

	QCheckBox *mReverse;				// show oldest message first
	QCheckBox *mIncoming;			// only show incoming messages

	QString mSuperBuffer,			// A buffer speeds up a lot
				mUser;					// Nickname of the user.


	QStringList logFileNames;		//List of logFileNames
	
	void readLog(int start);		// display the log from -start-
	int countMessages();				// count the amount of messages in the log
	void buildWidget(int count);
	void addMessage( KopeteMessage::MessageDirection dir, QString nick, QString date, QString body);

private slots:
	void slotPrevClicked();			// < button clicked
	void slotNextClicked();			// > button clicked
	void slotBackClicked();			// << pressed
	void slotForwardClicked();		// >> pressed
	
	void slotSearchClicked();		// search button clicked
	void slotReversedToggled(bool);	// checkbox mReversed toggled
	void slotIncomingToggled(bool);	// checkbox mIncoming toggled

signals:
	void closing();					// let the world know i wanna die
};

#endif


/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 8
 * indent-tabs-mode: t
 * End:
 */
// vim: set noet ts=4 sts=4 sw=4:

