/*
    systemtray.cpp  -  Kopete Tray Dock Icon

    Copyright (c) 2002      by Nick Betcher           <nbetcher@kde.org>
    Copyright (c) 2002-2003 by Martijn Klingens       <klingens@kde.org>

    Kopete    (c) 2002-2003 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include "systemtray.h"

#include <qcursor.h>
//#include <qpixmap.h>
#include <qpoint.h>
#include <qtimer.h>
#include <qtooltip.h>

#include <kaboutdata.h>
#include <kapplication.h>
#include <kdebug.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kpopupmenu.h>

KopeteSystemTray* KopeteSystemTray::s_systemTray = 0L;

KopeteSystemTray* KopeteSystemTray::systemTray( QWidget *parent, const char* name )
{
	if( !s_systemTray )
		s_systemTray = new KopeteSystemTray( parent, name );

	return s_systemTray;
}

KopeteSystemTray::KopeteSystemTray(QWidget* parent, const char* name)
	: KSystemTray(parent,name)
{
//	kdDebug(14010) << "Creating KopeteSystemTray" << endl;
	QToolTip::add( this, kapp->aboutData()->shortDescription() );

	mIsBlinkIcon = false;
	mIsBlinking = false;
	mBlinkTimer = new QTimer(this, "mBlinkTimer" );

	//mKopeteIcon = kapp->miniIcon(); 
	mKopeteIcon = QPixmap( BarIcon( QString::fromLatin1( "kopete" ) ) );
	mBlinkIcon = KGlobal::iconLoader()->loadIcon( QString::fromLatin1( "newmsg" ), KIcon::User);
	mMovie = KGlobal::iconLoader()->loadMovie( QString::fromLatin1( "newmessage" ), KIcon::User);

	QObject::connect(mBlinkTimer, SIGNAL(timeout()), this, SLOT(slotBlink()));

	setPixmap(mKopeteIcon);
//	show();
}

KopeteSystemTray::~KopeteSystemTray()
{
//	kdDebug(14010) << "[KopeteSystemTray] ~KopeteSystemTray" << endl;
//	delete mBlinkTimer;
}

void KopeteSystemTray::mousePressEvent( QMouseEvent *me )
{
	if ( me->button() == QEvent::RightButton )
	{
		// We really dont need this, but may be useful for some plugin
		emit rightClicked ( QPoint(me->x(), me->y()) );
		// We need just to popup the menu
		//contextMenu()->popup(QCursor::pos());
	}
	else if ( me->button() == QEvent::MidButton )
	{
		// We really dont need this, but may be useful for some plugin
		if ( !mIsBlinking )
			emit middleClicked ( QPoint(me->x(), me->y()) );
		else // hack to get mid-button to open new message-windows
			emit doubleClicked ( QPoint(me->x(), me->y()) );
	}
	else // leave hide/show mainwindow for KSystemTray
	{
		emit leftClicked ( QPoint(me->x(), me->y()) );
	}
	if ( !mIsBlinking )
	{
		KSystemTray::mousePressEvent( me );
	}
}

void KopeteSystemTray::mouseDoubleClickEvent( QMouseEvent *me )
{
	if ( me->button() == QEvent::LeftButton )
	{
		if ( !mIsBlinking )
		{
			KSystemTray::mouseDoubleClickEvent( me );
		}
		emit doubleClicked ( QPoint(me->x(), me->y()) );
	}
}

void KopeteSystemTray::startBlink( QString icon )
{
	mBlinkIcon = QPixmap ( KGlobal::iconLoader()->loadIcon( icon , KIcon::User) );
	startBlinkPrivate();
}

void KopeteSystemTray::startBlink( QPixmap icon )
{
	mBlinkIcon = icon;
	startBlinkPrivate();
}

void KopeteSystemTray::startBlink( QMovie icon )
{
	kdDebug(14010) << k_funcinfo << "starting movie." << endl;
	setMovie(icon);
	mIsBlinking = true;
}

void KopeteSystemTray::startBlink()
{
//	mBlinkIcon = QPixmap ( KGlobal::iconLoader()->loadIcon( QString::fromLatin1( "newmsg" ), KIcon::User) );
//	startBlinkPrivate();
	startBlink(mMovie);
}

void KopeteSystemTray::startBlinkPrivate()
{

	if (mBlinkTimer->isActive() == false)
	{
		mIsBlinkIcon = true;
		mIsBlinking = true;
		mBlinkTimer->start(1000, false);
	}
	else
	{
		mBlinkTimer->stop();
		mIsBlinkIcon = true;
		mIsBlinking = true;
		mBlinkTimer->start(1000, false);
	}
}

void KopeteSystemTray::stopBlink()
{
	if(movie())
	{
		kdDebug(14010) << k_funcinfo << "stopping movie." << endl;
		setPixmap(mKopeteIcon);
		mIsBlinkIcon = false;
		mIsBlinking=false;
		return;
	}

	if (mBlinkTimer->isActive() == true)
	{
		mBlinkTimer->stop();
		setPixmap(mKopeteIcon);
		mIsBlinkIcon = false;
		mIsBlinking = false;

	}
	else
	{
		setPixmap(mKopeteIcon);
		mIsBlinkIcon = false;
		mIsBlinking = false;
	}
}

void KopeteSystemTray::slotBlink()
{
	if (mIsBlinkIcon == true)
	{
		setPixmap(mKopeteIcon);
		mIsBlinkIcon = false;
	}
	else
	{
		setPixmap(mBlinkIcon);
		mIsBlinkIcon = true;
	}
}

#include "systemtray.moc"

// vim: set noet ts=4 sts=4 sw=4:

