/***************************************************************************
                          autoawayplugin.cpp  -  description
                             -------------------
    begin                : mi abr 10 2002
    copyright            : (C) 2002 by Duncan Mac-Vicar Prett
    email                : duncan@kde.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include <kgenericfactory.h>
#include <kstandarddirs.h>

#include "autoawayplugin.h"
#include "autoawaypreferences.h"
#include "kopeteaway.h"
#include "kopeteidentitymanager.h"
#include "xautolock.h"

K_EXPORT_COMPONENT_FACTORY( kopete_autoaway, KGenericFactory<AutoAwayPlugin> );

AutoAwayPlugin::AutoAwayPlugin( QObject *parent, const char *name,
	const QStringList & /* args */ )
: KopetePlugin( parent, name )
{
	mPrefs = new AutoAwayPreferences ( "date", this );
	mWatcher = new XAutoLock();

	mActive = false;

	connect( mWatcher, SIGNAL(timeout()), this, SLOT(slotTimeout()) );
	connect( mWatcher, SIGNAL(activity()), this, SLOT(slotActivity()) );
	connect( mPrefs, SIGNAL(saved()), this, SLOT(slotSettingsChanged()) );

	mWatcher->setTimeout( mPrefs->awayTimeout() * 60);
	mWatcher->start();
}

AutoAwayPlugin::~AutoAwayPlugin()
{
}

void AutoAwayPlugin::init()
{
}

bool AutoAwayPlugin::unload()
{
	kdDebug(14300) << "[AutoAway Plugin] : Stopping Watcher" << endl;
	mWatcher->stop();
	kdDebug(14300) << "[AutoAway Plugin] : Freeing memory" << endl;
	delete mWatcher;
	return KopetePlugin::unload();
}

void AutoAwayPlugin::slotTimeout()
{
	if ( !KopeteAway::globalAway() )
	{
		kdDebug(14300) << "[AutoAway Plugin] : Timeout and no user activity, going away" << endl;
		mActive = true;
		KopeteIdentityManager::manager()->setAwayAll();
	}
}

void AutoAwayPlugin::slotActivity()
{
	if ( mActive && mPrefs->goAvailable() )
	{
		kdDebug(14300) << "[AutoAway Plugin] : User activity!, going available" << endl;
		mActive = false;
		KopeteIdentityManager::manager()->setAvailableAll();
	}
}

void AutoAwayPlugin::slotSettingsChanged()
{
	mWatcher->setTimeout( mPrefs->awayTimeout() * 60 );
}

#include "autoawayplugin.moc"

// vim: set noet ts=4 sts=4 sw=4:

