/*
    imp.cpp

    Kopete Importer plugin

    Copyright (c) 2002 by James Grant   <topace@lightbox.org>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include "imp.moc"

#include "imp.h"

#include <iostream>

#include <qobject.h>
#include <qstringlist.h>
#include <qfile.h>
#include <qvaluelist.h>
#include <qstring.h>
#include <qtextstream.h>
#include <qdir.h>
#include <qregexp.h>

Imp::Imp(const QString &key,const QString &location)
{

	if(key=="licq")
	{
//		cout << "importing licq contacts from "<<location<<endl;
		//need to open the directory and all the  .uin files within the directory
		QDir d(location,"*.uin",QDir::Name,QDir::Files);
//		cout << "d is " << d.count() << endl;
    		for ( unsigned int i = 0; i < d.count(); i++ )
		{
			QString uinstr=d[i];
			uinstr.truncate(uinstr.length()-4);

			QFile file(location+"/"+d[i]);
			if(file.open(IO_ReadOnly)) 
			{
				QRegExp rx("Alias = (.*)$");
				QTextStream stream(&file);
				QString line;
				while(!stream.eof())
				{
					line=stream.readLine();
					if(rx.search(line)>=0)
					{
//						cout << "  Alias is "<<rx.cap(1)<<endl;
						QString alias=rx.cap(1);
						contacts[uinstr]=QPair<QString,QString>("icq",alias);
					}

				}
				file.close();
			}
		}
	}
	else if(key=="trillian")
	{
		QFile file(location);
		if(file.open(IO_ReadOnly)) 
		{
			QRegExp rxICQ("<buddy uri=\"ICQ:1%3A([0-9]*)%3A.*>(.*)</buddy>");
			QRegExp rxMSN("<buddy uri=\"MSN:1%3A(.*)%3A.*>(.*)</buddy>");
			QTextStream stream(&file);
			QString line;
			while(!stream.eof())
			{
				line=stream.readLine();
				if(rxICQ.search(line)>=0)
				{
//					cout <<"UIN: "<< rxICQ.cap(1) << " Alias: " <<rxICQ.cap(2)<<endl;
					QString uinstr=rxICQ.cap(1);
					QString alias=rxICQ.cap(2);
					contacts[uinstr]=QPair<QString,QString>("icq",alias);
				}
				else if(rxMSN.search(line)>=0)
				{
//					cout <<"identifier: "<< rxMSN.cap(1) << " Alias: " <<rxMSN.cap(2)<<endl;
					QString identifier=rxMSN.cap(1);
					QString alias=rxMSN.cap(2);
					contacts[identifier]=QPair<QString,QString>("msn",alias);
				}

			}
			file.close();
		}
	}
	else if(key=="proteus")
	{
		QFile file(location);
		if(file.open(IO_ReadOnly)) 
		{
			QRegExp RegAccountIdentifier("<key>Account Identifier</key>");
			QRegExp RegIdentifier("<key>Identifier</key>");
			QRegExp RegAlias("<key>(Alias|Nickname)</key>");
			QRegExp RegEndDict("</dict>");

			QRegExp RegInteger("<integer>([0-9]*)</integer>");
			QRegExp RegString("<string>(.*)</string>");

			QTextStream stream(&file);
			QString line;

			QString alias;
			QString identifier;
			QString accountidentifier;

			while(!stream.eof())
			{
				line=stream.readLine();
				if(RegAccountIdentifier.search(line)>=0)
				{
					line=stream.readLine();
					RegInteger.search(line);
					accountidentifier=RegInteger.cap(1);
				}
				else if(RegIdentifier.search(line)>=0)
				{
					line=stream.readLine();
					RegString.search(line);
					identifier=RegString.cap(1);
				}
				else if(RegAlias.search(line)>=0)
				{
					line=stream.readLine();
					RegString.search(line);
					alias=RegString.cap(1);
				}
				else if(RegEndDict.search(line)>=0 && accountidentifier && identifier && alias)
				{
				
					if(accountidentifier=="10")
						accountidentifier="icq";
					else if(accountidentifier=="11")
						accountidentifier="msn";
					else if(accountidentifier=="12")
						accountidentifier="aim";

//					cout <<"Account Identifier: " << accountidentifier << " Identifier: "<< identifier << " Alias: " << alias <<endl;

					contacts[identifier]=QPair<QString,QString>(accountidentifier,alias);

					accountidentifier=QString::null;
					identifier=QString::null;
					alias=QString::null;
				}
			}
			file.close();
		}
	}
	else if(key=="gaim_icq")
	{
		QFile file(location);
		if(file.open(IO_ReadOnly)) 
		{
			QRegExp rxICQ("b ([0-9]*):(.*)");
			QTextStream stream(&file);
			QString line;
			while(!stream.eof())
			{
				line=stream.readLine();
				if(rxICQ.search(line)>=0)
				{
					QString uinstr=rxICQ.cap(1);
					//this will chop the : off the beginning of the alias
					QString alias=rxICQ.cap(2).right(rxICQ.cap(2).length());
//					cout <<"UIN: "<< uinstr << " Alias: [" <<alias<<"]"<<endl;
	
					contacts[uinstr]=QPair<QString,QString>("icq",alias);
				}
			}
			file.close();
		}
	}
	else
	{
//		cout << "Unimplemented program key" << endl;
	}
}

Imp::~Imp()
{
}

int Imp::numContacts()
{
	return contacts.count();
}

QMap<QString,QPair<QString,QString> > Imp::getContacts() {
	return contacts;
}


/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 8
 * indent-tabs-mode: t
 * End:
 */
// vim: set noet ts=4 sts=4 sw=4:

