/*
    importer.cpp

    Kopete Importer plugin

    Copyright (c) 2002 by James Grant   <topace@lightbox.org>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include "importeruibase.h"
#include "importerplugin.h"
#include "importer.h"
#include "pluginloader.h"

#include <qlayout.h>
#include <kconfig.h>
#include <kglobal.h>
#include <klocale.h>
#include <klineedit.h>
#include <kdebug.h>
#include <qlabel.h>
#include <qpushbutton.h>

#include <kfiledialog.h>
#include <kdirselectdialog.h>
#include <kmessagebox.h>

#include <knuminput.h>

#include "../../protocols/icq_new/icqprotocol.h"
#include "../../protocols/msn/msnprotocol.h"
#include "imp.h"

Importer::Importer(const QString &pixmap,QObject *parent)
				: ConfigModule(i18n("Importer"),i18n("Importer Plugin"),pixmap,parent)
{
	( new QVBoxLayout( this ) )->setAutoAdd( true );

//	kdDebug(14304)<<"Importer::Importer:"<<"Loading Importer Plugin"<<endl;
	ImporterDialog = new ImporterUI(this);

	QMap<QString,QString>::ConstIterator i;
		QMap<QString,QString> m;

		m = ImporterPlugin::plugin()->programsMap();

		for ( i = m.begin(); i != m.end() ; ++i )
			ImporterDialog->m_ProgramsComboBox->insertItem( i.data(), ImporterPlugin::plugin()->programsIndex(i.key()) );

	connect(ImporterDialog->m_ChooseFilePushButton,SIGNAL(clicked()),this,SLOT(slotChooseFile()));
	connect(ImporterDialog->m_ImportContactsPushButton,SIGNAL(clicked()),this,SLOT(slotDoImport()));
	connect(ImporterDialog->m_ProgramsComboBox,SIGNAL(activated(int)),this,SLOT(slotProgramChanged(int)));
}

Importer::~Importer()
{
//	kdDebug(14304)<<"Importer::~Importer"<<"UnLoading Importer Plugin"<<endl;
}

void Importer::slotProgramChanged(int i)
{
	QString key=ImporterPlugin::plugin()->programsKey(i);

	if(key=="_empty")
		ImporterDialog->m_Filename->setText("");
	else if(key=="licq")
		ImporterDialog->m_Filename->setText(".licq/users");
}

void Importer::slotChooseFile()
{
	int selectType=1;
	//0=directory
	//1=file

	kdDebug(14304)<<"Choosing a file to open"<<endl;
	if(ImporterDialog->m_ProgramsComboBox->currentItem()==0)
	{
		KMessageBox::sorry(this,i18n("Please choose a program to import contacts from first"),i18n("Program Not Chosen"));
		return;
	}
	QString key=ImporterPlugin::plugin()->programsKey(ImporterDialog->m_ProgramsComboBox->currentItem());
	kdDebug(14304)<<"key="<<key<<endl;
	QString filter;
	QString startpath;
	if(key=="licq"){
		startpath=".licq/users";
		selectType=0;
	}

	if(key=="trillian"){
		filter="Buddies.xml";
		startpath="";
		selectType=1;
	}

	if(key=="proteus"){
		filter="*.plist";
		startpath="";
		selectType=1;
	}

	if(key=="gaim_icq"){
		filter="*.3.blist";
		startpath=".gaim";
		selectType=1;
	}

	if(key=="gaim_msn"){
		filter="*.4.blist";
		startpath=".gaim";
		selectType=1;
	}

	if(selectType==0)
		ImporterDialog->m_Filename->setText((KDirSelectDialog::selectDirectory(startpath,false,ImporterDialog,i18n("Select Directory to Import Contacts From"))).path());
	else if(selectType==1)
		ImporterDialog->m_Filename->setText(KFileDialog::getOpenFileName(startpath,filter,ImporterDialog,i18n("Select File to Import Contacts From")));
}

void Importer::slotDoImport()
{
	if(ImporterDialog->m_Filename->text().length()<=0)
	{
		KMessageBox::sorry(this,i18n("Please choose a file to import first"),i18n("File Not Chosen"));
		return;
	}
	if(ImporterDialog->m_ProgramsComboBox->currentItem()==0)
	{
		KMessageBox::sorry(this,i18n("Please choose a program to import contacts from"),i18n("Program Not Chosen"));
		return;
	}
	kdDebug(14304)<<"Importer::slotDoImport: Everything is okay, doing import"<<endl;

	QString key=ImporterPlugin::plugin()->programsKey(ImporterDialog->m_ProgramsComboBox->currentItem());

	ICQProtocol *icqp = static_cast<ICQProtocol*>( LibraryLoader::pluginLoader()->searchByID( QString( "ICQProtocol" ) ) );
	MSNProtocol *msnp = static_cast<MSNProtocol*>( LibraryLoader::pluginLoader()->searchByID( QString( "MSNProtocol" ) ) );
	Imp *i=new Imp(key,ImporterDialog->m_Filename->text());

	kdDebug(14304)<<"Importer::slotDoImport:" << i->numContacts() << " Contacts to Import"<< endl;

	QMap<QString,QPair<QString,QString> >::ConstIterator it;
	QMap<QString,QPair<QString,QString> > contacts=i->getContacts();
	bool warnicq=false;
	bool warnmsn=false;
	bool icq_added=false;
	bool msn_added=false;
	for(it=contacts.begin(); it!= contacts.end(); ++it)
	{
		kdDebug(14304) << "Importer::slotDoImport: Adding " << it.key() << " and " << it.data().first << " and " << it.data().second << endl;
		if(it.data().first=="icq") {
			if(icqp!=0L && icqp->isConnected())
			{
				icq_added=true;
				icqp->addContact(
					QString::number( it.key().toUInt() ),
					it.data().second,
					0,
					i18n( "Imported %1 contacts" ).arg(key),
					false);
			}
			else
				warnicq=true;
		}
		else if(it.data().first=="msn"){
			if(msnp!=0L && msnp->isConnected())
			{
				msn_added=true;
				msnp->addContact( 
					it.key(),
					QString::null,
					0,
					i18n("Imported %1 contacts").arg(key),
					false );
			}
			else
				warnmsn=true;
		}
	}
	if( warnicq )
	{
		KMessageBox::sorry( this, i18n( "Sorry, you must have the ICQ "
			"protocol loaded and the protocol must be connected to be able "
			"to import ICQ contacts" ),
			i18n( "ICQ Protocol Not Loaded or Not Connected" ) );
	}
	else if( icq_added )
	{
		KMessageBox::information( this,
			i18n( "ICQ contacts successfully imported" ),
			i18n( "ICQ Import Successful" ) );
	}

	if( warnmsn )
	{
		KMessageBox::sorry( this, i18n( "Sorry, you must have the MSN "
			"protocol loaded and the protocol must be connected to be able "
			"to import MSN contacts"),
			i18n( "MSN Protocol Not Loaded or Not Connected" ) );
	}
	else if( msn_added )
	{
		KMessageBox::information( this,
			i18n( "MSN contacts successfully imported" ),
			i18n( "MSN Import Successful" ) );
	}
}

#include "importer.moc"

// vim: set noet ts=4 sts=4 sw=4:

