/* $Id: libgadu.c,v 1.7 2003/01/07 14:02:14 akrille Exp $ */

/*
 *  (C) Copyright 2001-2002 Wojtek Kaniewski <wojtekka@irc.pl>,
 *                          Robert J. Wony <speedy@ziew.org>,
 *                          Arkadiusz Mikiewicz <misiek@pld.org.pl>
 *                          Tomasz Chiliski <chilek@chilan.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License Version
 *  2.1 as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <errno.h>
#ifndef _AIX
#  include <string.h>
#endif
#include <stdarg.h>
#ifdef sun
#  include <sys/filio.h>
#endif
#include "config.h"
#include "compat.h"
#include "libgadu.h"

int gg_debug_level = GG_DEBUG_NET|GG_DEBUG_TRAFFIC|GG_DEBUG_DUMP|GG_DEBUG_FUNCTION|GG_DEBUG_MISC;

int gg_dcc_port = 0;
unsigned long gg_dcc_ip = 0;

/*
 * zmienne opisujce parametry proxy http.
 */
char *gg_proxy_host = NULL;
int gg_proxy_port = 0;
int gg_proxy_enabled = 0;
int gg_proxy_http_only = 0;
char *gg_proxy_username = NULL;
char *gg_proxy_password = NULL;

#ifndef lint
static char rcsid[]
#ifdef __GNUC__
__attribute__ ((unused))
#endif
= "$Id: libgadu.c,v 1.7 2003/01/07 14:02:14 akrille Exp $";
#endif

/*
 * gg_libgadu_version()
 *
 * zwraca wersj libgadu.
 */
const char *gg_libgadu_version()
{
	return GG_LIBGADU_VERSION;
}

/*
 * gg_fix32()
 *
 * zamienia kolejno bajtw w liczbie 32-bitowej tak, by odpowiadaa
 * kolejnoci bajtw w protokole GG. ze wzgldu na LE-owo serwera,
 * zamienia tylko na maszynach BE-wych.
 *
 *  - x - liczba do zamiany
 *
 * liczba z odpowiedni kolejnoci bajtw.
 */
uint32_t gg_fix32(uint32_t x)
{
#ifndef WORDS_BIGENDIAN
	return x;
#else
	return (uint32_t)
		(((x & (uint32_t) 0x000000ffU) << 24) |
                 ((x & (uint32_t) 0x0000ff00U) << 8) |
                 ((x & (uint32_t) 0x00ff0000U) >> 8) |
                 ((x & (uint32_t) 0xff000000U) >> 24));
#endif
}

/*
 * gg_fix16()
 *
 * zamienia kolejno bajtw w liczbie 16-bitowej tak, by odpowiadaa
 * kolejnoci bajtw w protokole GG. ze wzgldu na LE-owo serwera,
 * zamienia tylko na maszynach BE-wych.
 *
 *  - x - liczba do zamiany
 *
 * liczba z odpowiedni kolejnoci bajtw.
 */
uint16_t gg_fix16(uint16_t x)
{
#ifndef WORDS_BIGENDIAN
	return x;
#else
	return (uint16_t)
		(((x & (uint16_t) 0x00ffU) << 8) |
                 ((x & (uint16_t) 0xff00U) >> 8));
#endif
}

/*
 * gg_login_hash() // funkcja wewntrzna
 *
 * liczy hash z hasa i danego seeda.
 *
 *  - password - haso do hashowania
 *  - seed - warto podana przez serwer
 *
 * hash.
 */
unsigned int gg_login_hash(const unsigned char *password, unsigned int seed)
{
#if 0
	unsigned int hash;

	for (hash = 1; *password; password++)
		hash *= (*password) + 1;
	hash *= seed;

	return hash;
#endif

	unsigned int x, y, z;

	y = seed;

	for (x = 0; *password; password++) {
		x = (x & 0xffffff00) | *password;
		y ^= x;
		y += x;
		x <<= 8;
		y ^= x;
		x <<= 8;
		y -= x;
		x <<= 8;
		y ^= x;

		z = y & 0x1F;
		y = (y << z) | (y >> (32 - z));
	}

	return y;
}

/*
 * gg_resolve() // funkcja wewntrzna
 *
 * tworzy potok, forkuje si i w drugim procesie zaczyna resolvowa
 * podanego hosta. zapisuje w sesji deskryptor potoku. jeli co tam
 * bdzie gotowego, znaczy, e mona wczyta struct in_addr. jeli
 * nie znajdzie, zwraca INADDR_NONE.
 *
 *  - fd - wskanik gdzie wrzuci deskryptor
 *  - pid - gdzie wrzuci pid procesu potomnego
 *  - hostname - nazwa hosta do zresolvowania
 *
 * 0, -1.
 */
int gg_resolve(int *fd, int *pid, const char *hostname)
{
	int pipes[2], res;
	struct in_addr a;

	gg_debug(GG_DEBUG_FUNCTION, "** gg_resolve(%p, %p, \"%s\");\n", fd, pid, hostname);

	if (!fd || !pid) {
		errno = EFAULT;
		return -1;
	}

	if (pipe(pipes) == -1)
		return -1;

	/* if ((res = fork()) == -1)
		return -1; */
	res = 0;

	if (!res) {
		if ((a.s_addr = inet_addr(hostname)) == INADDR_NONE) {
			struct hostent *he;

			if (!(he = gg_gethostbyname(hostname)))
				a.s_addr = INADDR_NONE;
			else {
				memcpy((char*) &a, he->h_addr, sizeof(a));
				free(he);
			}
		}

		write(pipes[1], &a, sizeof(a));

		/* exit(0); */
	}

	close(pipes[1]);

	*fd = pipes[0];
	*pid = res;

	return 0;
}

#ifdef HAVE_PTHREAD

struct gg_resolve_pthread_data {
	char *hostname;
	int fd;
};

static void *gg_resolve_pthread_thread(void *arg)
{
	struct gg_resolve_pthread_data *d = arg;
	struct in_addr a;

	if ((a.s_addr = inet_addr(d->hostname)) == INADDR_NONE) {
		struct hostent *he;

		if (!(he = gg_gethostbyname(d->hostname)))
			a.s_addr = INADDR_NONE;
		else {
			memcpy((char*) &a, he->h_addr, sizeof(a));
			free(he);
		}
	}

	write(d->fd, &a, sizeof(a));
	close(d->fd);

	free(d->hostname);
	d->hostname = NULL;

	free(d);

	pthread_exit(NULL);

	return NULL;	/* eby kompilator nie marudzi */
}

/*
 * gg_resolve_pthread() // funkcja wewntrzna
 *
 * tworzy potok, nowy wtek i w nim zaczyna resolvowa podanego hosta.
 * zapisuje w sesji deskryptor potoku. jeli co tam bdzie gotowego,
 * znaczy, e mona wczyta struct in_addr. jeli nie znajdzie, zwraca
 * INADDR_NONE.
 *
 *  - c - sesja
 *  - hostname - nazwa hosta do zresolvowania
 *
 * 0, -1.
 */
int gg_resolve_pthread(struct gg_common *c, const char *hostname)
{
	struct gg_resolve_pthread_data *d;
	int pipes[2];

	gg_debug(GG_DEBUG_FUNCTION, "** gg_resolve_pthread(%p, \"%s\");\n", c, hostname);

	if (!c || !hostname) {
		gg_debug(GG_DEBUG_MISC, "// gg_resolve_pthread() invalid arguments\n");
		errno = EFAULT;
		return -1;
	}

	if (pipe(pipes) == -1) {
		gg_debug(GG_DEBUG_MISC, "// gg_resolve_pthread() unable to create pipes (errno=%d, %s)\n", errno, strerror(errno));
		return -1;
	}

	if (!(d = malloc(sizeof(*d))) || !(d->hostname = strdup(hostname))) {
		gg_debug(GG_DEBUG_MISC, "// gg_resolve_pthread() out of memory\n");
		return -1;
	}

	d->fd = pipes[1];

	if (pthread_create(&c->resolver, NULL, gg_resolve_pthread_thread, d) == -1) {
		gg_debug(GG_DEBUG_MISC, "// gg_resolve_phread() unable to create thread\n");
		return -1;
	}

	c->fd = pipes[0];

	return 0;
}

#endif

/*
 * gg_recv_packet() // funkcja wewntrzna
 *
 * odbiera jeden pakiet i zwraca wskanik do niego. pami po nim
 * naley zwolni za pomoc free().
 *
 *  - sess - opis sesji
 *
 * w przypadku bdu NULL, kod bdu w errno.
 */
void *gg_recv_packet(struct gg_session *sess)
{
	struct gg_header h;
	char *buf = NULL;
	int ret = 0, offset, size = 0;

	gg_debug(GG_DEBUG_FUNCTION, "** gg_recv_packet(%p);\n", sess);

	if (!sess) {
		errno = EFAULT;
		return NULL;
	}

	if (sess->recv_left < 1) {
		while (ret != sizeof(h)) {
			ret = read(sess->fd, &h, sizeof(h));

			gg_debug(GG_DEBUG_MISC, "// gg_recv_packet() header recv(%d,%p,%d) = %d\n", sess->fd, &h, sizeof(h), ret);

			if (!ret) {
				gg_debug(GG_DEBUG_MISC, "// gg_recv_packet() header recv() failed: connection broken\n");
				return NULL;
			}

			if (ret == -1) {
				if (errno == EINTR) {
					gg_debug(GG_DEBUG_MISC, "// gg_recv_packet() header recv() interrupted system call, resuming\n");
					continue;
				}

				gg_debug(GG_DEBUG_MISC, "// gg_recv_packet() header recv() failed: errno=%d, %s\n", errno, strerror(errno));

				return NULL;
			}
		}

		h.type = fix32(h.type);
		h.length = fix32(h.length);
	} else {
		memcpy(&h, sess->recv_buf, sizeof(h));
	}

	/* jakie sensowne limity na rozmiar pakietu */
	if (h.length < 0 || h.length > 65535) {
		gg_debug(GG_DEBUG_MISC, "// gg_recv_packet() invalid packet length (%d)\n", h.length);
		errno = ERANGE;
		return NULL;
	}

	if (sess->recv_left > 0) {
		gg_debug(GG_DEBUG_MISC, "// gg_recv_packet() resuming last gg_recv_packet()\n");
		size = sess->recv_left;
		offset = sess->recv_done;
		buf = sess->recv_buf;
	} else {
		if (!(buf = malloc(sizeof(h) + h.length + 1))) {
			gg_debug(GG_DEBUG_MISC, "// gg_recv_packet() not enough memory for packet data\n");
			return NULL;
		}

		memcpy(buf, &h, sizeof(h));

		offset = 0;
		size = h.length;
	}

	while (size > 0) {
		ret = read(sess->fd, buf + sizeof(h) + offset, size);
		gg_debug(GG_DEBUG_MISC, "// gg_recv_packet() body recv(%d,%p,%d) = %d\n", sess->fd, buf + sizeof(h) + offset, size, ret);
		if (ret > -1 && ret <= size) {
			offset += ret;
			size -= ret;
		} else if (ret == -1) {
			gg_debug(GG_DEBUG_MISC, "// gg_recv_packet() body recv() failed (errno=%d, %s)\n", errno, strerror(errno));
			if (errno == EAGAIN) {
				gg_debug(GG_DEBUG_MISC, "// gg_recv_packet() %d bytes received, %d left\n", offset, size);
				sess->recv_buf = buf;
				sess->recv_left = size;
				sess->recv_done = offset;
				return NULL;
			}
			if (errno != EINTR) {
/*				errno = EINVAL; */
				free(buf);
				return NULL;
			}
		}
	}

	sess->recv_left = 0;

	if ((gg_debug_level & GG_DEBUG_DUMP)) {
		int i;

		gg_debug(GG_DEBUG_DUMP, "// gg_recv_packet()", h.type);
		for (i = 0; i < sizeof(h) + h.length; i++)
			gg_debug(GG_DEBUG_DUMP, " %.2x", (unsigned char) buf[i]);
		gg_debug(GG_DEBUG_DUMP, "\n");
	}

	return buf;
}

/*
 * gg_send_packet() // funkcja wewntrzna
 *
 * konstruuje pakiet i wysya go w do serwera.
 *
 *  - sock - deskryptor gniazda
 *  - type - typ pakietu
 *  - payload_1 - pierwsza cz pakietu
 *  - payload_length_1 - dugo pierwszej czci
 *  - payload_2 - druga cz pakietu
 *  - payload_length_2 - dugo drugiej czci
 *  - ... - kolejne czci pakietu i ich dugoci
 *  - NULL - kocowym parametr (konieczny!)
 *
 * jeli si powiodo, zwraca 0, w przypadku bdu -1. jeli errno == ENOMEM,
 * zabrako pamici. inaczej by bd przy wysyaniu pakietu. dla errno == 0
 * nie wysano caego pakietu.
 */
int gg_send_packet(int sock, int type, ...)
{
	struct gg_header *h;
	char *tmp;
	int tmp_length;
	void *payload;
	int payload_length;
	va_list ap;
	int res;

	gg_debug(GG_DEBUG_FUNCTION, "** gg_send_packet(%d, 0x%.2x, ...)\n", sock, type);

	tmp_length = 0;

	if (!(tmp = malloc(sizeof(struct gg_header)))) {
		gg_debug(GG_DEBUG_MISC, "// gg_send_packet() not enough memory for packet header\n");
		return -1;
	}

	h = (struct gg_header*) tmp;
	h->type = fix32(type);
	h->length = fix32(0);

	va_start(ap, type);

	payload = va_arg(ap, void *);

	while (payload) {
		payload_length = va_arg(ap, int);

		if (payload_length < 0)
			gg_debug(GG_DEBUG_MISC, "// gg_send_packet() invalid payload length (%d)\n", payload_length);

		if (!(tmp = realloc(tmp, sizeof(struct gg_header) + tmp_length + payload_length))) {
                        gg_debug(GG_DEBUG_MISC, "// gg_send_packet() not enough memory for payload\n");
                        return -1;
                }

		memcpy(tmp + sizeof(struct gg_header) + tmp_length, payload, payload_length);
		tmp_length += payload_length;

		payload = va_arg(ap, void *);
	}

	va_end(ap);

	h = (struct gg_header*) tmp;
	h->length = fix32(tmp_length);

	if ((gg_debug_level & GG_DEBUG_DUMP)) {
                int i;

                gg_debug(GG_DEBUG_DUMP, "// gg_send_packet()", fix32(h->type));
                for (i = 0; i < sizeof(struct gg_header) + fix32(h->length); i++)
                        gg_debug(GG_DEBUG_DUMP, " %.2x", (unsigned char) tmp[i]);
                gg_debug(GG_DEBUG_DUMP, "\n");
        }

	tmp_length += sizeof(struct gg_header);

	if ((res = write(sock, tmp, tmp_length)) < tmp_length) {
		gg_debug(GG_DEBUG_MISC, "// gg_send_packet() write() failed. res = %d, errno = %d (%s)\n", res, errno, strerror(errno));
		free(tmp);
		return -1;
	}

	free(tmp);
	return 0;
}

/*
 * gg_session_callback() // funkcja wewntrzna
 *
 * wywoywany z gg_session->callback, wykonuje gg_watch_fd() i pakuje
 * do gg_session->event jego wynik.
 */
static int gg_session_callback(struct gg_session *s)
{
	if (!s) {
		errno = EINVAL;
		return -1;
	}

	return ((s->event = gg_watch_fd(s)) != NULL) ? 0 : -1;
}

/*
 * gg_login()
 *
 * rozpoczyna procedur czenia si z serwerem. reszt obsuguje si przez
 * gg_watch_fd().
 *
 * UWAGA! program musi obsuy SIGCHLD, jeli czy si asynchronicznie,
 * eby poprawnie zamkn proces resolvera.
 *
 *  - p - struktura opisujca pocztkowy stan. wymagane pola: uin,
 *    password
 *
 * w przypadku bdu NULL, jeli idzie dobrze (async) albo poszo
 * dobrze (sync), zwrci wskanik do zaalokowanej struct gg_session.
 */
struct gg_session *gg_login(const struct gg_login_params *p)
{
	struct gg_session *sess = NULL;
	char *hostname;
	int port;

	if (!p) {
		gg_debug(GG_DEBUG_FUNCTION, "** gg_login(%p);\n", p);
		errno = EINVAL;
		return NULL;
	}

	gg_debug(GG_DEBUG_FUNCTION, "** gg_login(%p: [uin=%u, async=%d, ...]);\n", p, p->uin, p->async);

	if (!(sess = malloc(sizeof(*sess)))) {
		gg_debug(GG_DEBUG_MISC, "// gg_login() not enough memory for session data\n");
		goto fail;
	}

	memset(sess, 0, sizeof(*sess));

	if (!p->password || !p->uin) {
		gg_debug(GG_DEBUG_MISC, "// gg_login() invalid arguments. uin and password needed\n");
		errno = EINVAL;
		goto fail;
	}

	if (!(sess->password = strdup(p->password))) {
		gg_debug(GG_DEBUG_MISC, "// gg_login() not enough memory for password\n");
		goto fail;
	}

	sess->uin = p->uin;
	sess->state = GG_STATE_RESOLVING;
	sess->check = GG_CHECK_READ;
	sess->timeout = GG_DEFAULT_TIMEOUT;
	sess->async = p->async;
        sess->type = GG_SESSION_GG;
	sess->initial_status = p->status;
	sess->initial_descr = p->status_descr;
	sess->callback = gg_session_callback;
	sess->destroy = gg_free_session;
	sess->port = (p->server_port) ? p->server_port : GG_DEFAULT_PORT;
	sess->server_addr = p->server_addr;
	sess->external_port = p->external_port;
	sess->external_addr = p->external_addr;
	sess->protocol_version = (p->protocol_version) ? p->protocol_version : GG_DEFAULT_PROTOCOL_VERSION;
	if (p->has_audio)
		sess->protocol_version |= GG_HAS_AUDIO_MASK;
	sess->client_version = (p->client_version) ? strdup(p->client_version) : NULL;

	if (gg_proxy_enabled) {
		hostname = gg_proxy_host;
		sess->proxy_port = port = gg_proxy_port;
	} else {
		hostname = GG_APPMSG_HOST;
		port = GG_APPMSG_PORT;
	}

	if (!p->async) {
		struct in_addr a;

		if (!p->server_addr || !p->server_port) {
			if ((a.s_addr = inet_addr(hostname)) == INADDR_NONE) {
				struct hostent *he;

				if (!(he = gg_gethostbyname(hostname))) {
					gg_debug(GG_DEBUG_MISC, "// gg_login() host \"%s\" not found\n", hostname);
					goto fail;
				} else {
					memcpy((char*) &a, he->h_addr, sizeof(a));
					free(he);
				}
			}
		} else {
			a.s_addr = p->server_addr;
			port = p->server_port;
		}

		sess->hub_addr = a.s_addr;

		if ((sess->fd = gg_connect(&a, port, 0)) == -1) {
			gg_debug(GG_DEBUG_MISC, "// gg_login() connection failed (errno=%d, %s)\n", errno, strerror(errno));
			goto fail;
		}

		if (p->server_addr && p->server_port)
			sess->state = GG_STATE_CONNECTING_HUB;
		else
			sess->state = GG_STATE_CONNECTING_GG;

		while (sess->state != GG_STATE_CONNECTED) {
			struct gg_event *e;

			if (!(e = gg_watch_fd(sess))) {
				gg_debug(GG_DEBUG_MISC, "// gg_login() critical error in gg_watch_fd()\n");
				goto fail;
			}

			if (e->type == GG_EVENT_CONN_FAILED) {
				errno = EACCES;
				gg_debug(GG_DEBUG_MISC, "// gg_login() could not login\n");
				gg_event_free(e);
				goto fail;
			}

			gg_event_free(e);
		}

		return sess;
	}

	if (!sess->server_addr || gg_proxy_enabled) {
#ifndef HAVE_PTHREAD
		if (gg_resolve(&sess->fd, &sess->pid, hostname)) {
#else
		if (gg_resolve_pthread((struct gg_common*) sess, hostname)) {
#endif
			gg_debug(GG_DEBUG_MISC, "// gg_login() resolving failed (errno=%d, %s)\n", errno, strerror(errno));
			goto fail;
		}
	} else {
		if ((sess->fd = gg_connect(&sess->server_addr, sess->port, sess->async)) == -1) {
			gg_debug(GG_DEBUG_MISC, "// gg_login() direct connection failed (errno=%d, %s)\n", errno, strerror(errno));
			goto fail;
		}
		sess->state = GG_STATE_CONNECTING_GG;
		sess->check = GG_CHECK_WRITE;
	}

	return sess;

fail:
	free(sess);
	return NULL;
}

/*
 * gg_free_session()
 *
 * prbuje zamkn poczenia i zwalnia pami zajmowan przez sesj.
 *
 *  - sess - opis sesji
 */
void gg_free_session(struct gg_session *sess)
{
	if (!sess)
		return;

	/* XXX dopisa zwalnianie i zamykanie wszystkiego, co mogo zosta */

	if (sess->password)
		free(sess->password);

	if (sess->client_version)
		free(sess->client_version);

	free(sess);
}

/*
 * gg_change_status()
 *
 * zmienia status uytkownika. przydatne do /away i /busy oraz /quit.
 *
 *  - sess - opis sesji
 *  - status - nowy status uytkownika
 *
 * 0, -1.
 */
int gg_change_status(struct gg_session *sess, int status)
{
	struct gg_new_status p;

	gg_debug(GG_DEBUG_FUNCTION, "** gg_change_status(%p, %d);\n", sess, status);

	if (!sess) {
		errno = EFAULT;
		return -1;
	}

	if (sess->state != GG_STATE_CONNECTED) {
		errno = ENOTCONN;
		return -1;
	}

	p.status = fix32(status);

	sess->status = status;

	return gg_send_packet(sess->fd, GG_NEW_STATUS, &p, sizeof(p), NULL);
}

/*
 * gg_change_status_descr()
 *
 * zmienia status uytkownika na opisowy.
 *
 *  - sess - opis sesji
 *  - status - nowy status uytkownika
 *  - descr - opis statusu
 *
 * 0, -1.
 */
int gg_change_status_descr(struct gg_session *sess, int status, const char *descr)
{
	struct gg_new_status p;

	gg_debug(GG_DEBUG_FUNCTION, "** gg_change_status_descr(%p, %d, \"%s\");\n", sess, status, descr);

	if (!sess || !descr) {
		errno = EFAULT;
		return -1;
	}

	if (sess->state != GG_STATE_CONNECTED) {
		errno = ENOTCONN;
		return -1;
	}

	p.status = fix32(status);

	sess->status = status;

	return gg_send_packet(sess->fd, GG_NEW_STATUS, &p, sizeof(p), descr, (strlen(descr) > GG_STATUS_DESCR_MAXSIZE) ? GG_STATUS_DESCR_MAXSIZE : strlen(descr), NULL);
}

/*
 * gg_change_status_descr_time()
 *
 * zmienia status uytkownika na opisowy z godzin powrotu.
 *
 *  - sess - opis sesji
 *  - status - nowy status uytkownika
 *  - descr - opis statusu
 *  - time - czas w formacie uniksowym
 *
 * 0, -1.
 */
int gg_change_status_descr_time(struct gg_session *sess, int status, const char *descr, int time)
{
	struct gg_new_status p;
	uint32_t newtime;

	gg_debug(GG_DEBUG_FUNCTION, "** gg_change_status_descr_time(%p, %d, \"%s\", %d);\n", sess, status, descr, time);

	if (!sess || !descr || !time) {
		errno = EFAULT;
		return -1;
	}

	if (sess->state != GG_STATE_CONNECTED) {
		errno = ENOTCONN;
		return -1;
	}

	p.status = fix32(status);

	sess->status = status;

	newtime = fix32(time);


	return gg_send_packet(sess->fd, GG_NEW_STATUS, &p, sizeof(p), descr, (strlen(descr) > GG_STATUS_DESCR_MAXSIZE) ? GG_STATUS_DESCR_MAXSIZE : strlen(descr), &newtime, sizeof(newtime), NULL);
}

/*
 * gg_logoff()
 *
 * wylogowuje uytkownika i zamyka poczenie, ale nie zwalnia pamici.
 *
 *  - sess - opis sesji
 */
void gg_logoff(struct gg_session *sess)
{
	if (!sess)
		return;

	gg_debug(GG_DEBUG_FUNCTION, "** gg_logoff(%p);\n", sess);

	if (GG_S_NA(sess->status & ~GG_STATUS_FRIENDS_MASK))
		gg_change_status(sess, GG_STATUS_NOT_AVAIL);

	if (sess->fd != -1) {
		shutdown(sess->fd, 2);
		close(sess->fd);
		sess->fd = -1;
	}
}

/*
 * gg_send_message_ctcp()
 *
 * wysya wiadomo do innego uytkownika. zwraca losowy numer
 * sekwencyjny, ktry mona zignorowa albo wykorzysta do potwierdzenia.
 *
 *  - sess - opis sesji
 *  - msgclass - rodzaj wiadomoci
 *  - recipient - numer adresata
 *  - message - tre wiadomoci
 *  - message_len - dugo
 *
 * numer sekwencyjny wiadomoci lub -1 w przypadku bdu.
 */
int gg_send_message_ctcp(struct gg_session *sess, int msgclass, uin_t recipient, const unsigned char *message, int message_len)
{
	struct gg_send_msg s;

	gg_debug(GG_DEBUG_FUNCTION, "** gg_send_message_ctcp(%p, %d, %u, ...);\n", sess, msgclass, recipient);

	if (!sess) {
		errno = EFAULT;
		return -1;
	}

	if (sess->state != GG_STATE_CONNECTED) {
		errno = ENOTCONN;
		return -1;
	}

	s.recipient = fix32(recipient);
	s.seq = fix32(0);
	s.msgclass = fix32(msgclass);

	return gg_send_packet(sess->fd, GG_SEND_MSG, &s, sizeof(s), message, message_len, NULL);
}

/*
 * gg_send_message()
 *
 * wysya wiadomo do innego uytkownika. zwraca losowy numer
 * sekwencyjny, ktry mona ola albo wykorzysta do potwierdzenia.
 *
 *  - sess - opis sesji
 *  - msgclass - rodzaj wiadomoci
 *  - recipient - numer adresata
 *  - message - tre wiadomoci
 *
 * numer sekwencyjny wiadomoci lub -1 w przypadku bdu.
 */
int gg_send_message(struct gg_session *sess, int msgclass, uin_t recipient, const unsigned char *message)
{
	gg_debug(GG_DEBUG_FUNCTION, "** gg_send_message(%p, %d, %u, %p)\n", sess, msgclass, recipient, message);

	return gg_send_message_richtext(sess, msgclass, recipient, message, NULL, 0);
}

/*
 * gg_send_message_richtext()
 *
 * wysya kolorow wiadomo do innego uytkownika. zwraca losowy numer
 * sekwencyjny, ktry mona ola albo wykorzysta do potwierdzenia.
 *
 *  - sess - opis sesji
 *  - msgclass - rodzaj wiadomoci
 *  - recipient - numer adresata
 *  - message - tre wiadomoci
 *  - format - informacje o formatowaniu
 *  - formatlen - dugo informacji o formatowaniu
 *
 * numer sekwencyjny wiadomoci lub -1 w przypadku bdu.
 */
int gg_send_message_richtext(struct gg_session *sess, int msgclass, uin_t recipient, const unsigned char *message, const unsigned char *format, int formatlen)
{
	struct gg_send_msg s;

	gg_debug(GG_DEBUG_FUNCTION, "** gg_send_message_richtext(%p, %d, %u, %p, %p, %d);\n", sess, msgclass, recipient, message, format, formatlen);

	if (!sess) {
		errno = EFAULT;
		return -1;
	}

	if (sess->state != GG_STATE_CONNECTED) {
		errno = ENOTCONN;
		return -1;
	}

	s.recipient = fix32(recipient);
	if (!sess->seq)
		sess->seq = 0x01740000 | (rand() & 0xffff);
	s.seq = fix32(sess->seq);
	s.msgclass = fix32(msgclass);
	sess->seq += (rand() % 0x300) + 0x300;

	if (gg_send_packet(sess->fd, GG_SEND_MSG, &s, sizeof(s), message, strlen(message) + 1, format, formatlen, NULL) == -1)
		return -1;

	return fix32(s.seq);
}

/*
 * gg_send_message_confer()
 *
 * wysya wiadomo do kilku uytkownikow (konferencja). zwraca losowy numer
 * sekwencyjny, ktry mona ola albo wykorzysta do potwierdzenia.
 *
 *  - sess - opis sesji
 *  - msgclass - rodzaj wiadomoci
 *  - recipients_count - ilo adresatw
 *  - recipients - numerki adresatw
 *  - message - tre wiadomoci
 *
 * numer sekwencyjny wiadomoci lub -1 w przypadku bdu.
 */
int gg_send_message_confer(struct gg_session *sess, int msgclass, int recipients_count, uin_t *recipients, const unsigned char *message)
{
	gg_debug(GG_DEBUG_FUNCTION, "** gg_send_message_confer(%p, %d, %d, %p, %p);\n", sess, msgclass, recipients_count, recipients, message);

	return gg_send_message_confer_richtext(sess, msgclass, recipients_count, recipients, message, NULL, 0);
}

/*
 * gg_send_message_confer_richtext()
 *
 * wysya kolorow wiadomo do kilku uytkownikow (konferencja). zwraca
 * losowy numer sekwencyjny, ktry mona zignorowa albo wykorzysta do
 * potwierdzenia.
 *
 *  - sess - opis sesji
 *  - msgclass - rodzaj wiadomoci
 *  - recipients_count - ilo adresatw
 *  - recipients - numerki adresatw
 *  - message - tre wiadomoci
 *  - format - informacje o formatowaniu
 *  - formatlen - dugo informacji o formatowaniu
 *
 * numer sekwencyjny wiadomoci lub -1 w przypadku bdu.
 */
int gg_send_message_confer_richtext(struct gg_session *sess, int msgclass, int recipients_count, uin_t *recipients, const unsigned char *message, const unsigned char *format, int formatlen)
{
	struct gg_send_msg s;
	struct gg_msg_recipients r;
	int i, j, k;
	uin_t *recps;

	gg_debug(GG_DEBUG_FUNCTION, "** gg_send_message_confer_richtext(%p, %d, %d, %p, %p, %p, %d);\n", sess, msgclass, recipients_count, recipients, message, format, formatlen);

	if (!sess) {
		errno = EFAULT;
		return -1;
	}

	if (sess->state != GG_STATE_CONNECTED) {
		errno = ENOTCONN;
		return -1;
	}

	r.flag = 0x01;
	r.count = fix32(recipients_count - 1);

	if (!sess->seq)
		sess->seq = 0x01740000 | (rand() & 0xffff);
	s.seq = fix32(sess->seq);
	s.msgclass = fix32(msgclass);

	recps = malloc(sizeof(uin_t) * recipients_count);
	for (i = 0; i < recipients_count; i++) {

		s.recipient = fix32(recipients[i]);

		for (j = 0, k = 0; j < recipients_count; j++)
			if (recipients[j] != s.recipient) {
				recps[k] = fix32(recipients[j]);
				k++;
			}

		if (!i)
			sess->seq += (rand() % 0x300) + 0x300;

		if (gg_send_packet(sess->fd, GG_SEND_MSG, &s, sizeof(s), message, strlen(message) + 1, &r, sizeof(r), recps, (recipients_count - 1) * sizeof(uin_t), format, formatlen, NULL) == -1) {
			free(recps);
			return -1;
		}
	}

	free(recps);

	return fix32(s.seq);
}

/*
 * gg_ping()
 *
 * wysya do serwera pakiet ping.
 *
 *  - sess - opis sesji
 *
 * 0, -1.
 */
int gg_ping(struct gg_session *sess)
{
	gg_debug(GG_DEBUG_FUNCTION, "** gg_ping(%p);\n", sess);

	if (!sess) {
		errno = EFAULT;
		return -1;
	}

	if (sess->state != GG_STATE_CONNECTED) {
		errno = ENOTCONN;
		return -1;
	}

	return gg_send_packet(sess->fd, GG_PING, NULL);
}

/*
 * gg_notify_ex()
 *
 * wysya serwerowi list kontaktw (wraz z odpowiadajcymi im typami userw),
 * dziki czemu wie, czyj stan nas interesuje.
 *
 *  - sess - identyfikator sesji
 *  - userlist - wskanik do tablicy numerw
 *  - types - wskanik do tablicy typw uytkownikw
 *  - count - ilo numerkw
 *
 * 0, -1.
 */
int gg_notify_ex(struct gg_session *sess, uin_t *userlist, char *types, int count)
{
	struct gg_notify *n;
	uin_t *u;
	char *t;
	int i, res = 0;

	gg_debug(GG_DEBUG_FUNCTION, "** gg_notify_ex(%p, %p, %p, %d);\n", sess, userlist, types, count);

	if (!sess) {
		errno = EFAULT;
		return -1;
	}

	if (sess->state != GG_STATE_CONNECTED) {
		errno = ENOTCONN;
		return -1;
	}

	if (!userlist || !count)
		return 0;

	if (!(n = (struct gg_notify*) malloc(sizeof(*n) * count)))
		return -1;

	for (u = userlist, t = types, i = 0; i < count; u++, t++, i++) {
		n[i].uin = fix32(*u);
		n[i].dunno1 = *t;
	}

	if (gg_send_packet(sess->fd, GG_NOTIFY, n, sizeof(*n) * count, NULL) == -1)
		res = -1;

	free(n);

	return res;
}

/*
 * gg_notify()
 *
 * wysya serwerowi list kontaktw, dziki czemu wie, czyj stan nas
 * interesuje.
 *
 *  - sess - identyfikator sesji
 *  - userlist - wskanik do tablicy numerw
 *  - count - ilo numerkw
 *
 * 0, -1.
 */
int gg_notify(struct gg_session *sess, uin_t *userlist, int count)
{
	struct gg_notify *n;
	uin_t *u;
	int i, res = 0;

	gg_debug(GG_DEBUG_FUNCTION, "** gg_notify(%p, %p, %d);\n", sess, userlist, count);

	if (!sess) {
		errno = EFAULT;
		return -1;
	}

	if (sess->state != GG_STATE_CONNECTED) {
		errno = ENOTCONN;
		return -1;
	}

	if (!userlist || !count)
		return 0;

	if (!(n = (struct gg_notify*) malloc(sizeof(*n) * count)))
		return -1;

	for (u = userlist, i = 0; i < count; u++, i++) {
		n[i].uin = fix32(*u);
		n[i].dunno1 = 3;
	}

	if (gg_send_packet(sess->fd, GG_NOTIFY, n, sizeof(*n) * count, NULL) == -1)
		res = -1;

	free(n);

	return res;
}

/*
 * gg_add_notify_ex()
 *
 * dodaje do listy kontaktw dany numer w trakcie poczenia.
 * dodawanemu uytkownikowi okrelamy jego typ (patrz protocol.html)
 *
 *  - sess - identyfikator sesji
 *  - uin - numer
 *  - type - typ
 *
 * 0, -1.
 */
int gg_add_notify_ex(struct gg_session *sess, uin_t uin, char type)
{
	struct gg_add_remove a;

	gg_debug(GG_DEBUG_FUNCTION, "** gg_add_notify_ex(%p, %u, %d);\n", sess, uin, type);

	if (!sess) {
		errno = EFAULT;
		return -1;
	}

	if (sess->state != GG_STATE_CONNECTED) {
		errno = ENOTCONN;
		return -1;
	}

	a.uin = fix32(uin);
	a.dunno1 = type;

	return gg_send_packet(sess->fd, GG_ADD_NOTIFY, &a, sizeof(a), NULL);
}

/*
 * gg_add_notify()
 *
 * dodaje do listy kontaktw dany numer w trakcie poczenia.
 *
 *  - sess - identyfikator sesji
 *  - uin - numer
 *
 * 0, -1.
 */
int gg_add_notify(struct gg_session *sess, uin_t uin)
{
	return gg_add_notify_ex(sess, uin, 3);
}

/*
 * gg_remove_notify_ex()
 *
 * usuwa z listy kontaktw w trakcie poczenia
 * usuwanemu uytkownikowi okrelamy jego typ (patrz protocol.html)
 *
 *  - sess - identyfikator sesji
 *  - uin - numer
 *  - type - typ
 *
 * 0, -1.
 */
int gg_remove_notify_ex(struct gg_session *sess, uin_t uin, char type)
{
	struct gg_add_remove a;

	gg_debug(GG_DEBUG_FUNCTION, "** gg_remove_notify_ex(%p, %u, %d);\n", sess, uin, type);

	if (!sess) {
		errno = EFAULT;
		return -1;
	}

	if (sess->state != GG_STATE_CONNECTED) {
		errno = ENOTCONN;
		return -1;
	}

	a.uin = fix32(uin);
	a.dunno1 = type;

	return gg_send_packet(sess->fd, GG_REMOVE_NOTIFY, &a, sizeof(a), NULL);
}

/*
 * gg_remove_notify()
 *
 * usuwa z listy kontaktw w trakcie poczenia
 *
 *  - sess - identyfikator sesji
 *  - uin - numer
 *
 * 0, -1.
 */
int gg_remove_notify(struct gg_session *sess, uin_t uin)
{
	return gg_remove_notify_ex(sess, uin, 3);
}

/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 8
 * indent-tabs-mode: notnil
 * End:
 *
 * vim: shiftwidth=8:
 */
