/***************************************************************************
						client.cpp  -  description
							-------------------
	begin                : Sun Mar 10 2002
	copyright            : (C) 2002 by Vladimir Shutoff
	email                : shutoff@mail.ru
***************************************************************************/

/***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************/

#include "client.h"
#include "log.h"

#include <qsocket.h>
#include <qsocketdevice.h>
#include <qsocketnotifier.h>
#include <qdns.h>
#include <qtimer.h>
#include <qstring.h>
#include <qfileinfo.h>
#include <qdir.h>
#include <qstringlist.h>
#include <qtextcodec.h>
#include <qregexp.h>

#include <errno.h>

#include <string>

#undef HAVE_KEXTSOCK_H

#ifdef USE_KDE
#include <kglobal.h>
#include <kcharsets.h>
#include <kdebug.h>
#endif

#ifdef HAVE_KEXTSOCK_H
#include <kextsock.h>
#include <ksockaddr.h>
#endif

class KICQSockets : public SocketFactory
{
public:
    KICQSockets() {}

    virtual Socket *createSocket();
    virtual ServerSocket *createServerSocket();
};

KICQClient::KICQClient(QObject *parent, const char *name)
		: QObject(parent, name), ICQClient(new KICQSockets)
{
	init();
	QTimer *timer = new QTimer(this);
	QObject::connect(timer, SIGNAL(timeout()), this, SLOT(timer()));
	timer->start(1000);
/*	resolver = new QDns;
	resolver->setRecordType(QDns::Ptr);
	QObject::connect(resolver, SIGNAL(resultsReady()), this, SLOT(resolve_ready()));
*/
}

KICQClient::~KICQClient()
{
	close();
//	delete resolver;
	delete factory();
}

void KICQClient::timer()
{
	idle();
}

Socket *KICQSockets::createSocket()
{
	return new ICQClientSocket;
}

ServerSocket *KICQSockets::createServerSocket()
{
	return new ICQServerSocket(MinTCPPort, MaxTCPPort);
}

bool KICQClient::markAsRead(ICQMessage *msg)
{
	ICQUser *u = getUser(msg->getUin());
	if (u == NULL) return false;
	for (list<unsigned long>::iterator it = u->unreadMsgs.begin(); it != u->unreadMsgs.end(); it++)
	{
		if (*it == msg->Id)
		{
			u->unreadMsgs.remove(*it);
			emit messageRead(msg);
			return true;
		}
	}
	return false;
}


void KICQClient::process_event(ICQEvent *e)
{
	switch (e->type())
	{
		case EVENT_MESSAGE_RECEIVED:
		{
				ICQMessage *msg = e->message();
				if (msg == NULL)
				{
					kdDebug(14110) << "Message event without message" << endl;
					return;
				}

				unsigned long uin = msg->getUin();
				switch (msg->Type())
				{
					case ICQ_READxAWAYxMSG:
					case ICQ_READxOCCUPIEDxMSG:
					case ICQ_READxNAxMSG:
					case ICQ_READxDNDxMSG:
					case ICQ_READxFFCxMSG:
						getAutoResponse(msg->getUin(), msg);
						return;
				}
				if (uin == owner->Uin) uin = 0;
				if (e->state == ICQEvent::Fail)
				{
					ICQUser *u = getUser(e->Uin());
					if (u)
					{
							list<unsigned long>::iterator it;
							for (it = u->unreadMsgs.begin(); it != u->unreadMsgs.end(); ++it)
									if ((*it) == msg->Id) break;

							if (it != u->unreadMsgs.end())
							{
									u->unreadMsgs.remove(msg->Id);
									emit messageRead(msg);
							}
					}
				}
				else
				{
					ICQUser *u = getUser(e->Uin(), true);
					if (msg->Id == 0)
					{
					// TODO: servermessage, handle that
					}
					else
					{
						switch (msg->Type())
						{
							case ICQ_MSGxCHAT:
								switch (owner->uStatus & 0xFF)
								{
									case ICQ_STATUS_ONLINE:
									case ICQ_STATUS_FREEFORCHAT:
										break;
									default:
										getAutoResponse(uin, msg);
										return;
								}
								break;

							default:
									break;
						}
					}
					u->unreadMsgs.push_back(msg->Id);
					emit messageReceived(msg);
				}
				break;
		}
/*
		case EVENT_STATUS_CHANGED:
		{
			ICQUser *u = getUser(e->Uin());
			if (u)
			{
				if (u->IP() && (u->HostName.size() == 0))
				{
					resolveAddr a;
					a.uin   = e->Uin();
					a.bReal = false;
					resolveQueue.push_back(a);
				}

				if (u->RealIP() && (u->RealHostName.size() == 0))
				{
					resolveAddr a;
					a.uin   = e->Uin();
					a.bReal = true;
					resolveQueue.push_back(a);
				}
				start_resolve();
			}
			break;
		}
*/
		default:
				break;
	}
	emit event(e);
}

/*
void KICQClient::start_resolve()
{
	if (resolver->isWorking()) return;

	for (; !resolveQueue.empty();)
	{
		resolveAddr &a = *resolveQueue.begin();
		ICQUser *u = getUser(a.uin);
		if (u == NULL)
		{
			resolveQueue.erase(resolveQueue.begin());
			continue;
		}
		unsigned long ip;
		if (a.bReal)
			ip = u->RealIP;
		else
			ip = u->IP;

		if (ip == 0)
		{
			resolveQueue.erase(resolveQueue.begin());
			continue;
		}
		resolver->setLabel(QHostAddress(htonl(ip)));
		return;
	}
}

void KICQClient::resolve_ready()
{
	if (resolveQueue.empty()) return;
	resolveAddr &a = *resolveQueue.begin();
	ICQUser *u = getUser(a.uin);

	if (u && resolver->hostNames().count())
	{
		if (a.bReal)
			u->RealHostName = resolver->hostNames().first().latin1();
		else
			u->HostName = resolver->hostNames().first().latin1();

		ICQEvent e(EVENT_STATUS_CHANGED, a.uin);
		emit event(&e);
	}
	resolveQueue.erase(resolveQueue.begin());
	start_resolve();
}
*/
QTextCodec *KICQClient::codecForUser(unsigned long uin)
{
	ICQUser *u = getUser(uin);
	if (u){
		if (u->Encoding.c_str()){
			QTextCodec *res = QTextCodec::codecForName(u->Encoding.c_str());
			if (res){
				return res;
			}
		}
	}
	if (owner->Encoding.c_str()){
		QTextCodec *res = QTextCodec::codecForName(owner->Encoding.c_str());
		if (res){
			return res;
		}
	}
	return QTextCodec::codecForLocale();
}

string KICQClient::to8Bit(unsigned long uin, const QString &str)
{
	return to8Bit(codecForUser(uin), str);
}

QString KICQClient::from8Bit(unsigned long uin, const string &str, const char *strCharset)
{
	return from8Bit(codecForUser(uin), str, strCharset);
}

string KICQClient::to8Bit(QTextCodec *codec, const QString &str)
{
	int lenOut = str.length();
	string res;
	if (lenOut == 0) return res;
	res = (const char*)(codec->makeEncoder()->fromUnicode(str, lenOut));
	return res;
}

QString KICQClient::from8Bit(QTextCodec *codec, const string &str, const char *strCharset)
{
	if (!strcmp(codec->name(), serverCharset(codec->name())))
		return codec->makeDecoder()->toUnicode(str.c_str(), str.size());
	string s = str;
	if (strCharset && strcasecmp(strCharset, codec->name())){
		toServer(s, strCharset);
		fromServer(s, codec->name());
	}
	return codec->makeDecoder()->toUnicode(s.c_str(), s.size());
}

void KICQClient::setUserEncoding(unsigned long uin, int i)
{
	if (userEncoding(uin) == i) return;
	ICQUser *u = NULL;
	if ((uin == 0) || (uin == owner->Uin)){
		u = owner;
	}else{
		u = getUser(uin);
	}
	if (u == NULL) return;
	QString name;
	if (i > 0){
		name = (*encodings)[i-1];
		int left = name.find(" ( ");
		if (left >= 0) name = name.mid(left + 3);
		int right = name.find(" )");
		if (right >= 0) name = name.left(right);
		u->Encoding = name.latin1();
	}
	emit encodingChanged(uin);
	ICQEvent e(EVENT_INFO_CHANGED, uin);
	process_event(&e);
	if (uin == owner->Uin)
		emit encodingChanged(0);
}

int KICQClient::userEncoding(unsigned long uin)
{
	QTextCodec *codec = codecForUser(uin);
	int n = 1;
	for (QStringList::Iterator it = encodings->begin(); it != encodings->end(); ++it, n++){
		QString name = *it;
		int left = name.find(" ( ");
		if (left >= 0) name = name.mid(left + 3);
		int right = name.find(" )");
		if (right >= 0) name = name.left(right);
		if (!strcasecmp(name.latin1(), codec->name()))
			return n;
	}
	return 0;
}

// ==============================================================================

#ifdef HAVE_KEXTSOCK_H
ICQClientSocket::ICQClientSocket(KExtendedSocket *s)
#else
ICQClientSocket::ICQClientSocket(QSocket *s)
#endif
{
	sock = s;
	if (sock == NULL)
#ifdef HAVE_KEXTSOCK_H
		sock = new KExtendedSocket;
	sock->setSocketFlags(KExtendedSocket::outputBufferedSocket );
#else
		sock = new QSocket(this);
#endif
#ifdef HAVE_KEXTSOCK_H
	QObject::connect(sock, SIGNAL(connectionSuccess()), this, SLOT(slotConnected()));
	QObject::connect(sock, SIGNAL(lookupFinished(int)), this, SLOT(slotLookupFinished(int)));
	QObject::connect(sock, SIGNAL(connectionFailed(int)), this, SLOT(slotError(int)));
	QObject::connect(sock, SIGNAL(closed(int)), this, SLOT(slotError(int)));
#else
	QObject::connect(sock, SIGNAL(connected()), this, SLOT(slotConnected()));
	QObject::connect(sock, SIGNAL(connectionClosed()), this, SLOT(slotConnectionClosed()));
	QObject::connect(sock, SIGNAL(error(int)), this, SLOT(slotError(int)));
#endif
	QObject::connect(sock, SIGNAL(readyRead()), this, SLOT(slotReadReady()));
	QObject::connect(sock, SIGNAL(bytesWritten(int)), this, SLOT(slotBytesWritten(int)));
	bInWrite = false;
#ifdef HAVE_KEXTSOCK_H
	if (s) sock->enableRead(true);
#endif
}

ICQClientSocket::~ICQClientSocket()
{
	close();
	delete sock;
}

void ICQClientSocket::close()
{
#ifdef HAVE_KEXTSOCK_H
	sock->closeNow();
#else
	sock->close();
#endif
}

void ICQClientSocket::slotLookupFinished(int)
{
}

int ICQClientSocket::read(char *buf, unsigned int size)
{
	int res = sock->readBlock(buf, size);
	if (res < 0){
#ifdef HAVE_KEXTSOCK_H
		if ((errno == EWOULDBLOCK) || (errno == 0))
			return 0;
#endif
		if (notify) notify->error_state(ErrorRead);
		return -1;
	}
	return res;
}

void ICQClientSocket::write(const char *buf, unsigned int size)
{
	bInWrite = true;
	int res = sock->writeBlock(buf, size);
	bInWrite = false;
	if (res != (int)size){
		if (notify) notify->error_state(ErrorWrite);
		return;
	}
	if (sock->bytesToWrite() == 0)
		QTimer::singleShot(0, this, SLOT(slotBytesWritten()));
}

void ICQClientSocket::connect(const char *host, int port)
{
#ifdef HAVE_KEXTSOCK_H
	sock->setAddress(host, port);
	if (sock->lookup() < 0){
		if (notify) notify->error_state(ErrorConnect);
	}
	if (sock->startAsyncConnect() < 0){
		if (notify) notify->error_state(ErrorConnect);
	}
#else
	sock->connectToHost(host, port);
#endif
}

void ICQClientSocket::slotConnected()
{
	kdDebug(14110) << k_funcinfo << "called." << endl;
	if (notify) notify->connect_ready();
#ifdef HAVE_KEXTSOCK_H
	sock->setBlockingMode(false);
	sock->enableRead(true);
#endif
}

void ICQClientSocket::slotConnectionClosed()
{
	if (notify) notify->error_state(ErrorConnectionClosed);
}

void ICQClientSocket::slotReadReady()
{
	if (notify) notify->read_ready();
}

void ICQClientSocket::slotBytesWritten(int)
{
	slotBytesWritten();
}

void ICQClientSocket::slotBytesWritten()
{
	if (bInWrite) return;
	if (sock->bytesToWrite() == 0) notify->write_ready();
}

unsigned long ICQClientSocket::localHost()
{
#ifdef HAVE_KEXTSOCK_H
	unsigned long res = 0;
	const KSocketAddress *addr = sock->localAddress();
	if (addr && addr->inherits("KInetSocketAddress")){
		const KInetSocketAddress *addr_in = static_cast<const KInetSocketAddress*>(addr);
		const sockaddr_in *a = addr_in->addressV4();
		if (a) res = htonl(a->sin_addr.s_addr);
	}
	return res;
#else
	return sock->address().ip4Addr();
#endif
}

void ICQClientSocket::slotError(int err)
{
#ifdef HAVE_KEXTSOCK_H
    if (!(err & KBufferedIO::involuntary)) return;
    kdDebug(14110) << "Connection closed by peer" << endl;
#else
    if (!err) return;
    kdDebug(14110) << "Slot error " << err << endl;
#endif
    if (notify) notify->error_state(ErrorSocket);
}

void ICQClientSocket::pause(unsigned t)
{
	QTimer::singleShot(t * 1000, this, SLOT(slotBytesWritten()));
}

ICQServerSocket::ICQServerSocket(unsigned short minPort, unsigned short maxPort)
{
#ifdef HAVE_KEXTSOCK_H
	sock = new KExtendedSocket;
	connect(sock, SIGNAL(readyAccept()), this, SLOT(activated()));
	for (m_nPort = minPort; m_nPort <= maxPort; m_nPort++){
		sock->reset();
		sock->setBlockingMode(false);
		sock->setSocketFlags(KExtendedSocket::passiveSocket);
		sock->setPort(m_nPort);
		if (sock->listen() == 0)
			break;
	}
	if (m_nPort > maxPort){
		delete sock;
		sock = NULL;
		return;
	}
#else
	sn = NULL;
	sock = new QSocketDevice;
	for (m_nPort = minPort; m_nPort <= maxPort; m_nPort++){
		if (sock->bind(QHostAddress(), m_nPort))
			break;
	}
	if ((m_nPort > maxPort) || !sock->listen(50)){
		delete sock;
		sock = NULL;
		return;
	}
	sn = new QSocketNotifier(sock->socket(), QSocketNotifier::Read, this);
	connect(sn, SIGNAL(activated(int)), this, SLOT(slotActivated(int)));
#endif
}

ICQServerSocket::~ICQServerSocket()
{
#ifndef HAVE_KEXTSOCK_H
	if (sn) delete sn;
#endif
	if (sock) delete sock;
}

void ICQServerSocket::slotActivated(int)
{
#ifndef HAVE_KEXTSOCK_H
	if (sock == NULL) return;
	int fd = sock->accept();
	if (fd >= 0){
		if (notify){
			QSocket *s = new QSocket;
			s->setSocket(fd);
			notify->accept(new ICQClientSocket(s));
		}else{
#ifdef WIN32
			::closesocket(fd);
#else
			::close(fd);
#endif
		}
	}
#endif
}

void ICQServerSocket::slotActivated()
{
#ifdef HAVE_KEXTSOCK_H
	KExtendedSocket *s = NULL;
	sock->accept(s);
	if (s == NULL) return;
	if (notify){
		notify->accept(new ICQClientSocket(s));
	}else{
		delete s;
	}
#endif
}

void KICQClient::getAutoResponse(unsigned long uin, ICQMessage *msg)
{
	string res;
	ICQUser *u = getUser(uin);
	unsigned long status = owner->uStatus;

	if (status & ICQ_STATUS_DND)
	{
		if (u)
			res = u->AutoResponseDND;
		if (*res.c_str() == 0)
			res = owner->AutoResponseDND;
	}
	else if (status & ICQ_STATUS_OCCUPIED)
	{
		if (u)
			res = u->AutoResponseOccupied;
		if (*res.c_str() == 0)
			res = owner->AutoResponseOccupied;
	}
	else if (status & ICQ_STATUS_NA)
	{
		if (u)
			res = u->AutoResponseNA;
		if (*res.c_str() == 0)
			res = owner->AutoResponseNA;
	}
	else if (status & ICQ_STATUS_FREEFORCHAT)
	{
		if (u)
			res = u->AutoResponseFFC;
		if (*res.c_str() == 0)
			res = owner->AutoResponseFFC;
	}
	else
	{
		if (u)
			res = u->AutoResponseAway;
		if (*res.c_str() == 0)
			res = owner->AutoResponseAway;
	}
	declineMessage(msg, QString::fromLocal8Bit(res.c_str()));
}


unsigned long KICQClient::getFileSize(QString name, QString base, vector<fileName> &files)
{
	kdDebug(14110) << "[KICQClient] getFileSize(QString, QString, vector)" << endl;
	QString fName = base + "/" + name;

	QFileInfo fInfo(fName);
	if (!fInfo.exists()) return 0;
	if (!fInfo.isDir())
	{
		if (fInfo.isReadable())
		{
			fileName f;
			QString n = name;
			n = n.replace(QRegExp("/"), "\\");
			f.name = n.local8Bit();
			f.localName = fName.local8Bit();
			f.size = fInfo.size();
			files.push_back(f);
			return fInfo.size();
		}
		kdDebug(14110) << "File " << name << " is not readable" << endl;
		return 0;
	}
	
	QDir d(fName);
	const QFileInfoList *f = d.entryInfoList();
	if (f == NULL)
		return 0;
	
	QFileInfoList ff = *f;
	unsigned long res = 0;
	
	for (const QFileInfo *fi = ff.first(); fi != NULL; fi = ff.next())
	{
		QString baseName = fi->fileName();
		if ((baseName == "") || (baseName == ".") || (baseName == "..") || fi->isSymLink()) continue;
		QString fName = name;
		if (fName.length())
		{
			if (fName[(int)(fName.length()-1)] != '/') fName += "/";
		}
		fName += baseName;
		res += getFileSize(fName, base, files);
	}
	
	kdDebug(14110) << "[KICQClient] getFileSize() return value=" << res << endl;
	return res;
}


unsigned long KICQClient::getFileSize(QString fName, vector<fileName> &files)
{
	if (fName.isEmpty()) return 0;
	for (int i = fName.length() - 1; i >= 0; i--)
	{
		if (fName[i] != '/')
			continue;
		return getFileSize(fName.mid(i+1), fName.left(i), files);
	}
	return 0;
}

// called by libicq!
unsigned long KICQClient::getFileSize(const char *name, vector<fileName> &files)
{
	unsigned long res = 0;
	bool bInQuote = false;
	int start = 0;
	
	QString file = QString::fromLocal8Bit(name);

	for (int i = 0; i < (int)file.length(); i++)
	{
		if (file[i] == '\"')
		{
			QString fname = file.mid(start, i - start);
			if (!fname.isEmpty())
				res += getFileSize(fname, files);
			bInQuote = !bInQuote;
			start = i + 1;
			continue;
		}
		if ((file[i].isSpace() || (file[i] == ',')) && !bInQuote){
			QString fname = file.mid(start, i - start);
			if (!fname.isEmpty())
				res += getFileSize(fname, files);
			start = i + 1;
			continue;
		}
	}
	QString fname = file.mid(start);
	if (!fname.isEmpty())
		res += getFileSize(fname, files);

	return res;
}

bool KICQClient::createFile(ICQFile *f, int mode)
{
	kdDebug(14110) << "Client::createFile(), f->Name=" << f->Name.c_str() << ", mode=" << mode << endl;

	bool bTruncate = false;
	int size = 0;

	QString name = QString::fromLocal8Bit(f->localName.c_str());
	kdDebug(14110) << "Client::createFile(), local filename=" << name << "." << endl;

	if (name.isEmpty() || (name[(int)(name.length() - 1)] != '/'))
		name += "/";

	QString shortName = QString::fromLocal8Bit(f->Name.c_str());
	int s = shortName.findRev(':');

	if (s >= 0)
	{
		shortName = shortName.mid(s + 1);
		kdDebug(14110) << "[Client] createFile(); found a ':', shortName=" << shortName.local8Bit() << endl;
	}

	shortName = shortName.replace(QRegExp("\\"), "/");
	if (shortName[0] == '/')
	{
		int s = shortName.findRev('/');
		if (s >= 0) shortName = shortName.mid(s + 1);
		kdDebug(14110) << "[Client] createFile(); shortName is absolute path, shortName=" << shortName.local8Bit() << endl;
	}

	s = shortName.findRev("/../");
	if (s >= 0)
		shortName = shortName.mid(s + 4);
	if (shortName.left(3) == "../")
		shortName = shortName.mid(3);

	shortName = shortName.replace(QRegExp("//"), "/");
	shortName = shortName.replace(QRegExp("/./"), "/");

	name += shortName;
	
	kdDebug(14110) << "[Client] createFile(); final shortName=" << shortName.local8Bit() << endl;

	f->Name = shortName.local8Bit();

	QFile *file = new QFile(name);
	
	QFileInfo info(*file);
	if ( info.exists() )
	{
		if (info.isDir())
		{
			kdDebug(14110) << "[Client] createFile(); file is a dir and already existing, ABORT" << endl;
			if (f->autoAccept)
				return false;
			emit fileNoCreate(f, name);
			return false;
		}
		
		size = info.size();
		if ((mode == FT_REPLACE) || (mode == FT_RESUME))
		{
			if ((mode == FT_REPLACE) || (info.size() > f->Size))
			{
				bTruncate = true;
				size = 0;
			}
		}
		else if (f->autoAccept)
		{
			kdDebug(14110) << "[Client] createFile(); autoAccept is on, FIXME unfinished" << endl;
			//
			emit fileNoCreate(f, name);
			return false;
			//
/*
			ICQUser *u = getUser(f->getUin());
			if (u == NULL)
				u = owner;

			SIMUser *_u = static_cast<SIMUser*>(u);
			if (!_u->AcceptFileOverride)
				_u = static_cast<SIMUser*>(owner);

			if (_u->AcceptFileOverwrite || (info.size() > f->Size))
			{
				bTruncate = true;
				size = 0;
			}
*/
		}
		else
		{
			emit fileExist(f, name, info.size() < f->Size);
			return false;
		}
	}
	
	QString path = name;
	int p = path.findRev('/');
	if (p >= 0)
	{
		path = path.left(p);
		kdDebug(14110) << "[Client] createFile(); Trying to create path to save file to, path=" << path << endl;

		if (!path.isEmpty())
		{
/*			string p;
			p = path.local8Bit();
			makedir((char*)p.c_str());
*/
			QDir qd = QDir();
			bool mkdirReturn = qd.mkdir (path);
			if (mkdirReturn)
				kdDebug(14110) << "[Client] createFile(); mkdir SUCCESS" << endl;
			else
				kdDebug(14110) << "[Client] createFile(); mkdir FAILED" << endl;
		}
	}
	
	if (!file->open(IO_WriteOnly | (bTruncate ? IO_Truncate : 0)))
	{
		if (f->autoAccept)
			return false;
		emit fileNoCreate(f, name);
		return false;
	}
	kdDebug(14110) << "[KICQClient] createFile finally finished, Name=" << f->Name.c_str() <<
		", shortName()=" << f->shortName().c_str() <<
		", localName=" << f->localName.c_str() << endl;

	file->at(size);
	f->state = size;
//	f->setPos(size);
	f->p = (unsigned long)file;
	return true;
}

bool KICQClient::openFile(ICQFile *f)
{
	kdDebug(14110) << " Client::openFile(), f->Name=" << f->Name.c_str() << "." << endl;

	QFile *file = new QFile(QString::fromLocal8Bit(f->Name.c_str()));
	if (!file->open(IO_ReadOnly))
	{
		delete file;
		return false;
	}
	f->p = (unsigned long)file;
	return true;
}

bool KICQClient::seekFile(ICQFile *f, unsigned long pos)
{
	kdDebug(14110) << " Client::seekFile(), f->Name=" << f->Name.c_str() << "." << endl;
	emit fileProcess(f);
	return ((QFile*)(f->p))->at(pos);
}

bool KICQClient::readFile(ICQFile *f, Buffer &b, unsigned short size)
{
	kdDebug(14110) << " Client::readFile(), f->Name=" << f->Name.c_str() << ", size=" << size << "." << endl;

	b.allocate(b.writePos() + size, 1024);
	char *p = b.Data(b.writePos());
	int readn = ((QFile*)(f->p))->readBlock(p, size);
	if (readn < 0)
		return false;
	b.setWritePos(b.writePos() + readn);
	emit fileProcess(f);
	return true;
}

bool KICQClient::writeFile(ICQFile *f, Buffer &b)
{
	kdDebug(14110) << " Client::writeFile(), f->Name=" << f->Name.c_str() << "." << endl;
	
	if (f->p == 0) return false;
	unsigned size = b.size() - b.readPos();
	char *p = b.Data(b.readPos());
	bool res = (((QFile*)(f->p))->writeBlock(p, size) >= 0);
	emit fileProcess(f);
	return res;
}

void KICQClient::closeFile(ICQFile *f)
{
	kdDebug(14110) << " Client::closeFile(), f->Name=" << f->Name.c_str() << "." << endl;

	if (f->p)
		delete (QFile*)(f->p);
	f->p = 0;
}

#include "client.moc"

