/*
    icqcontact.cpp  -  ICQ Protocol Plugin

    Copyright (c) 2002 by Nick Betcher <nbetcher@kde.org>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include "icqcontact.h"
#include "icqprotocol.h"
#include "icquserinfo.h"
#include "icquserreason.h"
#include "icqpreferences.h"
#include "client.h"
#include "icqclient.h"

#include "kopetemessagemanager.h"
#include "kopetemessagemanagerfactory.h"
#include "kopetemetacontact.h"
#include "kopetestdaction.h"

#include <qapplication.h>
#include <qregexp.h>
#include <qstylesheet.h>

#include <kdebug.h>
#include <klocale.h>
//#include <kfiledialog.h>
#include <kmessagebox.h>


ICQContact::ICQContact( QString UIN, QString nickname, QStringList groups, KopeteProtocol *p, KopeteMetaContact *mc )
	: KopeteContact( p, UIN, mc )
{
//	kdDebug( 14110 ) << k_funcinfo << "UIN=" << UIN << ", nickname=" << nickname << ", metacontact=" << metaContact() << endl;

	mUser = 0L;
	userInfoDialog = 0L;
	userReasonDialog = 0L;
//	mAuthEvent = 0L;
//	mAddedEvent = 0L;
	mMsgManager = 0L;
	actionCollection = 0L;

//	kdDebug(14110) << "[ICQContact] Creating ICQUser..." << endl;
	mUser = static_cast<ICQProtocol*>( protocol() )->engine()->getUser( UIN.toULong(), true );
	if (!mUser)
	{
		kdDebug(14110) << k_funcinfo << "FAILED creating ICQUser for " << nickname << "(" << UIN << ")." << endl;
		return;
	}
//	kdDebug(14110) << "[ICQContact] SUCCESS creating ICQUser." << endl;

//	kdDebug(14110) << "[ICQContact] No. of groups " << groups.count() << endl;
	if ( groups.count() > 0 )
	{
		// TODO: Handle more than one group gracefully
		QString _groupName = groups.first();

		ICQGroup *group = static_cast<ICQProtocol*>( protocol() )->findGroup(_groupName, true);
		if ( group )
		{
//			kdDebug(14110) << k_funcinfo << "[ICQContact] assigning group to user, _groupName=" << _groupName << endl;
			mUser->GrpId = group->Id;
			static_cast<ICQProtocol*>(protocol())->engine()->moveUser(mUser, group);
		}
		else
		{
			kdDebug(14110) << k_funcinfo << "ERROR, Could not create or find group!" << _groupName << endl;
		}
	}

	/* ========================================================================== */

	mUser->Alias = nickname.local8Bit();
	mUser->Nick = nickname.local8Bit();

	if ( nickname == UIN || nickname.isEmpty() )
	{
//		kdDebug(14110) << k_funcinfo << "Contact only got UIN as displayName(), requesting user info" << endl;
		static_cast<ICQProtocol*>(protocol())->engine()->addInfoRequest(mUser->Uin, false);
	}
	
	if (nickname.isEmpty())
		setDisplayName( QString::number(mUser->Uin) );
	else
		setDisplayName( nickname );

//	kdDebug(14110) << k_funcinfo << "displayName()=" << displayName() << endl;
//	kdDebug(14110) << k_funcinfo << "appending to theContacts" << endl;
	theContacts.append(this);

	// ==========================================================================

	initActions();

//	kdDebug(14110) << k_funcinfo << "Added ICQContact with Nick= " << mUser->Nick.c_str() << ", Alias= " << mUser->Alias.c_str() << endl;
	statusHasChanged( mUser );
}

ICQContact::~ICQContact()
{
}

// ==============================================================================

void ICQContact::addToGroup( KopeteGroup *group )
{
	kdDebug( 14110 ) << k_funcinfo << "displayName() = " << group->displayName() << endl;
	kdDebug( 14110 ) << k_funcinfo << "DOING NOTHING" << endl;
}

void ICQContact::removeFromGroup( KopeteGroup *group )
{
	kdDebug( 14110 ) << k_funcinfo << "displayName() = " << group->displayName() << endl;
	kdDebug( 14110 ) << k_funcinfo << "DOING NOTHING" << endl;
}

void ICQContact::moveToGroup( KopeteGroup *from, KopeteGroup *to )
{
	if( !to )
	{
		kdDebug( 14110 ) << k_funcinfo << "No destination group, giving up!" << endl;
		return;
	}

	if( !from )
	{
		kdDebug( 14110 ) << k_funcinfo << "No source group, giving up although the move might work!" << endl;
		return;
	}

	QString fromGroupName = from->displayName();
	QString toGroupName = to->displayName();
	ICQProtocol *p = static_cast<ICQProtocol*>(protocol());

	kdDebug(14110) << k_funcinfo << "moving " << displayName() << " (" << uin() <<
		") from KopeteGroup " << fromGroupName <<
		" to KopeteGroup " << toGroupName << endl;

	if( (toGroupName.isNull() || to->type() != KopeteGroup::Classic) && to->pluginData(protocol(),"id").isEmpty() )
	{
		//if this contact is in the last group and the contact moved to top level, do nothing
		//		(exepted if the group 0 is the top-level group)
		kdDebug(14110) << "[ICQContact] moving to top-level group, as ICQ does not"
		" support contacts with no groups we just move to group 'General'" << endl;

		toGroupName = "General";
	}

	ICQGroup *srcGroup = p->engine()->getGroup ( mUser->GrpId, false );
	if ( !srcGroup )
	{
		kdDebug(14110) << "[ICQContact] ERROR, Cannot find the source-group user " << displayName() << " is in !!!" << endl;
	}
	else
	{
		kdDebug(14110) << "[ICQContact] ICQContact " <<
			displayName() << " (" << uin() <<
			") is currently in group " << srcGroup->Name.c_str() <<
			" [" << mUser->GrpId << "]" << endl;

		if ( fromGroupName.lower() != QString(srcGroup->Name.c_str()).lower() )
			kdDebug(14110) << "[ICQContact] WARN, source-group is NOT the real group this contact is in, we just ignore that fact" << endl;
	}

	// search for group to move to and optionally just create it
	ICQGroup *dstGroup = p->findGroup( toGroupName, true );
	if ( dstGroup )
	{
		kdDebug(14110) << "[ICQContact] moveToGroup() to group: " << toGroupName << " [" << dstGroup->Id << "]" << endl;
		mUser->GrpId = dstGroup->Id;
		p->engine()->moveUser( mUser, dstGroup );
	}
	else
	{
		kdDebug(14110) << "[ICQContact] ERROR, moveToGroup(), could not find/create group to move to" << endl;
	}

	onlineActionNotify();
//	kdDebug(14110) << k_funcinfo << "END" << endl;
}

// ==============================================================================

void ICQContact::initActions()
{
	actionReadReason = new KAction(i18n("Read Status Reason"), "info", 0,
		this, SLOT(slotUserReason()), this, "actionReadReason");
	actionSendAuth = new KAction(i18n("Send Authentication"), "send", 0,
		this, SLOT(slotSendAuth()), this, "actionSendAuth");
//	actionToggleIgnoreMode = new KToggleAction(i18n("Ignore User"), "ignore", 0,
//		this, SLOT(slotToggleIgnoreMode()), this, "actionToggleIgnoreMode");
}

KActionCollection *ICQContact::customContextMenuActions()
{
	if(actionCollection)
		delete actionCollection;
	actionCollection = new KActionCollection(this);

//	kdDebug(14110) << "[ICQContact] WaitAuth is " << mUser->WaitAuth << endl;
	actionCollection->insert(actionSendAuth);
//	actionCollection->insert(actionToggleIgnoreMode);
	if ( status() == Away )
		actionCollection->insert(actionReadReason);

	return actionCollection;
}

/*
void ICQContact::slotToggleIgnoreMode(void)
{
	kdDebug(14110) << "[ICQContact] slotToggleIgnoreMode(), users ignoreId=" << mUser->IgnoreId << "." << endl;

	//FIXME: EMERGENCY EXIT until all contacts always have a group set
	if ( mUser->GrpId == 0 )
	{
		kdDebug(14110) << "[ICQContact] ERROR, ICQUser has no group set, ignoring is impossible, unignoring him would delete the user!" << endl;
		return;
	}
	//END FIXME

	if (mUser->IgnoreId == 0)
	{
		int r = KMessageBox::questionYesNo( 0L,
			i18n(
				"<qt>Add %1 to ignore list.</qt>").arg(displayName()),
			i18n("Add to ignore list"));
		
		if ( r == KMessageBox::Yes )
			static_cast<ICQProtocol*>( protocol() )->engine()->setInIgnore(mUser, true);
		
	}
	else // remove from ignorelist
	{
		static_cast<ICQProtocol*>( protocol() )->engine()->setInIgnore(mUser, false);
	}
}
*/


void ICQContact::slotSendAuth(void)
{
	kdDebug(14110) << "[ICQContact] slotSendAuth()" << endl;
	ICQAuthGranted *sendMessage = new ICQAuthGranted();
	sendMessage->Uin.push_back( mUser->Uin );
	// We store them waiting for ack or error
	static_cast<ICQProtocol*>( protocol() )->addPendingMessage(sendMessage, this);
	static_cast<ICQProtocol*>( protocol() )->engine()->sendMessage(sendMessage);
}


void ICQContact::slotSendMsg(const KopeteMessage &message)
{
//	kdDebug(14110) << "[ICQContact] slotSendMsg()" << endl;
	if (message.body().isEmpty() ) // no text, do nothing
		return;

	QString msgString;
	msgString = "<font>" + message.escapedBody() + "<br>\n</font>";
//	msgString = "<font>" + message.escapedBody() + "\r\n</font>";
	// escapedBody() does this now (mETz 10.01.2003)
//	msgString.replace(QRegExp("\\r?\\n"), "<br>\n");

	kdDebug(14110) << k_funcinfo << "outgoing escaped message {" << msgString << "}" << endl;

	ICQMsg *sendMessage = new ICQMsg();
	sendMessage->Uin.push_back( mUser->Uin );
	sendMessage->Direct = true;

	// hope it works now, works fine with trillian users and umlauts+euro-symbol and with
	// pasted chinese :))
	sendMessage->Charset = "utf-8";
	sendMessage->Message = msgString.utf8();

//	kdDebug(14110) << "[ICQContact] outgoing utf-8 message {" << sendMessage->Message.c_str() << "}" << endl;
//	kdDebug(14110) << "[ICQContact] FG: " << message.fg().name() << ", BG: " << message.bg().name() << endl;
	bool useColor = (message.fg().isValid() || message.bg().isValid()) && (message.fg() != message.bg());
	if ( useColor )
	{
		QColor fg = message.fg().isValid() ? message.fg() : QColor(0,0,0);
		QColor bg = message.bg().isValid() ? message.bg() : QColor(255,255,255);
//		kdDebug(14110) << "[ICQContact] USING => fg: " << fg.name() << " bg: " << bg.name() << endl;
		sendMessage->BackColor = ( bg.rgb() & 0xFFFFFF);
		sendMessage->ForeColor = ( fg.rgb() & 0xFFFFFF);
	}

//	kdDebug(14110) << ""[ICQContact] SENDING: [[" << sendMessage->Message.c_str() << "]]" << endl;

	// We store the message waiting for ack or error
	static_cast<ICQProtocol*>( protocol() )->addPendingMessage(sendMessage, this);
	static_cast<ICQProtocol*>( protocol() )->engine()->sendMessage(sendMessage);
}

unsigned int ICQContact::uin(void) const
{
	if (mUser)
		return mUser->Uin;
	return 0;
}

QString ICQContact::data() const
{
	if (mUser)
		return QString::number( mUser->Uin );
	return QString();
}

void ICQContact::execute()
{
	msgManager()->readMessages();
}

KopeteMessageManager* ICQContact::msgManager()
{
	if ( mMsgManager )
		return mMsgManager;
	else
	{
		mMsgManager = KopeteMessageManagerFactory::factory()->create( protocol()->myself(), theContacts, protocol(), KopeteMessageManager::ChatWindow);
		connect(mMsgManager, SIGNAL(destroyed()), this, SLOT(slotMsgManagerDeleted()));
		connect(mMsgManager, SIGNAL(messageSent(const KopeteMessage&, KopeteMessageManager *)), this, SLOT(slotSendMsg(const KopeteMessage &)));
		return mMsgManager;
	}
}

void ICQContact::slotMsgManagerDeleted(void)
{
	kdDebug(14110) << k_funcinfo << endl;
	mMsgManager = 0L;
}

void ICQContact::messageAck(ICQMessage *msg)
{
//	kdDebug(14110) << "[ICQContact] Got ACK for message sent to " << displayName() << "(" << uin() << ")" << endl;
	newMessage(msg, true);
}

void ICQContact::newMessage(ICQMessage *message, bool own)
{
	QString messageBody = "";
	QDateTime time;

	ICQUser *u = static_cast<ICQProtocol*>( protocol() )->engine()->getUser(message->getUin(), false);
	if (!u) return;

	switch(message->Type())
	{
		case ICQ_MSGxMSG:
		{
			ICQMsg *m = static_cast<ICQMsg *>(message);
			if (!m)
				return;

			time.setTime_t(m->Time);

//			kdDebug(14110) << "[ICQContact] MSG ORG : {" << m->Message.c_str() << "} Charset='" << m->Charset.c_str() << "'" << endl;

			// FIXME: below should work but does not for received umlauts (mETz, 10.01.2003)
//			messageBody = QString::fromUtf8( static_cast<ICQProtocol*>( protocol() )->engine()->from8Bit(uin(), m->Message, m->Charset.c_str()) );

			if( m->Charset == "utf-8" )
			{
//				kdDebug(14110) << "[ICQContact] got UTF-8 Message!" << endl;
				messageBody = QString::fromUtf8(m->Message.c_str());
			}
			else
			{
//				kdDebug(14110) << k_funcinfo << "Got normal Message!" << endl;
				messageBody = QString::fromLocal8Bit(m->Message.c_str());
			}

//			kdDebug(14110) << "[ICQContact] MSG DEC : {" << messageBody << "}" << endl;

			// for contacts that send rtf/html formatted messages we need to strip off some things
			// for ourselves outgoing messages we always strip off tags
			// FIXME: This is not a good solution but it works(tm)!
			if ( u->hasCap(1) || own )
			{
//				kdDebug(14110) << k_funcinfo << "Peer has RTF support, removing additional linebreaks" << endl;
				int p = -1;
#if (QT_VERSION-0 >= 0x030100)
				messageBody.replace( "<br>\n", "<br>" );
#else
				while ( (p = messageBody.find("<br>\n",0,true)) != -1 )
				{
					messageBody = messageBody.remove( p, 5 );
					messageBody = messageBody.insert( p, "<br>" );
				}
				p = -1;
#endif
				// FIXME: This removes a <br> at the end of a html message
				// This way is error prone and will break if the messaging format changes
				// slightly.
				p = messageBody.findRev("<br></font>",-1,true);
//				kdDebug(14110) << "[ICQContact] p = " << p << ", bodylength= " << messageBody.length() << endl;
				if ( p == static_cast<int>( messageBody.length()-11) )
					messageBody = messageBody.remove( p, 4 );
			}

//			kdDebug(14110) << "[ICQContact] MSG ESC   : {" << messageBody << "}" << endl;

			KopeteContact *fromContact = 0L;
			KopeteContactPtrList toContacts;

			if ( own )  // outgoing message
			{
				fromContact = protocol()->myself();
				toContacts = theContacts;
			}
			else // incoming message
			{
				fromContact = this;
				toContacts.append ( protocol()->myself() );
			}
 
			KopeteMessage msg(time, fromContact, toContacts, messageBody, own ? KopeteMessage::Outbound : KopeteMessage::Inbound, KopeteMessage::RichText );

			if ( m->ForeColor != m->BackColor ) // black on black does not look good
			{
				msg.setFg(QColor(m->ForeColor));
				msg.setBg(QColor(m->BackColor));
			}
			msgManager()->appendMessage(msg);
			break;
		} // END case ICQ_MSGxMSG

		case ICQ_MSGxURL:
		{
			ICQUrl *m = static_cast<ICQUrl *>(message);
			if (!m)
				return;

			KopeteContactPtrList tmpMyself;
			tmpMyself.append ( protocol()->myself() );

			QString messageAndPrefix = "<b>[URL:] " + QStyleSheet::escape(QString::fromLocal8Bit(m->URL.c_str())) + "</b>\n" + QString::fromLocal8Bit(m->Message.c_str());
			KopeteMessage msg(this, tmpMyself, messageAndPrefix, KopeteMessage::Inbound, KopeteMessage::RichText);
			msgManager()->appendMessage(msg);
			break;
		}

		case ICQ_MSGxWEBxPANEL:
		{
			ICQWebPanel *m = static_cast<ICQWebPanel*>(message);
			if(!m) return;

			KopeteContactPtrList tmpMyself;
			tmpMyself.append ( protocol()->myself() );
/*
			const char *encoding = msg->Charset.c_str();
			if (*encoding == 0)
				encoding = codec->name();
			msg_text = m->Message.c_str();
			SIMClient::toUTF(msg_text, encoding);
*/
			QString messageAndPrefix =
				"<b>[WebPanel:]</b> (" + QString::fromLocal8Bit(m->Email.c_str()) + ") " +
				QString::fromLocal8Bit(m->Message.c_str());
			
			// Special name-handling for webpanel because these messages come from no real UIN
			// Name is passed by the packet and cannot be retrieved using the UIN
			QString id;
			if ( m->Name != "")
				id = QString::fromLocal8Bit(m->Name.c_str());
			else
				id = QString::number(mUser->Uin);
			setDisplayName( id );

			KopeteMessage msg(this, tmpMyself, messageAndPrefix, KopeteMessage::Inbound, KopeteMessage::RichText);
			msgManager()->appendMessage(msg);
			break;
		} // END case ICQ_MSGxWEBxPANEL

		case ICQ_MSGxEMAILxPAGER:
		{
			ICQEmailPager *m = static_cast<ICQEmailPager*>(message);
			if(!m) return;

			KopeteContactPtrList tmpMyself;
			tmpMyself.append ( protocol()->myself() );
/*
			const char *encoding = msg->Charset.c_str();
			if (*encoding == 0)
				encoding = codec->name();
			msg_text = m->Message.c_str();
			SIMClient::toUTF(msg_text, encoding);
*/
			QString messageAndPrefix =
				QString::fromLocal8Bit("<b>[EmailPager:]</b> (") +
				QString::fromLocal8Bit(m->Email.c_str()) +
				QString::fromLocal8Bit(") ") +
				QString::fromLocal8Bit(m->Message.c_str());

			// Special name-handling for webpanel because these messages come from no real UIN
			// Name is passed by the packet and cannot be retrieved using the UIN
			QString id;
			if ( m->Name != "")
				id = QString::fromLocal8Bit(m->Name.c_str());
			else
				id = QString::number(mUser->Uin);
			setDisplayName( id );

			KopeteMessage msg(this, tmpMyself, messageAndPrefix, KopeteMessage::Inbound, KopeteMessage::RichText);
			msgManager()->appendMessage(msg);
			break;
		}

		case ICQ_MSGxAUTHxGRANTED:
		{
			// ignore :)
		} // END case ICQ_MSGxAUTHxGRANTED

		default:
		{
			kdDebug(14110) << k_funcinfo << "Received UNHANDLED message-type, type=" << message->Type() <<
				" for user " << displayName() << "(" << uin() << ") [" << mUser->GrpId << "]" << endl;
			break;
		}
	}
}

/*
bool ICQContact::sendSMS(QString &msgText, QString &PhoneNo)
{
	ICQProtocol *p = static_cast<ICQProtocol*>( protocol() );

	ICQSMS *m = new ICQSMS();
	m->Uin.push_back( mUser->Uin );
	
	string text(msgText.utf8());
	text = p->engine()->clearHTML(text);
	s = QString::fromUtf8(text.c_str());

	// TODO: Split messages if they are too long
//	msgTail = trim(s);
//	m->Message = smsChunk();

	m->Message = trim(s);
	m->Phone = PhoneNo.local8Bit();
	m->Charset = p->engine()->codecForUser(getUin())->name();

	p->addPendingMessage(m, this);
	return p->engine()->sendMessage(m);
}
*/

void ICQContact::statusHasChanged(ICQUser *u)
{
	if( u->Uin != mUser->Uin )
		return;

//	kdDebug(14110) << "[ICQContact] " << displayName() << "(" << mUser->Uin << ") [" << mUser->GrpId << "] changed status to: " << statusText() << endl;

//	if ( status() == Away ) // if user is away check for the away-message
//			static_cast<ICQProtocol*>( protocol() )->engine()->addResponseRequest( mUser->Uin );

	// we can only send to people being online
	//TODO: File sending is unfinished
//	setFileCapable( (status()!=Offline) );

	emit statusChanged( this, status() );
}


void ICQContact::infoUpdated(int type)
{
	// happens on init, ICQContact is already in list of contacts
	// but ICQUser in libicq is not created yet.
	if ( !mUser )
		return;

/*
	// same as testing for mUser as that one
	// should be the corresponding ICQuser in libicq
	ICQUser *u = static_cast<ICQProtocol*>( protocol() )->engine()->getUser( mUser->Uin, false );
	if (!u)
		return;
*/

//	kdDebug(14110) << "[ICQContact] infoUpdated() for " << displayName() << "(" << mUser->Uin << ") [" << mUser->GrpId << "]" << endl;

	// After updating a user the engine tends to get rid of the group id
	// that the contact is associated with :(

/*
	QStringList grpList = metaContact()->groups().toStringList();
	QString grpName = grpList.first();
	if ( !grpName.isNull() )
	{
		ICQGroup *group = static_cast<ICQProtocol*>(protocol())->findGroup(grpName, false );
		if (group)
		{
			kdDebug(14110) << "[ICQContact] infoUpdated(), re-Set group for " <<
				displayName() << "(" << mUser->Uin << ") to " <<
				group->Name.c_str() << "[" << group->Id << "]" << endl;

			mUser->GrpId = group->Id;
		}
		else
		{
			kdDebug(14110) << "[ICQContact] infoUpdated() LOST GROUP for " << displayName() << "(" << mUser->Uin << ")" << endl;
		}
	}
	else
	{
		kdDebug(14110) << "[ICQContact] infoUpdated() NO GROUP for " << displayName() << "(" << mUser->Uin << ")" << endl;
	}
*/
/*
	if ( QString(mUser->Alias.c_str()).isEmpty() && !QString(mUser->Nick.c_str()).isEmpty() )
	{
		setDisplayName(QString::fromLocal8Bit(mUser->Nick.c_str()));
	}
*/
	if ( type == EVENT_SUBTYPE_FULLINFO )
	{
		kdDebug(14110) << k_funcinfo << "Got FULL info about '" << displayName() << "'(" << uin() << ") [" << mUser->GrpId << "]" << endl;
		QString newNick = QString::fromLocal8Bit(mUser->Alias.c_str());
		if (newNick.isEmpty())
			newNick = QString::fromLocal8Bit(mUser->Nick.c_str());

		if ( displayName() != newNick ) // contact changed his nickname
			setDisplayName(newNick);
		emit updatedInfoFull();
	}
	else if ( type == EVENT_SUBTYPE_AUTOREPLY )
	{
		kdDebug(14110) << k_funcinfo << "Got AUTOREPLY from '" << displayName() << "'(" << uin() << ") [" << mUser->GrpId << "]" << endl;
		emit updatedInfoPartly();
	}
}

QString ICQContact::statusIcon() const
{
	int status = mUser->uStatus;

	if ((status & 0xFFFF) == ICQ_STATUS_OFFLINE)
		return "icq_offline";
	if (status & ICQ_STATUS_DND)
		return "icq_dnd";
	if (status & ICQ_STATUS_OCCUPIED)
		return "icq_occupied";
	if (status & ICQ_STATUS_NA)
		return "icq_na";
	if (status & ICQ_STATUS_AWAY)
		return "icq_away";

	return "icq_online";
}

ICQContact::ContactStatus ICQContact::status() const
{
	if( mUser->uStatus == ICQ_STATUS_OFFLINE )
		return Offline;
	else if( (mUser->uStatus&0xFF) == ICQ_STATUS_ONLINE )
		return Online;
	else
		return Away;
}

QString ICQContact::statusText() const
{
	return static_cast<ICQProtocol*>( protocol() )->statusToString(mUser->uStatus);
}

int ICQContact::importance() const
{
	int status = mUser->uStatus;
	if ((status & 0xFFFF) == ICQ_STATUS_OFFLINE)
		return 0;
	if (status & ICQ_STATUS_DND)
		return 12;
	if (status & ICQ_STATUS_OCCUPIED)
		return 13;
	if (status & ICQ_STATUS_NA)
		return 10;
	if (status & ICQ_STATUS_FREEFORCHAT)
		return 20;
	if (status & ICQ_STATUS_AWAY)
		return 15;
	return 19;
}


void ICQContact::slotDeleteContact()
{
//	kdDebug(14110) << k_funcinfo << "called for " << displayName() << "(" << uin() << ") [" << mUser->GrpId << "]" << endl;

	onlineActionNotify();
	static_cast<ICQProtocol*>( protocol() )->engine()->deleteUser(mUser);
	mUser = 0L;
	deleteLater();
}

void ICQContact::slotUserInfo(void)
{
//	kdDebug(14110) << "[ICQContact] slotUserInfo()" << endl;
	if ( userInfoDialog )
	{
		userInfoDialog->raise();
	}
	else
	{
		userInfoDialog = new ICQUserInfo( this, displayName(), static_cast<ICQProtocol*>(protocol()));
		if ( !userInfoDialog ) // emergency exit
			return;
		connect( userInfoDialog, SIGNAL(updateNickname(const QString)), this, SLOT(slotUpdateNickname(const QString)) );
		connect( userInfoDialog, SIGNAL(closing()), this, SLOT(slotCloseUserInfoDialog()) );
		userInfoDialog->show();
	}
}

void ICQContact::slotCloseUserInfoDialog()
{
	userInfoDialog->delayedDestruct();
	userInfoDialog = 0L;
}


//void ICQContact::sendFile(const KURL &sourceURL, const QString &/*altFileName*/, const long unsigned int fileSize)
//{
/*
	QString filePath;

	// FIXME: can be removed soon, set ability to send file depending on contact-status instead!
	if ((mUser->uStatus & 0xFFFF) == ICQ_STATUS_OFFLINE)
	{
		KMessageBox::error( 0L,
		i18n(
			"<qt>%1 must be online before you can send him/her a file." 
			" Please wait until this user is online, and then try again.</qt>").arg(displayName()),
		i18n("User offline"));
		return;
	}
	
	//If no file was specified, then ask the user what file to send
	if( !sourceURL.isValid() ) 
	{
		filePath = KFileDialog::getOpenFileName();
	}
	else
	{
		filePath = sourceURL.path(-1);
	}
	
	if( !filePath.isEmpty() )
	{
		ICQFile *sendMessage = new ICQFile();
		if ( !sendMessage ) 
			return;
		sendMessage->Uin.push_back( mUser->Uin );
		sendMessage->Direct = true;
		sendMessage->Description = "None";
		sendMessage->Name = filePath.local8Bit();
		if (fileSize!=0L)
			sendMessage->Size = static_cast<unsigned long>(fileSize);

		kdDebug(14110) << "[ICQContact] slotSendFile(); Description=" << sendMessage->Description.c_str() << endl;
		kdDebug(14110) << "[ICQContact] slotSendFile(); Name=" << sendMessage->Name.c_str() << endl;
		kdDebug(14110) << "[ICQContact] slotSendFile(); Size=" << sendMessage->Size << endl;

		kdDebug(14110) << "[ICQContact] slotSendFile(); Adding ICQFile packet to output queue..." << endl;

		static_cast<ICQProtocol*>( protocol() )->addPendingMessage(sendMessage, this);
		if ( !static_cast<ICQProtocol*>( protocol() )->engine()->sendMessage(sendMessage) )
			kdDebug(14110) << "[ICQContact] slotSendFile(); sendMessage FAILED" << endl;
		else
			kdDebug(14110) << "[ICQContact] slotSendFile(); sendMessage WORKED" << endl;
	}
*/
//}


void ICQContact::slotUserReason(void)
{
//	kdDebug(14110) << "[ICQContact] slotUserReason()" << endl;
	if ( userReasonDialog )
	{
		userReasonDialog->raise();
	}
	else
	{
		userReasonDialog = new ICQUserReason( this, static_cast<ICQProtocol*>( protocol() ) );
		if ( !userReasonDialog ) // emergency exit
			return;
		connect( userReasonDialog, SIGNAL(closing()), this, SLOT(slotCloseUserReasonDialog()) );
		userReasonDialog->show();
	}
}


void ICQContact::slotCloseUserReasonDialog()
{
	userReasonDialog->delayedDestruct();
	if ( userReasonDialog )
		userReasonDialog = 0L;
}


void ICQContact::slotUpdateNickname(const QString newNickname)
{
	kdDebug(14110) << k_funcinfo << "called for user " << displayName() <<
		"(" << uin() << ") [" << mUser->GrpId << "]" << endl;

	if(newNickname == displayName()) // no changes in nick
		return;

	rename(newNickname);
	setDisplayName(newNickname);
}

void ICQContact::slotDeletingGroup(const QString &groupName)
{
	kdDebug(14110) << k_funcinfo << "called for user " << displayName() <<
		"(" << uin() << ") [" << mUser->GrpId << "]" << endl;

	ICQUser *u = static_cast<ICQProtocol*>( protocol() )->engine()->getUser(mUser->Uin, false);
	if (!u)
		return;

	ICQGroup *g = static_cast<ICQProtocol*>( protocol() )->engine()->getGroup(u->GrpId, false);
	if (!g)
		return;

	// Hardcoding Unknown is a hack, but I can't think of a better way to do it! Sorry :(
	if (QString::fromLocal8Bit(g->Name.c_str()).lower() == groupName.lower() || (QString(g->Name.c_str()).isEmpty() && groupName == "Unknown"))
	{
		static_cast<ICQProtocol*>( protocol() )->engine()->deleteUser(u);
		delete this;
		return;
	}
}

void ICQContact::rename( const QString &newName )
{
	kdDebug(14110) << k_funcinfo << "called. newName=" << newName << endl;
	onlineActionNotify();
	static_cast<ICQProtocol*>( protocol() )->engine()->renameUser(mUser, newName.local8Bit());
//	setDisplayName(newName);

}

void ICQContact::onlineActionNotify()
{
	if(!static_cast<ICQProtocol*>(protocol())->isConnected())
	{
		KMessageBox::information( 0L,
			i18n( "<qt>When you are offline, changes to your contact list don't update the "
			"server-side contact list. Your changes may be lost</qt>" ),
			i18n( "ICQ Plugin" ), "icq_OfflineContactList" );
	}
}

#include "icqcontact.moc"

// vim: set noet ts=4 sts=4 tw=4:

