/*
    icqcontact.h  -  ICQ Protocol Plugin

    Copyright (c) 2002 by Nick Betcher <nbetcher@kde.org>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#ifndef ICQCONTACT_H
#define ICQCONTACT_H

#include "kopetecontact.h"
#include "kopetemessage.h"
#include <kaction.h>
#include <kurl.h>

class KPopupMenu;
class ICQProtocol;
class KopeteMessageManager;
class ICQUserInfo;
class ICQUserReason;
class ICQMessage;
class ICQUser;

class ICQContact : public KopeteContact
{
	Q_OBJECT
	friend class ICQProtocol;

public:
	ICQContact( QString UIN, QString nickname, QStringList groups, KopeteProtocol *p, KopeteMetaContact *mc );
	~ICQContact();

	// Reimplementations of the members in KopeteContact
	virtual QString data() const;
	virtual ContactStatus status() const;
	QString statusText() const;
	QString statusIcon() const;
	int importance() const;
	unsigned int uin() const;
	KActionCollection *customContextMenuActions();
	bool isReachable() { return true; }

	// FIXME: Move this one to private and only allow const access through user()
	ICQUser *mUser;

	// returns the ICQUser for this ICQContact
	ICQUser* user() const { return mUser; };

	void messageAck(ICQMessage *);
	void rename( const QString &newName );

signals:
	void updatedInfoFull(); // gets emitted if full userinfo changed
	void updatedInfoPartly(); // gets emitted if away messages changed

public slots:
	virtual void execute();
	virtual void slotDeleteContact();
	virtual void slotUserInfo();
	void slotUserReason(void);
//	void sendFile(const KURL &sourceURL, const QString &altFileName, const long unsigned int fileSize);

private slots:
//	void slotToggleIgnoreMode(void);
	void slotSendAuth(void);
	void slotSendMsg(const KopeteMessage &);
	// FIXME: delete this method
	void slotDeletingGroup(const QString &groupName);
	void slotUpdateNickname(const QString newNickname);

	void slotCloseUserInfoDialog();
	void slotCloseUserReasonDialog();
	void slotMsgManagerDeleted();
	
	/**
	 * Called when contact got added to a group
	 */
	virtual void addToGroup( KopeteGroup *group );

	/**
	 * Called when contact got removed from a group
	 */
	virtual void removeFromGroup( KopeteGroup *group );

	/**
	 * Called when contact got moved to a different group
	 */
	virtual void moveToGroup( KopeteGroup *from, KopeteGroup *to );

private:
//	KopeteEvent *mAuthEvent;
//	KopeteEvent *mAddedEvent;
	KopeteMessageManager *mMsgManager;
	KopeteMessageManager *msgManager();
	QPtrList<KopeteContact> theContacts;
	KAction *actionReadReason;
	KAction *actionSendAuth;
//	KAction *actionSendFile;
//	KToggleAction *actionToggleIgnoreMode;
	KActionCollection *actionCollection;

	ICQUserInfo *userInfoDialog;
	ICQUserReason *userReasonDialog;

	void initActions();
	void onlineActionNotify(void);

protected:
	void newMessage(ICQMessage *message, bool own = false);
	void statusHasChanged(ICQUser *u); // the "has" is cheesy, but it avoids a conflict with KopeteContact
	void infoUpdated(int type);
};

#endif

// vim: set noet ts=4 sts=4 tw=4:

