 /*
    icqfiletransfer.cpp  -  ICQ Protocol Plugin

    Copyright (c) 2002 by Nick Betcher <nbetcher@kde.org>
    
    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
 
#include "icqfiletransfer.h"

#include "icqprotocol.h"
#include "kopetetransfermanager.h"
#include "kopeteprotocol.h"
#include "client.h"

#include <qtimer.h>

#include <kdebug.h>

ICQFileTransfer::ICQFileTransfer(ICQFile *file, KopeteTransfer *transfer, ICQProtocol *protocol, QObject *parent, const char *name)
	: QObject(parent, name)
{
	kdDebug(14110) << "[ICQFileTransfer] ICQFileTransfer(), file=" << file << "transfer=" << transfer << "protocol=" << protocol << endl;

	mFile = file;
	mTransfer = transfer;
	mProtocol = protocol;
	tryAbort = false;

	kdDebug(14110) << "[ICQFileTransfer] connecting to engine event() signal..." << endl;
	connect(mProtocol->engine(), SIGNAL(event(ICQEvent *)), this, SLOT(slotEvent(ICQEvent *)));
	kdDebug(14110) << "[ICQFileTransfer] connect done" << endl;
	
	kdDebug(14110) << "[ICQFileTransfer] creating timer..." << endl;
	mTimer = new QTimer(this);
	connect(mTimer, SIGNAL(timeout()), this, SLOT(slotUpdateProgress()));
	kdDebug(14110) << "[ICQFileTransfer] timer created" << endl;
	
	kdDebug(14110) << "[ICQFileTransfer] connecting to transfer canceled signal..." << endl;
	connect(transfer, SIGNAL(transferCanceled()), this, SLOT(slotCancelTransfer()));
	kdDebug(14110) << "[ICQFileTransfer] connect done." << endl;

	mTimer->start(500);
	kdDebug(14110) << "[ICQFileTransfer] END ICQFileTransfer()" << endl;
}

void ICQFileTransfer::slotCancelTransfer()
{
	kdDebug(14110) << "[ICQFileTransfer] slotCancelTransfer()" << endl;
	
	if (!mFile)
		return;

	tryAbort = true;
	mProtocol->engine()->cancelMessage(mFile);
}

void ICQFileTransfer::slotEvent(ICQEvent *e)
{
	kdDebug(14110) << "[ICQFileTransfer] slotEvent()" << endl;

	if (e->message() != mFile)
		return;

	if (e->state == ICQEvent::Success)
	{
		kdDebug(14110) << "[ICQFileTransfer]  e->state == ICQEvent::Success" << endl;
		if (e->type() != EVENT_DONE) return;

		mTransfer->slotPercentCompleted(100);

		kdDebug(14110) << "[ICQFileTransfer] instance being deleted." << endl;
		delete this;
		return;
	}
	else
	{
		if (tryAbort)
			mTransfer->setError(KopeteTransfer::CanceledLocal);
		else
			mTransfer->setError(KopeteTransfer::CanceledRemote);
	}
}

void ICQFileTransfer::slotUpdateProgress()
{
	kdDebug(14110) << "[ICQFileTransfer] slotUpdateProgress()" << endl;

	int percent = (int)( ((double)mFile->state / mFile->Size) * 100 );
	mTransfer->slotPercentCompleted(percent);
}

#include "icqfiletransfer.moc"

// vim: set noet ts=4 sts=4 sw=4:
