 /*
    icqpreferences.cpp  -  ICQ Protocol Plugin

    Copyright (c) 2002 by Duncan Mac-Vicar Prett <duncan@kde.org>
    
    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include "icqpreferences.h"
#include "icqpreferences.moc"

#include "icqprotocol.h"

#include "libicq/icqclient.h"
#include "libicq/socket.h"

#include "ui/prflogintabwidget.h"
#include "ui/prfconnectiontabwidget.h"
#include "ui/prfsettingstabwidget.h"

#include <qcombobox.h>
#include <qcheckbox.h>
#include <qlayout.h>
#include <qtabwidget.h>

#include <klineedit.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include <knuminput.h>
#include <kglobal.h>
#include <kconfig.h>
#include <klocale.h>
#include <qlabel.h>

ICQPreferences::ICQPreferences(const QString &pixmap, ICQProtocol *parent)
	: ConfigModule( i18n("ICQ Plugin"), i18n("Mirabilis ICQ Protocol"), pixmap, parent )
{
	protocol = parent;
	cfg = KGlobal::config();

	(new QVBoxLayout(this))->setAutoAdd(true);

	icqTabWidget = new QTabWidget( this, "ICQTabWidget" );

	loginTab = new loginTabWidget( icqTabWidget, "loginTab" );
	icqTabWidget->insertTab( loginTab, i18n( "&Login" ) );
	
	connectionTab = new connectionTabWidget( icqTabWidget, "connectionTab" );
	icqTabWidget->insertTab( connectionTab, i18n( "&Connection" ) );

	settingsTab = new settingsTabWidget( icqTabWidget, "settingsTab" );
	icqTabWidget->insertTab( settingsTab, i18n( "C&hat" ) );

	reopen();

	// Track changes of login tab ===============================================================
	connect ( loginTab->mUIN, SIGNAL(textChanged(const QString &)), this, SLOT(slotChangeNeedsReconnect()) );
	connect ( loginTab->mPass, SIGNAL(textChanged(const QString &)), this, SLOT(slotChangeNeedsReconnect()) );
	connect ( loginTab->mNick, SIGNAL(textChanged(const QString &)), this, SLOT(slotChangeNeedsReconnect()) );

	// Track changes of connection tab ==========================================================
	connect ( connectionTab->mServer, SIGNAL(textChanged(const QString&)), this, SLOT(slotChangeNeedsReconnect()) );
	connect ( connectionTab->mServerPort, SIGNAL(valueChanged(int)), this, SLOT(slotChangeNeedsReconnect()) );
/*
	connect ( connectionTab->mAutoConnect, SIGNAL(toggled(bool)), this, SLOT(slotChange()) );
	connect ( connectionTab->mAutoConnectStatus, SIGNAL(activated(int)), this, SLOT(slotChange()) );
*/
	connect ( connectionTab->mListenFromPort, SIGNAL(valueChanged(int)), this, SLOT(slotChangeNeedsReconnect()) );
	connect ( connectionTab->mListenToPort, SIGNAL(valueChanged(int)), this, SLOT(slotChangeNeedsReconnect()) );

	connect ( connectionTab->mUseProxy, SIGNAL(toggled(bool)), this, SLOT(slotChangeNeedsReconnect()) );
	connect ( connectionTab->mProxyType, SIGNAL(activated(int)), this, SLOT(slotChangeNeedsReconnect()) );
	connect ( connectionTab->mProxyServer, SIGNAL(textChanged(const QString &)), this, SLOT(slotChangeNeedsReconnect()) );
	connect ( connectionTab->mProxyPort, SIGNAL(valueChanged(int)), this, SLOT(slotChangeNeedsReconnect()) );

	connect ( connectionTab->mProxyAuth, SIGNAL(toggled(bool)), this, SLOT(slotChangeNeedsReconnect()) );
	connect ( connectionTab->mProxyAuthUser, SIGNAL(textChanged(const QString &)), this, SLOT(slotChangeNeedsReconnect()) );
	connect ( connectionTab->mProxyAuthPassword, SIGNAL(textChanged(const QString &)), this, SLOT(slotChangeNeedsReconnect()) );
	
	// Track changes of Settings tab =============================================================
/*
	connect ( connectionTab->chkLogAll, SIGNAL(toggled(bool)), this, SLOT(slotChange()) );
	connect ( connectionTab->sendByServer, SIGNAL(activated(int)), this, SLOT(slotChange()) );
	connect ( connectionTab->chkHideIP, SIGNAL(toggled(bool)), this, SLOT(slotChange()) );
	connect ( connectionTab->chkWebAware, SIGNAL(toggled(bool)), this, SLOT(slotChange()) );
	connect ( connectionTab->chkAuth, SIGNAL(toggled(bool)), this, SLOT(slotChange()) );
*/
	// ===========================================================================================

	connect ( connectionTab->mAutoConnect, SIGNAL(toggled(bool)), this, SLOT(slotAutoConnectToggled(bool)) );
	connect ( connectionTab->mUseProxy, SIGNAL(toggled(bool)), this, SLOT(slotUseProxyToggled(bool)) );
	connect ( connectionTab->mProxyAuth, SIGNAL(toggled(bool)), this, SLOT(slotProxyAuthToggled(bool)) );
	connect ( settingsTab->chkPorts, SIGNAL(toggled(bool)), this, SLOT(slotUseCustomPorts(bool)));
}


ICQPreferences::~ICQPreferences()
{
}


void ICQPreferences::reopen()
{
	cfg->setGroup("ICQ");
	mUIN				= cfg->readEntry ( "UIN", "0" );
	mPasswd				= cfg->readEntry ( "Password", "" );
	mNickName			= cfg->readEntry ( "Nick", "" );

	// Proxy-Settings
	// TODO: Change this to also use kde-proxy settings if wanted
	mUseProxy				= cfg->readBoolEntry ( "UseProxy", false );
	mProxyType				= cfg->readNumEntry( "ProxyType", 1 );
	mProxyServerAdress	= cfg->readEntry ( "ProxyServer", "" );
	mProxyPort				= cfg->readNumEntry ( "ProxyPort", 8080 );
	mProxyAuth 				= cfg->readBoolEntry ( "ProxyAuth", false );
	mProxyAuthUser 		= cfg->readEntry ( "ProxyAuthUser", "" );
	mProxyAuthPassword 	= cfg->readEntry ( "ProxyAuthPassword", "" );

	// Other stuff
	mLogAll				= cfg->readBoolEntry ( "LogAll", true );
	mSendServer			= cfg->readNumEntry ( "SendServer", 0 );
	mWebAware			= cfg->readBoolEntry ( "WebAware", false );
	mHideIP				= cfg->readBoolEntry ( "HideIP", true );
	mAuth					= cfg->readBoolEntry( "Auth", false);
	mUsePorts			= cfg->readBoolEntry( "UsePorts", false );
	mMinPort				= cfg->readNumEntry( "MinPort", 1024);
	mMaxPort				= cfg->readNumEntry( "MaxPort", 65535 );

	mRejURLMessages	= cfg->readBoolEntry( "Reject URL Messages", false );
	mRejWebMessages	= cfg->readBoolEntry( "Reject Web Messages", false );
	mRejEmailMessages = cfg->readBoolEntry( "Reject EmailExpress Messages", false );
	mRejNotOnList		= cfg->readBoolEntry( "Reject Users not on Contactlist", false );


	mAwayMessage		= cfg->readEntry ( "AwayMessage", "I'm currently away from my computer. Please leave a message for me when I return to my computer." );
	mNaMessage			= cfg->readEntry ( "NAMessage", "I'm currently away. Please leave a message for me when I return.");
	mDndMessage			= cfg->readEntry ( "DNDMessage", "I'm currently busy and would like to not be disturbed. If this is urgent, please leave me a message for when I get back.");
	mOccMessage			= cfg->readEntry ( "OCCMessage", "I'm occupied" );
	mFfcMessage			= cfg->readEntry ( "FFCMessage", "I'd love to hear what you have to say. Join my chat." );

	/* ====================================================================== */
	/* fill gui widgets ===== */

	if ( mUIN != "0" )
		loginTab->mUIN->setText ( mUIN );
	if ( mPasswd != "" )
		loginTab->mPass->setText ( mPasswd );
	if ( mNickName != "" )
		loginTab->mNick->setText ( mNickName );

	connectionTab->mServer->setText ( cfg->readEntry ( "Server", "login.icq.com" ) );
	connectionTab->mServerPort->setValue ( cfg->readNumEntry ( "Port", 5190 ) );

	connectionTab->mListenFromPort->setValue( cfg->readNumEntry ( "ListenFromPort", 1024 ) );
	connectionTab->mListenToPort->setValue( cfg->readNumEntry ( "ListenToPort", 65535 ) );

	connectionTab->mUseProxy->setChecked( mUseProxy );
	slotUseProxyToggled ( mUseProxy );
	connectionTab->mProxyType->setCurrentItem ( mProxyType-1 ); // TODO: check value read from config
	connectionTab->mProxyServer->setText ( mProxyServerAdress );
	connectionTab->mProxyPort->setValue ( mProxyPort );
	connectionTab->mProxyAuth->setChecked( mProxyAuth );
	slotProxyAuthToggled ( mProxyAuth );
	connectionTab->mProxyAuthUser->setText ( mProxyAuthUser );
	connectionTab->mProxyAuthPassword->setText ( mProxyAuthPassword );

	connectionTab->mAutoConnect->setChecked( cfg->readBoolEntry ( "AutoConnect", false ) );
	slotAutoConnectToggled( connectionTab->mAutoConnect->isChecked() );
	switch ( cfg->readNumEntry ( "AutoConnectStatus", 0 ) )
	{
		case ICQ_STATUS_ONLINE:
			connectionTab->mAutoConnectStatus->setCurrentItem( 0 );
			break;
		case ICQ_STATUS_AWAY:
			connectionTab->mAutoConnectStatus->setCurrentItem( 1 );
			break;
		case ICQ_STATUS_NA:
			connectionTab->mAutoConnectStatus->setCurrentItem( 2 );
			break;
		case ICQ_STATUS_DND:
			connectionTab->mAutoConnectStatus->setCurrentItem( 3 );
			break;
		case ICQ_STATUS_OCCUPIED:
			connectionTab->mAutoConnectStatus->setCurrentItem( 4 );
			break;
	}

	settingsTab->chkLogAll->setChecked ( mLogAll );
	settingsTab->sendByServer->setCurrentItem( mSendServer );
	settingsTab->chkHideIP->setChecked ( mHideIP );
	settingsTab->chkWebAware->setChecked ( mWebAware );
	settingsTab->chkAuth->setChecked( mAuth );
	
//	settingsTab->chkRejURLMessages->setChecked( mRejURLMessages );
	settingsTab->chkRejWebMessages->setChecked( mRejWebMessages );
	settingsTab->chkRejEmailMessages->setChecked( mRejEmailMessages );
	settingsTab->chkRejNotOnList->setChecked( mRejNotOnList );

	settingsTab->chkPorts->setChecked( mUsePorts );
	if (!mUsePorts)
	{
		settingsTab->minPort->setEnabled(false);
		settingsTab->maxPort->setEnabled(false);
	}
	settingsTab->minPort->setValue(mMinPort);
	settingsTab->maxPort->setValue(mMaxPort);

	mChanges = false;
}


void ICQPreferences::save()
{
	mUIN				= loginTab->mUIN->text();
	mPasswd				= loginTab->mPass->text();
	mNickName			= loginTab->mNick->text();

	mUseProxy			= connectionTab->mUseProxy->isChecked();
	mProxyType			= connectionTab->mProxyType->currentItem()+1; // TODO: check value
	mProxyServerAdress	= connectionTab->mProxyServer->text();
	mProxyPort			= connectionTab->mProxyPort->value();
	mProxyAuth			= connectionTab->mProxyAuth->isChecked();
	mProxyAuthUser		= connectionTab->mProxyAuthUser->text();
	mProxyAuthPassword	= connectionTab->mProxyAuthPassword->text();

	mSendServer			= settingsTab->sendByServer->currentItem();
	mLogAll				= settingsTab->chkLogAll->isChecked();
	mHideIP				= settingsTab->chkHideIP->isChecked();
	mWebAware			= settingsTab->chkWebAware->isChecked();
	mAuth 				= settingsTab->chkAuth->isChecked();
	mUsePorts			= settingsTab->chkPorts->isChecked();
	mMinPort				= settingsTab->minPort->value();
	mMaxPort				= settingsTab->maxPort->value();
	
//	mRejURLMessages	= settingsTab->chkRejURLMessages->isChecked();
	mRejWebMessages	= settingsTab->chkRejWebMessages->isChecked();
	mRejEmailMessages	= settingsTab->chkRejEmailMessages->isChecked();
	mRejNotOnList		= settingsTab->chkRejNotOnList->isChecked();

	cfg->setGroup ( "ICQ" );
	cfg->writeEntry ( "UIN", mUIN );
	cfg->writeEntry ( "Password", mPasswd );
	cfg->writeEntry ( "Nick", mNickName );

	cfg->writeEntry ( "Server", connectionTab->mServer->text() );
	cfg->writeEntry ( "Port", connectionTab->mServerPort->value() );

	cfg->writeEntry ( "ListenFromPort", connectionTab->mListenFromPort->value() );
	cfg->writeEntry ( "ListenToPort", connectionTab->mListenToPort->value() );

	cfg->writeEntry ( "UseProxy", mUseProxy );
	cfg->writeEntry ( "ProxyType", mProxyType );
	cfg->writeEntry ( "ProxyServer", mProxyServerAdress );
	cfg->writeEntry ( "ProxyPort", mProxyPort );
	cfg->writeEntry ( "ProxyAuth", mProxyAuth );
	cfg->writeEntry ( "ProxyAuthUser", mProxyAuthUser );
	cfg->writeEntry ( "ProxyAuthPassword", mProxyAuthPassword );
	cfg->writeEntry( "UsePorts", mUsePorts );
	cfg->writeEntry( "MinPort", mMinPort );
	cfg->writeEntry( "MaxPort", mMaxPort );

	cfg->writeEntry("AutoConnect", connectionTab->mAutoConnect->isChecked() );

	switch ( connectionTab->mAutoConnectStatus->currentItem() )
	{
		case 0:
			cfg->writeEntry("AutoConnectStatus", ICQ_STATUS_ONLINE );
			break;
		case 1:
			cfg->writeEntry("AutoConnectStatus", ICQ_STATUS_AWAY );
			break;
		case 2:
			cfg->writeEntry("AutoConnectStatus", ICQ_STATUS_NA );
			break;
		case 3:
			cfg->writeEntry("AutoConnectStatus", ICQ_STATUS_DND );
			break;
		case 4:
			cfg->writeEntry("AutoConnectStatus", ICQ_STATUS_OCCUPIED );
			break;
	}

	cfg->writeEntry ( "LogAll", mLogAll );
	cfg->writeEntry ( "SendServer", mSendServer );
	cfg->writeEntry ( "WebAware", mWebAware );
	cfg->writeEntry ( "HideIP", mHideIP );
	cfg->writeEntry( "Auth", mAuth);

	cfg->writeEntry ( "AwayMessage", mAwayMessage );
	cfg->writeEntry ( "NAMessage", mNaMessage );
	cfg->writeEntry ( "DNDMessage", mDndMessage );
	cfg->writeEntry ( "OCCMessage", mOccMessage );
	cfg->writeEntry ( "FFCMessage", mFfcMessage );
	
	cfg->writeEntry ( "Reject URL Messages", mRejURLMessages );
	cfg->writeEntry ( "Reject Web Messages", mRejWebMessages );
	cfg->writeEntry ( "Reject EmailExpress Messages", mRejEmailMessages );
	cfg->writeEntry ( "Reject Users not on Contactlist", mRejNotOnList );
	
	cfg->sync();
 
	emit saved();
	
	if ( mChanges == true && protocol->isConnected() )
		KMessageBox::information(0L, i18n("The changes you did become operative after you relogin with ICQ."), i18n("ICQ Changes During Online ICQ Session"),"icqChangesDuringOnline " );
	
	mChanges = false;
}

void ICQPreferences::setAwayMessage( QString msg )
{
	mAwayMessage = msg;
	cfg->writeEntry ( "AwayMessage", mAwayMessage );
	cfg->sync();
	emit saved();
}

void ICQPreferences::setNaMessage( QString msg )
{
	mNaMessage = msg;
	cfg->writeEntry ( "NAMessage", mNaMessage );
	cfg->sync();
	emit saved();
}

void ICQPreferences::setDndMessage( QString msg )
{
	mDndMessage = msg;
	cfg->writeEntry ( "DNDMessage", mDndMessage );
	cfg->sync();
	emit saved();
}

void ICQPreferences::setOccMessage( QString msg )
{
	mOccMessage = msg;
	cfg->writeEntry ( "OCCMessage", mOccMessage );
	cfg->sync();
	emit saved();
}

void ICQPreferences::setFfcMessage( QString msg )
{
	mFfcMessage = msg;
	cfg->writeEntry ( "FFCMessage", mFfcMessage );
	cfg->sync();
	emit saved();
}

void ICQPreferences::slotAutoConnectToggled( bool s )
{
	connectionTab->mAutoConnectStatus->setEnabled ( s );
}

void ICQPreferences::slotUseProxyToggled( bool s )
{
	connectionTab->mProxyType->setEnabled( s );
	connectionTab->mProxyServer->setEnabled( s );
	connectionTab->mProxyPort->setEnabled( s );
	connectionTab->mProxyAuth->setEnabled( s );
	if ( s ) 
		slotProxyAuthToggled ( connectionTab->mProxyAuth->isChecked() );
	else
		slotProxyAuthToggled ( false );
}

void ICQPreferences::slotProxyAuthToggled ( bool s )
{
	connectionTab->mProxyAuthUser->setEnabled( s );
	connectionTab->mProxyAuthPassword->setEnabled( s );
}


void ICQPreferences::slotChangeNeedsReconnect ( void )
{
	kdDebug(14110) << "[ICQPreferences] slotChangeNeedsReconnect()" << endl;
	mChanges = true;
}

void ICQPreferences::slotUseCustomPorts(bool s)
{
	settingsTab->lblMinPort->setEnabled(s);
	settingsTab->lblMaxPort->setEnabled(s);
	settingsTab->minPort->setEnabled(s);
	settingsTab->maxPort->setEnabled(s);
}

QString ICQPreferences::uin() const { return mUIN; }

QString ICQPreferences::nickName() const { return mNickName; }

QString ICQPreferences::passwd() const { return mPasswd; }

QString ICQPreferences::serverAdress() const { return connectionTab->mServer->text(); }

uint ICQPreferences::serverPort() const { return connectionTab->mServerPort->value(); }

uint ICQPreferences::listenFromPort() const { return connectionTab->mListenFromPort->value(); }

uint ICQPreferences::listenToPort() const { return connectionTab->mListenToPort->value(); }

bool ICQPreferences::logAll() const { return mLogAll; }

int ICQPreferences::sendServer() const { return mSendServer; }

bool ICQPreferences::notifyMessages() const { return mNotify; }

bool ICQPreferences::webAware() const { return mWebAware; }

bool ICQPreferences::hideIP() const { return mHideIP; }

bool ICQPreferences::auth() const { return mAuth; }

bool ICQPreferences::useProxy() const { return mUseProxy; }

uint ICQPreferences::proxyType() const { return mProxyType; }

QString ICQPreferences::proxyServerAdress() const { return mProxyServerAdress; }

uint ICQPreferences::proxyPort() const { return mProxyPort; }

bool ICQPreferences::proxyAuth() const { return mProxyAuth; }

QString ICQPreferences::proxyAuthUser() const { return mProxyAuthUser; }

QString ICQPreferences::proxyAuthPassword() const { return mProxyAuthPassword; }

QString ICQPreferences::awayMessage() const { return mAwayMessage; }

QString ICQPreferences::naMessage() const { return mNaMessage; }

QString ICQPreferences::dndMessage() const { return mDndMessage; }

QString ICQPreferences::occMessage() const { return mOccMessage; }

QString ICQPreferences::ffcMessage() const { return mFfcMessage; }

bool ICQPreferences::autoConnect() const { return connectionTab->mAutoConnect->isChecked(); }

Q_UINT16 ICQPreferences::autoConnectStatus() const { return connectionTab->mAutoConnectStatus->currentItem(); }

bool ICQPreferences::useCustomPorts() const { return mUsePorts; }

uint ICQPreferences::customMinPort() const { return mMinPort; }

uint ICQPreferences::customMaxPort() const { return mMaxPort; }

bool ICQPreferences::rejURLMessages() const { return mRejURLMessages; }

bool ICQPreferences::rejWebMessages() const { return mRejWebMessages; }

bool ICQPreferences::rejEmailMessages() const { return mRejEmailMessages; }

bool ICQPreferences::rejNotOnList() const { return mRejNotOnList; }
