 /*
    icqpreferences.h  -  ICQ Protocol Plugin

    Copyright (c) 2002 by Duncan Mac-Vicar Prett <duncan@kde.org>
    
    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#ifndef ICQPREFERENCES_H
#define ICQPREFERENCES_H

#include <configmodule.h>

class loginTabWidget;
class connectionTabWidget;
class settingsTabWidget;
class QTabWidget;
class KConfig;
class QComboBox;
class ICQProtocol;

/**
  *@author duncan
  */

class ICQPreferences : public ConfigModule
{
	Q_OBJECT
	public:
		ICQPreferences(const QString &pixmap,ICQProtocol *parent);
		~ICQPreferences();

		virtual void save();

		// these are to get config values
		// from outside of this class
		QString uin() const;
		QString nickName() const;
		QString passwd() const;
		QString serverAdress() const;
		uint serverPort() const;
		uint listenFromPort() const;
		uint listenToPort() const;
		bool logAll() const;
		int sendServer() const;
		bool notifyMessages() const;
		bool webAware() const;
		bool hideIP() const;
		bool auth() const;
		bool useProxy() const;
		uint proxyType() const;
		QString proxyServerAdress() const;
		uint proxyPort() const;
		bool proxyAuth() const;
		QString proxyAuthUser() const;
		QString proxyAuthPassword() const;
		QString awayMessage() const;
		QString naMessage() const;
		QString dndMessage() const;
		QString occMessage() const;
		QString ffcMessage() const;
		bool autoConnect() const;
		Q_UINT16 autoConnectStatus() const;
		bool useCustomPorts() const;
		uint customMinPort() const;
		uint customMaxPort() const;
		
		// settings to reject certain message types
		bool rejURLMessages() const;
		bool rejWebMessages() const;
		bool rejEmailMessages() const;
		bool rejNotOnList() const;

		void setAwayMessage( QString msg );
		void setNaMessage( QString msg );
		void setDndMessage( QString msg );
		void setOccMessage( QString msg );
		void setFfcMessage( QString msg );

	signals:
		void saved();

	public slots:
		void slotAutoConnectToggled( bool );
		void slotUseProxyToggled( bool );
		void slotProxyAuthToggled ( bool s );
		void slotChangeNeedsReconnect ( void );
		void slotUseCustomPorts(bool);
		virtual void reopen();

	private:
		QTabWidget *icqTabWidget;
		loginTabWidget *loginTab;
		connectionTabWidget *connectionTab;
		settingsTabWidget *settingsTab;
		KConfig *cfg;
		ICQProtocol *protocol;

		// all the config-settings
		QString mUIN;
		QString mNickName;
		QString mPasswd;
		
		// Proxy-Settings
		bool mUseProxy;
		uint mProxyType;
		QString mProxyServerAdress;
		uint mProxyPort;
		bool mProxyAuth;
		QString mProxyAuthUser;
		QString mProxyAuthPassword;

		QString mAwayMessage;
		QString mNaMessage;
		QString mDndMessage;
		QString mOccMessage;
		QString mFfcMessage;

		bool mLogAll;
		int mSendServer;
		bool mNotify;
		bool mWebAware;
		bool mHideIP;
		bool mAuth;
		bool mUsePorts;
		uint mMinPort;
		uint mMaxPort;

		bool mRejURLMessages;
		bool mRejWebMessages;
		bool mRejEmailMessages;
		bool mRejNotOnList;

		bool mChanges; // Tracking var to see if user changed something
};
#endif
