/*
	icqprotocol.h  -  ICQ Protocol Plugin

	Copyright (c) 2002 by Nick Betcher <nbetcher@kde.org>
	Copyright (c) 2002 by Vladimir Shutoff <shutoff@mail.ru>

	Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

	*************************************************************************
	*                                                                       *
	* This program is free software; you can redistribute it and/or modify  *
	* it under the terms of the GNU General Public License as published by  *
	* the Free Software Foundation; either version 2 of the License, or     *
	* (at your option) any later version.                                   *
	*                                                                       *
	*************************************************************************
	*/

#ifndef ICQPROTOCOL_H
#define ICQPROTOCOL_H

#include "kopeteprotocol.h"

class KAction;
class KActionMenu;
class KToggleAction;

class KopeteContact;
class KopeteFileTransferInfo;
class KopeteMetaContact;
class KopeteTransfer;

class ICQContact;
class ICQEvent;
class ICQFile;
class ICQGroup;
class ICQMessage;
class ICQPreferences;
class ICQChangeStatus;
class KICQClient;

/**
 * @author nbetcher
 */
class ICQProtocol : public KopeteProtocol
{
		Q_OBJECT

		public:
			ICQProtocol( QObject *parent, const char *name, const QStringList &args );
		~ICQProtocol();

		KICQClient *engine() { return mEngine; };
		ICQPreferences * prefs() { return mPrefs; };

		/* Plugin reimplementation */
		virtual void init();
		virtual bool unload();
		virtual void deserializeContact( KopeteMetaContact *metaContact, const QMap<QString, QString> &serializedData,
						const QMap<QString, QString> &addressBookData );

		/** KopeteProtocol reimplementation */
		virtual AddContactPage *createAddContactWidget(QWidget *parent);
		virtual bool isConnected() const;
		virtual void setAway(void);
		virtual void setAvailable(void);
		virtual bool isAway(void) const;

		bool addContactToMetaContact( const QString &contactId, const QString &displayName, KopeteMetaContact *parentContact );

		ICQPreferences *prefs() const { return mPrefs; };
		QString nickName() const { return mName; };
		QString statusToString ( unsigned int status ) const;

		void addPendingMessage(ICQMessage *msg, ICQContact *c) { pendingMessages.insert(msg, c); };

		ICQGroup *findGroup(const QString &groupName, bool create = false);

		virtual KopeteContact *myself() const;
		virtual const QString protocolIcon();

public slots:		
		virtual void connect();
		virtual void disconnect();

signals:
		void settingsChanged();
		void searchEvent( ICQEvent *);

private slots:
		/** Internal Functions */
		void slotGoOffline();
		void slotGoOnline();
		void slotSetAway(QString type);
 		void slotGoAway();
		void slotGoNA();
		void slotGoInvisible();
		void slotGoDND();
		void slotGoOccupied();
		void slotGroupAdded(const QString &);
		void slotUpdatedSettings(void);
		void slotEvent(ICQEvent *e);
		void slotFileAccepted(KopeteTransfer *, const QString& );
		void slotFileDeclined(const KopeteFileTransferInfo &);
		void slotFileExist(ICQFile*f, const QString &file, bool canResume);
		void slotEditOwnInfo(void);

private:
		/**
		 * Fill all KActions, used in constructor
		 */
		void initActions();
		void statusChange();
		/**
		 * Called when we got the serverside contactlist
		 * (happens when connecting to the server)
		 */
		void receivedContactlist( void );
		/**
		 * import the contact list from kopete 0.4.x
		 */
		void importOldContactList();

		KActionMenu *protocolActions(void);

		KActionMenu *protocolActionMenu;

		// our object to access libicq
		KICQClient *mEngine;

		// preferences window
		// also gives access to config-entries
		ICQPreferences *mPrefs;

		KAction* actionGoOnline;
		KAction* actionGoOffline;
		KAction* actionGoAway;
		KAction* actionGoNA;
		KToggleAction* actionGoInvisible;
		KAction* actionGoDND;
		KAction* actionGoOccupied;
		KAction *actionEditOwnInfo;

		KActionMenu *actionStatusMenu;

		// our own nickname
		QString mName;

		QMap< ICQMessage *, ICQContact *> pendingMessages;

		ICQContact *myContact;
		bool gotOwnInfo;

		KopeteTransfer *m_tmpFileTransfer;

		// Our changestatus object (dialog)
		ICQChangeStatus *mChangeStatus;
		
};

#endif

/*
 * Local variables:
 * mode: c++
 * c-indentation-style: k&r
 * c-basic-offset: 4
 * indent-tabs-mode: t
 * End:
 */

// vim: set noet ts=4 sts=4 tw=4:

