/*
    irc_channel_tabbar.cpp

    Copyright (c) 2002 by nbetcher               <nbetcher@usinternet.com>
    Copyright (c) 2002 by Andr Wbbeking        <Woebbeking@web.org>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/



#include "irc_channel_tabbar.h"

#include <qcolor.h>
#include <qpainter.h>
#include <qpalette.h>

#include <cassert>



IrcChannelTabBar::IrcChannelTabBar(QWidget* pParent)
    : QTabBar(pParent, "IrcChannelTabBar"),
      m_stDefaultTextColor(palette().color(QPalette::Active,
                                           QColorGroup::Foreground))
{
}



void
IrcChannelTabBar::paint(QPainter* pPainter,
                        QTab*     pTab,
                        bool      bSelected) const
{
    QPalette const stOldPalette(palette());

    QPalette stPalette(palette());
    stPalette.setColor(QColorGroup::Foreground, textColor(pTab));

    // as setPalette() calls update() you have to disable updates
    // to avoid endless recursion

    IrcChannelTabBar* that(const_cast<IrcChannelTabBar*>(this));

    bool const bUpdatesEnabled(isUpdatesEnabled());

    that->setUpdatesEnabled(false);
    that->setPalette(stPalette);

    QTabBar::paint(pPainter, pTab, bSelected);

    that->setPalette(stOldPalette);
    that->setUpdatesEnabled(bUpdatesEnabled);
}



QColor const
IrcChannelTabBar::textColor(QTab const* pTab) const
{
    QColor stColor(m_stDefaultTextColor);

    std::map<QTab const*, QColor>::const_iterator it = m_mapTabToTextColor.find(pTab);
    if (it != m_mapTabToTextColor.end())
    {
        stColor = it->second;
    }

    return stColor;
}



void
IrcChannelTabBar::setTextColor(QTab const*   pTab,
                               QColor const& rColor)
{
    if (pTab != 0)
    {
        // is it one of our tabs?
        assert(tabList()->find(pTab) >= 0);

        if (rColor != textColor(pTab))
        {
            m_mapTabToTextColor[pTab] = rColor;
            update();
        }
    }
}



QColor const
IrcChannelTabBar::defaultTextColor() const
{
    return m_stDefaultTextColor;
}



void
IrcChannelTabBar::setDefaultTextColor(QColor const& rColor)
{
    if (rColor != m_stDefaultTextColor)
    {
        m_stDefaultTextColor = rColor;
        update();
    }
}



void
IrcChannelTabBar::removeTab(QTab* pTab)
{
    m_mapTabToTextColor.erase(pTab);

    QTabBar::removeTab(pTab);
}
