/*
    irc_channel_tabbar.h

    Copyright (c) 2002 by nbetcher               <nbetcher@usinternet.com>
    Copyright (c) 2002 by Andr Wbbeking        <Woebbeking@web.org>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/



#ifndef IRC_CHANNEL_TABBAR_H
#define IRC_CHANNEL_TABBAR_H



#include <qtabbar.h>

#include <map>



class QPainter;



/**
 * This class extends QTabBar to paint the tab labels in any color.
 */
class IrcChannelTabBar : public QTabBar
{
public:

    /**
     * Constructor.
     */
    IrcChannelTabBar(QWidget* pParent);

    /**
     * Returns the text color for @p pTab.
     */
    QColor const textColor(QTab const* pTab) const;

    /**
     * Sets the text color for @p pTab to @p rColor.
     */
    void setTextColor(QTab const* pTab, QColor const& rColor);

    /**
     * Returns the default text color. This color is used if no text
     * color is set for a QTab. The start value is
     * palette().color(QPalette::Active, QColorGroup::Foreground).
     */
    QColor const defaultTextColor() const;

    /**
     * Sets the default text color to @p rColor.
     *
     * @see defaultTextColor()
     */
    void setDefaultTextColor(QColor const& rColor);

    /**
     * @internal
     */
    virtual void removeTab(QTab*);

protected:

    /**
     * @internal
     */
    virtual void paint(QPainter*, QTab*, bool bSelected) const;

private:

    //! this color is used if no color is set for a tab
    QColor m_stDefaultTextColor;

    //! text colors of the tabs (tab to color)
    std::map<QTab const*, QColor> m_mapTabToTextColor;
};



#endif
