/*
    irc_channel_tabwidget.cpp

    Copyright (c) 2002 by nbetcher               <nbetcher@usinternet.com>
    Copyright (c) 2002 by Andr Wbbeking        <Woebbeking@web.org>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include "irc_channel_tabwidget.h"
#include "irc_channel_tabbar.h"
#include <cassert>
#include "irc_view_base.h"

IrcChannelTabWidget::IrcChannelTabWidget(QWidget* pParent)
    : QTabWidget(pParent, "IrcChannelTabWidget"),
      m_stNormalColor(0, 0, 0),
      m_stActiveColor(32, 0, 255),
      m_stHighlightedColor(255, 0, 0)
{
    m_pTabBar = new IrcChannelTabBar(this);
    m_pTabBar->setDefaultTextColor(m_stNormalColor);
    setTabBar(m_pTabBar);

    setFocusPolicy(QWidget::NoFocus);

    connect(this, SIGNAL(currentChanged(QWidget*)),
            this, SLOT(slotCurrentPageChanged(QWidget*)));
}



IrcChannelTabWidget::ETabMode
IrcChannelTabWidget::tabMode(QWidget* pChild) const
{
    ETabMode eMode(eTM_Normal);
    if (QTab const* pTab = findTab(pChild))
    {
        QColor const& rColor = m_pTabBar->textColor(pTab);
        if (rColor == m_stHighlightedColor)
        {
            eMode = eTM_Highlighted;
        }
        else if (rColor == m_stActiveColor)
        {
            eMode = eTM_Active;
        }
        else
        {
            assert(rColor == m_stNormalColor);
        }
    }

    return eMode;
}



void
IrcChannelTabWidget::setTabMode(QWidget* pChild,
                                ETabMode eMode)
{
    if (QTab const* pTab = findTab(pChild))
    {
        // it is not possible to override Highlighted with Active mode
        if (eMode == eTM_Active && tabMode(pChild) == eTM_Highlighted)
        {
            return;
        }

        QColor stColor;
        switch (eMode)
        {
        case eTM_Normal:
            stColor = m_stNormalColor;
            break;

        case eTM_Active:
            stColor = m_stActiveColor;
            break;

        case eTM_Highlighted:
            stColor = m_stHighlightedColor;
            break;
        }

        m_pTabBar->setTextColor(pTab, stColor);
    }
}



// ------------------------------------------------------------------------------
// public slots
// ------------------------------------------------------------------------------



void
IrcChannelTabWidget::slotNextPage()
{
    if (count() > 0)
    {
        int iIndex(currentPageIndex() + 1);
        if (iIndex == count())
        {
            iIndex = 0;
        }

        setCurrentPage(iIndex);
    }
}



void
IrcChannelTabWidget::slotPrevPage()
{
    if (count() > 0)
    {
        int iIndex(currentPageIndex() - 1);
        if (iIndex < 0)
        {
            iIndex = count() - 1;
        }

        setCurrentPage(iIndex);
    }
}



// ------------------------------------------------------------------------------
// private slots
// ------------------------------------------------------------------------------



void
IrcChannelTabWidget::slotCurrentPageChanged(QWidget* pChild)
{
    if (pChild != 0)
    {
        setTabMode(pChild, eTM_Normal);
    }
}



// ------------------------------------------------------------------------------
// private methods
// ------------------------------------------------------------------------------



QTab const*
IrcChannelTabWidget::findTab(QWidget* pChild) const
{
    QTab const* pTab(m_pTabBar->tabAt(indexOf(pChild)));

    return pTab;
}



#include "irc_channel_tabwidget.moc"
