/*
    irc_channel_tabwidget.h

    Copyright (c) 2002 by nbetcher               <nbetcher@usinternet.com>
    Copyright (c) 2002 by Andr Wbbeking        <Woebbeking@web.org>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/



#ifndef IRC_CHANNEL_TABWIDGET_H
#define IRC_CHANNEL_TABWIDGET_H



#include <qtabwidget.h>



class IrcChannelTabBar;



/**
 * This class extends QTabWidget.
 */
class IrcChannelTabWidget : public QTabWidget
{
    Q_OBJECT

public:

    enum ETabMode
    {
        eTM_Normal,
        eTM_Active,
        eTM_Highlighted
    };

    /**
     * Constructor.
     */
    IrcChannelTabWidget(QWidget* pParent);

    /**
     * Returns the tab mode for page @p pChild.
     */
    ETabMode tabMode(QWidget* pChild) const;

    /**
     * Sets the tab mode for page @p pChild to @p eMode.
     */
    void setTabMode(QWidget* pChild, ETabMode eMode);

public slots:

    /**
     * Sets the next page as current page.
     */
    void slotNextPage();

    /**
     * Sets the previous page as current page.
     */
    void slotPrevPage();

private slots:

    /**
     * Sets the tab mode of the new current page @p pChild to eTM_Normal.
     */
    void slotCurrentPageChanged(QWidget* pChild);

private:

    /**
     * Returns the QTab for @p pChild.
     */
    QTab const* findTab(QWidget* pChild) const;

    QColor m_stNormalColor;
    QColor m_stActiveColor;
    QColor m_stHighlightedColor;

    IrcChannelTabBar* m_pTabBar;
};



#endif

