/*
    irc_view_base.cpp

    Copyright (c) 2002 by Andr Wbbeking        <Woebbeking@web.org>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include "irc_view_base.h"
#include "tabcompleter.h"
#include <qscrollbar.h>
#include <cassert>
#include <kdebug.h>

IrcViewBase::IrcViewBase(QWidget* pParent)
	: QWidget(pParent, "IrcViewBase"),
	m_pMessageView(0),
	m_pMessageEdit(0)
{

}

CustomTextEdit* IrcViewBase::messageView() const
{
	return m_pMessageView;
}



TabCompleter* IrcViewBase::messageEdit() const
{
	return m_pMessageEdit;
}

// ------------------------------------------------------------------------------
// protected methods
// ------------------------------------------------------------------------------

QTextEdit* IrcViewBase::createMessageView(QWidget* pParent)
{
	// this methode must not be called more than once
	assert(m_pMessageView == 0);

	m_pMessageView = new CustomTextEdit(pParent, "ChatView");
	m_pMessageView->setFocusPolicy(QWidget::NoFocus);
	m_pMessageView->setReadOnly(true);
	m_pMessageView->setTextFormat(Qt::RichText);
#if QT_VERSION >= 0x030100
	// Qt 3.1 or greater
	// AtWorkOrDocumentBoundry is awsome, makes it so long ass words or URLS don't go off of the page and give you a vertical scroll bar
	m_pMessageView->setWrapPolicy(QTextEdit::AtWordOrDocumentBoundary);
#else
	// Aww, no fun...
	m_pMessageView->setWrapPolicy(QTextEdit::Anywhere);
#endif
	return m_pMessageView;
}



TabCompleter* IrcViewBase::createMessageEdit(QWidget* pParent)
{
	// this methode must not be called more than once
	assert(m_pMessageEdit == 0);
	m_pMessageEdit = new TabCompleter(pParent, "MessageEdit");
	m_pMessageEdit->setMaxLength(500);

	return m_pMessageEdit;
}



void IrcViewBase::keyPressEvent(QKeyEvent* pEvent)
{
    // test if createChatView() was called
    assert(m_pMessageView != 0);

    if ((pEvent->state() & Qt::KeyButtonMask) == Qt::ShiftButton)
    {
        if (pEvent->key() == Qt::Key_PageDown)
        {
            m_pMessageView->verticalScrollBar()->addPage();
            pEvent->accept();
        }
        else if (pEvent->key() == Qt::Key_PageUp)
        {
            m_pMessageView->verticalScrollBar()->subtractPage();
            pEvent->accept();
        }
        else if (pEvent->key() == Qt::Key_Down)
        {
            m_pMessageView->verticalScrollBar()->addLine();
            pEvent->accept();
        }
        else if (pEvent->key() == Qt::Key_Up)
        {
            m_pMessageView->verticalScrollBar()->subtractLine();
            pEvent->accept();
        }
    }
}

CustomTextEdit::CustomTextEdit(QWidget *parent, const char *name)
	: KTextBrowser(parent, name)
{
	wasAtBottom = true;
}

void CustomTextEdit::append(const QString &text)
{
	// Make the maximum number of lines (paragraphs in this case) configurable
	if (paragraphs() >= 700)
	{
		// This fixes a bug in Qt that just sets the background black instead of removing the paragraph
		wasAtBottom = contentsY() >= (contentsHeight() - visibleHeight());
		setUpdatesEnabled(false);
		setReadOnly(false);
		removeParagraph(0);
		setReadOnly(true);
		if (wasAtBottom)
			scrollToBottom();
		setUpdatesEnabled(true);
	}
	KTextBrowser::append(text);
}

void CustomTextEdit::show()
{
	wasAtBottom = contentsY() >= (contentsHeight() - visibleHeight());
	KTextBrowser::show();
	if (wasAtBottom)
		scrollToBottom();
}

