/*
    irc_view_base.h

    Copyright (c) 2002 by Andr Wbbeking        <Woebbeking@web.org>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#ifndef IRC_VIEW_BASE_H
#define IRC_VIEW_BASE_H

#include <qwidget.h>
#include <ktextbrowser.h>

class TabCompleter;
class CustomTextEdit;

/**
 * This is the base class for all IRC views. It manages the message
 * view and edit and some key events for scrolling.
 *
 * The derived class have to create the layout.
 */
class IrcViewBase : public QWidget
{ 
public:

    /**
     * Constructor.
     */
    IrcViewBase(QWidget* pParent);

    /**
     * Returns the message view.
     */
    CustomTextEdit* messageView() const;

    /**
     * Returns the message edit.
     */
    TabCompleter* messageEdit() const;

protected:

    /**
     * Creates the message view with @p pParent as parent widget.
     *
     * Call this method only one time in a derived class.
     */
    QTextEdit* createMessageView(QWidget* pParent);

    /**
     * Creates the message edit with @p pParent as parent widget.
     *
     * Call this method only one time in a derived class.
     */
    TabCompleter* createMessageEdit(QWidget* pParent);

    /**
     * Processes SHIFT + Key_(Page)?(Up|Down) to scroll the message view.
     */
    virtual void keyPressEvent(QKeyEvent*);

private:

    //! the message view
    CustomTextEdit* m_pMessageView;

    //! the message edit
    TabCompleter* m_pMessageEdit;
};

class CustomTextEdit : public KTextBrowser
{
public:
	CustomTextEdit(QWidget * = 0, const char * = 0);
public slots:
	virtual void append(const QString &text);
	virtual void show();
private:
	bool wasAtBottom;
};

#endif // IRC_VIEW_BASE_H
