/***************************************************************************
                          ircchatview.cpp  -  description
                             -------------------
    begin                : Sun Feb 24 2002
    copyright            : (C) 2002 by nbetcher
    email                : nbetcher@usinternet.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#include "ircchatview.h"

#include "irc_channel_tabwidget.h"
#include "ircchatwindow.h"
#include "irccmdparser.h"
#include "irccontact.h"
#include "ircmessage.h"
#include "ircservercontact.h"
#include "kirc.h"
#include "kopetecontactlist.h"
#include "kopetemetacontact.h"
#include "messagetransport.h"
#include "tabcompleter.h"
#include "kopeteprotocol.h"

#include <kconfig.h>
#include <kglobal.h>
#include <klocale.h>
#include <kpopupmenu.h>
#include <kstandarddirs.h>
#include <kdebug.h>

#include <qlabel.h>
#include <qlayout.h>
#include <qlistbox.h>
#include <qsplitter.h>
#include <qstylesheet.h>
#include <qtextedit.h>
#include <qtoolbutton.h>
#include <qtooltip.h>
#include <qvaluelist.h>
#include <qvbox.h>



IRCChatView::IRCChatView(const QString &server, const QString &channel, IRCContact *irccontact, QWidget *parent)
	: IrcViewBase(parent)
{
	// create layout

	QVBoxLayout* pVBoxLayout(new QVBoxLayout(this, 2, 2, "MainLayout"));

	QHBoxLayout* pHBoxLayout(new QHBoxLayout(0, 0, 6, "Layout"));

	pHBoxLayout->addWidget(new QLabel(i18n("Topic:"), this));

	topicField = new QLineEdit(this, "topicField");
	pHBoxLayout->addWidget(topicField);

	QToolButton* closeTab = new QToolButton(this, "closeTab");
	closeTab->setText("X");
	pHBoxLayout->addWidget(closeTab);

	pVBoxLayout->addLayout(pHBoxLayout);

	mainSplitter = new QSplitter(QSplitter::Horizontal, this, "mainSplitter");

	createMessageView(mainSplitter);

	namesList = new QListBox(mainSplitter, "namesList");
	namesList->setFocusPolicy(QWidget::NoFocus);

	pVBoxLayout->addWidget(mainSplitter);

	createMessageEdit(this);
	pVBoxLayout->addWidget(messageEdit());

	// ----------------------------------------------------------------------

	m_channel = channel;

	m_ircContact = irccontact;
	setCaption(i18n("Chatting in channel %1").arg(m_channel));
	firstJoin = true;
	tryingPart = false;

	QObject::connect(m_ircContact->serverContact()->chatWindow()->mTabWidget, SIGNAL(currentChanged(QWidget *)), this, SLOT(slotPageChanged(QWidget *)));
	QObject::connect(m_ircContact->engine(), SIGNAL(userJoinedChannel(const QString &, const QString &)), this, SLOT(userJoinedChannel(const QString &, const QString &)));
	QObject::connect(m_ircContact->engine(), SIGNAL(incomingMessage(const QString &, const QString &, const QString &)), this, SLOT(incomingMessage(const QString &, const QString &, const QString &)));
	QObject::connect(m_ircContact->engine(), SIGNAL(incomingPartedChannel(const QString &, const QString &, const QString &)), this, SLOT(userPartedChannel(const QString &, const QString &, const QString &)));
	QObject::connect(m_ircContact->engine(), SIGNAL(incomingNamesList(const QString &, const QString &, const int)), this, SLOT(incomingNamesList(const QString &, const QString &, const int)));
	QObject::connect(m_ircContact->engine(), SIGNAL(incomingAction(const QString &, const QString &, const QString &)), this, SLOT(incomingAction(const QString &, const QString &, const QString &)));
	QObject::connect(m_ircContact->engine(), SIGNAL(incomingQuitIRC(const QString &,  const QString &)), this, SLOT(userQuitIRC(const QString &, const QString &)));
	QObject::connect(m_ircContact->engine(), SIGNAL(incomingNickChange(const QString &,  const QString &)), this, SLOT(nickNameChanged(const QString &, const QString &)));
	QObject::connect(m_ircContact->engine(), SIGNAL(incomingTopicChange(const QString &, const QString &, const QString &)), this, SLOT(incomingNewTopic(const QString &, const QString &, const QString &)));
	QObject::connect(m_ircContact->engine(), SIGNAL(incomingExistingTopic(const QString &,  const QString &)), this, SLOT(receivedExistingTopic(const QString &, const QString &)));
	QObject::connect(m_ircContact->engine(), SIGNAL(incomingNoNickChan(const QString &)), this, SLOT(incomingNoNickChan(const QString &)));
	QObject::connect(m_ircContact->engine(), SIGNAL(successfullyChangedNick(const QString &, const QString &)), this, SLOT(nickNameChanged(const QString &, const QString &)));
	QObject::connect(m_ircContact->engine(), SIGNAL(incomingKick(const QString &, const QString &, const QString &, const QString &)), this, SLOT(incomingKick(const QString &, const QString &, const QString &, const QString &)));
	QObject::connect(m_ircContact->engine(), SIGNAL(incomingNotice(const QString &, const QString &)), this, SLOT(incomingNotice(const QString &, const QString &)));
	QObject::connect(closeTab, SIGNAL(clicked()), this, SLOT(slotCloseTab()));
	QObject::connect(namesList, SIGNAL(rightButtonPressed( QListBoxItem *, const QPoint &)), this, SLOT(slotContextMenu( QListBoxItem *, const QPoint &)));

	QString message = i18n("Attempting to join channel %1 on server %2")
		.arg(channel).arg(server);
	messageView()->setText(message);

	if (m_ircContact->engine()->isLoggedIn())
		m_ircContact->engine()->joinChannel(channel);

	connect(messageEdit(), SIGNAL(returnPressed()), SLOT(slotReturnPressed()));
	connect(topicField, SIGNAL(returnPressed()), SLOT(slotChangeTopic()));

	namesList->setMinimumWidth(0);
	QValueList<int> original = mainSplitter->sizes();
	QValueList<int> sizes;
	// TODO: Read this setting from a config file!
	int userListSize = (int)(original[1] * .15);
	int chatViewSize = (int)(original[0] * .85);
	sizes.append(chatViewSize);
	sizes.append(userListSize);
	mainSplitter->setSizes(sizes);
}

void IRCChatView::slotContextMenu( QListBoxItem *item, const QPoint &point)
{
	if (item != 0)
	{
		onContact = item->text();
		if (onContact[0] == '@' || onContact[0] == '+')
			onContact.remove(0, 1);

		popup = new KPopupMenu();
		popup->insertTitle(onContact);
		ctcpMenu = new KPopupMenu();
		ctcpMenu->insertItem(i18n("Ping"), this, SLOT(slotPingUser()));
		ctcpMenu->insertItem(i18n("Version"), this, SLOT(slotVersionUser()));
		dccMenu = new KPopupMenu();
		dccMenu->insertItem(i18n("Send File..."), this, SLOT(slotDccSend()));
		dccMenu->insertItem(i18n("Chat"), this, SLOT(slotDccChat()));
		popup->insertItem(QIconSet(locate("data", "kopete/pics/irc_querymsg.xpm")), i18n("Open Query"), this, SLOT(slotOpenQuery()));
		popup->insertItem(QIconSet(locate("data", "kopete/pics/irc_ctcp.xpm")), i18n("CTCP"), ctcpMenu);
		popup->insertItem(QIconSet(locate("data", "kopete/pics/irc_dcc.xpm")), i18n("DCC"), dccMenu);
		popup->popup(point);
	}
}

void IRCChatView::slotDccChat()
{
	if (!onContact.isEmpty())
		m_ircContact->serverContact()->parser()->parseCommand(messageView(), IRCCmdParser::Chat, "dcc", QString("chat %1").arg(onContact), m_channel);
}

void IRCChatView::slotDccSend()
{
	if (!onContact.isEmpty())
		m_ircContact->serverContact()->parser()->parseCommand(messageView(), IRCCmdParser::Chat, "dcc", QString("send %1").arg(onContact), m_channel);
}

void IRCChatView::slotPingUser()
{
	if (!onContact.isEmpty())
		m_ircContact->serverContact()->parser()->parseCommand(messageView(), IRCCmdParser::Chat, "ctcp", QString("%1 ping").arg(onContact), m_channel);
}

void IRCChatView::slotVersionUser()
{
	if (!onContact.isEmpty())
		m_ircContact->serverContact()->parser()->parseCommand(messageView(), IRCCmdParser::Chat, "ctcp", QString("%1 version").arg(onContact), m_channel);
}

void IRCChatView::slotOpenQuery()
{
	if (!onContact.isEmpty())
	{
		KopeteProtocol *protocol = m_ircContact->protocol();
		QString contactID=QString(onContact+"@"+m_ircContact->serverName()).lower();

		KopeteContact *c = m_ircContact->protocol()->contacts()[contactID];
		if (c)
		{
			kdDebug(14120) << k_funcinfo << "Contact already exists " << onContact <<endl;
		}
		else
		{
			KopeteMetaContact *m = new KopeteMetaContact();
			m->setTemporary(true);
			kdDebug(14120) << k_funcinfo << "add contact: " << onContact <<endl;
			m->addContact(new IRCContact(m_ircContact->serverName(), onContact, 0, true, m_ircContact->serverContact(), m, protocol ));
			KopeteContactList::contactList()->addMetaContact(m);
		}
	}
}

void IRCChatView::slotCloseTab()
{
	if (m_ircContact->waitingPart() == true)
		m_ircContact->unloading();
	else
		m_ircContact->slotPart();
}

void IRCChatView::slotPageChanged(QWidget *widget)
{
	if (m_ircContact->tabPage() != 0)
	{
		if (m_ircContact->tabPage() == widget)
			messageEdit()->setFocus();
	}
}

void IRCChatView::incomingNoNickChan(const QString &nickname)
{
	QVBox *parent = m_ircContact->tabPage();
	if (parent != 0)
		if (m_ircContact->serverContact()->chatWindow()->mTabWidget->label(m_ircContact->serverContact()->chatWindow()->mTabWidget->indexOf(parent)).lower() != m_channel.lower())
			return;
	else
		return;

	QString message = i18n("No such nickname or channel: %1").arg(nickname);
	m_ircContact->serverContact()->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), m_ircContact->serverContact()->engine()->nickName(), IRCMessage::ServerMsg, QString(m_channel + "-" + m_ircContact->serverContact()->serverName()), messageView()));
}


void IRCChatView::nickNameChanged(const QString &oldname, const QString &newname)
{
	// Update the tab completion stuff
	messageEdit()->remove(oldname);
	messageEdit()->add(newname);

	// Now the difficult part
	int index = 0;
	while (index != (namesList->numRows()))
	{
		QListBoxItem *it = namesList->item(index);
		if (it != 0)
		{
			QString specialChar = "";
			QString oneRemoved = it->text().lower();
			if (oneRemoved[0] == '+' || oneRemoved[0] == '@')
			{
				specialChar.append(oneRemoved[0]);
				oneRemoved = oneRemoved.remove(0, 1);
			}
			if (oneRemoved.lower() == oldname.lower())
			{
				QString specialNew = specialChar;
				specialNew.append(newname);
				namesList->changeItem(specialNew, index);
				QString message = i18n("%1 is now known as %2").arg(QStyleSheet::escape(oldname)).arg(QStyleSheet::escape(newname));
				m_ircContact->serverContact()->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), m_ircContact->serverContact()->engine()->nickName(), IRCMessage::NickChange, QString(m_channel + "-" + m_ircContact->serverContact()->serverName()), messageView()));
				return;
			}
		}
		index++;
	}
	if (newname.lower() == m_ircContact->engine()->nickName().lower())
	{
		QString message = i18n("Your nick has successfully been changed from <b>%1</b> to <b>%2</b>.").arg(QStyleSheet::escape(oldname)).arg(QStyleSheet::escape(newname));
		m_ircContact->serverContact()->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), m_ircContact->serverContact()->engine()->nickName(), IRCMessage::NickChange, QString(m_channel + "-" + m_ircContact->serverContact()->serverName()), messageView()));
	}
}

void IRCChatView::incomingNamesList(const QString &channel, const QString &nickname, const int userClass)
{
	if (channel.lower() != m_channel.lower())
		return;

	int index = 0;
	while (index != namesList->numRows())
	{
		QListBoxItem *it = namesList->item(index);
		if (it != 0)
		{
			QString oneRemoved = it->text().lower();
			if (oneRemoved[0] == '+' || oneRemoved[0] == '@')
				oneRemoved = oneRemoved.remove(0, 1);

			if (oneRemoved.lower() == nickname.lower())
				delete it;

		}
		if (index == namesList->numRows())
			break;

		index++;
	}
	QString newNickname = nickname;
	switch(userClass)
	{
		case KIRC::Operator :
		{
			newNickname.prepend('@');
			break;
		}
		case KIRC::Voiced :
		{
			newNickname.prepend('+');
			break;
		}
	}

	messageEdit()->add(newNickname);
	namesList->insertItem(newNickname);
	namesList->sort();
}

void IRCChatView::incomingNotice(const QString &originating, const QString &message)
{
	if (m_ircContact->serverContact()->chatWindow()->mTabWidget->currentPage() == m_ircContact->tabPage())
		m_ircContact->serverContact()->messenger()->displayMessage(MessageTransport(message, originating, QString(""), QString(""), m_ircContact->serverContact()->engine()->nickName(), IRCMessage::Notice, QString(originating + "-" + m_ircContact->serverContact()->serverName()), messageView()));
}

void IRCChatView::slotReturnPressed()
{
	// Always make sure to return all if() statements, otherwise a message will be echoed that tells the user the command is understood (even though the command is ran)
	QString command = messageEdit()->text().section(' ', 0, 0);
	QString text = messageEdit()->text().section(' ', 1);
	if (command[0] == '/')
		command.remove(0, 1);
	else
	{
		QString message = messageEdit()->text();
		if (!message.stripWhiteSpace().isEmpty())
		{
			m_ircContact->engine()->messageContact(
					m_channel, message
				);
			messageEdit()->clear();
			return;
		}
	}
	m_ircContact->serverContact()->parser()->parseCommand(messageView(), IRCCmdParser::Chat, command, text, m_channel);
	messageEdit()->clear();
}

void IRCChatView::incomingAction(const QString &originating, const QString &target, const QString &message)
{
	if (target.lower() != m_channel.lower())
		return;

	QString origNick = originating.section('!', 0, 0);
	m_ircContact->serverContact()->messenger()->displayMessage(MessageTransport(message, origNick, QString(""), target, m_ircContact->serverContact()->engine()->nickName(), IRCMessage::Action, QString(m_channel + "-" + m_ircContact->serverContact()->serverName()), messageView()));
}

void IRCChatView::userJoinedChannel(const QString &user, const QString &channel)
{
	if (channel.lower() != m_channel.lower())
		return;

	if (firstJoin == true)
	{
		messageView()->setEnabled(true);
		namesList->setEnabled(true);
		messageEdit()->setEnabled(true);
		messageView()->clear();
		firstJoin = false;
	}
	QString nickname = user.section('!', 0, 0);
	m_ircContact->serverContact()->messenger()->displayMessage(MessageTransport(QString(""), nickname, user.section('!', 1), channel, m_ircContact->serverContact()->engine()->nickName(), IRCMessage::Join, QString(m_channel + "-" + m_ircContact->serverContact()->serverName()), messageView()));

	int index = 0;
	while (index != (namesList->numRows()))
	{
		QListBoxItem *it = namesList->item(index);
		if (it != 0)
		{
			QString oped;
			if (it->text()[0] == '+' || it->text()[0] == '@')
				oped = it->text().remove(0, 1);

			if (it->text().lower() == nickname.lower()
				|| oped.lower() == nickname.lower()
			)
				return;
		}
		index++;
	}
	namesList->insertItem(nickname);
	messageEdit()->add(nickname);
	namesList->sort();
}

void IRCChatView::userPartedChannel(const QString &user, const QString &channel, const QString &reason)
{
	if (channel.lower() != m_channel.lower())
		return;

	QString nickname = user.section('!', 0, 0);
	messageEdit()->remove(nickname);
	m_ircContact->serverContact()->messenger()->displayMessage(MessageTransport(reason,
			nickname,
			user.section('!', 1),
			channel,
			m_ircContact->serverContact()->engine()->nickName(),
			IRCMessage::Part,
			m_channel,
			messageView()
	));

	int index = 0;
	while (index != (namesList->numRows()))
	{
		QListBoxItem *it = namesList->item(index);
		if (it != 0)
		{
			QString oped;
			if (it->text()[0] == '+' || it->text()[0] == '@')
				oped = it->text().remove(0, 1);
			if (it->text().lower() == nickname.lower() || oped.lower() == nickname.lower())
			{
				delete it;
				break;
			}
		}
		index++;
	}
}

void IRCChatView::userQuitIRC(const QString &user, const QString &reason)
{
	QString nickname = user.section('!', 0, 0);
	messageEdit()->remove(nickname);
	int index = 0;
	bool found = false;
	while (index != (namesList->numRows()))
	{
		QListBoxItem *it = namesList->item(index);
		if (it != 0)
		{
			QString oped;
			if (it->text()[0] == '+' || it->text()[0] == '@')
				oped = it->text().remove(0, 1);

			if (it->text().lower() == nickname.lower() || oped.lower() == nickname.lower())
			{
				delete it;
				found = true;
				break;
			}
		}
		index++;
	}
	if (found == true)
		m_ircContact->serverContact()->messenger()->displayMessage(MessageTransport(reason, nickname, user.section('!', 1), QString(""), m_ircContact->serverContact()->engine()->nickName(), IRCMessage::Quit, QString(m_channel + "-" + m_ircContact->serverContact()->serverName()), messageView()));
}

void IRCChatView::incomingNewTopic(const QString &channel, const QString &originating, const QString &newTopic)
{
	if (channel.lower() == m_channel.lower())
	{
		topicField->setText(newTopic);
		QToolTip::add(topicField, newTopic);
		m_ircContact->serverContact()->messenger()->displayMessage(MessageTransport(newTopic , originating.section('!', 0, 0), QString(""), channel, m_ircContact->serverContact()->engine()->nickName(), IRCMessage::ChangedTopic, QString(m_channel + "-" + m_ircContact->serverContact()->serverName()), messageView()));
	}
}

void IRCChatView::receivedExistingTopic(const QString &channel, const QString &existingTopic)
{
	if (channel.lower() == m_channel.lower())
	{
		topicField->setText(existingTopic);
		QToolTip::add(topicField, existingTopic);
	}
}

void IRCChatView::incomingMessage(const QString &originating, const QString &target, const QString &message)
{
	if (target.lower() != m_channel.lower())
		return;

	if (originating.lower() == m_ircContact->engine()->nickName().lower())
		m_ircContact->serverContact()->messenger()->displayMessage(MessageTransport(message, originating.section('!', 0, 0), QString(""), target, m_ircContact->serverContact()->engine()->nickName(), IRCMessage::YouSayMsg, QString(m_channel + "-" + m_ircContact->serverContact()->serverName()), messageView()));
	else
		m_ircContact->serverContact()->messenger()->displayMessage(MessageTransport(message, originating.section('!', 0, 0), QString(""), target, m_ircContact->serverContact()->engine()->nickName(), IRCMessage::PrivMsg, QString(m_channel + "-" + m_ircContact->serverContact()->serverName()), messageView()));

	// This is for adjusting the colors of the tab when someone says something and this tab isn't focused
	if (m_ircContact->serverContact()->chatWindow()->mTabWidget->currentPage() != m_ircContact->tabPage())
	{
		IrcChannelTabWidget::ETabMode eMode(IrcChannelTabWidget::eTM_Active);
		if (message.contains(m_ircContact->serverContact()->engine()->nickName()) >= 1)
			eMode = IrcChannelTabWidget::eTM_Highlighted;

		m_ircContact->serverContact()->chatWindow()->mTabWidget->setTabMode(m_ircContact->tabPage(),
										    eMode);
	}
}

void IRCChatView::closeEvent( QCloseEvent *e )
{
	if (m_ircContact->waitingPart() == true)
		m_ircContact->unloading();
	else {
		m_ircContact->slotPart();
		e->ignore();
	}
}

void IRCChatView::slotChangeTopic()
{
	if (m_ircContact->engine()->isLoggedIn())
		m_ircContact->engine()->setTopic(m_channel, topicField->text());
}

void IRCChatView::incomingKick(
		const QString &user, const QString &channel,
		const QString &by, const QString &reason
	)
{
	if (channel.lower() != m_channel.lower())
		return;

	QString nickname = user.section('!', 0, 0);
	messageEdit()->remove(nickname);

	m_ircContact->serverContact()->messenger()->displayMessage(MessageTransport(
			reason,
			nickname,
			by,
			channel,
			m_ircContact->serverContact()->engine()->nickName(),
			IRCMessage::Kick,
			m_channel,
			messageView()
		));

	int index = 0;
	while (index != (namesList->numRows()))
	{
		QListBoxItem *it = namesList->item(index);
		if (it != 0)
		{
			QString oped;
			if (it->text()[0] == '+' || it->text()[0] == '@')
				oped = it->text().remove(0, 1);
			if (it->text().lower() == nickname.lower() || oped.lower() == nickname.lower())
			{
				delete it;
				break;
			}
		}
		index++;
	}
}

#include "ircchatview.moc"
/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 8
 * indent-tabs-mode: t
 * End:
 */
// vim: set noet ts=4 sts=4 sw=4:

