/***************************************************************************
                          irccmdparser.cpp  -  description
                             -------------------
    begin                : Sun Apr 7 2002
    copyright            : (C) 2002 by nbetcher
    email                : nbetcher@usinternet.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#include "irccmdparser.h"

#include "irccontact.h"
#include "ircprotocol.h"
#include "ircservercontact.h"
#include "kirc.h"
#include "kopetecontactlist.h"
#include "kopetemetacontact.h"
#include "messagetransport.h"

#include <klocale.h>
#include <kdebug.h>

#include <qfileinfo.h>
#include <qtextedit.h>
#include <qdict.h>

IRCCmdParser::IRCCmdParser(IRCProtocol *protocol, IRCServerContact *contact)
{
	mContact = contact;
	mProtocol = protocol;
	serverCommands << "join" << "quit" << "part" << "leave" << "nick" << "msg" << "ctcp" << "dcc" << "whois" << "wi" << "notice";
	chatCommands << "join" << "quit" << "part" << "leave" << "nick" << "msg" << "me" << "ctcp" << "kick" << "dcc" << "whois" << "wi" << "notice";
	queryCommands <<  "join" << "quit" << "part" << "leave" << "nick" << "msg" << "me" << "ctcp" << "dcc" << "whois" << "wi" << "notice";
}

void IRCCmdParser::parseCommand(QTextEdit *chatView, CommandType type, const QString &command, const QString &text, const QString &callingName)
{
	// callingName is the name of the window that is calling this: e.g. #kde or in case of a query: SomeNickName
	QString ID = callingName;
	if (type == IRCCmdParser::Server && serverCommands.contains(command) == false)
	{
		ID.prepend("@");
		QString message = i18n("The command <b>%1</b> can't be used in this window or this command is not implemented yet.").arg(command);
		mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerMsg, ID, chatView));
		return;
	}
	if (type == IRCCmdParser::Chat && chatCommands.contains(command) == false)
	{
		QString message = i18n("The command <b>%1</b> can't be used in this window or this command is not implemented yet.").arg(command);
		mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerMsg, ID, chatView));
		return;
	}
	if (type == IRCCmdParser::Query && queryCommands.contains(command) == false)
	{
		QString message = i18n("The command <b>%1</b> can't be used in this window or this command is not implemented yet.").arg(command);
		mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerMsg, ID, chatView));
		return;
	}
	
	if (command.lower() == "notice")
	{
		if (mContact->engine()->isLoggedIn() == false)
		{
			QString message = i18n("The command <b>%1</b> requires you to be connected to a server. Please press the \"Connect\" button in the toolbar.<br>").arg(command);
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerMsg, ID, chatView));
			return;
		}
		if (text.isEmpty())
		{
			QString message = i18n("Your /notice command is missing a recipient and message. The format is: /notice target message.");
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::DCC, ID, chatView));
			return;
		}
		mContact->engine()->sendNotice(text.section(' ', 0, 0), text.section(' ', 1));
		return;
	}
	
	if (command.lower() == "dcc")
	{
		if (mContact->engine()->isLoggedIn() == false)
		{
			QString message = i18n("The command <b>%1</b> requires you to be connected to a server. Please press the \"Connect\" button in the toolbar.<br>").arg(command);
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerMsg, ID, chatView));
			return;
		}
		if (text.isEmpty())
		{
			QString message = i18n("Your /dcc command is missing a recipient and DCC type. The format is: /dcc type Target.");
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::DCC, ID, chatView));
			return;
		}
		QString type = text.section(' ', 0, 0);
		if (type.isEmpty())
		{
			QString message = i18n("Your /dcc command is missing a DCC type. The format is: /dcc type Target.");
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::DCC, ID, chatView));
			return;
		}
		if (type.lower() == "chat")
		{
			mContact->initiateDcc(text.section(' ', 1, 1), QString(""), DCCServer::Chat);
			QString message = i18n("Establishing a DCC CHAT session with %1...").arg(text.section(' ', 1, 1));
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::DCC, ID, chatView));
			return;
		} else if (type.lower() == "send")
		{
			QString filename = text.section(' ', 2, 2);
			if (!filename.isEmpty())
			{
				QFileInfo finfo(filename);
				if (!finfo.exists())
				{
					QString message = i18n("The filename that you specified doesn't exist. If you don't supply a filename, you will be prompted to select the file from an Open dialog.");
					mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::DCC, ID, chatView));
					return;
				}
			}
			mContact->initiateDcc(text.section(' ', 1, 1), filename, DCCServer::File);
			QString message = i18n("Establishing a DCC SEND session with %1...").arg(text.section(' ', 1, 1));
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::DCC, ID, chatView));
			return;
		}
	} else if (command.lower() == "ctcp")
	{
		if (mContact->engine()->isLoggedIn() == false)
		{
			QString message = i18n("The command <b>%1</b> requires you to be connected to a server. Please press the \"Connect\" button in the toolbar.<br>").arg(command);
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerMsg, ID, chatView));
			return;
		}
		if (text.isEmpty())
		{
			QString message = i18n("Your /ctcp command is missing a recipient and CTCP type. The format is: /ctcp Target type.");
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::Ctcp, ID, chatView));
			return;
		}
		QString type = text.section(' ', 1, 1);
		if (type.isEmpty())
		{
			QString message = i18n("Your /ctcp command is missing a CTCP type. The format is: /ctcp Target type.");
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::Ctcp, ID, chatView));
			return;
		}
		if (type.lower() == "ping")
		{
			mContact->engine()->sendCtcpPing(text.section(' ', 0, 0));
			QString message = i18n("Sent a CTCP request of type %1 to %2").arg(type).arg(text.section(' ', 0, 0));
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::Ctcp, ID, chatView));
			return;
		}
		if (type.lower() == "version")
		{
			mContact->engine()->sendCtcpVersion(text.section(' ', 0, 0));
			QString message = i18n("Sent a CTCP request of type %1 to %2").arg(type).arg(text.section(' ', 0, 0));
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::Ctcp, ID, chatView));
			return;
		}
		QString message = i18n("Your /ctcp command was not understood because the CTCP type %1 is not implemented yet.").arg(type);
		mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::Ctcp, ID, chatView));
		return;
	} else if (command.lower() == "me")
	{
		if (mContact->engine()->isLoggedIn() == false)
		{
			QString message = i18n("The command <b>%1</b> requires you to be connected to a server. Please press the \"Connect\" button in the toolbar.<br>").arg(command);
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerMsg, ID, chatView));
			return;
		}
		if (text.isEmpty())
		{
			QString message = i18n("Your /me command is missing a message.");
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::UnknownMsg, ID, chatView));
			return;
		}
		mContact->engine()->actionContact(callingName, text);
		return;
	} else if (command.lower() == "quit")
	{
		if (mContact->engine()->isLoggedIn() == false)
		{
			QString message = i18n("The command <b>%1</b> requires you to be connected to a server. Please press the \"Connect\" button in the toolbar.<br>").arg(command);
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerMsg, ID, chatView));
			return;
		}
		if (text.isEmpty() == false)
			mContact->setQuitMessage(text);

		mContact->slotQuitServer();
		return;
	} else if ((command.lower() == "part") || (command.lower() == "leave"))
	{
		if (mContact->engine()->isLoggedIn() == false)
		{
			QString message = i18n("The command <b>%1</b> requires you to be connected to a server. Please press the \"Connect\" button in the toolbar.<br>").arg(command);
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerMsg, ID, chatView));
			return;
		}
		QString appending = text.section(' ', 0, 0);
		if (appending[0] == '#' || appending[0] == '!' || appending[0] == '&')
		{
			mContact->engine()->partChannel(appending, text.section(' ', 1));
		} else {
			if (type == IRCCmdParser::Server || type == IRCCmdParser::Query)
			{
				QString message2 = i18n("The command <b>%1</b> in this window requires an argument that begins with a '#', '!', or '&'. Example: /join #kopete.<br>").arg(command);
				mContact->messenger()->displayMessage(MessageTransport(message2, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerMsg, ID, chatView));
				return;
			}
			if (callingName.isEmpty() == false)
			{
				if (callingName[0] == '#' || callingName[0] == '!' || callingName == '&')
				{
					QString quitMessage;
					if (text.isEmpty())
						quitMessage = "Using Kopete IRC";
					else
						quitMessage = text;

					mContact->engine()->partChannel(callingName, quitMessage);
					QString parsed = QStyleSheet::escape(i18n("Attempting to quit channel. If this takes longer than expected, it may be that you have been disconnected from the server. In that case, please disconnect from the server by clicking the disconnect icon on the toolbar.<br>"));
					chatView->append(parsed);
				}
			}
		}
		return;
	} else if (command.lower() == "nick")
	{
		if (mContact->engine()->isLoggedIn() == false)
		{
			QString message = i18n("The command <b>%1</b> requires you to be connected to a server. Please press the \"Connect\" button in the toolbar.<br>").arg(command);
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerMsg, ID, chatView));
			return;
		}
		QString newnick = text.section(' ', 0, 0);
		if (newnick != mContact->engine()->nickName() && newnick.isEmpty() == false)
			mContact->engine()->changeNickname(newnick);

		return;
	} else if (command.lower() == "msg")
	{
		if (mContact->engine()->isLoggedIn() == false)
		{
			QString message = i18n("The command <b>%1</b> requires you to be connected to a server. Please press the \"Connect\" button in the toolbar.<br>").arg(command);
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerMsg, ID, chatView));
			return;
		}
		QString target = text.section(' ', 0, 0);
		QString message = text.section(' ', 1);
		if (message.isEmpty() || target.isEmpty())
		{
			QString message = i18n("Your /msg command is malformed.");
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::UnknownMsg, ID, chatView));
			return;
		}
		mContact->engine()->messageContact(target, message);
		return;
	} else if (command.lower() == "join")
	{
		if (mContact->engine()->isLoggedIn() == false)
		{
			QString message = i18n("The command <b>%1</b> requires you to be connected to a server. Please press the \"Connect\" button in the toolbar.<br>").arg(command.lower());
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerMsg, ID, chatView));
			return;
		}
		QString channel = text.section(' ', 0, 0);
		if (channel.isEmpty() == false && (channel.lower()[0] == '#' || '!' || '&'))
		{
			QStringList channels = QStringList::split(',', channel);
			QStringList::ConstIterator it = channels.begin();
			QStringList::ConstIterator end = channels.end();
			for (; it != end; ++it)
			{
				QString contactID=QString((*it)+"@"+mContact->serverName()).lower();
				kdDebug(14120) << "mContact->serverName() == " << mContact->serverName() << endl;
				//kdDebug(14120) << "looking for contactID " << contactID << endl;
				
				IRCContact *c = static_cast<IRCContact*>(mProtocol->contacts()[contactID]);
				if (c) {
					Q_ASSERT(!c->metaContact()->isTemporary());
					c->joinNow();
					kdDebug(14120) << k_funcinfo << "Contact already exists " << contactID <<endl;
				}
				else
				{
					kdDebug(14120) << k_funcinfo << "add contact: " << contactID <<endl;

					KopeteMetaContact *m = new KopeteMetaContact();
					m->setTemporary(true);
					m->addContact(new IRCContact(mContact->serverName(), (*it), 0, true, mContact, m, mProtocol));
					KopeteContactList::contactList()->addMetaContact(m);
				}
			}
		}
		else
		{
			QString message = i18n("The command <b>%1</b> requires you to prefix the channel name with a '#', '!' or a '&'.<br>").arg(command.lower().remove(0, 1));
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerMsg, ID, chatView));
			return;
		}
		return;
	} else if (command.lower() == "kick")
	{
		if (mContact->engine()->isLoggedIn() == false)
		{
			QString message = i18n("The command <b>%1</b> requires you to be connected to a server. Please press the \"Connect\" button in the toolbar.<br>").arg(command.lower().remove(0, 1));
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerMsg, ID, chatView));
			return;
		}
		QString user = text.section(' ', 0, 0);
		QString reason = text.section(' ', 1);

		if (user.isEmpty() == false)
		{

			if (reason.isEmpty())
				mContact->engine()->kickUser(user, callingName,mContact->engine()->nickName());
			else
				mContact->engine()->kickUser(user, callingName, reason);

		} else {
			QString message = i18n("The command <b>%1</b> requires you to specify a user to kick.<br>").arg(command.lower().remove(0, 1));
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerMsg, ID, chatView));
			return;
		}
		return;
	} else if (command.lower() == "whois" || command.lower() == "wi")
	{
		if (mContact->engine()->isLoggedIn() == false)
		{
			QString message = i18n("The command %1 requires you to be connected to a server. Please press the \"Connect\" button in the toolbar.<br>").arg(command.lower().remove(0, 1));
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerMsg, ID, chatView));
			return;
		}
		QString user = text.section(' ', 0, 0);

		if (user.isEmpty() == false)
			mContact->engine()->whoisUser(user);
		else {
			QString message = i18n("The command %1 requires you to specify a user.<br>").arg(command.lower().remove(0, 1));
			mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerMsg, ID, chatView));
			return;
		}
		return;
	}

	QString message = i18n("The command <b>%1</b> isn't understood. Please check your spelling and try again. It is possible this function is not yet implemented.<br>").arg(command.lower());
	mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerMsg, ID, chatView));
}
/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 8
 * indent-tabs-mode: t
 * End:
 */
// vim: set noet ts=4 sts=4 sw=4:

