/***************************************************************************
                          ircconsoleview.cpp  -  description
                             -------------------
    begin                : Sun Mar 24 2002
    copyright            : (C) 2002 by nbetcher
    email                : nbetcher@usinternet.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#include "ircconsoleview.h"

#include "irc_channel_tabwidget.h"
#include "ircchatwindow.h"
#include "irccmdparser.h"
#include "ircmessage.h"
#include "ircservercontact.h"
#include "kirc.h"
#include "messagetransport.h"
#include "tabcompleter.h"

#include <kconfig.h>
#include <kglobal.h>
#include <klocale.h>
#include <kstandarddirs.h>

#include <qlabel.h>
#include <qlayout.h>
#include <qtextedit.h>
#include <qtoolbutton.h>
#include <qtooltip.h>
#include <qvbox.h>



IRCConsoleView::IRCConsoleView(const QString &server, KIRC *engine, IRCServerContact *contact, QWidget *parent)
	: IrcViewBase(parent)
{
	QVBoxLayout* pVBoxLayout(new QVBoxLayout(this, 2, 2, "MainLayout"));

	QHBoxLayout* pHBoxLayout(new QHBoxLayout(0, 0, 6, "Layout"));

	pHBoxLayout->addWidget(new QLabel(i18n("Server:"), this));

	serverName = new QLabel(this);
	pHBoxLayout->addWidget(serverName);

	pHBoxLayout->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));

	QToolButton* closeTab = new QToolButton(this, "closeTab");
	closeTab->setText("X");
	pHBoxLayout->addWidget(closeTab);

	pVBoxLayout->addLayout(pHBoxLayout);

	createMessageView(this);
	pVBoxLayout->addWidget(messageView());

	createMessageEdit(this);
	pVBoxLayout->addWidget(messageEdit());



	m_tryingQuit = false;
	m_ircServerContact = contact;
	m_serverName = server;
	m_engine = engine;
	serverName->setText(m_serverName);
	QObject::connect(messageEdit(), SIGNAL(returnPressed()), this, SLOT(slotReturnPressed()));
	QObject::connect(m_engine, SIGNAL(incomingWelcome(const QString &)), this, SLOT(slotIncomingSrvMsg(const QString &)));
	QObject::connect(m_engine, SIGNAL(incomingYourHost(const QString &)), this, SLOT(slotIncomingSrvMsg(const QString &)));
	QObject::connect(m_engine, SIGNAL(incomingHostCreated(const QString &)), this, SLOT(slotIncomingSrvMsg(const QString &)));
	QObject::connect(m_engine, SIGNAL(incomingHostInfo(const QString &)), this, SLOT(slotIncomingSrvMsg(const QString &)));
	QObject::connect(m_engine, SIGNAL(incomingUsersInfo(const QString &)), this, SLOT(slotIncomingSrvMsg(const QString &)));
	QObject::connect(m_engine, SIGNAL(incomingOnlineOps(const QString &)), this, SLOT(slotIncomingSrvMsg(const QString &)));
	QObject::connect(m_engine, SIGNAL(incomingUnknownConnections(const QString &)), this, SLOT(slotIncomingSrvMsg(const QString &)));
	QObject::connect(m_engine, SIGNAL(incomingTotalChannels(const QString &)), this, SLOT(slotIncomingSrvMsg(const QString &)));
	QObject::connect(m_engine, SIGNAL(incomingHostedClients(const QString &)), this, SLOT(slotIncomingSrvMsg(const QString &)));
	QObject::connect(m_engine, SIGNAL(incomingMotd(const QString &)), this, SLOT(slotIncomingMotd(const QString &)));
	QObject::connect(m_engine, SIGNAL(connectedToServer()), this, SLOT(slotLoggedIn()));
	QObject::connect(m_engine, SIGNAL(incomingNoNickChan(const QString &)), this, SLOT(incomingNoNickChan(const QString &)));
	QObject::connect(m_engine, SIGNAL(incomingUnknown(const QString &)), this, SLOT(incomingUnknown(const QString &)));
	QObject::connect(m_engine, SIGNAL(incomingWhoIsChannels(const QString &, const QString &)), this, SLOT(incomingWhoIsChannels(const QString &, const QString &)));
	QObject::connect(m_engine, SIGNAL(incomingWhoIsIdle(const QString &, unsigned long)), this, SLOT(incomingWhoIsIdle(const QString &, unsigned long)));
	QObject::connect(m_engine, SIGNAL(incomingWhoIsOperator(const QString &)), this, SLOT(incomingWhoIsOperator(const QString &)));
	QObject::connect(m_engine, SIGNAL(incomingWhoIsServer(const QString &, const QString &, const QString &)), this, SLOT(incomingWhoIsServer(const QString &, const QString &, const QString &)));
	QObject::connect(m_engine, SIGNAL(incomingWhoIsUser(const QString &, const QString &, const QString &, const QString &)), this, SLOT(incomingWhoIsUser(const QString &, const QString &, const QString &, const QString &)));
	QObject::connect(m_engine, SIGNAL(repliedCtcp(const QString &, const QString &, const QString &)), this, SLOT(slotRepliedCtcp(const QString &, const QString &, const QString &)));
	QObject::connect(m_engine, SIGNAL(incomingCtcpReply(const QString &, const QString &, const QString &)), this, SLOT(slotCtcpReply(const QString &, const QString &, const QString &)));
	QObject::connect(m_engine, SIGNAL(incomingNotice(const QString &, const QString &)), this, SLOT(incomingNotice(const QString &, const QString &)));
	QObject::connect(contact->engine(), SIGNAL(connecting()), this, SLOT(slotConnecting()));
	QObject::connect(m_ircServerContact->chatWindow()->mTabWidget, SIGNAL(currentChanged(QWidget *)), this, SLOT(slotPageChanged(QWidget *)));
	QObject::connect(closeTab, SIGNAL(clicked()), this, SLOT(slotCloseTab()));
}

void IRCConsoleView::slotCloseTab()
{
	m_ircServerContact->setClosing(true);
	m_ircServerContact->slotDisconnectNow();
}

void IRCConsoleView::slotRepliedCtcp(const QString &type, const QString &target, const QString &reply)
{
	QString message = i18n("Received a CTCP request of type %1 from %2, replied with: %3").arg(type).arg(target).arg(reply);
	m_ircServerContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), m_ircServerContact->engine()->nickName(), IRCMessage::Ctcp, QString("@" + m_serverName), messageView()));
}

void IRCConsoleView::slotCtcpReply(const QString &type, const QString &target, const QString &messageReceived)
{
	QString message = i18n("Received a CTCP reply of type %1 from %2: %3").arg(type).arg(target).arg(messageReceived);
	m_ircServerContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), m_ircServerContact->engine()->nickName(), IRCMessage::Ctcp, QString("@" + m_serverName), messageView()));
}

void IRCConsoleView::slotPageChanged(QWidget *widget)
{
	if (m_ircServerContact->tabView() != 0)
		if (m_ircServerContact->tabView() == widget)
			messageEdit()->setFocus();
}

void IRCConsoleView::incomingUnknown(const QString &message)
{
	QString message2 = i18n("The server sent the unknown or unimplemented message: %1").arg(message);
	m_ircServerContact->messenger()->displayMessage(MessageTransport(message2, QString(""), QString(""), QString(""), m_ircServerContact->engine()->nickName(), IRCMessage::ServerMsg, QString("@" + m_serverName), messageView()));
}

void IRCConsoleView::incomingWhoIsChannels(const QString &nickname, const QString &channel)
{
	QString message = i18n("%1 is chatting in %2.").arg(nickname).arg(channel);
	m_ircServerContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), m_ircServerContact->engine()->nickName(), IRCMessage::Whois, QString("@" + m_serverName), messageView()));
}

void IRCConsoleView::incomingWhoIsIdle(const QString &nickname, unsigned long seconds)
{
	QString message = i18n("%1 has been idle for %2 seconds.").arg(nickname).arg(seconds);
	m_ircServerContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), m_ircServerContact->engine()->nickName(), IRCMessage::Whois, QString("@" + m_serverName), messageView()));
}

void IRCConsoleView::incomingWhoIsOperator(const QString &nickname)
{
	QString message = i18n("%1 is an IRC operator.").arg(nickname);
	m_ircServerContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), m_ircServerContact->engine()->nickName(), IRCMessage::Whois, QString("@" + m_serverName), messageView()));
}

void IRCConsoleView::incomingWhoIsUser(const QString &nickname, const QString &username, const QString &hostname, const QString &realname)
{
	QString message = i18n("%1 (%2@%3): %4").arg(nickname).arg(username).arg(hostname).arg(realname);
	m_ircServerContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), m_ircServerContact->engine()->nickName(), IRCMessage::Whois, QString("@" + m_serverName), messageView()));
}

void IRCConsoleView::incomingWhoIsServer(const QString &/*nickname*/, const QString &server, const QString &serverInfo)
{
	QString message = i18n("Server info for %1: %2").arg(server).arg(serverInfo);
	m_ircServerContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), m_ircServerContact->engine()->nickName(), IRCMessage::Whois, QString("@" + m_serverName), messageView()));
}

void IRCConsoleView::incomingNoNickChan(const QString &nickname)
{
	QString message = i18n("No such nickname or channel: %1").arg(nickname);
	m_ircServerContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), m_ircServerContact->engine()->nickName(), IRCMessage::ServerMsg, QString("@" + m_serverName), messageView()));
}

void IRCConsoleView::slotLoggedIn()
{
	QString message = i18n("Logged into server.");
	m_ircServerContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString("") , m_ircServerContact->engine()->nickName(), IRCMessage::ServerMsg, QString("@" + m_serverName), messageView()));
}

void IRCConsoleView::slotIncomingMotd(const QString &message)
{
	m_ircServerContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), m_ircServerContact->engine()->nickName(), IRCMessage::MotdMsg, QString("@" + m_serverName), messageView()));
}

void IRCConsoleView::slotIncomingSrvMsg(const QString &message)
{
	m_ircServerContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), m_ircServerContact->engine()->nickName(), IRCMessage::ServerInfo, QString("@" + m_serverName), messageView()));
}

void IRCConsoleView::slotReturnPressed()
{
	QString command = messageEdit()->text().section(' ', 0, 0);
	QString text = messageEdit()->text().section(' ', 1);
	if (command[0] == '/')
	{
		command.remove(0, 1);
		if (command.lower() == "server")
		{
			if (text.isEmpty())
			{
				QString message = i18n("You need to specify a server when performing a /server command. The syntax is: /server irc.yourserver.com");
				m_ircServerContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), m_ircServerContact->engine()->nickName(), IRCMessage::UnknownMsg, QString("@" + m_serverName), messageView()));
				messageEdit()->setText("");
				return;
			}
			if (m_ircServerContact->engine()->isLoggedIn() == true)
			{
				m_tryingQuit = true;
				m_pendingConnection = QStringList::split(' ', text);
				QObject::connect(m_ircServerContact->engine(), SIGNAL(successfulQuit()), this, SLOT(slotServerHasQuit()));
				m_ircServerContact->slotDisconnectNow();
			} else {
				m_tryingQuit = true;
				m_pendingConnection = QStringList::split(' ', text);
				slotServerHasQuit();
			}
		}
	} else {
		if (messageEdit()->text().isEmpty() == false)
		{
			QString message = i18n("All text inputted into server windows must be preceded with a '/' to indicate a command.");
			m_ircServerContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), m_ircServerContact->engine()->nickName(), IRCMessage::UnknownMsg, QString("@" + m_serverName), messageView()));
			messageEdit()->setText("");
			return;
		} else
			return;
	}
	m_ircServerContact->parser()->parseCommand(messageView(), IRCCmdParser::Server, command, text, m_serverName);
	messageEdit()->setText("");
}

void IRCConsoleView::slotHostFound()
{
	QString message = i18n("Server resolved to address", "%1 resolved to %2").arg(m_serverName).arg(m_engine->address().toString());
	m_ircServerContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), m_ircServerContact->engine()->nickName(), IRCMessage::ServerMsg, QString("@" + m_serverName), messageView()));
}

void IRCConsoleView::slotConnected()
{
	QString message = i18n("Connected to server, logging in... ");
	m_ircServerContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), m_ircServerContact->engine()->nickName(), IRCMessage::ServerMsg, QString("@" + m_serverName), messageView()));
}

void IRCConsoleView::slotConnectionClosed()
{
	QObject::disconnect(m_engine, SIGNAL(hostFound()), this, SLOT(slotHostFound()));
	QObject::disconnect(m_engine, SIGNAL(connected()), this, SLOT(slotConnected()));
	QObject::disconnect(m_engine, SIGNAL(connectionClosed()), this, SLOT(slotConnectionClosed()));
	QObject::disconnect(m_engine, SIGNAL(error(int)), this, SLOT(slotError(int)));
	QString message = QString("<b>%1</b>").arg(i18n("Disconnected from server."));
	m_ircServerContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), m_ircServerContact->engine()->nickName(), IRCMessage::ServerMsg, QString("@" + m_serverName), messageView()));
}

void IRCConsoleView::slotError(int errorNum)
{
	QObject::disconnect(m_engine, SIGNAL(hostFound()), this, SLOT(slotHostFound()));
	QObject::disconnect(m_engine, SIGNAL(connected()), this, SLOT(slotConnected()));
	QObject::disconnect(m_engine, SIGNAL(connectionClosed()), this, SLOT(slotConnectionClosed()));
	QObject::disconnect(m_engine, SIGNAL(error(int)), this, SLOT(slotError(int)));
	QString message = "<b>";
	switch (errorNum)
	{
		case QSocket::ErrConnectionRefused :
		{
			message.append(i18n("Connection to the server failed, connection refused."));
			break;
		};
		case QSocket::ErrHostNotFound :
		{
			message.append(i18n("Connection to the server failed, server hostname not found."));
			break;
		};
		case QSocket::ErrSocketRead :
		{
			message.append(i18n("FATAL! Connection interrupted, socket read error!"));
			break;
		};
	}
	message.append("</b>");
	m_ircServerContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), m_ircServerContact->engine()->nickName(), IRCMessage::ServerMsg, QString("@" + m_serverName), messageView()));
}

void IRCConsoleView::slotServerHasQuit()
{
	if (m_tryingQuit == true)
	{
		if (m_pendingConnection.isEmpty() == false)
		{
			m_serverName = m_pendingConnection[0];
			m_ircServerContact->setServerName( m_pendingConnection[0] );
			m_ircServerContact->newNickname(KGlobal::config()->readEntry("Nickname", "KopeteUser"));
			m_ircServerContact->chatWindow()->setCaption(m_serverName);
			m_ircServerContact->chatWindow()->mTabWidget->changeTab(m_ircServerContact->tabView(), m_pendingConnection[0]);
			serverName->setText(m_pendingConnection[0]);
			m_ircServerContact->slotConnectNow();
		}
	}
	m_tryingQuit = false;
	QObject::disconnect(m_ircServerContact->engine(), SIGNAL(successfulQuit()), this, SLOT(slotServerHasQuit()));
	m_pendingConnection.clear();
}

void IRCConsoleView::slotConnecting()
{
	QObject::connect(m_engine, SIGNAL(hostFound()), this, SLOT(slotHostFound()));
	QObject::connect(m_engine, SIGNAL(connected()), this, SLOT(slotConnected()));
	QObject::connect(m_engine, SIGNAL(connectionClosed()), this, SLOT(slotConnectionClosed()));
	QObject::connect(m_engine, SIGNAL(error(int)), this, SLOT(slotError(int)));
	QString message = QString("%1 %2").arg(i18n("Connecting to server")).arg(m_serverName);
	m_ircServerContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), m_ircServerContact->engine()->nickName(), IRCMessage::ServerMsg, QString("@" + m_serverName), messageView()));
}

void IRCConsoleView::incomingNotice(const QString &originating, const QString &message)
{
	if (m_ircServerContact->chatWindow()->mTabWidget->currentPage() == m_ircServerContact->tabView())
		m_ircServerContact->messenger()->displayMessage(MessageTransport(message, originating, QString(""), QString(""), m_ircServerContact->engine()->nickName(), IRCMessage::Notice, QString(originating + "-" + m_ircServerContact->serverName()), messageView()));
}

#include "ircconsoleview.moc"
/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 8
 * indent-tabs-mode: t
 * End:
 */
// vim: set noet ts=4 sts=4 sw=4:

