/***************************************************************************
                          ircdccreceive.cpp  -  description
                             -------------------
    begin                : Fri Apr 26 2002
    copyright            : (C) 2002 by nbetcher
    email                : nbetcher@usinternet.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 


#include "ircdccreceive.h"

#include "ircservercontact.h"
#include "kirc.h"
#include "messagetransport.h"

#include <klocale.h>
#include <kprogress.h>
#include <kpushbutton.h>
#include <ksqueezedtextlabel.h>

#include <qtextedit.h>
#include <qvbox.h>



IRCDCCReceive::IRCDCCReceive(const QString &nickname, const QString &filename, IRCServerContact *contact, QVBox *layout, DCCClient *chatObject)
	: IRCReceiveBase(layout)
{
	mChatObject = chatObject;
	mNickname = nickname;
	mContact = contact;
	mLayout = layout;
	connect(cmdAbort, SIGNAL(clicked()), this, SLOT(slotAbort()));
	connect(chatObject, SIGNAL(receiveAckPercent(int)), this, SLOT(slotReceivedProgress(int)));
	connect(chatObject, SIGNAL(terminating()), this, SLOT(slotChatEnding()));
	connect(chatObject, SIGNAL(connected()), this, SLOT(slotConnected()));
	connect(chatObject, SIGNAL(sendFinished()), this, SLOT(slotReceivedFinished()));
	chatView->setTextFormat(Qt::RichText);

	mContact->messenger()->displayMessage(MessageTransport(i18n("Connecting to port %1...").arg(QString::number(chatObject->hostPort())), mNickname, QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerInfo, QString("^" + mNickname), chatView));

	filenameLabel->setText(filename);
	nicknameLabel->setText(nickname);
}

void IRCDCCReceive::slotAbort()
{
	if (mChatObject != 0)
		mChatObject->dccCancel();

	// This should destroy 'this' too since QObject considers 'this' to be a child of mLayout
	delete mLayout;
}

void IRCDCCReceive::slotReceivedProgress(int percent)
{
	receivedProgress->setProgress(percent);
}

void IRCDCCReceive::slotReceivedFinished()
{
	mContact->messenger()->displayMessage(MessageTransport(i18n("File successfully received!"), mNickname, QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerInfo, QString("^" + mNickname), chatView));
	mChatObject->dccCancel();
}

void IRCDCCReceive::slotConnected()
{
	mContact->messenger()->displayMessage(MessageTransport(i18n("Connected, receiving file..."), mNickname, QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerInfo, QString("^" + mNickname), chatView));
}

void IRCDCCReceive::slotChatEnding()
{
	mContact->messenger()->displayMessage(MessageTransport(i18n("The remote user has ended the DCC SEND session. Close this window when finished."), mNickname, QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerInfo, QString("^" + mNickname), chatView));
	mChatObject = 0L;
}

IRCDCCReceive::~IRCDCCReceive()
{
	if (mChatObject != 0)
	{
		mChatObject->dccCancel();
		delete mChatObject;
	}
}

#include "ircdccreceive.moc"


/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 8
 * indent-tabs-mode: t
 * End:
 */
// vim: set noet ts=4 sts=4 sw=4:

