/***************************************************************************
                          ircdccsend.cpp  -  description
                             -------------------
    begin                : Wed Apr 24 2002
    copyright            : (C) 2002 by nbetcher
    email                : nbetcher@usinternet.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 


#include "ircdccsend.h"

#include "ircservercontact.h"
#include "kirc.h"
#include "messagetransport.h"

#include <klocale.h>
#include <kprogress.h>
#include <kpushbutton.h>
#include <ksqueezedtextlabel.h>

#include <qtextedit.h>
#include <qvbox.h>



IRCDCCSend::IRCDCCSend(const QString &nickname, const QString &filename, IRCServerContact *contact, QVBox *layout, DCCServer *chatObject)
	: IRCSendBase(layout)
{
	mChatObject = chatObject;
	mNickname = nickname;
	mContact = contact;
	mLayout = layout;
	connect(cmdAbort, SIGNAL(clicked()), this, SLOT(slotAbort()));
	connect(chatObject, SIGNAL(incomingAckPercent(int)), this, SLOT(slotAckClientProgress(int)));
	connect(chatObject, SIGNAL(sendingNonAckPercent(int)), this, SLOT(slotSendProgress(int)));
	connect(chatObject, SIGNAL(terminating()), this, SLOT(slotChatEnding()));
	connect(chatObject, SIGNAL(clientConnected()), this, SLOT(slotConnected()));
	connect(chatObject, SIGNAL(sendFinished()), this, SLOT(slotSendFinished()));
	chatView->setTextFormat(Qt::RichText);

	mContact->messenger()->displayMessage(MessageTransport(i18n("Listening on port %2...").arg(QString::number(mChatObject->port())), mNickname, QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerInfo, QString("^" + mNickname), chatView));

	mContact->messenger()->displayMessage(MessageTransport(i18n("Asking %1 to connect to port %2...").arg(nickname).arg(QString::number(mChatObject->port())), mNickname, QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerInfo, QString("^" + mNickname), chatView));

	mContact->engine()->requestDccConnect(nickname, filename, mChatObject->port(), DCCClient::File);

	filenameLabel->setText(filename);
	nicknameLabel->setText(nickname);
}

void IRCDCCSend::slotAbort()
{
	if (mChatObject != 0)
		mChatObject->abort();

	// This should destroy 'this' too since QObject considers 'this' to be a child of mLayout
	delete mLayout;
}

void IRCDCCSend::slotAckClientProgress(int percent)
{
	clientProgress->setProgress(percent);
}

void IRCDCCSend::slotSendProgress(int percent)
{
	sentProgress->setProgress(percent);
}

void IRCDCCSend::slotSendFinished()
{
	mContact->messenger()->displayMessage(MessageTransport(i18n("File successfully sent!"), mNickname, QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerInfo, QString("^" + mNickname), chatView));
	mChatObject->abort();
}

void IRCDCCSend::slotConnected()
{
	mContact->messenger()->displayMessage(MessageTransport(i18n("User connected, sending file..."), mNickname, QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerInfo, QString("^" + mNickname), chatView));
}

void IRCDCCSend::slotChatEnding()
{
	mContact->messenger()->displayMessage(MessageTransport(i18n("The remote user has ended the DCC SEND session. You will not be allowed to send anymore text. Close this window when finished."), mNickname, QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerInfo, QString("^" + mNickname), chatView));
	mChatObject = 0L;
}

IRCDCCSend::~IRCDCCSend()
{
	if (mChatObject != 0)
	{
		mChatObject->abort();
		delete mChatObject;
	}
}

#include "ircdccsend.moc"
/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 8
 * indent-tabs-mode: t
 * End:
 */
// vim: set noet ts=4 sts=4 sw=4:

