/***************************************************************************
                          ircdccview.h  -  description
                             -------------------
    begin                : Sat Apr 20 2002
    copyright            : (C) 2002 by nbetcher
    email                : nbetcher@usinternet.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#include "ircdccview.h"

#include "dcchandler.h"
#include "irc_channel_tabwidget.h"
#include "ircchatwindow.h"
#include "irccmdparser.h"
#include "ircmessage.h"
#include "ircservercontact.h"
#include "kirc.h"
#include "messagetransport.h"
#include "tabcompleter.h"

#include <klocale.h>

#include <qlabel.h>
#include <qlayout.h>
#include <qstring.h>
#include <qtextedit.h>
#include <qtoolbutton.h>
#include <qvbox.h>



IRCDCCView::IRCDCCView(const QString &nickname, IRCServerContact *contact, QVBox *layout, DCCClient *chatObject)
	: IrcViewBase(layout)
{
	createLayout();

	//Sorry to put yet another twist on how I do things. mLayout is used instead of making a IRCContact and using IRCContact::tabPage()
	mChatObject = chatObject;
	mNickname = nickname;
	mContact = contact;
	mLayout = layout;
	connect(chatObject, SIGNAL(connected()), this, SLOT(slotConnected()));
	connect(chatObject, SIGNAL(incomingDccMessage(const QString &, bool)), this, SLOT(incomingMessage(const QString &, bool)));
	connect(mContact->chatWindow()->mTabWidget, SIGNAL(currentChanged(QWidget *)), this, SLOT(slotPageChanged(QWidget *)));
	connect(chatObject, SIGNAL(terminating()), this, SLOT(slotChatEnding()));
	// Just connect this so that people know if the user isn't on
	connect(mContact->engine(), SIGNAL(incomingNoNickChan(const QString &)), this, SLOT(incomingNoNickChan(const QString &)));

	mContact->messenger()->displayMessage(MessageTransport(i18n("Attempting to connect to %1 on port %2...").arg(mChatObject->ipaddress().toString()).arg(QString::number(mChatObject->hostPort())), mNickname, QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerInfo, QString("^" + mNickname), messageView()));
}

IRCDCCView::IRCDCCView(const QString &nickname, IRCServerContact *contact, QVBox *layout, DCCServer *chatObject)
	: IrcViewBase(layout)
{
	createLayout();

	//Sorry to put yet another twist on how I do things. mLayout is used instead of making a IRCContact and using IRCContact::tabPage()
	mChatObject = chatObject->mClient;
	mNickname = nickname;
	mContact = contact;
	mLayout = layout;
	connect(mChatObject, SIGNAL(incomingDccMessage(const QString &, bool)), this, SLOT(incomingMessage(const QString &, bool)));
	connect(mContact->chatWindow()->mTabWidget, SIGNAL(currentChanged(QWidget *)), this, SLOT(slotPageChanged(QWidget *)));
	connect(mChatObject, SIGNAL(terminating()), this, SLOT(slotChatEnding()));
	// Just connect this so that people know if the user isn't on
	connect(mContact->engine(), SIGNAL(incomingNoNickChan(const QString &)), this, SLOT(incomingNoNickChan(const QString &)));
	connect(mChatObject, SIGNAL(connected()), this, SLOT(slotConnected()));

	mContact->messenger()->displayMessage(MessageTransport(i18n("Listening on port %2...").arg(QString::number(chatObject->port())), mNickname, QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerInfo, QString("^" + mNickname), messageView()));

	mContact->messenger()->displayMessage(MessageTransport(i18n("Asking %1 to connect to port %2...").arg(nickname).arg(QString::number(chatObject->port())), mNickname, QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerInfo, QString("^" + mNickname), messageView()));

	mContact->engine()->requestDccConnect(nickname, QString(""), chatObject->port(), DCCClient::Chat);
}

void IRCDCCView::incomingNoNickChan(const QString &nickname)
{
	QString message = QString("%1: %2").arg(i18n("No such nickname or channel")).arg(nickname);
	mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerMsg, QString("^" + mNickname), messageView()));
}

void IRCDCCView::slotConnected()
{
	mContact->messenger()->displayMessage(MessageTransport(i18n("Connected."), mNickname, QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerInfo, QString("^" + mNickname), messageView()));
}

void IRCDCCView::slotChatEnding()
{
	mContact->messenger()->displayMessage(MessageTransport(i18n("The remote user has ended the DCC CHAT session. You will not be allowed to send any more text. Close this window when finished."), mNickname, QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::ServerInfo, QString("^" + mNickname), messageView()));
	mChatObject = 0L;
}

void IRCDCCView::incomingMessage(const QString &message, bool fromMe)
{
	if (fromMe)
		mContact->messenger()->displayMessage(MessageTransport(message, mContact->engine()->nickName(), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::YouSayMsg, QString("^" + mNickname), messageView()));
	else
		mContact->messenger()->displayMessage(MessageTransport(message, mNickname, QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::PrivMsg, QString("^" + mNickname), messageView()));

	// This is for adjusting the colors of the tab when someone says something and this tab isn't focused
	if (mContact->chatWindow()->mTabWidget->currentPage() != mLayout)
		mContact->chatWindow()->mTabWidget->setTabMode(mLayout,
							       IrcChannelTabWidget::eTM_Active);
}

void IRCDCCView::slotReturnPressed()
{
	if (mChatObject == 0)
		return;

	// Always make sure to return all if() statements, otherwise a message will be echoed that tells the user the command is understood (even though the command is ran)
	QString command = messageEdit()->text().section(' ', 0, 0);
	QString text = messageEdit()->text().section(' ', 1);
	if (command[0] == '/')
		command.remove(0, 1);
	else {
		if (messageEdit()->text().isEmpty() == false)
		{
			// Do this so that people can't send messages when not connected, but it will allow them to copy any text out of the line edit for later use if they'd like
			if (mChatObject->state() != QSocket::Connected)
				return;

			if (!mChatObject->sendMessage(messageEdit()->text()))
			{
				QString message = i18n("There was an error sending your message to the DCC connection. You may not be connected any longer.");
				mContact->messenger()->displayMessage(MessageTransport(message, QString(""), QString(""), QString(""), mContact->engine()->nickName(), IRCMessage::UnknownMsg, QString("^" + mNickname), messageView()));
			}
			messageEdit()->clear();
			return;
		} else
			return;
	}
	mContact->parser()->parseCommand(messageView(), IRCCmdParser::Server, command, text, QString(""));
	messageEdit()->clear();
}

void IRCDCCView::slotPageChanged(QWidget *widget)
{
	if (mLayout != 0)
		if (mLayout == widget)
			messageEdit()->setFocus();
}

void IRCDCCView::slotCloseTab()
{
	if (mChatObject != 0)
		mChatObject->close();
	// This should destroy 'this' too since QObject considers 'this' to be a child of mLayout
	delete mLayout;
}



void
IRCDCCView::createLayout()
{
	// create layout

	QVBoxLayout* pVBoxLayout(new QVBoxLayout(this, 2, 2, "MainLayout"));

	QHBoxLayout* pHBoxLayout(new QHBoxLayout(0, 0, 6, "Layout"));

	pHBoxLayout->addWidget(new QLabel(i18n("DCC chat:"), this));

	pHBoxLayout->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));

	QToolButton* closeTab = new QToolButton(this, "closeTab");
	closeTab->setText("X");
	pHBoxLayout->addWidget(closeTab);

	pVBoxLayout->addLayout(pHBoxLayout);

	createMessageView(this);
	pVBoxLayout->addWidget(messageView());

	createMessageEdit(this);
	pVBoxLayout->addWidget(messageEdit());

	// connect slots

	connect(messageEdit(), SIGNAL(returnPressed()), this, SLOT(slotReturnPressed()));
	connect(closeTab, SIGNAL(clicked()), this, SLOT(slotCloseTab()));
}



#include "ircdccview.moc"
/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 8
 * indent-tabs-mode: t
 * End:
 */
// vim: set noet ts=4 sts=4 sw=4:

