/***************************************************************************
                          icqmessage.cpp  -  abstracts messages
                             -------------------
    begin                : Wed Jan 23 2002
    copyright            : (C) 2002 by nbetcher
    email                : nbetcher@usinternet.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#include "ircmessage.h"

#include "messagetransport.h"

#include <kconfig.h>
#include <kdebug.h>
#include <kdeversion.h>
#include <kglobal.h>
#include <klocale.h>
#include <kstandarddirs.h>
#include <kstringhandler.h>

#include <qdatetime.h>
#include <qfile.h>
#include <qstylesheet.h>
#include <qtextedit.h>
#include <qtextstream.h>
#include <qimage.h>

IRCMessage::IRCMessage()
{
	QMimeSourceFactory::defaultFactory()->addFilePath(locate("data", "kopete/pics/"));
	QMimeSourceFactory::defaultFactory()->setImage("private message", QImage(locate("data", "kopete/pics/irc_privmsg.png")));
	QMimeSourceFactory::defaultFactory()->setImage("join", QImage(locate("data", "kopete/pics/irc_join.png")));
	QMimeSourceFactory::defaultFactory()->setImage("part", QImage(locate("data", "kopete/pics/irc_part.png")));
	QMimeSourceFactory::defaultFactory()->setImage("quit", QImage(locate("data", "kopete/pics/irc_quit.png")));
	QMimeSourceFactory::defaultFactory()->setImage("server message", QImage(locate("data", "kopete/pics/irc_servermsg.png")));
	QMimeSourceFactory::defaultFactory()->setImage("unknown", QImage(locate("data", "kopete/pics/irc_unknowncmd.png")));
	QMimeSourceFactory::defaultFactory()->setImage("nickname change", QImage(locate("data", "kopete/pics/irc_nickchange.png")));
	QMimeSourceFactory::defaultFactory()->setImage("action", QImage(locate("data", "kopete/pics/irc_action.png")));
	QMimeSourceFactory::defaultFactory()->setImage("topic", QImage(locate("data", "kopete/pics/irc_topic.png")));
	QMimeSourceFactory::defaultFactory()->setImage("query message", QImage(locate("data", "kopete/pics/irc_querymsg.png")));
	QMimeSourceFactory::defaultFactory()->setImage("motd message", QImage(locate("data", "kopete/pics/irc_motdmsg.png")));
	QMimeSourceFactory::defaultFactory()->setImage("irc info", QImage(locate("data", "kopete/pics/irc_info.png")));
	QMimeSourceFactory::defaultFactory()->setImage("you say", QImage(locate("data", "kopete/pics/irc_yousay.png")));
	QMimeSourceFactory::defaultFactory()->setImage("ctcp message", QImage(locate("data", "kopete/pics/irc_ctcp.png")));
	QMimeSourceFactory::defaultFactory()->setImage("dcc message", QImage(locate("data", "kopete/pics/irc_dcc.png")));
}

void IRCMessage::displayMessage(MessageTransport transport)
{
	QString text;
	QString message = QStyleSheet::escape(transport.message());
	bool colored = false;
	bool sidePics = KGlobal::config()->readBoolEntry("UseSidePics", true);

	KGlobal::config()->setGroup("IRC");

	// Add markup to URLs, only works with KDE > 3.0.x
#if KDE_VERSION >= 306
	message = KStringHandler::tagURLs(message);
#endif

	KGlobal::config()->setGroup("IRC");
	if (KGlobal::config()->readBoolEntry("HighlightNickname", true) == true && message.contains(transport.ournick(), false) == true)
	{
		colored = true;
	}
	QString highlightPhrase = KGlobal::config()->readEntry("HighlightPhrase", "");
	if (KGlobal::config()->readBoolEntry("HighlightOthers", false) == true && message.contains(highlightPhrase, false) == true && highlightPhrase.isEmpty() == false && colored == false)
	{
		colored = true;
	}
	if (colored == true)
	{
		QColor color(175, 8, 8);
		message.prepend(QString("<font color=%1>").arg(KGlobal::config()->readColorEntry("HighlightColor", &color).name()));
		message.append("</font>");
	}

	QString timeStamp;
	if (KGlobal::config()->readBoolEntry("UseTimeStamps", true))
	{
		timeStamp = QStyleSheet::escape(QString(" [%1] ").arg(QTime::currentTime().toString()));
	}

	// This parses every character to find a color, bold, underline, or italic in the message
	bool unterminatedBold = false;
	bool unterminatedColor = false;
	bool unterminatedUnderline = false;
	for (unsigned int i = 0; i != message.length(); i++)
	{
		switch(QChar(message[i]).unicode())
		{
			case 2:
			{
				// Bold
				if (unterminatedBold == true)
				{
					message.remove(i, 1);
					message.insert(i, "</b>");
					unterminatedBold = false;
				} else {
					message.remove(i, 1);
					message.insert(i, "<b>");
					unterminatedBold = true;
				}
				break;
			}
			case 3:
			{
				message.remove(i, 1);
				if (unterminatedColor == true)
				{
					message.insert(i, "</font>");
					unterminatedColor = false;
					if (!QChar(message[i]).isNumber())
						continue;
				}
				// Colors! Or colours for you freaks out there ;)
				QString code;
				if (QChar(message[i + 1]).isNumber())
				{
					code = message[i];
					code.append(message[i + 1]);
					message.remove(i, 2);
				} else {
					code = message[i];
					message.remove(i, 1);
				}
				switch (code.toUInt())
				{
					case 0:
					{
						message.insert(i, "<font color=#FFFFFF>");
						unterminatedColor = true;
						break;
					}
					case 1:
					{
						message.insert(i, "<font color=#000000>");
						unterminatedColor = true;
						break;
					}
					case 2:
					{
						message.insert(i, "<font color=#00008C>");
						unterminatedColor = true;
						break;
					}
					case 3:
					{
						message.insert(i, "<font color=#006400>");
						unterminatedColor = true;
						break;
					}
					case 4:
					{
						message.insert(i, "<font color=#E60000>");
						unterminatedColor = true;
						break;
					}
					case 5:
					{
						message.insert(i, "<font color=#960000");
						unterminatedColor = true;
						break;
					}
					case 6:
					{
						message.insert(i, "<font color=#500050>");
						unterminatedColor = true;
						break;
					}
					case 7:
					{
						message.insert(i, "<font color=#FF5A00>");
						unterminatedColor = true;
						break;
					}
					case 8:
					{
						message.insert(i, "<font color=#FFFF00>");
						unterminatedColor = true;
						break;
					}
					case 9:
					{
						message.insert(i, "<font color=#00FF00>");
						unterminatedColor = true;
						break;
					}
					case 10:
					{
						message.insert(i, "<font color=#0096B4>");
						unterminatedColor = true;
						break;
					}
					case 11:
					{
						message.insert(i, "<font color=#AAAAFF>");
						unterminatedColor = true;
						break;
					}
					case 12:
					{
						message.insert(i, "<font color=#0F0FFF>");
						unterminatedColor = true;
						break;
					}
					case 13:
					{
						message.insert(i, "<font color=#C800C8>");
						unterminatedColor = true;
						break;
					}
					case 14:
					{
						message.insert(i, "<font color=#505050>");
						unterminatedColor = true;
						break;
					}
					case 15:
					{
						message.insert(i, "<font color=#AAAAAA>");
						unterminatedColor = true;
						break;
					}
				}
			break;
			}
			case 31:
			{
				// Underline
				if (unterminatedUnderline == true)
				{
					message.remove(i, 1);
					message.insert(i, "</u>");
					unterminatedUnderline = false;
				} else {
					message.remove(i, 1);
					message.insert(i, "<u>");
					unterminatedUnderline = true;
				}
				break;
			}
		}
	}
	
	if (unterminatedBold == true)
		message.append("</b>");

	if (unterminatedColor == true)
		message.append("</font>");

	if (unterminatedUnderline == true)
		message.append("</u>");

	switch (transport.type())
	{
		case 0:
		{
			if (sidePics == true)
				text.append("<img source=\"private message\">");

			text.append(timeStamp);
			text.append("<b>");
			text.append(QStyleSheet::escape(QString("<%1>").arg(transport.nickname())));
			text.append("</b> ");
			text.append(message);
			break;
		}
		case 1:
		{
			if (sidePics == true)
				text.append("<img source=\"join\">");

			text.append(timeStamp);
			text.append("<b>");
			text.append(QStyleSheet::escape(transport.nickname()));
			text.append("</b>");
			text.append(QStyleSheet::escape(i18n(" [%1] has joined ").arg(transport.hostname())));
			text.append(QStyleSheet::escape(transport.channel()));
			break;
		}
		case 2:
		{
			if (sidePics == true)
				text.append("<img source=\"part\">");

			text.append(timeStamp);
			text.append("<b>");
			text.append(QStyleSheet::escape(transport.nickname()));
			text.append("</b>");
			text.append(QStyleSheet::escape(i18n(" [%1] has left ").arg(transport.hostname())));
			text.append(QStyleSheet::escape(transport.channel()));
			text.append(QString(": %1").arg(message));
			break;
		}
		case 3:
		{
			if (sidePics == true)
				text.append("<img source=\"quit\">");

			text.append(timeStamp);
			text.append("<b>");
			text.append(QStyleSheet::escape(transport.nickname()));
			text.append("</b>");
			text.append(QStyleSheet::escape(i18n(" [%1] has quit IRC ").arg(transport.hostname())));
			text.append(QString(": %1").arg(message));
			break;
		}
		case 4:
		{
			if (sidePics == true)
				text.append("<img source=\"server message\">");

			text.append(timeStamp);
			text.append(transport.message());
			break;
		}
		case 5:
		{
			if (sidePics == true)
				text.append("<img source=\"unknown\">");

			text.append(timeStamp);
			text.append(transport.message());
			break;
		}
		case 6:
		{
			if (sidePics == true)
				text.append("<img source=\"nickname change\">");

			text.append(timeStamp);
			text.append(message);
			break;
		}
		case 7:
		{
			if (sidePics == true)
				text.append("<img source=\"action\">");

			text.append(timeStamp);
			text.append(QStyleSheet::escape(transport.nickname()));
			text.append(QString(" %1").arg(message));
			break;
		}
		case 8:
		{
			if (sidePics == true)
				text.append("<img source=\"topic\">");

			text.append(timeStamp);
			text.append(QString("<b>%1</b> has changed the topic for %2 to %3").
					arg(QStyleSheet::escape(transport.nickname())).
					arg(QStyleSheet::escape(transport.channel())).
					arg(message));
			break;
		}
		case 9:
		{
			if (sidePics == true)
				text.append("<img source=\"query message\">");

			text.append(timeStamp);
			text.append("<b>");
			text.append(QStyleSheet::escape(QString("<%1>").arg(transport.nickname())));
			text.append("</b> ");
			text.append(message);
			break;
		}
		case 10:
		{
			if (sidePics == true)
				text.append("<img source=\"you say\">");

			text.append(timeStamp);
			text.append("<b>");
			text.append(QStyleSheet::escape(QString("<%1>").arg(transport.nickname())));
			text.append("</b> ");
			text.append(message);
			break;
		}
		case 11:
		{
			if (sidePics == true)
				text.append("<img source=\"motd message\">");

			text.append(timeStamp);
			text.append(message);
			break;
		}
		case 12:
		{
			if (sidePics == true)
				text.append("<img source=\"irc info\">");

			text.append(timeStamp);
			text.append(message);
			break;
		}
		case 13:
		{
			if (sidePics == true)
				text.append("<img source=\"ctcp message\">");

			text.append(timeStamp);
			text.append(message);
			break;
		}
		case 14:
		{
			if (sidePics == true)
				text.append("<img source=\"part\">");

			text.append(timeStamp);
			text.append("<b>%1</b> was kicked from %2 by %3 (%4)").
					arg(QStyleSheet::escape(transport.nickname())).
					arg(transport.channel()).
					arg(transport.hostname()).
					arg(message);
			break;
		}
		case 15:
		{
			if (sidePics == true)
				text.append("<img source=\"dcc message\">");

			text.append(timeStamp);
			text.append(message);
			break;
		}
		case 16: /* Whois */
		{
			if (sidePics == true)
				text.append("<img source=\"irc info\">");

			text.append(timeStamp);
			text.append(message);
			break;
		}
		case 17:
		{
			if (sidePics == true)
				text.append("<img source=\"query message\">");

			text.append(timeStamp);
			text.append("NOTICE: <b>");
			text.append(QStyleSheet::escape(QString("<%1>").arg(transport.nickname())));
			text.append("</b> ");
			text.append(message);
			break;
		}
	}

#if QT_VERSION < 305
	// caused by a bug in Qt < 3.0.5 we need <br> to separate
	// the lines in QTextBrowser
 	text.append("<br>");
	transport.chatView()->append(text);
#else
	transport.chatView()->append(text);
	// this <br> is needed by the logs
 	text.append("<br>");
#endif

	KGlobal::config()->setGroup("IRC");
	if (KGlobal::config()->readBoolEntry("LogChannels", true) && transport.identifier()[0] == '#')
	{
		QDate now;
		QFile log(locateLocal("data", "kopete/irc_logs/%1@%2.html").arg(transport.identifier()).arg(now.toString()));
		if (log.open(IO_ReadWrite))
		{
			log.at(log.size());
			QTextStream t( &log );
			t.setEncoding(QTextStream::Locale);
			t << text << endl;
		}
	}
	if (KGlobal::config()->readBoolEntry("LogQueries", true) && transport.identifier()[0] != '#' && transport.identifier()[0] != '(' && transport.identifier()[0] != '^' && transport.identifier()[0] != '@')
	{
		QDate now;
		QFile log(locateLocal("data", "kopete/irc_logs/%1@%2.html").arg(transport.identifier()).arg(now.toString()));
		if (log.open(IO_ReadWrite))
		{
			log.at(log.size());
			QTextStream t( &log );
			t.setEncoding(QTextStream::Locale);
			t << text << endl;
		}
	}
	if (KGlobal::config()->readBoolEntry("LogDcc", false) && transport.identifier()[0] == '^')
	{
		QDate now;
		QFile log(locateLocal("data", "kopete/irc_logs/DCC!%1@%2.html").arg(transport.identifier()).arg(now.toString()));
		if (log.open(IO_ReadWrite))
		{
			log.at(log.size());
			QTextStream t( &log );
			t.setEncoding(QTextStream::Locale);
			t << text << endl;
		}
	}
	if (KGlobal::config()->readBoolEntry("LogServers", false) && transport.identifier()[0] == '@')
	{
		QDate now;
		QFile log(locateLocal("data", "kopete/irc_logs/%1@%2.html").arg(transport.identifier()).arg(now.toString()));
		if (log.open(IO_ReadWrite))
		{
			log.at(log.size());
			QTextStream t( &log );
			t.setEncoding(QTextStream::Locale);
			t << text << endl;
		}
	}
}


/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 8
 * indent-tabs-mode: t
 * End:
 */
// vim: set noet ts=4 sts=4 sw=4:

