#include "tabcompleter.h"

#include <qregexp.h>



// Njaard didn't just write Noatun ;)
TabCompleter::TabCompleter(QWidget *parent, const char *name)
	: QLineEdit(parent, name)
{
}

void TabCompleter::add(const QString &name)
{
	// add it in sorted
	for(QStringList::Iterator i(mWords.begin()); i != mWords.end(); ++i)
	{
		if ((*i).lower() < name.lower())
		{
			++i;
			mWords.insert(i, name);
			return;
		}
		else if ((*i).lower() == name.lower())
		{
			return;
		}
	}
	mWords.append(name);
}

void TabCompleter::remove(const QString &name)
{
	for(QStringList::Iterator i(mWords.begin()); i != mWords.end(); ++i)
	{
		if ((*i).lower() == name.lower())
		{
			mWords.remove(i);
			return;
		}
	}
}

void TabCompleter::keyPressEvent(QKeyEvent *e)
{

	if ((e->state() & Qt::KeyButtonMask) != 0)
	{
		QLineEdit::keyPressEvent(e);
		return;
	}

	if (e->key() == Key_Return )
	{
		mHistory.addEntry(text());
		QLineEdit::keyPressEvent(e);
		return;
	}

	if (e->key() == Key_Up )
	{
		setText(mHistory.prevEntry());
		return;
	}

	if (e->key() == Key_Down )
	{
		setText(mHistory.nextEntry());
		return;
	}

	// </remove this and die>
	
	if (e->key()!=Key_Tab)
	{
		pos = -1;
		inc=0;
		QLineEdit::keyPressEvent(e);
		return;
	}

	QString word;
	if (!inc)
	{
		int index=cursorPosition();
//		cerr << index << " " << pos << " " <<lenNew << " " << pos+lenNew<<endl;
		if (pos!=-1 && (index != pos+lenNew))
		{
			inc=0;
			pos=-1;
			return;
		}

		word=text();
		// we're either at EOL or EOW
		if (word.length()>(uint)index && word[index]!=' ')
			return;
		word.remove(index, (int)word.length());

		{
			QRegExp e(" ([^ ]*)$");
			if (e.search(word)>=0)
				word=e.cap(1);
		}

		pos=index-word.length();
		len=word.length();
	}
	else
	{
		word=text();
		word.remove(pos+len, lenNew-len);
		setText(word);
		word=text().mid(pos, len);
	}

	// now, find the first word that begins with the current one

	int oneFound=0;


	QString colon;
	if (pos==0)
		colon=": ";

	for (uint i=0; i<mWords.count(); i++)
	{
		if (mWords[i].lower().startsWith(word.lower()))
		{
			oneFound++;
			if (oneFound != inc+1)
				continue;
//			cerr << "trying" <<endl;
			lenNew=mWords[i].length();

			if (pos==0) lenNew+=2;

			QString line=text();
			line.replace(pos, len, mWords[i]+colon);
			setText(line);
			setCursorPosition(pos+lenNew);
			inc++;
			return;
		}
	}
	setCursorPosition(pos+len);
	inc=0;
}

bool TabCompleter::event( QEvent *e )
{
	if (e->type()==QEvent::KeyPress)
	{
		if (static_cast<QKeyEvent*>(e)->key()==Key_Tab)
		{
			keyPressEvent(static_cast<QKeyEvent*>(e));
			return true;
		}
	}

	return QLineEdit::event(e);
}

#include "tabcompleter.moc"

/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 8
 * indent-tabs-mode: t
 * End:
 */
// vim: set noet ts=4 sts=4 sw=4:

