/***************************************************************************
                          ircqueryiew.cpp  -  description
                             -------------------
    begin                : Sun Feb 24 2002
    copyright            : (C) 2002 by nbetcher
    email                : nbetcher@usinternet.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ircqueryview.h"

#include "irc_channel_tabwidget.h"
#include "ircchatwindow.h"
#include "irccmdparser.h"
#include "irccontact.h"
#include "ircmessage.h"
#include "ircservercontact.h"
#include "kirc.h"
#include "messagetransport.h"
#include "tabcompleter.h"

#include <klocale.h>

#include <qlabel.h>
#include <qlayout.h>
#include <qtextedit.h>
#include <qtoolbutton.h>
#include <qvbox.h>

IRCQueryView::IRCQueryView(const QString &, const QString &nickname, IRCServerContact *contact, QWidget *parent, IRCContact *mainContact)
	: IrcViewBase(parent)
{
	createLayout();

	m_ircContact = mainContact;
	m_serverContact = contact;
	m_nickname = nickname;

	if (m_serverContact->engine()->isLoggedIn() == false)
	{
		messageEdit()->setEnabled(false);
		connect(m_serverContact->engine(), SIGNAL(connectedToServer()), this, SLOT(slotConnected()));
	}
	connect(m_serverContact->engine(), SIGNAL(connectionClosed()), this, SLOT(slotServerQuit()));
	connect(m_serverContact->engine(), SIGNAL(connectedToServer()), this, SLOT(slotServerReady()));
	connect(m_serverContact->engine(), SIGNAL(incomingPrivMessage(const QString &, const QString &, const QString &)), this, SLOT(incomingMessage(const QString &, const QString &, const QString &)));
	connect(m_serverContact->engine(), SIGNAL(incomingPrivAction(const QString &, const QString &, const QString &)), this, SLOT(incomingAction(const QString &, const QString &, const QString &)));
	connect(m_serverContact->engine(), SIGNAL(incomingNoNickChan(const QString &)), this, SLOT(incomingNoNickChan(const QString &)));
	connect(m_serverContact->chatWindow()->mTabWidget, SIGNAL(currentChanged(QWidget *)), this, SLOT(slotPageChanged(QWidget *)));
	connect(m_serverContact->engine(), SIGNAL(incomingNotice(const QString &, const QString &)), this, SLOT(incomingNotice(const QString &, const QString &)));
}

void IRCQueryView::slotCloseTab()
{
	m_ircContact->unloading();
}

void IRCQueryView::slotPageChanged(QWidget *widget)
{
	if (m_ircContact->tabPage() != 0)
	{
		if (m_ircContact->tabPage() == widget)
		{
			messageEdit()->setFocus();
		}
	}
}

void IRCQueryView::slotServerQuit()
{
	messageEdit()->setEnabled(false);
}

void IRCQueryView::slotServerReady()
{
	messageEdit()->setEnabled(true);
}

void IRCQueryView::incomingNoNickChan(const QString &nickname)
{
	QVBox *parent = m_ircContact->tabPage();
	if (parent != 0)
	{
		if (m_serverContact->chatWindow()->mTabWidget->label(m_serverContact->chatWindow()->mTabWidget->indexOf(parent)).lower() != m_nickname.lower())
		{
			return;
		}
	} else {
		return;
	}
	QString message = i18n("No such nickname or channel: %1").arg(nickname);
	m_serverContact->messenger()->displayMessage(MessageTransport(message, QString::null, QString::null, QString::null, m_serverContact->engine()->nickName(), IRCMessage::ServerMsg, QString(m_nickname + "-" + m_serverContact->serverName()), messageView()));
}

void IRCQueryView::slotConnected()
{
	messageEdit()->setEnabled(false);
}

void IRCQueryView::slotReturnPressed()
{
	// Always make sure to return all if() statements, otherwise a message will be echoed that tells the user the command is understood (even though the command is ran)
	QString command = messageEdit()->text().section(' ', 0, 0);
	QString text = messageEdit()->text().section(' ', 1);
	if (command[0] == '/')
	{
		command.remove(0, 1);
	} else {
		if (messageEdit()->text().isEmpty() == false)
		{
			m_serverContact->engine()->messageContact(m_nickname, messageEdit()->text());
			messageEdit()->clear();
			return;
		} else {
			return;
		}
	}
	m_serverContact->parser()->parseCommand(messageView(), IRCCmdParser::Query, command, text, m_nickname);
	messageEdit()->clear();
}

void IRCQueryView::incomingAction(const QString &originating, const QString &target, const QString &message)
{
	QString from = originating.section('!', 0, 0);
	if (from.lower() != m_nickname.lower())
	{
		if (from.lower() != m_serverContact->engine()->nickName().lower())
		{
			return;
		} else {
			if (target.lower() != m_nickname.lower())
			{
				return;
			}
		}
	}
	m_serverContact->messenger()->displayMessage(MessageTransport(message, from, QString::null, QString::null, m_serverContact->engine()->nickName(), IRCMessage::Action, QString(m_nickname + "-" + m_serverContact->serverName()), messageView()));
}

void IRCQueryView::incomingMessage(const QString &originating, const QString &target, const QString &message)
{
	QString from = originating.section('!', 0, 0);
	if (from.lower() != m_nickname.lower())
	{
		if (from.lower() != m_serverContact->engine()->nickName().lower())
		{
			return;
		} else {
			if (target.lower() != m_nickname.lower())
			{
				return;
			}
		}
	}

	if (originating.lower() == m_serverContact->engine()->nickName().lower())
	{
		m_serverContact->messenger()->displayMessage(MessageTransport(message, originating.section('!', 0, 0), QString::null, target, m_serverContact->engine()->nickName(), IRCMessage::YouSayMsg, QString(m_nickname + "-" + m_serverContact->serverName()), messageView()));
	} else {
		m_serverContact->messenger()->displayMessage(MessageTransport(message, originating.section('!', 0, 0), QString::null, target, m_serverContact->engine()->nickName(), IRCMessage::QueryMsg, QString(m_nickname + "-" + m_serverContact->serverName()), messageView()));
	}
	if (m_serverContact->chatWindow()->mTabWidget->currentPage() != m_ircContact->tabPage())
	{
		IrcChannelTabWidget::ETabMode eMode(IrcChannelTabWidget::eTM_Active);
		if (message.contains(m_serverContact->engine()->nickName()) >= 1)
		{
			eMode = IrcChannelTabWidget::eTM_Highlighted;
		}

		m_serverContact->chatWindow()->mTabWidget->setTabMode(m_ircContact->tabPage(),
								      eMode);
	}
}

IRCQueryView::~IRCQueryView()
{
}

void
IRCQueryView::createLayout()
{
	// create layout

	QVBoxLayout* pVBoxLayout(new QVBoxLayout(this, 2, 2, "MainLayout"));

	QHBoxLayout* pHBoxLayout(new QHBoxLayout(0, 0, 6, "Layout"));

	pHBoxLayout->addWidget(new QLabel(tr2i18n("Query:"), this));

	pHBoxLayout->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));

	QToolButton* closeTab = new QToolButton(this, "closeTab");
	closeTab->setText(tr2i18n("X"));
	pHBoxLayout->addWidget(closeTab);

	pVBoxLayout->addLayout(pHBoxLayout);

	createMessageView(this);
	pVBoxLayout->addWidget(messageView());

	createMessageEdit(this);
	pVBoxLayout->addWidget(messageEdit());

	// connect slots

	connect(messageEdit(), SIGNAL(returnPressed()), this, SLOT(slotReturnPressed()));
	connect(closeTab, SIGNAL(clicked()), this, SLOT(slotCloseTab()));
}

void IRCQueryView::incomingNotice(const QString &originating, const QString &message)
{
	if (m_serverContact->chatWindow()->mTabWidget->currentPage() == m_serverContact->tabView())
		m_serverContact->messenger()->displayMessage(MessageTransport(message, originating, QString(""), QString(""), m_serverContact->engine()->nickName(), IRCMessage::Notice, QString(originating + "-" + m_serverContact->serverName()), messageView()));
}

#include "ircqueryview.moc"
/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 8
 * indent-tabs-mode: t
 * End:
 */
// vim: set noet ts=4 sts=4 sw=4:

