 /*
    jabbermessagemanager.cpp

    Copyright (c) 2002 by Till Gerken <till@tantalo.net>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include <kdebug.h>
#include <qptrlist.h>
 
#include "psi/libpsi/src/tasks.h"
#include "psi/libpsi/src/types.h"
 
#include "kopetecontact.h"
#include "kopetemessagemanager.h"
#include "kopetemessagemanagerfactory.h"
#include "jabberprotocol.h"
#include "jabbercontact.h"
#include "jabbermessagemanager.h"
#include "jabbermessagemanager.moc"

JabberMessageManager::JabberMessageManager(const KopeteContact *user,
										   KopeteContactPtrList others,
										   const KopeteMessageManager::WidgetType type,
										   const char *name)
					: KopeteMessageManager(user, others, JabberProtocol::protocol(), 0,
										   type, JabberProtocol::protocol(), name )
{

	// let Kopete's factory know about us
	KopeteMessageManagerFactory::factory()->addKopeteMessageManager(this);

	// this slot will forward messages to the protocol
	connect( this, SIGNAL( messageSent( const KopeteMessage&,
		KopeteMessageManager* ) ),
		this, SLOT( slotMessageSent( const KopeteMessage&,
		KopeteMessageManager* ) ) );

	// in order to use auto-deletion, we have to create a chat window
	// first or we will self-destruct ourselves right away
	//newChatView();
	//setCanBeDeleted(true);

}

KopeteContact *JabberMessageManager::getContact(const QString &name)
{

	KopeteContactPtrList memberList = members();

	for(KopeteContact *contact = memberList.first(); contact != 0L; contact = memberList.next())
	{
		if(contact->displayName() == name)
		{
			return contact;
		}
	}

	return 0L;

}

void JabberMessageManager::removeContact(const QString &name)
{

	KopeteContactPtrList memberList = members();
	
	for(KopeteContact *contact = memberList.first(); contact != 0L; contact = memberList.next())
	{
		if(contact->displayName() == name)
		{
			KopeteMessageManager::removeContact(contact);
		}
	}

}

void JabberMessageManager::km2jm(const KopeteMessage &km, Jabber::Message &jm)
{
	JabberContact *to = dynamic_cast<JabberContact *>(km.to().first());
	const JabberContact *from = dynamic_cast<const JabberContact *>(km.from());

	// ugly hack, Jabber::Message does not have a setFrom() method
	Jabber::Message jabMessage(Jabber::Jid(from->userId()));

	// if a resource has been selected for this contact,
	// send to this special resource - otherwise,
	// just send to the server and let the server decide
	if (!to->resource().isNull())
		jabMessage.setTo(Jabber::Jid(QString("%1/%2").arg(to->userId(), 1).arg(to->resource(), 2)));
	else
		jabMessage.setTo(Jabber::Jid(to->userId()));

	//jabMessage.setFrom(from->userId();
	jabMessage.setBody(km.plainBody());
	jabMessage.setSubject(km.subject());

	if(widgetType() == KopeteMessageManager::ChatWindow)
		jabMessage.setType("chat");
	else
		jabMessage.setType("normal");


	jm = jabMessage;

}

void JabberMessageManager::slotMessageSent(const KopeteMessage &message, KopeteMessageManager *)
{
	Jabber::Message jabMessage;

	kdDebug(14130) << "[JabberMessageManager] slotMessageSent called: " << message.body() << endl;

	km2jm(message, jabMessage);

	// pass message on to protocol backend
	JabberProtocol::protocol()->slotSendMessage(jabMessage);

	// append message to window
	appendMessage(message);

}

JabberMessageManager::~JabberMessageManager()
{
}
