 /*
    jabberprefs.cpp

    Copyright (c) 2002 by Daniel Stone <dstone@kde.org>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include <qcheckbox.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qcombobox.h>

#include <kconfig.h>
#include <kglobal.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <knuminput.h>

#include "dlgjabberprefs.h"
#include "jabberprefs.h"

JabberPreferences::JabberPreferences(const QString &pixmap,
				     JabberProtocol *parent)
:ConfigModule(i18n("Jabber Plugin"), i18n("Jabber Protocol"), pixmap,
	      parent)
{
	protocol = parent;
	(new QVBoxLayout(this))->setAutoAdd(true);
	preferencesDialog = new DlgJabberPrefs(parent, this);

	reopen();
	preferencesDialog->settings_changed = false;
}

JabberPreferences::~JabberPreferences()
{
}

void JabberPreferences::reopen()
{
	KGlobal::config()->setGroup("Jabber");
	preferencesDialog->mID->setText(KGlobal::config()->readEntry("UserID", i18n("<your Jabber ID>")));
	preferencesDialog->mPass->setText(KGlobal::config()->readEntry("Password", i18n("<your Jabber password>")));
	preferencesDialog->mResource->setText(KGlobal::config()->readEntry("Resource", "Kopete"));
	preferencesDialog->mServer->setText(KGlobal::config()->readEntry("Server", "jabber.org"));

	// START Order Important =====
	// !!! First set the checkbox, THEN the port
	// if done the other way round: checking the checkbox triggers +/- 1 for the port-value
	// ( see DlgJabberPrefs::sslToggled() )
	preferencesDialog->chkUseSSL->setChecked(KGlobal::config()->readBoolEntry("UseSSL", false));

	preferencesDialog->mPort->setValue( KGlobal::config()->readNumEntry("Port", 5222) );
	// END Order Important =====

	QString auth = KGlobal::config()->readEntry("AuthType", "digest");
	preferencesDialog->cmbAuth->setCurrentItem(0);
	if(auth == QString("plain"))
		preferencesDialog->cmbAuth->setCurrentItem(1);

	QString proxyType = KGlobal::config()->readEntry("ProxyType", "None");
	preferencesDialog->cbProxyType->setCurrentItem(0);
	if(proxyType == QString("HTTPS"))
		preferencesDialog->cbProxyType->setCurrentItem(1);
	else
	if(proxyType == QString("SOCKS4"))
		preferencesDialog->cbProxyType->setCurrentItem(2);
	else
	if(proxyType == QString("SOCKS5"))
		preferencesDialog->cbProxyType->setCurrentItem(3);

	preferencesDialog->leProxyName->setText(KGlobal::config()->readEntry("ProxyName", ""));
	preferencesDialog->spbProxyPort->setValue(KGlobal::config()->readNumEntry("ProxyPort", 8080));
	preferencesDialog->cbProxyAuth->setChecked(KGlobal::config()->readBoolEntry("ProxyAuth", false));
	preferencesDialog->leProxyUser->setText(KGlobal::config()->readEntry("ProxyUser", ""));
	preferencesDialog->leProxyPass->setText(KGlobal::config()->readEntry("ProxyPass", ""));

	preferencesDialog->mAutoConnect->setChecked(KGlobal::config()->readBoolEntry("AutoConnect", false));
	preferencesDialog->mLogAll->setChecked(KGlobal::config()->readBoolEntry("LogAll", true));
	preferencesDialog->mPreferEmail->setChecked(KGlobal::config()->readBoolEntry("EmailDefault", false));

}

void JabberPreferences::save()
{
	KConfig *config = KGlobal::config();

	if (preferencesDialog->settings_changed)
		if ( protocol->isConnected())
			// TODO please correct my english
			KMessageBox::information(0L, i18n("The changes you did become operative after you logout and login with Jabber."), i18n("Jabber Changes During Online Jabber Session"),"jabberChangesDuringOnline " );

	config->setGroup("Jabber");
	if (preferencesDialog->mID->text() != i18n("<your Jabber ID>")) {
		config->writeEntry("UserID", preferencesDialog->mID->text());
	}
	if (preferencesDialog->mPass->text() != i18n("<your Jabber password>")) {
		config->writeEntry("Password", preferencesDialog->mPass->text());
	}
	config->writeEntry("Resource", preferencesDialog->mResource->text());
	config->writeEntry("Server", preferencesDialog->mServer->text());

	config->writeEntry("Port", preferencesDialog->mPort->value());

	config->writeEntry("AutoConnect", preferencesDialog->mAutoConnect->isChecked());
	config->writeEntry("UseSSL", preferencesDialog->chkUseSSL->isChecked());
	config->writeEntry("LogAll", preferencesDialog->mLogAll->isChecked());
	config->writeEntry("EmailDefault", preferencesDialog->mPreferEmail->isChecked());

	switch(preferencesDialog->cmbAuth->currentItem())
	{
		case 0:		config->writeEntry("AuthType", "digest");
					break;
		case 1:		config->writeEntry("AuthType", "plain");
					break;
		default:	// this case should never happen, just
					// implemented for safety
					config->writeEntry("AuthType", "digest");
					break;
	}

	switch(preferencesDialog->cbProxyType->currentItem())
	{
		case 0:		config->writeEntry("ProxyType", "None");
					break;
		case 1:		config->writeEntry("ProxyType", "HTTPS");
					break;
		case 2:		config->writeEntry("ProxyType", "SOCKS4");
					break;
		case 3:		config->writeEntry("ProxyType", "SOCKS5");
					break;
		default:	// this case should never happen, just
					// implemented for safety
					config->writeEntry("ProxyType", "None");
					break;
	}

	config->writeEntry("ProxyName", preferencesDialog->leProxyName->text());
	config->writeEntry("ProxyPort", preferencesDialog->spbProxyPort->value());
	config->writeEntry("ProxyAuth", preferencesDialog->cbProxyAuth->isChecked());
	config->writeEntry("ProxyUser", preferencesDialog->leProxyUser->text());
	config->writeEntry("ProxyPass", preferencesDialog->leProxyPass->text());

	config->sync();

	preferencesDialog->settings_changed = false;

	emit saved();
}

#include "jabberprefs.moc"
/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 8
 * indent-tabs-mode: t
 * End:
 */
// vim: set noet ts=4 sts=4 sw=4:

