#include"console.h"

#include<qtimer.h>

#include<stdio.h>
#include<sys/time.h>
#include<sys/types.h>
#include<unistd.h>

#define STDIN 0

Console::Console(const QString &_prompt)
{
	index = 0;
	cmdline[0] = 0;
	prompt = _prompt;
	needprompt = 1;

	con_init();

	QTimer *t = new QTimer(this);
	connect(t, SIGNAL(timeout()), SLOT(eventloop()));
	t->start(1);
}

Console::~Console()
{
	con_deinit();
}

void Console::eventloop()
{
	int c;

	if(needprompt) {
		printf("%s", prompt.latin1());
		fflush(stdout);
		needprompt = 0;
	}

	if(con_kbhit()) {
		c = con_getch();
		if(c == 8 || c == 127) {
			if(index) {
				printf("\b \b");
				cmdline[--index] = 0;
			}
		}
		else if(c == '\n') {
			printf("\n");
			needprompt = 1;
			if(index) {
				cmd = cmdline;
				index = 0;
				cmdline[0] = 0;

				// send command signal
				command(cmd);
			}
		}
		else if(c >= 32 && c <= 126 && index < 255) {
			cmdline[index++] = c;
			cmdline[index] = 0;
			putchar(c);
		}
		fflush(stdout);
	}
}

void Console::setPrompt(const QString &str)
{
	prompt = str;
}

void Console::con_init()
{
	struct termios attr;

	// fd 0 is STDIN
	tcgetattr(STDIN, &attr);
	old_term_attr = attr;

	attr.c_lflag &= ~(ECHO);        // turn off the echo flag
	attr.c_lflag &= ~(ICANON);      // no wait for a newline
	attr.c_cc[VMIN] = 1;            // read at least 1 char

	// set the new attributes
	tcsetattr(0, TCSAFLUSH, &attr);
}

void Console::con_deinit()
{
	tcsetattr(STDIN, TCSANOW, &old_term_attr);
}

int Console::con_getch()
{
	int c;

	c = getchar();       /* now getchar won't wait for newline */

	return c;
}

int Console::con_kbhit()
{
	struct timeval zero_time;
	fd_set getch_readfds;

	zero_time.tv_sec = 0;
	zero_time.tv_usec = 0;

	FD_ZERO(&getch_readfds);
	FD_SET(STDIN, &getch_readfds);

	select(STDIN+1, &getch_readfds, NULL, NULL, &zero_time);

	return (FD_ISSET(STDIN, &getch_readfds));
}

void Console::reshowPrompt()
{
	printf("%s%s", prompt.latin1(), cmdline);
	fflush(stdout);

	needprompt = 0;
}

void Console::erasePrompt()
{
	// erase line
	printf("\r%c[2K", 27);
	fflush(stdout);
}

void Console::append(const QString &str)
{
	erasePrompt();
	printf("%s", str.latin1());
	fflush(stdout);
	reshowPrompt();
}

