//#define TEST_PRIVATE

#include"jabbertest.h"
#include"stream.h"
#include"dtcp.h"
#include"jidstream.h"
#include<qtimer.h>

#include<stdlib.h>


JabberTest::JabberTest()
:QObject(0)
{
	use_debug = true;

	console = new Console("JabberTest> ");
	connect(console, SIGNAL(command(const QString &)), SLOT(doCommand(const QString &)));

	client = new Client;

	// setup direct-tcp
	DTCPManager *m = client->dtcpManager();
	QStringList l;
	l += "127.0.0.1";
	m->setHostList(l);
	m->setServer(8000);

	connect(client, SIGNAL(debugText(const QString &)), SLOT(clientDebug(const QString &)));
	connect(client, SIGNAL(error(const StreamError &)), SLOT(clientError(const StreamError &)));
	connect(client, SIGNAL(closeFinished()), SLOT(clientCloseFinished()));
	connect(client, SIGNAL(handshaken()), SLOT(clientHandshaken()));
	connect(client, SIGNAL(resourceAvailable(const Jid &, const Resource &)), SLOT(clientResourceAvailable(const Jid &, const Resource &)));
	connect(client, SIGNAL(resourceUnavailable(const Jid &, const Resource &)), SLOT(clientResourceUnavailable(const Jid &, const Resource &)));
	connect(client, SIGNAL(messageReceived(const Message &)), SLOT(clientMessageReceived(const Message &)));
	connect(client, SIGNAL(authFinished(bool, int, const QString &)), SLOT(authFinished(bool, int, const QString &)));
	connect(client, SIGNAL(rosterRequestFinished(bool, int, const QString &)), SLOT(rosterRequestFinished(bool, int, const QString &)));
}

JabberTest::~JabberTest()
{
	delete client;
	delete console;
}

void JabberTest::doCommand(const QString &cmd)
{
	QStringList argv = QStringList::split(' ', cmd);
	int argc = argv.count();

	if(argv[0] == "quit")
		bail();
	else if(argv[0] == "login") {
		if(argc < 4) {
			console->append("login [host] [user] [pass]\n");
		}
		else {
			host = argv[1];
			user = argv[2];
			pass = argv[3];

			console->append("Connecting...\n");
			client->connectToHost(host);
		}
	}
	else if(argv[0] == "logout") {
		if(client->isActive()) {
			QString str;
			str += "<presence type=\"unavailable\">\n";
			str += " <status>Logged out</status>\n";
			str += "</presence>\n";

			client->send(str);
			client->close();
			console->append("Logging out...\n");
		}
		else
			console->append("Not connected.\n");
			
	}
	else if(argv[0] == "w") {
		int type = 0;
		if(argc > 1) {
			if(argv[1] == "online")
				type = 0;
			else if(argv[1] == "away")
				type = 1;
			else if(argv[1] == "offline")
				type = 2;
			else if(argv[1] == "all")
				type = 3;
		}
		displayRoster(type);
	}
	else if(argv[0] == "m") {
		if(argc < 3) {
			console->append("m [to] [words...]\n");
		}
		else {
			if(!client->isAuthenticated())
				console->append("Not logged in.\n");
			else {
				QString body;
				for(int n = 2; n < argc; ++n) {
					if(n != 2)
						body += ' ';
					body += argv[n];
				}
				Message m(argv[1]);
				m.setBody(body);
				client->sendMessage(m);
			}
		}
	}
	else if(argv[0] == "jstest") {
		if(argc < 2) {
			console->append("jstest [jid]\n");
		}
		else {
			if(!client->isAuthenticated())
				console->append("Not logged in.\n");
			else {
				Jid j(argv[1]);
				JidStream *s = new JidStream(client);
				s->connectToJid(j, "abcdefg");
			}
		}
	}
	else if(argv[0] == "jslist") {
		console->append("---- JidStream listing begin ----\n");

		JidStreamListIt it(*client->jsManager()->streamList());
		for(JidStream *i; (i = it.current()); ++it) {
			QString s;
			if(i->state() == JidStream::Negotiating)
				s = "Neg";
			else if(i->state() == JidStream::Establishing)
				s = "Est";
			else if(i->state() == JidStream::Connected)
				s = "Con";
			else if(i->state() == JidStream::Closing)
				s = "Clo";
			else if(i->state() == JidStream::Idle)
				s = "Idl";
			else if(i->state() == JidStream::Waiting)
				s = "Wai";

			QString dstr; dstr.sprintf("%s %s {%s}\n", s.latin1(), i->peer().full().latin1(), i->key().latin1());
			console->append(dstr);
		}

		console->append("---- JidStream listing end   ----\n\n");
	}
	else if(argv[0] == "dtcplist") {
		console->append("---- DTCPConnection listing begin ----\n");

		DTCPConnectionListIt it(*client->dtcpManager()->connectionList());
		for(DTCPConnection *c; (c = it.current()); ++it) {
			QString s;
			if(c->state() == DTCPConnection::Idle)
				s = "Idl";
			else if(c->state() == DTCPConnection::Asking)
				s = "Ask";
			else if(c->state() == DTCPConnection::Establishing)
				s = "Est";
			else if(c->state() == DTCPConnection::Waiting)
				s = "Wai";
			else if(c->state() == DTCPConnection::Active)
				s = "Act";

			QString dstr; dstr.sprintf("%s %s [%s]\n", s.latin1(), c->peer().full().latin1(), c->key().latin1());
			console->append(dstr);
		}

		console->append("---- DTCPConnection listing end   ----\n\n");
	}
	else if(argv[0] == "debug") {
		if(argc < 2) {
			console->append("debug [on/off]\n");
		}
		else {
			if(argv[1] == "on")
				use_debug = true;
			else
				use_debug = false;
		}
	}
	else if(argv[0] == "test") {
#ifdef TEST_PRIVATE
		#include "private.cpp"
                console->append("Connecting...\n");
                client->connectToHost(host);
#else
		console->append("No private test code.\n");
#endif
	}
	else if(argv[0] == "clear") {
		system("clear");
	}
	else if(argv[0] == "help") {
		QString str;
		str += "  login [host] [user] [pass]      connect\n";
		str += "  logout                          disconnect\n";
		str += "  w [online|away|offline|all]     display roster (default=online)\n";
		str += "  m [to] [words...]               send message\n";
		str += "  jstest [jid]                    jidstream test\n";
		str += "  jslist                          list jidstreams\n";
		str += "  dtcplist                        list dtcp connections\n";
		str += "  debug [on/off]                  toggle debug (default=on)\n";
		str += "  test                            test login\n";
		str += "  clear                           clear the screen\n";
		str += "  help                            this help text\n";
		str += "  quit                            quit the program\n";
		str += "\n";
		console->append(str);
	}
	else {
		console->append(QString("Unrecognized command: [%1].  Try 'help'.\n").arg(argv[0]));
	}
}

void JabberTest::clientDebug(const QString &str)
{
	if(use_debug)
		console->append(str);
}

void JabberTest::clientError(const StreamError &e)
{
	console->append(QString("%1\n").arg(e.toString()));

	if(e.isWarning())
		client->continueAfterWarning();
}

void JabberTest::clientCloseFinished()
{
	console->append("Disconnected.\n");
}

void JabberTest::clientHandshaken()
{
	console->append("/\\/\\/ handshaken!\n\nLogging in...\n");

	client->authDigest(user, pass, "Test");
}

void JabberTest::clientResourceAvailable(const Jid &j, const Resource &r)
{
	LiveRoster::ConstIterator it = client->roster().find(j, false);
	if(it == client->roster().end())
		return;
	const LiveRosterItem &i = *it;

	QString str;
	if(i.name().isEmpty())
		str = i.jid().full();
	else
		str = i.name();

	console->append(QString("%1 is online (%2)\n").arg(str).arg(r.name()));
}

void JabberTest::clientResourceUnavailable(const Jid &j, const Resource &r)
{
	LiveRoster::ConstIterator it = client->roster().find(j, false);
	if(it == client->roster().end())
		return;
	const LiveRosterItem &i = *it;

	QString str;
	if(i.name().isEmpty())
		str = i.jid().full();
	else
		str = i.name();

	console->append(QString("%1 went offline (resource = [%2])\n").arg(str).arg(r.name()));
}

void JabberTest::clientMessageReceived(const Message &m)
{
	QString str = "[Message Received]\n";
	str += QString("From: ") + m.from().full() + '\n';
	str += QString("Subj: ") + m.subject() + '\n';
	str += QString("Body: ") + m.body() + '\n';
	str += '\n';
	console->append(str);
}

void JabberTest::authFinished(bool success, int code, const QString &str)
{
	if(success) {
		QString p = client->user() + '@' + client->host() + " > ";
		console->setPrompt(p);
		console->erasePrompt();
		console->reshowPrompt();
		console->append("/\\/\\/ Success :-)\n\nRequesting roster...\n");
		client->rosterRequest();
	}
	else {
		console->append(QString("/\\/\\/ Failed.  Reason: [%1] %1\n").arg(code).arg(str));
		client->close();
	}
}

void JabberTest::rosterRequestFinished(bool success, int code, const QString &str)
{
	if(success) {
		const LiveRoster &r = client->roster();
		int x = r.count();
		console->append(QString("/\\/\\/ Got %1 entries.\n\n").arg(x));
	}
	else {
		console->append(QString("/\\/\\/ Failed.  Reason: [%1] %1\n\n").arg(code).arg(str));
	}

	client->send("<presence/>\n");
}

void JabberTest::changePasswordFinished()
{
	JT_Register *r = (JT_Register *)sender();
	if(r->success())
		printf("password changed.\n");
	else
		printf("failed.\n");
}

void JabberTest::displayRoster(int type)
{
	const LiveRoster &r = client->roster();
	for(LiveRoster::ConstIterator it = r.begin(); it != r.end(); ++it) {
		const LiveRosterItem &i = *it;

		QString substr;
		switch(i.subscription().type()) {
			case Subscription::Both:
				substr = "<-->";  break;
			case Subscription::From:
				substr = "  ->";  break;
			case Subscription::To:
				substr = "<-  ";  break;
			case Subscription::None:
			default:
				substr = "----";  break;
		}

		QString statusStr;
		ResourceList::ConstIterator it = i.priority();
		int state;
		if(it == i.resourceList().end()) {
			statusStr = "Offline";
			state = 2;
		}
		else {
			if((*it).status().isAway()) {
				statusStr = "Away";
				state = 1;
			}
			else {
				statusStr = "Online";
				state = 0;
			}
		}

		if(type != 3) {
			if(state != type)
				continue;
		}

		QString str;
		str.sprintf("  %7s ", statusStr.latin1());
		str += substr + ' ';
		if(!i.name().isEmpty())
			str += i.name() + ' ';
		str += QString("<") + i.jid().full() + '>';
		str += '\n';

		console->append(str);
	}
	console->append("\n");
}

void JabberTest::bail()
{
	console->erasePrompt();
	printf("quitting.\n");
	client->close();
	quit();
}

