/*
 * qssl.h - Qt OpenSSL plugin
 * Copyright (C) 2001, 2002  Justin Karneges
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef QSSL_H
#define QSSL_H

#include<qobject.h>

class QString;
class QSSLFilter;

class QSSL
{
public:
	QSSL() {}
	virtual ~QSSL() {}

	virtual QSSLFilter *createFilter()=0;
};

#ifdef Q_WS_WIN
extern "C" __declspec(dllexport) QSSL *createQSSL();
#else
extern "C" QSSL *createQSSL();
#endif

class QSSLFilter : public QObject
{
	Q_OBJECT

public:
	QSSLFilter() {}
	virtual ~QSSLFilter() {}

	virtual void reset()=0;
	virtual bool begin()=0;

	// send data
	virtual void send(const QByteArray &)=0;
	// check/recv data
	virtual bool isRecvData()=0;
	virtual QByteArray recv()=0;

	// pass incoming socket data to this function
	virtual void putIncomingSSLData(const QByteArray &)=0;
	// check/read outgoing socket data with this function
	virtual bool isOutgoingSSLData()=0;
	virtual QByteArray getOutgoingSSLData()=0;

	virtual void setSSLTrustedCertStoreDir(const QString &)=0;

	virtual void continueAfterWarning()=0;

signals:
	void handshaken();
	void readyRead();
	void outgoingSSLDataReady();
	void error(bool isWarning, const QString &);
};

#endif
