#ifndef QSSL_P_H
#define QSSL_P_H

#include"qssl.h"

class _QSSL : public QSSL
{
public:
	_QSSL();
	~_QSSL();

	QSSLFilter *createFilter();
//! \if _hide_doc_
private:
	class QSSLPrivate *d;
//! \endif _hide_doc_
};

class _QSSLFilter : public QSSLFilter
{
	Q_OBJECT

public:
	_QSSLFilter();
	~_QSSLFilter();

	void reset();
	bool begin();

	// send data
	void send(const QByteArray &);
	// check/recv data
	bool isRecvData();
	QByteArray recv();

	// pass incoming socket data to this function
	void putIncomingSSLData(const QByteArray &);
	// check/read outgoing socket data with this function
	bool isOutgoingSSLData();
	QByteArray getOutgoingSSLData();

	void setSSLTrustedCertStoreDir(const QString &);

	void continueAfterWarning();

signals:
	void handshaken();
	void readyRead();
	void outgoingSSLDataReady();
	void error(bool isWarning, const QString &);

private slots:
	void sslUpdate();

private:
	enum { SSL_IDLE, SSL_CONNECT, SSL_HANDSHAKING, SSL_ACTIVE };
	void finishHandshake();
	void processSendQueue();
	void sslReadAll();
	void doError();
	void doWarning();
	QString sslErrMessage(int errcode);

	//! \if _hide_doc_
	class QSSLFilterPrivate *d;
	//! \endif
};

#endif
