/***************************************************************************
                   dlgjabberprefs.cpp  -  Jabber Preferences Dialog
                             -------------------
    begin                : Sun Aug 25 2002
    copyright            : (C) 2002 by Daniel Stone, Till Gerken,
                           Kopete Development team
    email                : dstone@kde.org, till@tantalo.net
 ***************************************************************************

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qcheckbox.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include "dlgpreferences.h"
#include "jabberprotocol.h"
#include "dlgjabberprefs.h"

DlgJabberPrefs::DlgJabberPrefs(JabberProtocol *owner, QWidget *parent, const char *name) : DlgPreferences(parent, name)
{

	plugin = owner;

	// Connection TAB
	connect (mID, SIGNAL(textChanged(const QString &)), this, SLOT(configChanged()));
	connect (mPass, SIGNAL(textChanged(const QString &)), this, SLOT(configChanged()));
	connect (mResource, SIGNAL(textChanged(const QString &)), this, SLOT(configChanged()));
	connect (mServer, SIGNAL(textChanged(const QString &)), this, SLOT(configChanged()));
	connect (mPort, SIGNAL(valueChanged(int)), this, SLOT(configChanged()));
	connect (mAutoConnect, SIGNAL(toggled(bool)), this, SLOT(configChanged()));
	connect (chkUseSSL, SIGNAL(toggled(bool)), this, SLOT(configChanged()));
	
	// Chat TAB
	connect (mPreferEmail, SIGNAL(toggled(bool)), this, SLOT(configChanged()));
	connect (mLogAll, SIGNAL(toggled(bool)), this, SLOT(configChanged()));

	connect (cmbAuth, SIGNAL(activated(int)), this, SLOT(configChanged()));

	connect (cbProxyType, SIGNAL(activated(int)), this, SLOT(configChanged()));
	connect (leProxyName, SIGNAL(textChanged(const QString &)), this, SLOT(configChanged()));
	connect (spbProxyPort, SIGNAL(valueChanged(int)), this, SLOT(configChanged()));
	connect (cbProxyAuth, SIGNAL(toggled(bool)), this, SLOT(configChanged()));
	connect (leProxyUser, SIGNAL(textChanged(const QString &)), this, SLOT(configChanged()));
	connect (leProxyPass, SIGNAL(textChanged(const QString &)), this, SLOT(configChanged()));

	connect (btnRegister, SIGNAL(clicked()), this, SLOT(registerClicked()));
	connect (chkUseSSL, SIGNAL(toggled(bool)), this, SLOT(sslToggled(bool)));

}

DlgJabberPrefs::~DlgJabberPrefs()
{

}

void DlgJabberPrefs::configChanged()
{
	settings_changed = true;
}

void DlgJabberPrefs::registerClicked()
{

	plugin->registerUser();

}

void DlgJabberPrefs::sslToggled(bool value)
{

	if (value)
		mPort->stepUp();
	else
		mPort->stepDown();

}

#include "dlgjabberprefs.moc"

/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 8
 * indent-tabs-mode: t
 * End:
 */
// vim: set noet ts=4 sts=4 sw=4:
