/*
    msnpreferences.cpp - MSN Preferences Widget

    Copyright (c) 2002 by Duncan Mac-Vicar Prett <duncan@kde.org>
    Copyright (c) 2002 by Martijn Klingens       <klingens@kde.org>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include "msnpreferences.h"

#include <qcheckbox.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlayout.h>

#include <kconfig.h>
#include <kdebug.h>
#include <kglobal.h>
#include <klineedit.h>
#include <klocale.h>
#include <knuminput.h>

MSNPreferences::MSNPreferences( const QString &pixmap, QObject *parent )
: ConfigModule( i18n( "MSN Plugin" ), i18n( "Microsoft Network Protocol" ),
	pixmap, parent )
{
	( new QVBoxLayout( this ) )->setAutoAdd( true );
	preferencesDialog = new msnPrefsUI(this);
	reopen();
}

MSNPreferences::~MSNPreferences()
{
}

void MSNPreferences::reopen()
{
	KGlobal::config()->setGroup("MSN");
	preferencesDialog->mID->setText(
		KGlobal::config()->readEntry( "UserID", QString::null ) );
	preferencesDialog->mPass->setText(
		KGlobal::config()->readEntry( "Password", QString::null ) );
	preferencesDialog->mServer->setText(
		KGlobal::config()->readEntry( "Server", "messenger.hotmail.com" ) );
	preferencesDialog->mPort->setValue(
		KGlobal::config()->readNumEntry( "Port", 1863 ) );
	preferencesDialog->mAutoConnect->setChecked(
		KGlobal::config()->readBoolEntry( "AutoConnect", false ) );
	preferencesDialog->mLogAll->setChecked(
		KGlobal::config()->readBoolEntry( "LogAll", true ) );
//	preferencesDialog->mExportGroups->setChecked(
//		KGlobal::config()->readBoolEntry( "ExportGroups", false ) );
	preferencesDialog->mMailNotifications->setChecked(
		KGlobal::config()->readBoolEntry( "MailNotifications", true ) );
	preferencesDialog->mRaiseWindow->setChecked(
		KGlobal::config()->readBoolEntry( "OpenWindow", true ) );
                              
}


void MSNPreferences::save()
{
	KConfig *config=KGlobal::config();
	config->setGroup("MSN");
	config->writeEntry("UserID", preferencesDialog->mID->text());
	config->writeEntry("Password", preferencesDialog->mPass->text());
	config->writeEntry("Server", preferencesDialog->mServer->text());
	config->writeEntry("Port", preferencesDialog->mPort->text());
	config->writeEntry("AutoConnect", preferencesDialog->mAutoConnect->isChecked());
	config->writeEntry("LogAll", preferencesDialog->mLogAll->isChecked());
//	config->writeEntry("ExportGroups", preferencesDialog->mExportGroups->isChecked());
	config->writeEntry("MailNotifications", preferencesDialog->mMailNotifications->isChecked());
	config->writeEntry("OpenWindow", preferencesDialog->mRaiseWindow->isChecked());
	config->sync();
	emit saved();

}

QString MSNPreferences::msnId()
{
	KGlobal::config()->setGroup("MSN");
	return KGlobal::config()->readEntry( "UserID", "" );

}

QString MSNPreferences::password()
{
	KGlobal::config()->setGroup("MSN");
	return KGlobal::config()->readEntry( "Password", "" );
}

QString MSNPreferences::publicName()
{
	KGlobal::config()->setGroup("MSN");
	return KGlobal::config()->readEntry( "Nick", "Kopete User" );
}

bool MSNPreferences::autoConnect()
{
	KGlobal::config()->setGroup("MSN");
	return KGlobal::config()->readBoolEntry( "AutoConnect", false );
}

/*bool MSNPreferences::exportGroups()
{
	//I put this false by default because export groups can made error.
	//currently, there are no way to delete groups whit kopete
	//and don't forget that we can create MAX 30 groups (Olivier)
	KGlobal::config()->setGroup("MSN");
	return KGlobal::config()->readBoolEntry("ExportGroups", false );
}*/

void MSNPreferences::setPublicName(QString s)
{
	KConfig *config=KGlobal::config();
	config->setGroup( "MSN" );
	config->writeEntry( "Nick", s );
	config->sync();
}

bool MSNPreferences::openWindow()
{
	KGlobal::config()->setGroup( "MSN" );
	return KGlobal::config()->readBoolEntry( "OpenWindow", true );
}

bool MSNPreferences::mailNotifications()
{
	KGlobal::config()->setGroup( "MSN" );
	return KGlobal::config()->readBoolEntry( "MailNotifications", true );
}

#include "msnpreferences.moc"

// vim: set noet ts=4 sts=4 sw=4:
