/*
    oscarpreferences.cpp  -  Oscar Protocol Plugin

    Copyright (c) 2002 by Tom Linsky <twl6@po.cwru.edu>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include "oscarpreferences.h"
#include "oscarpreferences.moc"
#include "oscarsocket.h"

#include <qlayout.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qcheckbox.h>

#include <kglobal.h>
#include <klocale.h>
#include <kdebug.h>

OscarPreferences::OscarPreferences(const QString &pixmap,QObject *parent)
							: ConfigModule(i18n("Oscar Plugin"),i18n("AOL Oscar Protocol"),pixmap,parent)
{
	cfg = KGlobal::config();

	( new QVBoxLayout( this ) )->setAutoAdd( true );
	preferencesDialog = new OscarPrefsUI(this);

	load();
}

OscarPreferences::~OscarPreferences()
{
}

void OscarPreferences::load ( void )
{
	QString tempUser, tempPW;
	bool tempAutoConnect, tempLogAll;

	tempUser = screenName();
	tempPW = password();
	tempAutoConnect	= autoConnect();
	tempLogAll = logAll();
	mProfile = profile();
	/* Logic for importing old aim preferences ================== */
	// Try to read old AIM settings if OSCAR is not configured and AIM group is found in config
	if ( tempUser == i18n("Your AIM screenname here") && tempPW == ""  && cfg->hasGroup("AIM") )
	{
		cfg->setGroup("AIM");
		tempUser			= cfg->readEntry ( "ScreenName", i18n("Your AIM screenname here") );
		tempPW				= cfg->readEntry ( "Password", "" );
		tempAutoConnect		= cfg->readBoolEntry("AutoConnect", false);
		tempLogAll				= cfg->readBoolEntry("LogAll", true);

		if  ( tempUser != i18n("Your AIM screenname here") ) // read some usable screenname
		{
			kdDebug(14150) << "[OscarPreferences] SUCCESS importing old aim-prefs, removing old AIM-group from Config" << endl;
			cfg->deleteGroup ( "AIM", true ); // remove whole AIM-Group from config
			cfg->sync();
		}
		cfg->setGroup("Oscar");
	}
	/* =========================================================== */

	preferencesDialog->mSN->setText( tempUser );
	preferencesDialog->mPass->setText( tempPW );

	preferencesDialog->mServer->setText( server() );
	preferencesDialog->mPort->setValue( port() );

	preferencesDialog->mAutoConnect->setChecked( tempAutoConnect );
	preferencesDialog->mReportIdle->setChecked( reportIdle() );
	preferencesDialog->mIdleTimeout->setValue( idleTimeout() );

	preferencesDialog->chkLogAll->setChecked( tempLogAll );
}

void OscarPreferences::save()
{
	cfg->setGroup("Oscar");
	cfg->writeEntry("ScreenName", preferencesDialog->mSN->text());
	cfg->writeEntry("Password", preferencesDialog->mPass->text());
	cfg->writeEntry("Server", preferencesDialog->mServer->text());
	cfg->writeEntry("Port", preferencesDialog->mPort->text());
	cfg->writeEntry("LogAll", preferencesDialog->chkLogAll->isChecked());
	cfg->writeEntry("AutoConnect", preferencesDialog->mAutoConnect->isChecked());
	cfg->writeEntry("ReportIdle", preferencesDialog->mReportIdle->isChecked());
	cfg->writeEntry("IdleTimeout", preferencesDialog->mIdleTimeout->text());
	cfg->writeEntry("UserProfile", mProfile);
	cfg->sync();

	emit saved();
}


/** sets the user's profile! */
void OscarPreferences::setProfile( const QString &prof )
{
	mProfile = prof;
	cfg->setGroup("Oscar");
	cfg->writeEntry("UserProfile", mProfile);
	cfg->sync();
}

/** Returns true if we should log all conversations */
bool OscarPreferences::logAll() const
{
	cfg->setGroup( "Oscar" );
	return cfg->readBoolEntry( "LogAll", true );
}

/** Returns true if we should report idle time to server, false otherwise */
bool OscarPreferences::reportIdle() const
{
	cfg->setGroup( "Oscar" );
	return cfg->readBoolEntry( "ReportIdle", true );
}

/** Amount of time we should wait before reporting idle time to server */
int OscarPreferences::idleTimeout() const
{
	cfg->setGroup( "Oscar" );
	return cfg->readNumEntry( "IdleTimeout", 10 );
}

/** Gets the screen name from the preferences and returns it */
QString OscarPreferences::screenName()
{
	cfg->setGroup("Oscar");
	return cfg->readEntry ( "ScreenName", i18n("Your AIM screenname here") );
}

/** Gets the password from the preferences and returns it */
QString OscarPreferences::password()
{
	cfg->setGroup("Oscar");
	return cfg->readEntry ( "Password", "" );
}

/** Gets the autoconnect setting from the preferences and returns it */
bool OscarPreferences::autoConnect()
{
	cfg->setGroup("Oscar");
	return cfg->readBoolEntry("AutoConnect", false);
}

/** Gets the user profile from the preferences and returns it */
QString OscarPreferences::profile()
{
	cfg->setGroup("Oscar");
	return cfg->readEntry( "UserProfile", i18n("Visit the kopete Web site at <a href=http://kopete.kde.org>http://kopete.kde.org</a>") );
}

/** Reads the oscar server from the preferences and returns it */
QString OscarPreferences::server()
{
	cfg->setGroup("Oscar");
	return cfg->readEntry("Server", OSCAR_SERVER);
}

/** Reads the server port from the preferences and returns it */
int OscarPreferences::port()
{
	cfg->setGroup("Oscar");
	return cfg->readNumEntry("Port", OSCAR_PORT);
}

// vim: set ts=4 sts=4 sw=4 noet:

