/*
    oscarpreferences.h  -  Oscar Protocol Plugin

    Copyright (c) 2002 by Tom Linsky <twl6@po.cwru.edu>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#ifndef OSCARPREFERENCES_H
#define OSCARPREFERENCES_H

#include <configmodule.h>
#include <kconfig.h>
#include "oscarprefs.h"

class OscarPrefsUI;
class KConfig;

class OscarPreferences : public ConfigModule
{
	Q_OBJECT

	public:
		OscarPreferences ( const QString &pixmap, QObject *parent = 0 );
		~OscarPreferences();

    /** Returns true if we should log all conversations */
		bool logAll() const;
		/** Returns true if we should report idle time to server, false otherwise */
		bool reportIdle() const;
		/** Amount of time we should wait before reporting idle time to server */
		int idleTimeout() const;

		virtual void save();
		void load();

		/** sets the user's profile! */
		void setProfile( const QString &prof );

	  /** Gets the autoconnect setting from the preferences and returns it */
	  bool autoConnect();
	  /** Gets the password from the preferences and returns it */
	  QString password();
	  /** Gets the screen name from the preferences and returns it */
  	QString screenName();
	  /** Reads the server port from the preferences and returns it */
  	int port();
  	/** Reads the oscar server from the preferences and returns it */
  	QString server();
  	/** Gets the user profile from the preferences and returns it */
  	QString profile();

	signals:
		void saved();

	private:
		OscarPrefsUI *preferencesDialog;
		KConfig *cfg;

		QString mProfile;
};

#endif
