/*
    oscaruserinfo.cpp  -  Oscar Protocol Plugin

    Copyright (c) 2002 by Tom Linsky <twl6@po.cwru.edu>

    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include "oscaruserinfo.h"
#include "oscaruserinfo.moc"

#include "oscarprotocol.h"

#include <qlayout.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qstring.h>

#include <klocale.h>
#include <kmessagebox.h>
#include <kstandarddirs.h>
#include <ktextbrowser.h>
#include <kdebug.h>
#include <kapplication.h>

OscarUserInfo::OscarUserInfo(const QString name, const QString nick,
		OscarProtocol *protocol, AIMBuddy &buddy)
		: OscarUserInfoBase()
{
//	TODO: Let me know if this has problems when commented --Nick: setWFlags(Qt::WType_Modal);
	// Chris: What do these do?
	QMimeSourceFactory::defaultFactory()->addFilePath(
			kapp->dirs()->findDirs("data","kopete/")[0]);
	QMimeSourceFactory::defaultFactory()->addFilePath(
			kapp->dirs()->findDirs("data","kopete/pics/")[0]);
	mName = name;
	mNick = nick;
	mProtocol = protocol;
	// Set the caption
	setCaption( i18n("User Information on %1").arg(name) );

	// Save
	QObject::connect(cmdSave, SIGNAL(clicked()),
			this, SLOT(slotSaveClicked()));
	// Close
	QObject::connect(cmdClose, SIGNAL(clicked()),
			this, SLOT(slotCloseClicked()));
	// Engine got user profile
	QObject::connect(mProtocol->engine, SIGNAL(gotUserProfile(UserInfo,QString)),
			this, SLOT(slotSearchFound(UserInfo, QString)));

	screenNameLabel->setText(name);
	if (nick.isEmpty()){
		nickNameLE->setText(name);
	} else {
		nickNameLE->setText(nick);
	}

	// If we're conneced
	if (mProtocol->isConnected()){
		// And our buddy is not offline
		if (buddy.status() != TAIM_OFFLINE){
			QString location = locate("data","kopete/loading.html");
			location.prepend("/");
			QFile fd(location);
			if (fd.open(IO_ReadOnly)){
				QTextStream s(&fd);
				QString loadingCode = s.read();
				userInfoView->setText(loadingCode);
				fd.close();
			}
			mProtocol->engine->sendUserProfileRequest(name);
		}
	}
}

// This constructor is called when we want to edit
// our own profile
OscarUserInfo::OscarUserInfo(const QString name, const QString nick,
		OscarProtocol *protocol, const QString &profile)
		: OscarUserInfoBase()
{
	QMimeSourceFactory::defaultFactory()->addFilePath(
			kapp->dirs()->findDirs("data","kopete/")[0]);
	QMimeSourceFactory::defaultFactory()->addFilePath(
			kapp->dirs()->findDirs("data","kopete/pics/")[0]);

	mName = name;
	mNick = nick;
	setCaption( i18n("User Information on %1").arg(name) );

	mProtocol = protocol;
	// Save
	QObject::connect(cmdSave, SIGNAL(clicked()),
			this, SLOT(slotSaveClicked()));
	// Close
	QObject::connect(cmdClose, SIGNAL(clicked()),
			this, SLOT(slotCloseClicked()));

	screenNameLabel->setText(name);
	if (nick.isEmpty()){
		nickNameLE->setText(name);
	} else {
		nickNameLE->setText(nick);
	}
	cmdSave->setText("&Save profile");
	userInfoView->setReadOnly(false);
	userInfoView->setTextFormat(PlainText);
	userInfoView->setText(profile);
}

void OscarUserInfo::slotSaveClicked()
{
	emit updateNickname(nickNameLE->text());
	setCaption( i18n("User Information on %1").arg(nickNameLE->text()) );
	if (!userInfoView->isReadOnly()){
		mProtocol->setUserProfile(userInfoView->text());
	}
}

void OscarUserInfo::slotCloseClicked()
{
	delete this;
}

void OscarUserInfo::slotSearchFound(UserInfo /*u*/, QString profile)
{
	userInfoView->setText(profile);
	//disconnect so we can have more than one user profile window open with
	//different users' info in them
	QObject::disconnect(mProtocol->engine, SIGNAL(gotUserProfile(UserInfo,QString)),
			this, SLOT(slotSearchFound(UserInfo, QString)));
}
