#include "smspreferencesbase.h"
#include "serviceloader.h"
#include "smsglobal.h"
#include "smscontact.h"

#include <qlayout.h>
#include <qcombobox.h>
#include <qpushbutton.h>

#include <kmessagebox.h>
#include <klocale.h>

SMSPreferencesBase::SMSPreferencesBase( SMSContact* contact, QWidget* parent, const char* name )
	: QWidget( parent, name )
{
	(new QVBoxLayout(this, QBoxLayout::Down))->setAutoAdd(true);

	preferencesDialog = new smsPrefsUI(this);

	service = 0L;
	configWidget = 0L;
	m_contact = contact;

	preferencesDialog->serviceName->insertStringList(ServiceLoader::services());

	connect (preferencesDialog->serviceName, SIGNAL(activated(const QString &)),
		this, SLOT(setServicePreferences(const QString &)));
	connect (preferencesDialog->descButton, SIGNAL(clicked()),
		this, SLOT(showDescription()));

	reopen();
}

SMSPreferencesBase::~SMSPreferencesBase()
{
	if (service != 0L)
		delete service;
}

void SMSPreferencesBase::reopen()
{
	QString sName = SMSGlobal::readConfig("SMS", "ServiceName", m_contact);

	for (int i=0; i < preferencesDialog->serviceName->count(); i++)
	{
		if (preferencesDialog->serviceName->text(i) == sName)
		{
			preferencesDialog->serviceName->setCurrentItem(i);
			break;
		}
	}
	setServicePreferences(preferencesDialog->serviceName->currentText());
}

void SMSPreferencesBase::save()
{
	SMSGlobal::writeConfig("SMS", "ServiceName", m_contact,
		preferencesDialog->serviceName->currentText());

	emit saved();
}

void SMSPreferencesBase::setServicePreferences(const QString& serviceName)
{
	if (service != 0L)
		delete service;

	if (configWidget != 0L)
		delete configWidget;
	
	service = ServiceLoader::loadService(serviceName, m_contact);

	if ( service == 0L)
		return;
	
	connect (this, SIGNAL(saved()), service, SLOT(savePreferences()));

	configWidget = service->configureWidget(this);
	configWidget->show();
}

void SMSPreferencesBase::showDescription()
{
	SMSService* s = ServiceLoader::loadService(preferencesDialog->serviceName->currentText(), 0L);

	QString d = s->description();

	KMessageBox::information(0L, d, i18n("Description"));
}

#include "smspreferencesbase.moc"
/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 8
 * indent-tabs-mode: t
 * End:
 */
// vim: set noet ts=4 sts=4 sw=4:

